#import "CategorylistTableView.h"
#import "ViewCategory.h"

@implementation CategorylistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

    // Set click action
	[self setTarget:self];
	[self setAction:@selector(clickItem:)];

	// Set sort descriptors
    NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"title" ascending:YES selector:NSSelectorFromString(@"caseInsensitiveCompare:")];
    [categorylistArrayController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
    [sortDescriptor release];

}
//=======================================================================
// action
//=======================================================================
//------------------------------------
// clickItem
//------------------------------------
- (IBAction)clickItem:(id)sender
{

	[viewCategory searchItem:nil];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// deselectItem
//------------------------------------
- (void)deselectItem
{
	[self deselectRow:[self selectedRow]];
}
//=======================================================================
// delegate
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
        objectValueForTableColumn:(NSTableColumn *)aTableColumn
        row:(int)rowIndex
{

	// get record
//	id record = [[categorylistArrayController arrangedObjects] objectAtIndex:rowIndex];

	// icon
	if([[aTableColumn identifier] isEqualToString:@"icon"]){
		return [NSImage imageNamed: @"icon_playlist"];
	}

    return nil;
}

//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[categorylistArrayController arrangedObjects] count];
}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{    

	// not display
	if([self isDisplay] == NO){
		return;
	}

	BOOL isAction = NO;
	BOOL isSelect = NO;
	NSString *keys = [theEvent charactersIgnoringModifiers];
	int row = 0;

	if (keys && [keys length] > 0){
		unichar c = [keys characterAtIndex:0];
		if(c == NSUpArrowFunctionKey){
			if([self selectedRow] == -1){
				row = [self numberOfRows] - 1;
			}else{
				row = [self selectedRow] - 1;
			}
			if(row >= 0){
				[self selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
				isSelect = YES;
			}
			isAction = YES;
		}
		else if(c == NSDownArrowFunctionKey){
			if([self selectedRow] == -1){
				row = 0;
			}else{
				row = [self selectedRow] + 1;
			}
			if(row < [self numberOfRows]){
				[self selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
				isSelect = YES;
			}
			isAction = YES;
		}
	}

	// search
	if(isSelect == YES){
		[viewCategory searchItem:nil];
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}

}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmCategorylist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
//--------------------------------------
// isDisplay
//--------------------------------------
- (BOOL)isDisplay
{
	NSSize viewSize = [[self enclosingScrollView] contentSize];
//	NSLog(@"width=%f", viewSize.width);
//	NSLog(@"height=%f", viewSize.height);

	if(viewSize.width > 0 && viewSize.height > 0){
		return YES;
	}else{
		return NO;
	}
}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[super dealloc];
}
@end
