#import "ClonelistTableView.h"
#import "ViewClone.h"
#import "RoundImageCell.h"
#import "ItemDescTextCell.h"
#import "TableColumnManager.h"
#import "TableColumnExtension.h"
#import "ThumbsExtension.h"
#import "CellAttributeExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "ImageBindExtension.h"
#import "ConvertExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation ClonelistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

	// set row height
	float rowHeight = [self defaultFloatValue:@"optRowHeightClone"];
	[sliderRowHeight setFloatValue:rowHeight];
	[self setRowHeight:rowHeight];

	// image column
	NSTableColumn *imageColumn = [self tableColumnWithIdentifier:@"image"];
	RoundImageCell *imageCell = [[[RoundImageCell alloc] init] autorelease];
	[imageColumn setDataCell:imageCell];

	// description column
	NSTableColumn *descColumn = [self tableColumnWithIdentifier:@"description"];
	ItemDescTextCell *descTextCell = [[[ItemDescTextCell alloc] init] autorelease];
	[descColumn setDataCell:descTextCell];

	// set click action
	[self setTarget:viewClone];
	[self setDoubleAction:@selector(playItem:)];

	// change description column & sort descriptor
	[self setColumnDescriptionHeaderTitle:self langType:[self defaultLangType]];
	[self setColumnDescriptionSortDescriptor:self
									aController:clonelistArrayController
									langType:[self defaultLangType]
	];

	// set table column state
	[self setTableColumnState:self key:@"stateTableColumnClonelist4_0"];

	// init column controller
	if (tcm_ == nil) {
		tcm_ = [[TableColumnManager alloc] initWithTable:self];
	}

	// showTableColumns
	// set default hide columns
	NSArray *defaultHideCols = [NSArray arrayWithObjects:
									@"auditIcon",
									@"auditStatus",
									@"addVersion",
									nil
								];
	[self showTableColumns:tcm_ aTableView:self defaultHideCols:defaultHideCols key:@"stateTableColumnClonelist4_0"];

	// set sort descriptor
	[self setArrayControllerSortDescriptor:clonelistArrayController key:@"sortDescColumnClonelist4_0"];

	// set header menu
	[[self headerView] setMenu:cmClonelistHeader];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[self window]];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED_CLONE object:nil];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_STATUS_DID_CHANGED_CLONE object:nil];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_CLONE object:nil];
	[nc addObserver:self selector:@selector(handleLangTypeDidChanged:) name:DEFAULT_NOTIF_LANG_TYPE_DID_CHANGED object:nil];

	// set registerForDraggedTypes
//	[self registerForDraggedTypes:[NSArray arrayWithObjects:ClonelistPboardType, nil]];

	// set drag operation
//	[self setDraggingSourceOperationMask:NSDragOperationLink forLocal:NO];

}
//------------------------------------
// changeRowHeight
//------------------------------------
- (IBAction)changeRowHeight:(id)sender
{
	[self setRowHeight:[sender floatValue]];

	if([self numberOfSelectedRows] > 0){
		[self scrollRowToVisible:[self selectedRow]];
	}

}
//------------------------------------
// change table column state
//------------------------------------
- (IBAction)changeColumnState:(id)sender;
{
	[self setColumnState:tcm_ sender:sender];
}
//------------------------------------
// isShowColumn
//------------------------------------
- (BOOL)isShowColumn:(NSString*)identifier
{
	return [self isShowColumnState:tcm_ identifier:identifier];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{

	// table column state
	[self saveTableColumnState:self key:@"stateTableColumnClonelist4_0"];
	// save sort descriptor
	[self saveArrayControllerSortDescriptor:clonelistArrayController key:@"sortDescColumnClonelist4_0"];

}
//------------------------------------
// handleItemInfoDidChanged
//------------------------------------
-(void)handleItemInfoDidChanged:(NSNotification *)notification
{
	[self reloadData];
}
//------------------------------------
// handleLangTypeDidChanged
//------------------------------------
-(void)handleLangTypeDidChanged:(NSNotification *)notification
{
	// change description column & sort descriptor
	[self setColumnDescriptionHeaderTitle:self langType:[self defaultLangType]];
	[self setColumnDescriptionSortDescriptor:self
									aController:clonelistArrayController
									langType:[self defaultLangType]
	];

//	[self reloadData];
	[clonelistArrayController rearrangeObjects];
}
//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
		objectValueForTableColumn:(NSTableColumn *)aTableColumn
		row:(int)rowIndex
{

	// get record
	id record = [[clonelistArrayController arrangedObjects] objectAtIndex:rowIndex];

	NSString *curRomId = [record valueForKey: @"cloneId"];
	BOOL isExist = [[record valueForKey: @"isExist"] boolValue];
/*
	BOOL isExist = NO;
	if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
		isExist = YES;
	}
*/
/*
	// status
	if([[aTableColumn identifier] isEqualToString:@"statusIcon"]){
		return [self getStatusIconImage:YES isExist:isExist];
	}
*/
	// image
	if([[aTableColumn identifier] isEqualToString:@"image"]){

		// get thumbs image
		NSImage *image = [self getThumbsImage:[self getThumbsImagePath:curRomId
																imageType:[self defaultThumbsImageTypeClone]
																fileIndex:0
																maxIndex:nil
												]
									hasImage:nil
						];
		// isExist
//		if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
//			isExist = YES;
//		}
		RoundImageCell *imageCell = [aTableColumn dataCell];
		[imageCell setImage:image];
		[imageCell setRomId:curRomId];
		[imageCell setImageType:[self defaultThumbsImageTypeClone]];
		[imageCell setIsExist:isExist];
	}
	// description
	else if([[aTableColumn identifier] isEqualToString:@"description"]){

		NSString *title = @"-";
		NSString *manufacture = @"-";
		NSString *year = @"-";
		int labelColorNo = 0;

		// title
		title = [self getTitleString:[self defaultLangType]
								title:[record valueForKey:@"title"]
								titlejp:[record valueForKey:@"titlejp"]
				];
		manufacture = [record valueForKey:@"manufacture"];
		year = [record valueForKey:@"year"];
		// fileSize
//		NSString *fileSizeStr = @"-";
//		int fileSize = [[record valueForKey: @"fileSize"] intValue];
//		if(fileSize > 0){
//			fileSizeStr = [self convertFileSizeToString:fileSize];
//		}
		// current exist
		BOOL isCurExist = NO;
		if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
			isCurExist = YES;
		}
		labelColorNo = 0;

		// statusIcon
		NSImage *imageStatus = [self getStatusIconImage:YES isExist:isExist isCurExist:isCurExist];

		NSString *desc1 = [NSString stringWithFormat:@"%@ / %@ ", manufacture, year];
//		NSString *desc2 = [NSString stringWithFormat:@"Size: %@", fileSizeStr];
		NSString *desc2 = @"";
		ItemDescTextCell *descTextCell = [aTableColumn dataCell];
		[descTextCell setStringValue:[NSString stringWithFormat:@"%@\t%@\t%@", title, desc1, desc2]];
		[descTextCell setIsExist:isExist];
		[descTextCell setLabelColorNo:labelColorNo];
		[descTextCell setImage:imageStatus];
		return descTextCell;

	}
	// defaultIcon
	else if([[aTableColumn identifier] isEqualToString:@"defaultIcon"]){
		// parent object
		id romObject = [record valueForKey:@"rlt_romlist"];
		if(romObject){
			if([curRomId isEqualToString:[romObject valueForKey:@"defaultRomId"]]){
				return [NSImage imageNamed: @"icon_checked"];
			}
		}
		return nil;
	}
	// fileSize
	else if([[aTableColumn identifier] isEqualToString:@"fileSize"]){
		NSString *fileSizeStr = @"-";
		int fileSize = [[record valueForKey: @"fileSize"] intValue];
		// convert file size
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		return fileSizeStr;
	}
	// auditIcon
	else if([[aTableColumn identifier] isEqualToString:@"auditIcon"]){
		return [self getAuditIconImage:[record valueForKey: @"audit"]];
	}

	return nil;
}
//------------------------------------
// toolTipForCell
//------------------------------------
- (NSString *)tableView:(NSTableView *)aTableView toolTipForCell:(NSCell *)aCell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)aTableColumn row:(int)row mouseLocation:(NSPoint)mouseLocation
{
	if( [[aTableColumn identifier] isEqualToString:@"description"]
	){
		id record = [[clonelistArrayController arrangedObjects] objectAtIndex:row];
		return [self createToolTipString:record isClone:YES];
	}
	return nil;
}
//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[clonelistArrayController arrangedObjects] count];
}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}

//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{	 
	BOOL isAction = NO;

	// no alternateKeyMask
	if(!([theEvent modifierFlags] & NSAlternateKeyMask)){
		// return key
		if([theEvent keyCode] == 36){
			[viewClone playItem:nil];
			isAction = YES;
		}
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmClonelist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
/*
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	int wheelDelta;

	wheelDelta = [theEvent deltaY];

	if(wheelDelta <= 0){
		[clonelistArrayController selectNext:nil];
	}else{
		[clonelistArrayController selectPrevious:nil];
	}
}
*/
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
