#import "ConvertExtension.h"
#import "FileAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation NSObject(convertExtension_)

//------------------------------------
// converToAppHomePath
//------------------------------------
- (NSString*)converToAppHomePath
{
	return [[[NSBundle mainBundle] bundlePath] stringByDeletingLastPathComponent];
}
//------------------------------------
// converToBezelHomePath
//------------------------------------
- (NSString*)converToBezelHomePath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToAppHomePath], @"ExtraBezel"];
}
//------------------------------------
// converToBezelTemplatesPath
//------------------------------------
- (NSString*)converToBezelTemplatesPath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToBezelHomePath], @"Templates"];
}
//------------------------------------
// converToBezelLayoutsPath
//------------------------------------
- (NSString*)converToBezelLayoutsPath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToBezelTemplatesPath], @"Layouts"];
}
//------------------------------------
// converToBezelImagesPath
//------------------------------------
- (NSString*)converToBezelImagesPath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToBezelTemplatesPath], @"Images"];
}
//------------------------------------
// converToBezelIconsPath
//------------------------------------
- (NSString*)converToBezelIconsPath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToBezelTemplatesPath], @"Icons"];
}

//------------------------------------
// converToBezelIconFilePath
//------------------------------------
- (NSString*)converToBezelIconFilePath:(NSString*)fileName
{
	return [NSString stringWithFormat:@"%@/%@.png", [self converToBezelIconsPath], fileName];
}
//------------------------------------
// converToBezelTemplateFilePath
//------------------------------------
- (NSString*)converToBezelTemplateFilePath:(NSString*)fileName
{
	return [NSString stringWithFormat:@"%@/%@.lay", [self converToBezelLayoutsPath], fileName];
}
//------------------------------------
// converToBezelTemplateListPath
//------------------------------------
- (NSString*)converToBezelTemplateListPath;
{
	return [NSString stringWithFormat:@"%@/list.xml", [self converToBezelTemplatesPath]];
}

//------------------------------------
// converToBezelTempPath
//------------------------------------
- (NSString*)converToBezelTempPath
{
	return [NSString stringWithFormat:@"%@/%@", [self converToBezelHomePath], @"BezelsTemp"];
}
//------------------------------------
// converToBezelTempFolderPath
//------------------------------------
- (NSString*)converToBezelTempFolderPath:(NSString*)folderPath romId:(NSString*)romId
{
	return [NSString stringWithFormat:@"%@/%@", folderPath, romId];
}
//------------------------------------
// converToBezelTempFilePath
//------------------------------------
- (NSString*)converToBezelTempFilePath:(NSString*)folderPath romId:(NSString*)romId
{
	return [NSString stringWithFormat:@"%@/%@.lay", folderPath, romId];
}
//------------------------------------
// converToBezelFileNameWithSubImage
//------------------------------------
- (NSString*)converToBezelFileNameWithSubImage:(NSString*)fileName
{
	return [NSString stringWithFormat:@"%@_sub", fileName];
}

//------------------------------------
// convert file size string
//------------------------------------
- (NSString*)convertFileSizeToString:(double)fileSize
{
	NSString *fileSizeName = @"";

	// fileSize < 1MB
/*
	if(fileSize < 1024){
		fileSize = fileSize / 1024;
		fileSizeName = @"KB";
	}
*/
	if(fileSize < (1024 * 1024)){
		fileSize = fileSize / 1024;
		fileSizeName = @"KB";
	}
	// fileSize < 1GB
	else if(fileSize < (1024 * 1024 * 1024)){
		fileSize = fileSize / (1024 * 1024);
		fileSizeName = @"MB";
	}
	else{
		fileSize = fileSize / (1024 * 1024 * 1024);
		fileSizeName = @"GB";
	}

	return [NSString stringWithFormat: @"%.1f %@",fileSize,fileSizeName];

}
//------------------------------------
// convertTimeToString
//------------------------------------
- (NSString*)convertTimeToString:(int)timeSec
{

/*
	output format
	 1:11:11
	   11:11
	    1:11
	    0:11
	    0:01
*/

	NSString *timeStr;
	NSString *hourStr;
	NSString *minStr;
	NSString *secStr;

	int hour = 0;
	int min = 0;
	int sec = 0;

	if(timeSec >= 3600){
		hour = timeSec / 3600;
		timeSec = timeSec % 3600;
	}

	if(timeSec >= 60){
		min = timeSec / 60;
		timeSec = timeSec % 60;
	}

	sec = timeSec;

	// hour format
	hourStr = [NSString stringWithFormat:@"%d",hour];

	// min format
	if(hour > 0){
		if(min < 10){
			minStr = [NSString stringWithFormat:@"0%d",min];
		}else{
			minStr = [NSString stringWithFormat:@"%d",min];
		}
	}else{
		minStr = [NSString stringWithFormat:@"%d",min];
	}

	// sec format
	if(sec < 10){
		secStr = [NSString stringWithFormat:@"0%d",sec];
	}else{
		secStr = [NSString stringWithFormat:@"%d",sec];
	}

	// time format
	if(hour > 0){
		timeStr = [NSString stringWithFormat:@"%@:%@:%@",hourStr, minStr, secStr];
	}else{
		timeStr = [NSString stringWithFormat:@"%@:%@", minStr, secStr];
	}

	return timeStr;

}
//------------------------------------
// convertDateToString
//------------------------------------
- (NSString*)convertDateToString:(NSDate*)date formatString:(NSString*)formatString
{
	NSDateFormatter *formatter = [
					[[NSDateFormatter alloc] 
						initWithDateFormat:formatString
						allowNaturalLanguage:FALSE
					]
	autorelease];

	return [formatter stringFromDate:date];
}
//------------------------------------
// convertToComma
//------------------------------------
- (NSString*)convertToComma:(int)value
{
	NSNumber *number=[NSNumber numberWithInt:value];
	NSNumberFormatter *formatter=[[[NSNumberFormatter alloc] init] autorelease];

//	[formatter setThousandSeparator:@","];
//	[formatter setHasThousandSeparators:YES];
	[formatter setFormat:@"#,##0;0;-#,##0"];

	return [formatter stringForObjectValue:number];
}
//------------------------------------
// convertToZeroFormat
//------------------------------------
- (NSString*)convertToZeroFormat:(int)value
{
	NSNumber *number=[NSNumber numberWithInt:value];
	NSNumberFormatter *formatter=[[[NSNumberFormatter alloc] init] autorelease];

	[formatter setFormat:@"00000000;00000000;-00000000"];

	return [formatter stringForObjectValue:number];
}
//------------------------------------
// convertToResultString
//------------------------------------
- (NSString*)convertToResultString:(int)startIndex endIndex:(int)endIndex totalResults:(int)totalResults
{
	
	return [NSString stringWithFormat:@"%@ - %@ of %@",
								[self convertToComma:startIndex],
								[self convertToComma:endIndex],
								[self convertToComma:totalResults]
			];
}

//------------------------------------
// decodeToPercentEscapesString
//------------------------------------
- (NSString*)decodeToPercentEscapesString:(NSString*)string
{
	// %XX -> char
	NSString *encodedStr = [self encodeFromPercentEscapesString:string];

	// char -> %XX
	NSString *decodedStr = (NSString *) CFURLCreateStringByAddingPercentEscapes(
				kCFAllocatorDefault,
				(CFStringRef) encodedStr,
				nil,
				nil,
				kCFStringEncodingUTF8);

	return decodedStr;
}
//------------------------------------
// encodeFromPercentEscapesString
//------------------------------------
- (NSString*)encodeFromPercentEscapesString:(NSString*)string
{
	// %XX -> char
	NSString *encodedStr = (NSString *) CFURLCreateStringByReplacingPercentEscapesUsingEncoding(
				kCFAllocatorDefault,
				(CFStringRef) string,
				CFSTR(""),
				kCFStringEncodingUTF8);

	return encodedStr;
}
//------------------------------------
// replaceCharacters
//------------------------------------
- (NSString*)replaceCharacter:(NSString*)string str1:(NSString*)str1 str2:(NSString*)str2
{
	NSMutableString *muString = [NSMutableString stringWithString:string];

	[muString replaceOccurrencesOfString:str1 withString:str2 options:0 range:NSMakeRange(0,[muString length])];

	return muString;
}
//------------------------------------
// checkIsExistCharacterSet
//------------------------------------
- (BOOL)checkIsExistCharacterSet:(NSString*)string cSet:(NSCharacterSet*)cSet
{
	BOOL ret = NO;
	NSRange rng = [string rangeOfCharacterFromSet:cSet];
	if(rng.length > 0){
		ret = YES;
	}
	return ret;
}
//------------------------------------
// trimKeywordFromString
//------------------------------------
- (NSString*)trimKeywordFromString:(NSString*)string
{

	NSString *sepStr = @"\t";
	NSArray *sepStrs = [[[NSArray alloc] initWithObjects:
						@"[", @"]",
						@"<", @">",
						@"(", @")",
						@"/",
						nil
					] autorelease];

	int i;

	// replace sepStrs to sepStr
	if(![string isEqualToString:sepStr]){
		for (i = 0; i < [sepStrs count]; i++){
			string = [self replaceCharacter:string str1:[sepStrs objectAtIndex:i] str2:sepStr];
		}
		NSArray *array = [string componentsSeparatedByString:sepStr];
		string = [[array objectAtIndex:0] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	}

	return string;
}

//------------------------------------
// getDefaultRomSet
//------------------------------------
- (id)getDefaultRomSet:(id)record isClone:(BOOL*)isClone
{
	NSString *romId = [record valueForKey:@"romId"];
	NSString *defaultRomId = [record valueForKey:@"defaultRomId"];
	id attribute = nil;

	// default set is clone
	if(![romId isEqualToString:defaultRomId] && [record valueForKey:@"rlt_clonelist"]){
		attribute = [record valueForKey:@"rlt_clonelist"];
		*isClone = YES;
	}else{
		attribute = record;
		*isClone = NO;
	}
	return attribute;
}

//------------------------------------
// getTitleString
//------------------------------------
- (NSString*)getTitleString:(int)langType
							title:(NSString*)title
							titlejp:(NSString*)titlejp
{
	NSString *string = @"";

	// JP
	if(langType == DEFAULT_LANG_TYPE_JP){
		if(![titlejp isEqualToString:@""]){
			string = titlejp;
		}else{
			string = [NSString stringWithFormat:@"%@ [en]", title];
		}
	}
	// EN
	else{
		string = title;
	}

	return string;
}
//------------------------------------
// createToolTipString
//------------------------------------
- (NSString*)createToolTipString:(id)record isClone:(BOOL)isClone
{

	NSString *romId = @"";
	if(isClone == YES){
		romId = [record valueForKey:@"cloneId"];
	}else{
		romId = [record valueForKey:@"romId"];
	}

	NSString *title = [self getTitleString:[self defaultLangType]
							title:[record valueForKey:@"title"]
							titlejp:[record valueForKey:@"titlejp"]
			];
	NSString *manufacture = [record valueForKey:@"manufacture"];
	NSString *year = [record valueForKey:@"year"];
	NSString *fileSizeStr = @"-";
	int fileSize = [[record valueForKey: @"fileSize"] intValue];
	if(fileSize > 0){
		fileSizeStr = [self convertFileSizeToString:fileSize];
	}
	NSString *fileDate = [record valueForKey:@"fileDate"];
	NSString *filePath = [self createCurrentRomPath:romId];
	if(!fileDate || [fileDate isEqualToString:@""]){
		fileDate = @"-";
	}
	if(!filePath || [filePath isEqualToString:@""]){
		filePath = @"-";
	}

	NSString *string = [NSString stringWithFormat:@"%@\n%@\n%@\n%@"
								, title
								, [NSString stringWithFormat:@"%@ / %@", manufacture, year]
								, [NSString stringWithFormat:@"Size: %@\nDate : %@", fileSizeStr, fileDate]
								, [NSString stringWithFormat:@"Path: %@", filePath]
						];


	return string;
}
//------------------------------------
// getLastSeparatedString
//------------------------------------
- (NSString*)getLastSeparatedString:(NSString*)str sep:(NSString*)sep
{
	NSString *retStr = str;

	NSArray *cols = [str componentsSeparatedByString:sep];
	if([cols count] > 0){
		retStr = [cols objectAtIndex:[cols count] - 1];
	}

	return retStr;
}
@end