#import "DialogExtension.h"

@implementation NSObject(dialogExtension_)

//------------------------------------
// displayMessage
//------------------------------------
- (int)displayMessage:(NSString *)messageStyle
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{

	// post notification
	[self postDialogOpenedNotification];

	NSAlert *alert;
	int i,result;
	NSArray *btnArray;

	alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if([messageStyle isEqualToString:@"alert"]){
		[alert setIcon:[NSImage imageNamed:@"icon_alert"]];
	}
	else if([messageStyle isEqualToString:@"error"]){
		[alert setIcon:[NSImage imageNamed:@"icon_stop"]];
	}
	else if([messageStyle isEqualToString:@"info"]){
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}
	else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageWithIcon
//------------------------------------
- (int)displayMessageWithIcon:(NSString *)iconName
					messageText:(NSString *)messageText
					infoText:(NSString *)infoText
					btnList:(NSString *)btnList
{

	// post notification
	[self postDialogOpenedNotification];

	NSAlert *alert;
	int i,result;
	NSArray *btnArray;

	alert = [[[NSAlert alloc] init] autorelease];

	[alert setAlertStyle:NSWarningAlertStyle];
	[alert setMessageText:messageText];
	if(![infoText isEqualToString:@""]){
		[alert setInformativeText:infoText];
	}

	if(![iconName isEqualToString:@""]){
		[alert setIcon:[NSImage imageNamed:iconName]];
	}else{
		[alert setIcon:[NSImage imageNamed:@"icon_info"]];
	}

	btnArray = [btnList componentsSeparatedByString:@","];
	for(i = 0; i < [btnArray count]; i++){
		[alert addButtonWithTitle:[btnArray objectAtIndex:i]];
	}

	result = [alert runModal];
	return result;

}
//------------------------------------
// displayMessageAlertAppIsNotFound 
//------------------------------------
- (int)displayMessageAlertAppIsNotFound:(NSString*)filePath
{
	NSString *infoText = @"";
	if(![filePath isEqualToString:@""]){
		infoText = [NSString stringWithFormat:@"Check file \"%@\"", filePath];
	}

	return [self displayMessage:@"alert"
						messageText:@"Application is not found."
						infoText:infoText
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertFileIsNotFound 
//------------------------------------
- (int)displayMessageAlertFileIsNotFound:(NSString*)filePath
{

	NSString *infoText = @"";
	if(![filePath isEqualToString:@""]){
		infoText = [NSString stringWithFormat:@"Check file \"%@\"", filePath];
	}

	return [self displayMessage:@"alert"
						messageText:@"File is not found."
						infoText:infoText
						btnList:@"Cancel"
			];
}
//------------------------------------
// displayMessageAlertRomFileIsNotFound 
//------------------------------------
- (int)displayMessageAlertRomFileIsNotFound:(NSString*)romId isParent:(BOOL)isParent
{
	NSString *message = @"";
	if(isParent == YES){
		message = [NSString stringWithFormat:@"Parent rom file \"%@.zip\" is not found.", romId];
	}else{
		message = [NSString stringWithFormat:@"Rom file \"%@.zip\" is not found.", romId];
	}
	return [self displayMessage:@"alert"
						messageText:message
						infoText:@"Continue play?"
						btnList:@"Play,Cancel"
			];
}
//------------------------------------
// displayMessageAlertFolderIsNotMatch 
//------------------------------------
- (int)displayMessageAlertFolderIsNotMatch:(NSString*)filePath1 filePath2:(NSString*)filePath2;
{
	NSString *infoText = [NSString stringWithFormat:@"Clone : %@\nParent: %@"
							, filePath1
							, filePath2
						];

	return [self displayMessage:@"alert"
						messageText:@"Clone rom is placed in different folder of parent rom.\nGame may not run correctly."
						infoText:infoText
						btnList:@"Continue,Cancel"
			];
}

//------------------------------------
// displayMessageAlertExecutionFile
//------------------------------------
- (int)displayMessageAlertExecutionFile
{
	return [self displayMessage:@"alert"
						messageText:@"Execution file is not found."
						infoText:@"Please set execution file in Preferences."
						btnList:@"Preferences,Cancel"
			];
}
//------------------------------------
// displayMessageAlertRunningTask
//------------------------------------
- (int)displayMessageAlertRunningTask
{
	return [self displayMessage:@"alert"
						messageText:@"Game is already running."
						infoText:@"Please quit game."
						btnList:@"Cancel"
			];
}

//------------------------------------
// displayMessageAlertFolder
//------------------------------------
- (int)displayMessageAlertFolder:(NSString*)folderName
{
	return [self displayMessage:@"alert"
						messageText:[NSString stringWithFormat:@"%@ folder is not found.", folderName]
						infoText:@"Please set folder in Preferences."
						btnList:@"Preferences,Cancel"
			];
}
//------------------------------------
// selectFilePathWithOpenPanel
//------------------------------------
- (NSArray*)selectFilePathWithOpenPanel:(BOOL)isFile
						isDir:(BOOL)isDir
						isMultiSel:(BOOL)isMultiSel
						isPackage:(BOOL)isPackage
						isAlias:(BOOL)isAlias
						canCreateDir:(BOOL)canCreateDir
						defaultPath:(NSString*)defaultPath
{

	// post notification
	[self postDialogOpenedNotification];

	NSOpenPanel* openPanel;
	int	result;
	NSArray* filePaths = [[[NSArray alloc] init] autorelease];
//	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	openPanel = [NSOpenPanel openPanel];
	[openPanel setCanChooseFiles:isFile];
	[openPanel setCanChooseDirectories:isDir];
	[openPanel setAllowsMultipleSelection:isMultiSel];
	[openPanel setTreatsFilePackagesAsDirectories:isPackage];
	[openPanel setResolvesAliases:isAlias];
	[openPanel setCanCreateDirectories:canCreateDir];

	result = [openPanel runModalForDirectory:defaultPath file:nil types:nil];

	if (result == NSFileHandlingPanelOKButton) {
		filePaths = [openPanel filenames];
	}

	return filePaths;

}
//------------------------------------
// selectFilePathWithSavePanel
//------------------------------------
- (NSString*)selectFilePathWithSavePanel:(NSString*)fileName
						canCreateDir:(BOOL)canCreateDir
						defaultPath:(NSString*)defaultPath
{

	// post notification
	[self postDialogOpenedNotification];

	NSSavePanel* savePanel;
	int	result;
	NSString *filePath = @"";

	savePanel = [NSSavePanel savePanel];
	[savePanel setCanCreateDirectories:canCreateDir];

	result = [savePanel runModalForDirectory:defaultPath file:fileName];

	if (result == NSFileHandlingPanelOKButton) {
		filePath = [savePanel filename];
	}

	return filePath;
}
//------------------------------------
// postDialogOpenedNotification
//------------------------------------
- (void)postDialogOpenedNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:WINDOW_NOTIF_DIALOG_DID_OPENED object:self];
}
@end