#import "ExecuteMAME.h"
#import "ViewPrefs.h"
#import "TBArrayController.h"
#import "ViewLog.h"
#import "FileAttributeExtension.h"
#import "DialogExtension.h"
#import "ImageBindExtension.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"
#import "MAMEAttributeExtension.h"

@implementation ExecuteMAME

//=======================================================================
// Initialize App
//=======================================================================
- (void)awakeFromNib
{

	//
	// set state from user default
	//
	// window rect
	[self setWindowRect:replaySettingWindow key:@"rectWindowReplaySetting"];
	[self setWindowRect:bezelSettingWindow key:@"rectWindowBezelSetting"];

	[self setLogString:@""];
	[self setPlayParams:nil];
	[self setReplayParams:nil];
	[self setBezelParams:nil];
	[self setTask:nil];

	// set key equivalent
	[btnReplayPlay setKeyEquivalent:@"\r"];
	[btnReplayCancel setKeyEquivalent:@"\033"];

	// set first responder
	[bezelSettingWindow makeFirstResponder:tbvBezelList];
	// set key equivalent
	[btnBezelPlay setKeyEquivalent:@"\r"];
	[btnBezelCancel setKeyEquivalent:@"\033"];

	// alloc bezelList
	bezelList_ = [[NSMutableArray alloc] init];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:replaySettingWindow];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:bezelSettingWindow];

}
//=======================================================================
// IBAction
//=======================================================================
//------------------------------------
// openReplaySettingWindow
//------------------------------------
- (IBAction)openReplaySettingWindow:(id)sender
{
	[replaySettingWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// openBezelSettingWindow
//------------------------------------
- (IBAction)openBezelSettingWindow:(id)sender
{
	[bezelSettingWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// closeReplaySettingWindow
//------------------------------------
- (IBAction)closeReplaySettingWindow:(id)sender
{
	[replaySettingWindow close];
}
//------------------------------------
// closeBezelSettingWindow
//------------------------------------
- (IBAction)closeBezelSettingWindow:(id)sender
{
	[bezelSettingWindow close];
}

//------------------------------------
// executeMameWithReplay
//------------------------------------
- (IBAction)executeMameWithReplay:(id)sender
{

	if( [self defaultExecuteMameNo] == MAME_EXEC_TYPE_SDLMAME &&
//		[self defaultBoolValue:@"optSdlMameEnable"] == YES &&
		[self defaultBoolValue:@"optSdlMameReplayEnable"] == YES
	){
		// ok
	}
	// error
	else{
		[self displayMessage:@"alert"
					messageText:@"Can not run replay mode."
					infoText:@"SDLMAME is not selected or disabled replay mode."
					btnList:@"Cancel"
		];
		return;
	}

	int result;

	int replayType = [replayTypeMatrix selectedTag];
	NSString *replayFile = @"";

	// record
	if(replayType == MAME_REPLAY_TYPE_RECORD){
		replayFile = [txtRecordFile stringValue];
	}
	// playback
	else if(replayType == MAME_REPLAY_TYPE_PLAYBACK){
		replayFile = [pbtnPlaybackFiles titleOfSelectedItem];
	}
	// unknown
	else{
		return;
	}

	// trim space
	replayFile = [replayFile stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
	
	// replayFile error
	if(	[replayFile isEqualToString:@""] ||
		[[replayFile substringToIndex:1] isEqualToString:@"."] ||
		[[replayFile substringToIndex:1] isEqualToString:@"/"]
	){
		// record
		if(replayType == MAME_REPLAY_TYPE_RECORD){
			[self displayMessage:@"alert"
						messageText:@"Record file is not correctly."
						infoText:@"Please set record file."
						btnList:@"Cancel"
			];
		}
		// playback
		else if(replayType == MAME_REPLAY_TYPE_PLAYBACK){
			[self displayMessage:@"alert"
						messageText:@"Playback file is not correctly."
						infoText:@"Please select playback file."
						btnList:@"Cancel"
			];
		}
		return;
	}

	//
	// check & confirm
	//
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];
	NSString *curPath = [appPath stringByDeletingLastPathComponent];
	NSString *folderPath = [NSString stringWithFormat:@"%@/inp", curPath];
	NSString *filePath = [NSString stringWithFormat:@"%@/%@", folderPath, replayFile];

	// folder is file
	if([self checkFileExist:folderPath] == FILE_EXIST_FILE){
		[self displayMessage:@"alert"
					messageText:@"\"inp\" folder is not folder."
					infoText:[NSString stringWithFormat:@"Please check %@ ", folderPath]
					btnList:@"Cancel"
		];
		return;
	}

	// record
	if(replayType == MAME_REPLAY_TYPE_RECORD){
		// confirm
		result = [self displayMessageWithIcon:@"icon_replay_record"
								messageText:[NSString stringWithFormat:@"Record file \"%@\" is selected.", replayFile]
								infoText:@"Do you record your play?"
								btnList:@"Record,Cancel"
				];
		// cancel
		if(result == NSAlertSecondButtonReturn){
			return;
		}

		// file
		if([self checkFileExist:filePath] == FILE_EXIST_FILE){
			result = [self displayMessage:@"alert"
								messageText:[NSString stringWithFormat:@"Record file \"%@\" is already exists.", replayFile]
								infoText:@"Do you replace it?"
								btnList:@"Replace,Cancel"
					];
			// cancel
			if(result == NSAlertSecondButtonReturn){
				return;
			}
		}
		// folder
		else if([self checkFileExist:filePath] == FILE_EXIST_DIR){
			[self displayMessage:@"alert"
								messageText:[NSString stringWithFormat:@"Record file \"%@\" is folder.", replayFile]
								infoText:@"Please change record file."
								btnList:@"Cancel"
			];
			return;
		}
		// not found
		else{
			// ok
		}

	}
	// playback
	else if(replayType == MAME_REPLAY_TYPE_PLAYBACK){

		// not found
		if([self checkFileExist:filePath] != FILE_EXIST_FILE){
			[self displayMessage:@"alert"
										messageText:@"Playback file is not found."
										infoText:@"Please select Playback file."
										btnList:@"Cancel"
			];
			return;
		}
		// confirm
		result = [self displayMessageWithIcon:@"icon_replay_playback"
									messageText:[NSString stringWithFormat:@"Playback file \"%@\" is selected.", replayFile]
									infoText:@"Do you playback from this file?"
									btnList:@"Playback,Cancel"
				];
		// cancel
		if(result == NSAlertSecondButtonReturn){
			return;
		}
	}
	// unknown
	else{
		return;
	}
	
	// set params
	NSDictionary *replayParams = [NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:YES], @"isReplay",
									[NSNumber numberWithInt:replayType], @"replayType",
									replayFile, @"replayFile",
									nil
								];
	[self setReplayParams:replayParams];

	// close window
	[self closeReplaySettingWindow:nil];

	// execute mame
	// bezel
	if([self defaultBoolValue:@"optSdlMameBezelEnable"] == YES){
		[self playItemWithBezel:[self playParams]];
	}else{
		[self executeMame:[self playParams] replayParams:replayParams bezelParams:[self bezelParams]];
	}

}
//------------------------------------
// executeMameWithBezel
//------------------------------------
- (IBAction)executeMameWithBezel:(id)sender
{
	
	if([tbvBezelList numberOfSelectedRows] <= 0){
		return;
	}
	int index = [[tbvBezelList selectedRowIndexes] firstIndex];
	if(index < 0 || index > [bezelList_ count] - 1){
		return;
	}

	id record = [bezelList_ objectAtIndex:index];

	NSMutableDictionary *bezelParams = [record valueForKey:@"bezelParams"];

	BOOL showSubImage;
	if([btnBezelShowSubImage state] == 0){
		showSubImage = NO;
	}else{
		showSubImage = YES;
	}

	float bezelAlpha = [sliderBezelAlpha floatValue];

	// add params
	[bezelParams setValue:[NSNumber numberWithBool:showSubImage] forKey:@"showSubImage"];
	[bezelParams setValue:[NSNumber numberWithFloat:bezelAlpha] forKey:@"bezelAlpha"];
	[self setBezelParams:bezelParams];

	// close window
	[self closeBezelSettingWindow:nil];

	// execute mame
	[self executeMame:[self playParams] replayParams:[self replayParams] bezelParams:bezelParams];

}
//------------------------------------
// changeReplayType
//------------------------------------
- (IBAction)changeReplayType:(id)sender
{
	int replayType = [replayTypeMatrix selectedTag];

	if(replayType == MAME_REPLAY_TYPE_RECORD){
		[txtRecordFile setEnabled:YES];
		[pbtnPlaybackFiles setEnabled:NO];
	}
	else if(replayType == MAME_REPLAY_TYPE_PLAYBACK){
		[txtRecordFile setEnabled:NO];
		[pbtnPlaybackFiles setEnabled:YES];
	}
	else{
		[txtRecordFile setEnabled:NO];
		[pbtnPlaybackFiles setEnabled:NO];
	}

}
//------------------------------------
// changeBezelImageView
//------------------------------------
- (IBAction)changeBezelImageView:(id)sender
{

	if([tbvBezelList numberOfSelectedRows] <= 0){
		return;
	}
	int index = [[tbvBezelList selectedRowIndexes] firstIndex];
	if(index < 0 || index > [bezelList_ count] - 1){
		return;
	}

	id record = [bezelList_ objectAtIndex:index];

	NSMutableDictionary *bezelParams = [record valueForKey:@"bezelParams"];

	NSString *layout = [bezelParams valueForKey:@"layout"];
	BOOL isBezel = [[bezelParams valueForKey:@"isBezel"] boolValue];
	BOOL isSubImage = [[bezelParams valueForKey:@"isSubImage"] boolValue];

	BOOL showSubImage;
	if([btnBezelShowSubImage state] == 0){
		showSubImage = NO;
	}else{
		showSubImage = YES;
	}

	float bezelAlpha = [sliderBezelAlpha floatValue];

	NSDictionary *playParams = [self playParams];
	NSString *romId = [playParams valueForKey:@"romId"];
//	BOOL isClone = [[playParams valueForKey:@"isClone"] boolValue];
	int itemType = [[playParams valueForKey:@"itemType"] intValue];
	int imageType;
	if(itemType == ITEM_TYPE_MAIN){
		imageType = [self defaultThumbsImageTypeMain];
	}else{
		imageType = [self defaultThumbsImageTypeClone];
	}

	// change default snap
	if( imageType != DEFAULT_IMAGE_TYPE_SNAP &&
		imageType != DEFAULT_IMAGE_TYPE_TITLE){
		imageType = DEFAULT_IMAGE_TYPE_SNAP;
	}

	// create preview image
	NSImage *image = [self createBezelPreviewImage:romId
										bezelParams:bezelParams
										showSubImage:showSubImage
										imageType:imageType
										bezelAlpha:bezelAlpha
					];
	[imageBezel setImage:image];
	[imageBezel setBgColor:[NSColor blackColor]];

	[btnBezelShowSubImage setEnabled:isSubImage];
	[sliderBezelAlpha setEnabled:isBezel];

	// set default
	[self setDefaultStringValue:layout key:@"optBezelLayout"];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playItem
//------------------------------------
- (void)playItem:(NSDictionary*)playParams
{

	// no params
	if(playParams == nil){
		return;
	}

	// clear params
	[self setPlayParams:nil];
	[self setReplayParams:nil];
	[self setBezelParams:nil];

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:MAME_NOTIF_EXEC_DID_STARTED object:self];

	int mameNo = [[playParams valueForKey:@"mameNo"] intValue];

	// execute mame
	// sdlmame
	if(mameNo == MAME_EXEC_TYPE_SDLMAME){
		// replay
//		if( [self defaultBoolValue:@"optSdlMameEnable"] == YES &&
//			[self defaultBoolValue:@"optSdlMameReplayEnable"] == YES
		if([self defaultBoolValue:@"optSdlMameReplayEnable"] == YES
		){
			[self playItemWithReplay:playParams];
		}
		// bezel
		else if([self defaultBoolValue:@"optSdlMameBezelEnable"] == YES){
			[self playItemWithBezel:playParams];
		}
		// no
		else{
			[self executeMame:playParams replayParams:nil bezelParams:nil];
		}

	}
	// other mame
	else{
		[self executeMame:playParams replayParams:nil bezelParams:nil];
	}

}

//------------------------------------
// playItemWithReplay
//------------------------------------
- (BOOL)playItemWithReplay:(NSDictionary*)playParams
{

	int result;

	//
	// app path
	//
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	// set params
	[self setPlayParams:playParams];

	//
	// create inp items
	//
	// remove items
	[pbtnPlaybackFiles removeAllItems];

	// add item
	[pbtnPlaybackFiles addItemWithTitle:@"-"];

	NSString *title = [playParams valueForKey:@"title"];
	NSString *romId = [playParams valueForKey:@"romId"];
	NSString *curPath = [appPath stringByDeletingLastPathComponent];
	NSString *folderPath = [NSString stringWithFormat:@"%@/inp", curPath];
	NSString *playBackFileName = [NSString stringWithFormat:@"%@.inp", romId];

//	NSLog(@"folderPath=%@", folderPath);

	// folderPath is folder
	if([self checkFileExist:folderPath] == FILE_EXIST_DIR){

		int index = 0;
		int selectedIndex = 0;
		NSString *fileName;
		NSString *filePath;
		NSFileManager *myFile = [NSFileManager defaultManager];

		NSArray *files = [myFile directoryContentsAtPath:folderPath];
		NSEnumerator *enumFiles = [files objectEnumerator];

		while (fileName = [enumFiles nextObject]) {

			// invalid file
			if( [fileName isEqualToString:@""] ||
				[[fileName substringToIndex:1] isEqualToString:@"."] ||
				[fileName isEqualToString:@"Icon\x0d"]
			){
				continue;
			}

			filePath = [NSString stringWithFormat:@"%@/%@", folderPath, fileName];

			// not file
			if([self checkFileExist:filePath] != FILE_EXIST_FILE){
				continue;
			}

			index++;
			[pbtnPlaybackFiles addItemWithTitle:fileName];

			if([fileName isEqualToString:playBackFileName]){
				selectedIndex = index;
			}
		}

		// select item
		[pbtnPlaybackFiles selectItemAtIndex:selectedIndex];
	}

	[replaySettingWindow setTitle:title];

	[txtRecordFile setStringValue:playBackFileName];

	[self changeReplayType:nil];
	[self openReplaySettingWindow:nil];

	return YES;
}
//------------------------------------
// playItemWithBezel
//------------------------------------
- (BOOL)playItemWithBezel:(NSDictionary*)playParams
{

	int result;

	//
	// app path
	//
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	//
	// images path
	//
	NSString *imagesPath = [self converToBezelImagesPath];
	// not found
	if([self checkFileExist:imagesPath] != FILE_EXIST_DIR){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Can not play with extra bezel."
										infoText:@"Bezel images folder is not found."
										btnList:@"Cancel"
		];
		return NO;
	}
	//
	// layouts path
	//
	NSString *layoutsPath = [self converToBezelLayoutsPath];
	// not found
	if([self checkFileExist:layoutsPath] != FILE_EXIST_DIR){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Can not play with extra bezel."
										infoText:@"Bezel layouts folder is not found."
										btnList:@"Cancel"
		];
		return NO;
	}

	//
	// list path
	//
	NSString *listPath = [self converToBezelTemplateListPath];
	// not found
	if([self checkFileExist:listPath] != FILE_EXIST_FILE){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Can not play with extra bezel."
										infoText:@"list.xml is not found."
										btnList:@"Cancel"
		];
		return NO;
	}

	// set params
	[self setPlayParams:playParams];

	NSString *title = [playParams valueForKey:@"title"];

	//
	// create bezelList
	//
	[bezelList_ removeAllObjects];

	NSString *bezelTitle;
	NSString *layout;
	NSString *icon;
	NSString *iconPath;

	id record;
	int i;
	int selectedIndex = 0;

	NSString *bezelLayout = [self defaultStringValue:@"optBezelLayout"];

	// read list file
	NSString *listString = [[[NSString alloc] initWithContentsOfFile:listPath] autorelease];
	if(listString == nil){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Can not play with extra bezel."
										infoText:@"Failed reading from file."
										btnList:@"Cancel"
		];
		return NO;
	}

	BOOL isSuccess = NO;
	NSString *logString = @"";
	NSArray *menuItems = [self getBezelTemplateItems:listString
												logString:&logString
												isSuccess:&isSuccess
							];

	if(isSuccess == NO){
		int result = [self displayMessage:@"alert"
								messageText:@"Can not create bezel list."
								infoText:@"Please check log."
								btnList:@"Cancel,Log"
					];
		if(result == NSAlertSecondButtonReturn){
			// open log window
			[viewLog setTitle:@"log"];
			[viewLog setLogString:logString];
			[viewLog setFileName:@""];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return NO;
	}

	for(i = 0; i < [menuItems count]; i++){

		record = [menuItems objectAtIndex:i];

		bezelTitle = [record valueForKey:@"title"];
		layout = [record valueForKey:@"layout"];
		icon = [record valueForKey:@"icon"];

		// set image
		NSImage *iconImage = nil;
		if(![icon isEqualToString:@""]){
			iconPath = [self converToBezelIconFilePath:icon];
			if([self checkFileExist:iconPath] == FILE_EXIST_FILE){
				iconImage = [[[NSImage alloc] initByReferencingFile:iconPath] autorelease];	
				[iconImage setSize:NSMakeSize(16,16)];
				[iconImage setScalesWhenResized:YES];
			}
		}

		// add object
		[bezelList_ addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithInt:i], @"index" ,
				bezelTitle, @"title" ,
				record, @"bezelParams" ,
				iconImage, @"iconImage" ,		// for nil last
				nil
			]
		];

		// selectedIndex
		if([layout isEqualToString:bezelLayout]){
			selectedIndex = i;
		}

	}
/*
	}else{
		// add object
		[bezelList_ addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithInt:0], @"index" ,
				@"No Bezels", @"title" ,
				nil, @"image" ,
				nil, @"bezelParams" ,
				nil
			]
		];
	}
*/
	[tbvBezelList reloadData];

	// select row
	if(selectedIndex < [bezelList_ count]){
		[tbvBezelList selectRow:selectedIndex byExtendingSelection:NO];
	}

	[bezelSettingWindow setTitle:title];

	// create bezel image
	[self changeBezelImageView:nil];

	// open window
	[self openBezelSettingWindow:nil];

	return YES;
}

//------------------------------------
// executeMame
//------------------------------------
- (void)executeMame:(NSDictionary*)playParams
				replayParams:(NSDictionary*)replayParams
				bezelParams:(NSDictionary*)bezelParams
{
	// no params
	if(playParams == nil){
		return;
	}

	// play params
	int mameNo = [[playParams valueForKey:@"mameNo"] intValue];
	NSString *romId = [playParams valueForKey:@"romId"];
	NSString *parentId = [playParams valueForKey:@"parentId"];

	// post notification
//	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
//	[nc postNotificationName:MAME_NOTIF_EXEC_DID_STARTED object:self];


	BOOL isPlay = NO;

	//
	// execute Mame
	//
	// sdlmame
	if(mameNo == MAME_EXEC_TYPE_SDLMAME){
		isPlay = [self executeSdlMame:romId
							parentId:parentId
							bezelParams:bezelParams
							replayParams:replayParams
				];
	}
	// mameosx
	else if(mameNo == MAME_EXEC_TYPE_MAMEOSX){
		isPlay = [self executeMameOSX:romId parentId:parentId];
	}
	// macmame
	else if(mameNo == MAME_EXEC_TYPE_MACMAME){
		isPlay = [self executeMacMame:romId parentId:parentId];
	}
	// unknown
	else{
		return;
	}
	
	// update play count
	if(isPlay == YES){
		[self updatePlayCount:parentId];
	}

}

//------------------------------------
// executeSdlMame
//------------------------------------
- (BOOL)executeSdlMame:(NSString*)romId
				parentId:(NSString*)parentId
				bezelParams:(NSDictionary*)bezelParams
				replayParams:(NSDictionary*)replayParams
{

    int result;

	// check task is running
	if([self task] && [[self task] isRunning] == YES){
		[self displayMessageAlertRunningTask];
		return NO;
	}

	//
	// app path
	//
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	// alert debug options
	if([self defaultBoolValue:@"optSdlMameEnable"] == YES){
		if(	[self defaultBoolValue:@"optSdlMameDebugAlert"] == YES &&
			(
				[self defaultBoolValue:@"optSdlMameDebug"] == YES ||
				[self defaultBoolValue:@"optSdlMameDebugErrorLog"] == YES ||
				[self defaultBoolValue:@"optSdlMameDebugVerbose"] == YES ||
				[self defaultBoolValue:@"optSdlMameDebugUpdateInPause"] == YES
			)
		){
			result = [self displayMessage:@"alert"
								messageText:@"Debug options are enabled."
								infoText:@"Continue play?"
								btnList:@"Play,Cancel"
			];
			if(result != NSAlertFirstButtonReturn){
				return NO;
			}
		}
	}

	// current path
	NSString *curPath = [appPath stringByDeletingLastPathComponent];

	// bezel params
	BOOL isBezel = NO;
//	BOOL showSubImage = NO;
	
	if(bezelParams != nil){
		isBezel = [[bezelParams valueForKey:@"isBezel"] boolValue];
//		showSubImage = [[bezelParams valueForKey:@"showSubImage"] boolValue];
	}

	// replay params
	BOOL isReplay = NO;
	int replayType = MAME_REPLAY_TYPE_NONE;
	NSString *replayFile = @"";
	if(replayParams != nil){
		isReplay = [[replayParams valueForKey:@"isReplay"] boolValue];
		replayType = [[replayParams valueForKey:@"replayType"] intValue];
		replayFile = [replayParams valueForKey:@"replayFile"];
	}


	//
	// args
	//
	NSMutableArray *args = [[[NSMutableArray alloc] init] autorelease];
	
	// romId
	[args addObject:romId];

	// other options
	[args addObjectsFromArray:[self createSdlMameArguments]];

	// -rompath
	if([self defaultBoolValue:@"optSdlMameEnable"] == YES){
		if([self defaultBoolValue:@"optSdlMameRomPath"] == YES){

			// check default roms folder
			NSString *defaultFolderPathString = [self getDefaultRomsFolderPathString];

			// not exist
			if([defaultFolderPathString isEqualToString:@""]){
				result = [self displayMessageAlertFolder:@"Roms"];
				// open prefs tab
				if(result == NSAlertFirstButtonReturn){
					[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
				}
				return NO;
			}

			// rom file not exist
			NSString *romPath = [self createCurrentRomPath:romId];
			if([romPath isEqualToString:@""]){
				result = [self displayMessageAlertRomFileIsNotFound:romId isParent:NO];
				// cancel
				if(result == NSAlertSecondButtonReturn){
					return NO;
				}
			}else{
				// parent rom file not exist
				NSString *parentPath = [self createCurrentRomPath:parentId];
				if([parentPath isEqualToString:@""]){
					result = [self displayMessageAlertRomFileIsNotFound:parentId isParent:YES];
					// cancel
					if(result == NSAlertSecondButtonReturn){
						return NO;
					}
				}
			}
			[args addObject:@"-rompath"];
			[args addObject:defaultFolderPathString];
		}

		// -snapshot_directory
		if([self defaultBoolValue:@"optSdlMameSnapPath"] == YES){
			NSString *snapFolder = [self defaultSettingFolderPath:MAME_FOLDER_TYPE_SNAPS];
			// exist
			if([self checkFileExist:snapFolder] == FILE_EXIST_DIR){
				[args addObject:@"-snapshot_directory"];
				[args addObject:snapFolder];
			}
		}
	}

	// bezel
	if(isBezel == YES){
		NSString *bezelPath = [self createBezelFilePath:parentId
												romId:romId
												bezelParams:bezelParams
							];
//		NSLog(@"bezelPath=%@", bezelPath);
		if(![bezelPath isEqualToString:@""]){
			[args addObject:@"-artpath"];
			[args addObject:bezelPath];
			[args addObject:@"-use_bezels"];
		}
	}

	//
	// replay
	//
	if(isReplay == YES && ![replayFile isEqualToString:@""]){

		// record
		if(replayType == MAME_REPLAY_TYPE_RECORD){
			[args addObject:@"-record"];
			[args addObject:replayFile];
		}
		// playback
		else if(replayType == MAME_REPLAY_TYPE_PLAYBACK){
			[args addObject:@"-playback"];
			[args addObject:replayFile];
		}

	}

//	NSLog(@"%@", args);

	// clear log
	[self setLogString:@""];

	//
	// set task
	//
	NSTask *task = [[[NSTask alloc] init] autorelease];
//	task = [[NSTask alloc] init];

	// tack output
	[task setStandardOutput:[NSPipe pipe]];

	// tack output
	[task setStandardError:[task standardOutput]];

	// notification
	[[NSNotificationCenter defaultCenter] addObserver:self 
											selector:@selector(handleTaskResult:) 
											name:NSFileHandleReadCompletionNotification 
											object:[[task standardOutput] fileHandleForReading]
//											object:task
	];

	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];

	//
	// launch task 
	//
	[task setLaunchPath:appPath];
	[task setCurrentDirectoryPath:curPath];
	[task setArguments:args];
	[task launch];

	[self setTask:task];

	return YES;

}

//------------------------------------
// executeMameOSX
//------------------------------------
- (BOOL)executeMameOSX:(NSString*)romId parentId:(NSString*)parentId
{

	int result;

	// check task is running
	if([self task] && [[self task] isRunning] == YES){
		[self displayMessageAlertRunningTask];
		return NO;
	}

	//
	// app path
	//
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_MAMEOSX];

	// app is not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	//
	// args
	//
	NSMutableArray *args = [[[NSMutableArray alloc] init] autorelease];

	// -Game
	[args addObject:@"-Game"];
	[args addObject:romId];
	// other args
	[args addObjectsFromArray:[self createMameOSXArguments]];

	// -rompath
	if([self defaultBoolValue:@"optMameOSXEnable"] == YES){
		if([self defaultBoolValue:@"optMameOSXRomPath"] == YES){

			// check default roms folder
			NSString *defaultFolderPathString = [self getDefaultRomsFolderPathString];

			// not exist
			if([defaultFolderPathString isEqualToString:@""]){
				result = [self displayMessageAlertFolder:@"Roms"];
				// open prefs tab
				if(result == NSAlertFirstButtonReturn){
					[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
				}
				return NO;
			}

			// rom file not exist
			NSString *romPath = [self createCurrentRomPath:romId];
			if([romPath isEqualToString:@""]){
				result = [self displayMessageAlertRomFileIsNotFound:romId isParent:NO];
				// cancel
				if(result == NSAlertSecondButtonReturn){
					return NO;
				}
			}else{
				// parent rom file not exist
				NSString *parentPath = [self createCurrentRomPath:parentId];
				if([parentPath isEqualToString:@""]){
					result = [self displayMessageAlertRomFileIsNotFound:parentId isParent:YES];
					// cancel
					if(result == NSAlertSecondButtonReturn){
						return NO;
					}
				}
			}
			[args addObject:@"-RomPath"];
			[args addObject:defaultFolderPathString];
		}
	}

//	NSLog(@"%@", args);

	// clear log
	[self setLogString:@""];

	//
	// set task
	//
	NSTask* task = [[[NSTask alloc] init] autorelease];
//	task = [[NSTask alloc] init];

	// tack output
	[task setStandardOutput:[NSPipe pipe]];

	// tack error
	[task setStandardError:[task standardOutput]];

	// notification
	[[NSNotificationCenter defaultCenter] addObserver:self 
											selector:@selector(handleTaskResult:) 
											name:NSFileHandleReadCompletionNotification 
											object:[[task standardOutput] fileHandleForReading]
//											object:task
	];

	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];

	//
	// launch task 
	//
	NSString *launchPath = [[NSBundle bundleWithPath:appPath] executablePath];
	NSString *curPath = [appPath stringByDeletingLastPathComponent];

	[task setLaunchPath:launchPath];
	[task setCurrentDirectoryPath:curPath];
	[task setArguments:args];
	[task launch];

	[self setTask:task];

	return YES;
}
//------------------------------------
// executeMacMame
//------------------------------------
- (BOOL)executeMacMame:(NSString*)romId parentId:(NSString*)parentId
{

    int result;

	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_MACMAME];

	// app is not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	// check default roms folder
	NSString *defaultFolderPath = [self getDefaultRomsFolderPath];

	// not exist
	if([self checkFileExist:defaultFolderPath] != FILE_EXIST_DIR){
		result = [self displayMessageAlertFolder:@"Roms"];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	NSString *romPath = [self createCurrentRomPath:romId];
	int exist = [self checkFileExist:romPath];

	// not found
	if(exist == FILE_EXIST_NOT_FOUND){
		[self displayMessage:@"alert"
					messageText:@"Rom file is not found."
					infoText:@"Please check rom file in Roms Folder."
					btnList:@"Cancel"
		];
		return NO;
	}
	// file
	else if(exist == FILE_EXIST_FILE){
		// ok
	}
	// folder
	else if(exist == FILE_EXIST_DIR){
		result = [self displayMessage:@"alert"
						messageText:@"Rom file is not zip format."
						infoText:@"MacMAME might not run."
						btnList:@"Continue,Cancel"
				];
		if(result == NSAlertFirstButtonReturn){
			// continue
		}else{
			return NO;
		}
	}
	// unknown
	else{
		return NO;
	}

	// parent rom file not exist
	NSString *parentPath = [self createCurrentRomPath:parentId];
	if([parentPath isEqualToString:@""]){
		result = [self displayMessageAlertRomFileIsNotFound:parentId isParent:YES];
		// cancel
		if(result == NSAlertSecondButtonReturn){
			return NO;
		}
	}

	NSString *romFolderPath = defaultFolderPath;
	NSString *parentFolderPath = defaultFolderPath;

	// get file placed folder path
	if(![romPath isEqualToString:@""]){
		romFolderPath = [romPath stringByDeletingLastPathComponent];
	}
	if(![parentPath isEqualToString:@""]){
		parentFolderPath = [parentPath stringByDeletingLastPathComponent];
	}
	// not match
	if( ![romFolderPath isEqualToString:@""] &&
		![parentFolderPath isEqualToString:@""] &&
		![romFolderPath isEqualToString:parentFolderPath]){
		result = [self displayMessageAlertFolderIsNotMatch:romPath filePath2:parentPath];
		// cancel
		if(result != NSAlertFirstButtonReturn){
			return NO;
		}
	}

	// launch app
	return [[NSWorkspace sharedWorkspace] openFile:romPath withApplication:appPath];

}
//------------------------------------
// updatePlayCount
//------------------------------------
- (void)updatePlayCount:(NSString*)romId
{

 	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"romId == %@", romId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];

	if([fetchArray count] > 0){
		id record = [fetchArray objectAtIndex:0];
		int playCount = [[record valueForKey:@"playCount"] intValue];
		playCount++;
		[record setValue:[NSNumber numberWithInt:playCount] forKey:@"playCount"];
	}
}

//------------------------------------
// handleTaskResult
//------------------------------------
- (void)handleTaskResult:(NSNotification *)aNotification
{

//	NSTask *task = [aNotification object];
//	NSFileHandle *handle = [[task standardOutput] fileHandleForReading];
	NSTask *task = [self task];
	if(task == nil){
		return;
	}

//	NSLog(@"userInfo=%@", [[aNotification userInfo] description]);
	NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];

	if([data length]){

		[self setLogString:[[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];

	}else{

		// terminate task
		[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object:[[task standardOutput] fileHandleForReading]];
		[task terminate];

		// error log
		NSData *errData;
		while((errData = [[[task standardOutput] fileHandleForReading] availableData]) && [errData length]){
			[self setLogString:[[[NSString alloc] initWithData:errData encoding:NSUTF8StringEncoding] autorelease]];
		}

		// status
		if([task terminationStatus]){
			NSString *status = [NSString stringWithFormat:@"%d",[task terminationStatus]];
			int result = [self displayMessage:@"alert"
									messageText:[NSString stringWithFormat:@"Execute error status = %@", status]
									infoText:@"Please check log."
									btnList:@"Cancel,Log"
						];
			if(result == NSAlertSecondButtonReturn){
				// open log window
				[viewLog setLogString:[self logString]];
				[viewLog setTitle:@"log"];
				[viewLog setFileName:@""];
				[viewLog setCanSave:NO];
				[viewLog openLogWindow:nil];
			}

		}
    }
//	[handle readInBackgroundAndNotify];
	[[aNotification object] readInBackgroundAndNotify];

}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	if([notification object] == replaySettingWindow){
		[self saveWindowRect:replaySettingWindow key:@"rectWindowReplaySetting"];
	}
	else if([notification object] == bezelSettingWindow){
		[self saveWindowRect:bezelSettingWindow key:@"rectWindowBezelSetting"];
	}

}

//------------------------------------
// task
//------------------------------------
- (void)setTask:(NSTask*)task
{
	[task retain];
    [task_ release];
    task_ = task;
}
- (NSTask*)task
{
    return task_;
}

//------------------------------------
// playParams
//------------------------------------
- (void)setPlayParams:(NSDictionary*)playParams
{
	[playParams retain];
    [playParams_ release];
    playParams_ = playParams;

}
- (NSDictionary*)playParams
{
	return playParams_;
}
//------------------------------------
// replayParams
//------------------------------------
- (void)setReplayParams:(NSDictionary*)replayParams
{
	[replayParams retain];
    [replayParams_ release];
    replayParams_ = replayParams;

}
- (NSDictionary*)replayParams
{
	return replayParams_;
}
//------------------------------------
// bezelParams
//------------------------------------
- (void)setBezelParams:(NSDictionary*)bezelParams
{
	[bezelParams retain];
    [bezelParams_ release];
    bezelParams_ = bezelParams;

}
- (NSDictionary*)bezelParams
{
	return bezelParams_;
}
//------------------------------------
// bezelList
//------------------------------------
- (NSMutableArray*)bezelList
{
	return bezelList_;
}

//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
    [logString_ release];
    logString_ = logString;

}
- (NSString*)logString
{
    return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	logString_ = [logString_ stringByAppendingString:[[[NSString alloc] initWithString:logString] autorelease]];
}
//------------------------------------
// isSelectSdlmame
//------------------------------------
- (BOOL)isSelectSdlmame
{
	if([self defaultExecuteMameNo] == MAME_EXEC_TYPE_SDLMAME){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[task_ release];
	[playParams_ release];
	[replayParams_ release];
	[bezelParams_ release];
	[bezelList_ release];
	[logString_ release];
    [super dealloc];
}

@end
