#import "FileAttributeExtension.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"

@implementation NSObject(fileAttributeExtension_)
//------------------------------------
// checkFileExist
//------------------------------------
- (int)checkFileExist:(NSString*)filePath
{

	int ret = FILE_EXIST_NOT_FOUND;
	BOOL isDir;

	if([[NSFileManager defaultManager] fileExistsAtPath:filePath isDirectory:&isDir] == YES){
		if(isDir == YES){
			ret = FILE_EXIST_DIR;
		}else{
			ret = FILE_EXIST_FILE;
		}
	}

	return ret;
 
}
//------------------------------------
// checkRomExist
//------------------------------------
- (int)checkRomExist:(NSString*)romId
{

	return [self checkFileExist:[self createCurrentRomPath:romId]];
 
}
/*
//------------------------------------
// createCurrentRomPath
//------------------------------------
- (NSString*)createCurrentRomPath:(NSString*)romId
{

	NSString *romPath = @"";

	NSString *folderPath = [self defaultSettingFolderPath:MAME_FOLDER_TYPE_ROMS];
	NSString *filePath = [self createRomFilePath:romId folderPath:folderPath];

	// file
	if([self checkFileExist:filePath] == FILE_EXIST_FILE){
		romPath = filePath; 
	}
	// folder
	else{
		filePath = [self createRomFolderPath:romId folderPath:folderPath];
		if([self checkFileExist:filePath] == FILE_EXIST_DIR){
			romPath = filePath; 
		}
	}

	return romPath;
 
}
*/
//------------------------------------
// createCurrentRomPath
//------------------------------------
- (NSString*)createCurrentRomPath:(NSString*)romId
{

	NSString *romPath = @"";

	NSArray *folderItems = [self defaultArrayValue:@"pathRomsFolderItems"];

	NSString *filePath;
	NSString *folderPath;
	BOOL enabled;
	id record;

	int i;

	for(i = 0; i < [folderItems count]; i++){

		record = [folderItems objectAtIndex:i];
		enabled = [[record valueForKey:@"enabled"] boolValue];
		if(enabled == NO){
			continue;
		}

		folderPath = [record valueForKey:@"filePath"];

		// create filePath
		filePath = [self createRomPath:romId folderPath:folderPath];

		// exists
		if(![filePath isEqualToString:@""]){
			romPath = filePath;
			break; 
		}

	}

	return romPath;
 
}
//------------------------------------
// createCurrentRomPath
//------------------------------------
- (NSString*)createRomPath:(NSString*)romId folderPath:(NSString*)folderPath
{

	NSString *romPath = @"";
	NSString *filePath = [self createRomFilePath:romId folderPath:folderPath];

	// file
	if([self checkFileExist:filePath] == FILE_EXIST_FILE){
		romPath = filePath;
	}
	// folder
	else{
		filePath = [self createRomFolderPath:romId folderPath:folderPath];
		if([self checkFileExist:filePath] == FILE_EXIST_DIR){
			romPath = filePath; 
		}
	}

	return romPath;
}
//------------------------------------
// createRomFilePath
//------------------------------------
- (NSString*)createRomFilePath:(NSString*)romId folderPath:(NSString*)folderPath
{

	NSString *filePath = @"";

	// file path
	if(![folderPath isEqualToString:@""]){
		filePath = [NSString stringWithFormat: @"%@/%@.zip", folderPath, romId];
	}

	return filePath;
 
}
//------------------------------------
// createRomFolderPath
//------------------------------------
- (NSString*)createRomFolderPath:(NSString*)romId folderPath:(NSString*)folderPath
{

	NSString *filePath = @"";

	// folder path
	if(![folderPath isEqualToString:@""]){
		filePath = [NSString stringWithFormat: @"%@/%@", folderPath, romId];
	}

	return filePath;
 
}
//------------------------------------
// getFileAttribute
//------------------------------------
- (NSMutableDictionary *)getFileAttribute:(NSString*)filePath
{

	NSMutableDictionary *fileDict = [[[NSMutableDictionary alloc] init] autorelease];
	NSArray *dateArray = [[[NSArray alloc] init] autorelease];
	NSFileManager *myFile = [ NSFileManager defaultManager];

	BOOL fileExist = NO;
	NSNumber *fileTypeCode;
	NSString *fileName = @"";
	NSString *fileType = @"";
	NSMutableString *fileTypeName = @"";
	NSDate *fileDate;
	NSMutableString *fileDateStr;
	NSString *fileTimeStr;
	NSDictionary *fileAttb;

	double fileSize;
	NSString* fileSizeStr;
	int ret;

	// 
	ret = [self checkFileExist:filePath];

	if(ret != FILE_EXIST_NOT_FOUND){

		fileExist = YES;
		fileName = [myFile displayNameAtPath:filePath];
		fileAttb = [myFile fileAttributesAtPath:[filePath stringByExpandingTildeInPath] 
							traverseLink:YES];
//		NSLog(@"NSFileType=%@", [fileAttb valueForKey:@"NSFileType"]);
//		NSLog(@"fileSize=%f", [[fileAttb valueForKey:@"NSFileSize"] doubleValue]);
		// file
		if([fileAttb valueForKey:@"NSFileType"] == NSFileTypeRegular){
			fileType = @"file";
			fileSize = [[fileAttb valueForKey:@"NSFileSize"] intValue];
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		// folder
		else if([fileAttb valueForKey:@"NSFileType"] == NSFileTypeDirectory){
			fileType = @"folder";
			// calc folder size
			NSDirectoryEnumerator *enumDir = [myFile enumeratorAtPath:filePath];
			while([enumDir nextObject]) {
				fileSize += [[[enumDir fileAttributes] objectForKey:NSFileSize] intValue];
			}
//			fileSize = [[fileAttb valueForKey:@"NSFileSize"] intValue];
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}

		fileTypeCode = [fileAttb objectForKey:@"NSFileHFSTypeCode"];
		if(fileTypeCode) {
			fileTypeName = [NSMutableString stringWithString:NSFileTypeForHFSTypeCode([fileTypeCode unsignedLongValue])];
			// erase "'" Code
			[fileTypeName replaceOccurrencesOfString:@"'" withString:@"" options:0 range:NSMakeRange(0,[fileTypeName length])];
		}

		fileDate = [fileAttb valueForKey:@"NSFileModificationDate"];
		dateArray = [[fileDate description] componentsSeparatedByString:@" "];
		fileDateStr = [NSMutableString stringWithString:[dateArray objectAtIndex:0]];
		[fileDateStr replaceOccurrencesOfString:@"-" withString:@"/" options:0 range:NSMakeRange(0,[fileDateStr length])];		

		fileTimeStr = [dateArray objectAtIndex:0];

		[fileDict setValue:filePath forKey:@"filePath"];
		[fileDict setValue:fileName forKey:@"fileName"];
		[fileDict setValue:fileType forKey:@"fileType"];
		[fileDict setValue:fileTypeName forKey:@"fileTypeName"];
		[fileDict setValue:[NSNumber numberWithDouble:fileSize] forKey:@"fileSize"];
		[fileDict setValue:fileSizeStr forKey:@"fileSizeStr"];
		[fileDict setValue:fileDate forKey:@"fileDate"];
		[fileDict setValue:fileDateStr forKey:@"fileDateStr"];
		[fileDict setValue:fileTimeStr forKey:@"fileTimeStr"];

	}

	[fileDict setValue:[NSNumber numberWithBool:fileExist] forKey:@"fileExist"];

	return fileDict;
}
//------------------------------------
// getFileTypeName
//------------------------------------
- (NSString*)getFileTypeName:(NSString*)filePath
{
	NSMutableString *fileTypeName = @"";

	NSDictionary *fileAttb = [[NSFileManager defaultManager] 
							fileAttributesAtPath:[filePath stringByExpandingTildeInPath] 
							traverseLink:YES];

	NSNumber *fileTypeCode = [fileAttb objectForKey:@"NSFileHFSTypeCode"];
	if(fileTypeCode) {
		fileTypeName = [NSMutableString stringWithString:NSFileTypeForHFSTypeCode([fileTypeCode unsignedLongValue])];
		// erase "'" Code
		[fileTypeName replaceOccurrencesOfString:@"'" withString:@"" options:0 range:NSMakeRange(0,[fileTypeName length])];
	}

	return fileTypeName;
}


//------------------------------------
// resolveAliasFileWithPath
//------------------------------------
- (NSString*)resolveAliasFileWithPath:(NSString*)path
{
	FSRef aRef;
	BOOL aIsTargetFolder, aWasAliased;
	id aURL = nil;
	if(CFURLGetFSRef((CFURLRef)[NSURL fileURLWithPath:path], &aRef) == 0){
		return nil;
	}
	if(FSResolveAliasFile(&aRef, YES, (Boolean*)&aIsTargetFolder, (Boolean*)&aWasAliased) != noErr){
		return nil;
	}
	if(aWasAliased){
		if((aURL = [(NSURL*)CFURLCreateFromFSRef(kCFAllocatorDefault, &aRef) autorelease]) == nil){
		return nil;
		}
		path = [aURL path];
	}
	return path;
}
//------------------------------------
// checkFileIsCollect
//------------------------------------
- (int)checkFileIsCollect:(NSString*)filePath fileType:(NSString*)fileType
{

	NSFileManager *myFile = [ NSFileManager defaultManager];
	BOOL isDir;

	// folder not exist
	if(![myFile fileExistsAtPath:filePath isDirectory:&isDir]){
		return FILE_COLLECT_NOT_FOUND;
	}
	// not directory
	if( ([fileType isEqualToString:@"file"] && isDir == YES) ||
		([fileType isEqualToString:@"folder"] && isDir == NO)
		){
		return FILE_COLLECT_NOT_MATCH;
	}

	return FILE_COLLECT_MATCH;

}
//------------------------------------
// checkAndCreateFolder
//------------------------------------
- (int)checkAndCreateFolder:(NSString*)filePath
{

	NSFileManager *myFile = [ NSFileManager defaultManager];
	BOOL isDir;
	BOOL isSuccess = NO;

	// check directory
	if([myFile fileExistsAtPath:filePath isDirectory:&isDir]){
		// not directory
		if(isDir == NO){
			return FOLDER_CHECK_NOT_FOLDER;
		}
	}
	// create directory
	else{
		isSuccess = [self createNewDirectory:filePath];	
		// failed
		if(isSuccess == NO){
			return FOLDER_CHECK_CREATE_FAILED;
		}
	}

	return FOLDER_CHECK_SUCCESS;
}
//------------------------------------
// createNewDirectory
//------------------------------------
- (BOOL)createNewDirectory:(NSString*)filePath
{
	if([filePath isEqualToString:@""]){
		return NO;
	}

	return [[NSFileManager defaultManager] createDirectoryAtPath:filePath attributes:[self createfileAttbDict]];
}
//------------------------------------
// createNewFileWithString
//------------------------------------
- (BOOL)createNewFileWithString:(NSString*)filePath string:(NSString*)string
{
	
	if([filePath isEqualToString:@""] || [string isEqualToString:@""]){
		return NO;
	}

	NSFileManager *myFile = [ NSFileManager defaultManager];
	
	NSMutableData *data = [[[NSMutableData alloc] initWithCapacity:1] autorelease];
	[data appendBytes:[string cString] length:[string cStringLength]];

	return [myFile createFileAtPath:filePath contents:data attributes:[self createfileAttbDict]];

/*
	// use fileHandle 
	NSMutableData *data = [[[NSMutableData alloc] autorelease] initWithCapacity:1];
	[data appendBytes:[string cString] length:[string cStringLength]];

	NSFileHandle *fileHandle = [ NSFileHandle fileHandleForUpdatingAtPath: filePath];

	[fileHandle writeData: data];
	[fileHandle closeFile];
	return YES;
*/
}
//------------------------------------
// createfileAttbDict
//------------------------------------
- (NSDictionary*)createfileAttbDict
{
	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSDate date], NSFileModificationDate,
				@"owner", @"NSFileOwnerAccountName",
				@"group", @"NSFileGroupOwnerAccountName",
				nil, @"NSFilePosixPermissions",
				[NSNumber numberWithBool:NO], @"NSFileExtensionHidden",
			nil];
}

//------------------------------------
// deleteLastString
//------------------------------------
- (NSString*)deleteLastString:(NSString *)str delStr:(NSString*)delStr isDel:(BOOL*)isDel
{

	int strLen = [str length];
	int delStrLen = [delStr length];

	// trancate delStr
	if([[str substringFromIndex:(strLen - delStrLen)] isEqualToString:delStr]){
		*isDel = YES;
		return [str substringToIndex:(strLen - delStrLen)];
	}else{
		*isDel = NO;
		return str;
	}

}

@end