#import "HelperExtension.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation NSObject(helperExtension_)

//------------------------------------
// showFileInFinder
//------------------------------------
- (void)showFileInFinder:(NSString*)filePath
{
	// file is not found
	if([self checkFileExist:filePath] == FILE_EXIST_NOT_FOUND){
		[self displayMessageAlertFileIsNotFound:filePath];
		return;
	}

	[[NSWorkspace sharedWorkspace] selectFile:filePath inFileViewerRootedAtPath:nil];

}
//------------------------------------
// openURLWithBrowser
//------------------------------------
- (BOOL)openURLWithBrowser:(NSString*)url
{

	if(!url){
		return NO;
	}

	// decode
	url = [self decodeToPercentEscapesString:url];
//	NSLog(@"url=%@", url);

	// open browser
	return [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:url]];

}
//------------------------------------
// searchItemWithParams
//------------------------------------
- (BOOL)searchItemWithParams:(NSDictionary*)params
{

	// get params
	int keywordType = [[params valueForKey:@"keywordType"] intValue];
	NSString *url = [params valueForKey:@"url"];
	NSString *romId = [params valueForKey:@"romId"];
	NSString *title = [params valueForKey:@"title"];
	NSString *titlejp = [params valueForKey:@"titlejp"];
	BOOL isTrim = [[params valueForKey:@"isTrim"] boolValue];

	NSString *keyword = @"";

	// ID
	if(keywordType == 0){
		keyword = romId;
	}
	// title
	else if(keywordType == 1){
		keyword = title;
	}
	// titlejp
	else if(keywordType == 2){
		keyword = titlejp;
	}
	
	// trim keyword
	if(isTrim == YES){
		keyword = [self trimKeywordFromString:keyword];
	}

	// append keyword
	url = [url stringByAppendingString:keyword];

//	NSLog(@"url=%@", url);

	// search with browser
	return [self openURLWithBrowser:url];

}
//------------------------------------
// executeAppleScript
//------------------------------------
- (NSDictionary*)executeAppleScript:(NSString*)script
{
	// call apple script
	NSDictionary *errorInfo = nil;
	NSAppleScript *as = [[[NSAppleScript alloc] initWithSource:script] autorelease];
	[as executeAndReturnError:&errorInfo];

	return errorInfo;
}

//------------------------------------
// getChildNode
//------------------------------------
- (NSXMLNode*)getChildNode:(NSXMLNode*)node name:(NSString*)name
{
	NSEnumerator *e = [[node children] objectEnumerator];

	NSXMLNode *childNode;
	while (childNode = [e nextObject]){
		if ([[childNode name] isEqualToString:name]){
			return childNode;
		}
	}

	return nil;
}
//------------------------------------
// getChildNodes
//------------------------------------
- (NSArray*)getChildNodes:(NSXMLNode*)node
{
	NSMutableArray *nodes = [[NSMutableArray arrayWithCapacity:[[node children] count]] retain];
	NSEnumerator *e = [[node children] objectEnumerator];

	NSXMLNode *childNode;
	while (childNode = [e nextObject]){
		[nodes addObject:[childNode stringValue]];
	}
	return [nodes autorelease];
}
//------------------------------------
// postItemInfoDidChangedMainNotification
//------------------------------------
- (void)postItemInfoDidChangedMainNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED_MAIN object:nil];
}
//------------------------------------
// postItemInfoDidChangedCloneNotification
//------------------------------------
- (void)postItemInfoDidChangedCloneNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED_CLONE object:nil];
}
//------------------------------------
// postItemStatusDidChangedMainNotification
//------------------------------------
- (void)postItemStatusDidChangedMainNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_STATUS_DID_CHANGED_MAIN object:self];
}
//------------------------------------
// postItemStatusDidChangedCloneNotification
//------------------------------------
- (void)postItemStatusDidChangedCloneNotification
{
	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:ITEM_NOTIF_STATUS_DID_CHANGED_CLONE object:self];
}
@end