#import "ImageItem.h"

@implementation ImageItem
//------------------------------------
// init
//------------------------------------
- (id)initWithFilePath:(NSString*)filePath
				image:(NSImage*)image
				notifType:(int)notifType
{
    if (self = [super init])
	{
		[self setImage:image];
		notifType_ = notifType;			// not used

		if([self createConnection:filePath] == NO){
//			return nil;
		}

    }
    return self;
}

//------------------------------------
// createConnection
//------------------------------------
- (BOOL)createConnection:(NSString*)filePath
{

//	NSLog(@"filePath=%@", filePath);

	// get image
	NSURL *url = [NSURL fileURLWithPath:filePath];

	NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url
															cachePolicy:NSURLRequestUseProtocolCachePolicy
															timeoutInterval:5
									];

	[self setConnection:[[[NSURLConnection alloc] initWithRequest:request delegate:self] autorelease]];

	if([self connection] != nil){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// didReceiveResponse
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	// none
}
//------------------------------------
// didReceiveData
//------------------------------------
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
	if(!imageData_) imageData_ = [[NSMutableData data] retain];
	[imageData_ appendData:data];
}
//------------------------------------
// didFailWithError
//------------------------------------
- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
	NSString *filePath = [[NSBundle mainBundle] pathForResource:@"image_noimage" ofType:@"png"];
	NSImage *image = [[[NSImage alloc] initByReferencingFile:filePath] autorelease];
	[self setImage:image];	
	[self notifyImageItemDidLoad:self];
}
//------------------------------------
// connectionDidFinishLoading
//------------------------------------
- (void)connectionDidFinishLoading:(NSURLConnection*)connection
{
	NSImage *image = [[[NSImage alloc] initWithData:imageData_] autorelease];
	[self setImage:image];	
	[self notifyImageItemDidLoad:self];
}
//------------------------------------
// notifyDownloadItemChange
//------------------------------------
- (void)notifyImageItemDidLoad:(ImageItem*)item
{
	NSNotificationCenter * nc = [NSNotificationCenter defaultCenter];
	if(notifType_ == IMAGE_ITEM_NOTIF_TABLE_MAIN){
		[nc postNotificationName:IMAGE_ITEM_NOTIF_NAME_TABLE_MAIN object:item];
	}
	else if(notifType_ == IMAGE_ITEM_NOTIF_TABLE_CLONE){
		[nc postNotificationName:IMAGE_ITEM_NOTIF_NAME_TABLE_CLONE object:item];
	}
	else if(notifType_ == IMAGE_ITEM_NOTIF_MATRIX_MAIN){
		[nc postNotificationName:IMAGE_ITEM_NOTIF_NAME_MATRIX_MAIN object:item];
	}
	else if(notifType_ == IMAGE_ITEM_NOTIF_MATRIX_GALLERY){
		[nc postNotificationName:IMAGE_ITEM_NOTIF_NAME_MATRIX_GALLERY object:item];
	}
}

//------------------------------------
// cancelConnection
//------------------------------------
- (void)cancelConnection
{
	if(connection_ != nil){
		[connection_ cancel];
	}
}

//------------------------------------
// connection
//------------------------------------
- (void)setConnection:(NSURLConnection*)connection
{
	[connection retain];
	[connection_ release];
	connection_ = connection;
}
- (NSURLConnection*)connection
{
	return connection_;
}
//------------------------------------
// image
//------------------------------------
- (void)setImage:(NSImage*)image
{
	[image retain];
	[image_ release];
	image_ = image;
}
- (NSImage*)image
{
	return image_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[self cancelConnection];

	[image_ release];
	[imageData_ release];
	[connection_ release];
    [super dealloc];
}

@end
