#import "ImageScrollView.h"
#import "FileAttributeExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

static NSString *defaultKeyName1 = @"optDrawBackGroundImage";
static NSString *defaultKeyName2 = @"pathBackGroundImage";
static NSString *defaultKeyName3 = @"optBackGroundImageDrawType";
static NSString *defaultKeyName4 = @"optBackGroundImageAlpha";

@implementation ImageScrollView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self changeImage];
	[self setDrawsBackground:NO];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:defaultKeyName1 options:0 context:nil];
	[defaults addObserver:self forKeyPath:defaultKeyName2 options:0 context:nil];
	[defaults addObserver:self forKeyPath:defaultKeyName3 options:0 context:nil];
	[defaults addObserver:self forKeyPath:defaultKeyName4 options:0 context:nil];
}
//=======================================================================
// method
//=======================================================================
//------------------------------------
// changeImage
//------------------------------------
- (void)changeImage
{

	NSImage *image = nil;
	NSColor *bgColor = [self backgroundColor];

	// enable image
	if([self defaultBoolValue:defaultKeyName1] == YES){
		image = [self createBaseImage];
		if(image){
			// tile
			if([self defaultIntValue:defaultKeyName3] == DEFAULT_BG_IMAGE_DRAW_TYPE_TILE){
				float alpha = [self defaultFloatValue:defaultKeyName4];
				bgColor = [NSColor colorWithPatternImage:[self createPatternImage:image alpha:alpha]];
			}
			// prop / fit
			else{
				bgColor = [NSColor colorWithCalibratedWhite:0.1 alpha:1.0];
			}
		}
	}

	[self setImage:image];
	[self setBgColor:bgColor];
	[self setNeedsDisplay:YES];
}
//------------------------------------
// createBaseImage
//------------------------------------
- (NSImage*)createBaseImage
{
	NSImage *image = nil;
	NSString *imagePath = [self defaultSettingFilePath:MAME_FILE_TYPE_BACKGROUND_IMAGE];
	if([self checkFileExist:imagePath] == FILE_EXIST_FILE){
		image = [[[NSImage alloc] initByReferencingFile:imagePath] autorelease];
	}
	return image;
}
//------------------------------------
// createPatternImage
//------------------------------------
- (NSImage*)createPatternImage:(NSImage*)image alpha:(float)alpha
{
	NSImage *patternImage = [[[NSImage alloc] initWithSize:[image size]] autorelease];

	// draw image
	[patternImage lockFocus];

	[image compositeToPoint:NSZeroPoint operation:NSCompositeSourceOver fraction:alpha];

	[patternImage unlockFocus];

	return patternImage;

}
//------------------------------------
// drawRect
//------------------------------------
- (void)drawRect:(NSRect)rect
{

	NSRect contentRect = [[self contentView] frame];

	// draw background
	[[self bgColor] set];
	NSRectFill(contentRect);

	NSImage *image = [self image];
	int drawType = [self defaultIntValue:defaultKeyName3];

	if(image &&
		(
			drawType == DEFAULT_BG_IMAGE_DRAW_TYPE_PROP ||
			drawType == DEFAULT_BG_IMAGE_DRAW_TYPE_FIT
		)
	){

		NSPoint	imagePoint = NSZeroPoint;
		imagePoint.x = contentRect.origin.x;
		imagePoint.y = contentRect.origin.y + contentRect.size.height;

		float alpha = [self defaultFloatValue:defaultKeyName4];

		// fit
		if(drawType == DEFAULT_BG_IMAGE_DRAW_TYPE_FIT){
			[image setSize:NSMakeSize(contentRect.size.width, contentRect.size.height)];
			[image setScalesWhenResized:YES];
		}else{
			// set image size
			[image setSize:[self setCellImageSize:[image size]
										frameSize:contentRect.size
										paddingSize:NSMakeSize(0,0)
							]
			];
			[image setScalesWhenResized:YES];

			// set image rect
			NSRect imageRect = [self setCellImageRect:[image size]
											frameRect:contentRect
								];

			imagePoint.x += imageRect.origin.x;
			imagePoint.y -= imageRect.origin.y;
		}

	//	NSLog(@"image size=%.1f, %.1f", [image size].width, [image size].height);
	//	NSLog(@"image point=%.1f, %.1f", imagePoint.x, imagePoint.y);

		[NSGraphicsContext saveGraphicsState];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver fraction:alpha];

		[NSGraphicsContext restoreGraphicsState];
	}

//	[image release];

	[super drawRect:rect];

}
//------------------------------------
// isOpaque
//------------------------------------
- (BOOL)isOpaque
{
	return NO;
}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if( [keyPath isEqualToString:defaultKeyName1] ||
		[keyPath isEqualToString:defaultKeyName2] ||
		[keyPath isEqualToString:defaultKeyName3]
	){
		[self changeImage];
	}
	else if([keyPath isEqualToString:defaultKeyName4]
	){
		// tile
		if([self defaultIntValue:defaultKeyName3] == DEFAULT_BG_IMAGE_DRAW_TYPE_TILE){
			[self changeImage];
		}else{
			[self setNeedsDisplay:YES];
		}
	}
}
//------------------------------------
// image
//------------------------------------
- (void)setImage:(NSImage*)image
{
	[image retain];
	[image_ release];
	image_ = image;
}

- (NSImage*)image
{
    return image_;
}
//------------------------------------
// bgColor
//------------------------------------
- (void)setBgColor:(NSColor*)bgColor
{
	[bgColor retain];
	[bgColor_ release];
	bgColor_ = bgColor;
}

- (NSColor*)bgColor
{
    return bgColor_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:defaultKeyName1];
	[defaults removeObserver:self forKeyPath:defaultKeyName2];
	[defaults removeObserver:self forKeyPath:defaultKeyName3];
	[defaults removeObserver:self forKeyPath:defaultKeyName4];

	[image_ release];
	[bgColor_ release];
    [super dealloc];
}

@end
