#import "ItemDescTextCell.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation ItemDescTextCell
//------------------------------------
// init
//------------------------------------
- (id)init
{
    if (self = [super init])
	{
		[self setIsExist:NO];
		[self setLabelColorNo:0];
		[self setImage:nil];
    }
    return self;
}

//------------------------------------
// drawInteriorWithFrame
//------------------------------------
- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{

	NSRect insetRect = NSInsetRect(cellFrame,0,0);

	//
	// ParagraphStyle
	//
	NSMutableParagraphStyle *lineStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
	[lineStyle setLineBreakMode:NSLineBreakByTruncatingTail];
//	[lineStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];

	//
	// color
	//
	NSColor *titleColor = [NSColor blackColor];
	NSColor *descColor = [NSColor grayColor];

	// success
	if([self isExist] == NO){
		titleColor = [NSColor grayColor];
	}

	//
	// attribute
	//
	// title
	float fontSize = [self defaultFloatValue:@"optFontSizeList"];
	float titleFontSize = fontSize + 1.0;
	NSMutableDictionary *titleAttr = [[[NSMutableDictionary alloc] initWithObjectsAndKeys:
											 titleColor, NSForegroundColorAttributeName,
											 [NSFont systemFontOfSize:titleFontSize], NSFontAttributeName,
											 lineStyle, NSParagraphStyleAttributeName,
											 nil
										] autorelease];
											
	// desc1
	float descFontSize = fontSize - 0.5;
	NSMutableDictionary *descAttr = [[[NSMutableDictionary alloc] initWithObjectsAndKeys:
												descColor, NSForegroundColorAttributeName,
												[NSFont systemFontOfSize:descFontSize], NSFontAttributeName,
												lineStyle, NSParagraphStyleAttributeName,
												nil
										] autorelease];
											

	//
	// stringValue
	//
	NSArray *values = [[self stringValue] componentsSeparatedByString:@"\t"];

	//
	// title
	//
	NSString *title = @"";
	if([values count] >= 1){
		title = [values objectAtIndex:0];
	}
	NSSize titleSize = [title sizeWithAttributes:titleAttr];
	
	NSString *desc1 = @"";
//	NSString *desc2 = @"";

	//
	// desc1
	//
	if([values count] >= 1){
		desc1 = [values objectAtIndex:1];
	}
	//
	// desc2
	//
//	if([values count] >= 2){
//		desc2 = [values objectAtIndex:2];
//	}

	NSSize desc1Size = [desc1 sizeWithAttributes:descAttr];
//	NSSize desc2Size = [desc2 sizeWithAttributes:descAttr];

	//
	// rect
	//
	float vPadding = 4.0;
	float hPadding = 2.0;
	NSSize imageSize = NSMakeSize(16, 13);

//	float totalHeight = titleSize.height + desc1Size.height + vPadding;
	
	NSRect textRect = NSMakeRect(insetRect.origin.x + hPadding,
//								  insetRect.origin.y + (insetRect.size.height - totalHeight) / 2,
								  insetRect.origin.y,
								  insetRect.size.width - hPadding,
//								  totalHeight
								  insetRect.size.height
								);
	
	NSRect titleRect = NSMakeRect(
//									textRect.origin.x, 
									textRect.origin.x + imageSize.width, 
//									textRect.origin.y + (textRect.size.height / 2) - titleSize.height - 2,
									textRect.origin.y,
									textRect.size.width - imageSize.width,
									titleSize.height
								);

	NSRect desc1Rect = NSMakeRect(textRect.origin.x + imageSize.width,
//									textRect.origin.y + (textRect.size.height / 2) + 2,
									textRect.origin.y + titleRect.size.height + vPadding,
									textRect.size.width - imageSize.width,
									desc1Size.height
								);
/*
	NSRect desc2Rect = NSMakeRect(textRect.origin.x + imageSize.width,
//									textRect.origin.y + (textRect.size.height / 2) + 2,
									desc1Rect.origin.y + desc1Rect.size.height + 2,
									textRect.size.width - imageSize.width,
									desc2Size.height
								);
*/

    // draw image
    if([self image]){
		NSPoint	imagePoint;

		imagePoint.x = insetRect.origin.x + hPadding;
		imagePoint.y = insetRect.origin.y + imageSize.height;

		[NSGraphicsContext saveGraphicsState];

		// draw image
		[[self image] compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		[NSGraphicsContext restoreGraphicsState];
	}

	//
	// label color
	//
	if([self labelColorNo] > 0){
		NSColor *labelColor = [self getLabelColor:[self labelColorNo]];
		if(labelColor != nil){
			NSRect labelRect = titleRect;
			NSSize stringSize = titleSize;
			labelRect.origin.x -= hPadding;	

			if(!NSEqualSizes(stringSize, NSZeroSize)){
				labelRect.size.width = stringSize.width + 5;
			}else{
				labelRect.size.width = labelRect.size.height;
			}
			if(labelRect.size.width > cellFrame.size.width - imageSize.width){
				labelRect.size.width = cellFrame.size.width - imageSize.width;
			}

			[labelColor set];
			NSBezierPath *labelBoxPath = [self setRoundPathForRect:labelRect radius:8];
			[labelBoxPath fill];
		}
	}

	//
	// highlighted
	//
	if([self isHighlighted]){
		[titleAttr setValue:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];
		[descAttr setValue:[NSColor whiteColor] forKey:NSForegroundColorAttributeName];
	}

	//
	// draw text
	//
	float total1Height = titleRect.size.height + desc1Rect.size.height + vPadding;
//	float total2Height = titleRect.size.height + desc1Rect.size.height + vPadding + desc2Rect.size.height;

	[title drawInRect:titleRect withAttributes:titleAttr];

	if(total1Height <= textRect.size.height){
		[desc1 drawInRect:desc1Rect withAttributes:descAttr];
	}
//	if(total2Height <= textRect.size.height){
//		[desc2 drawInRect:desc2Rect withAttributes:descAttr];
//	}
}
//------------------------------------
// isExist
//------------------------------------
- (void)setIsExist:(BOOL)isExist
{
	isExist_ = isExist;
}
- (BOOL)isExist
{
	return isExist_;
}
//------------------------------------
// labelColorNo
//------------------------------------
- (void)setLabelColorNo:(int)labelColorNo
{
	labelColorNo_ = labelColorNo;
}
- (int)labelColorNo
{
	return labelColorNo_;
}
//------------------------------------
// image
//------------------------------------
- (void)setImage:(NSImage*)image
{
	[image retain];
    [image_ release];
    image_ = image;
}
- (NSImage*)image
{
	return image_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[super dealloc];
}
@end
