#import "ItemSearchField.h"
#import "ViewMain.h"
#import "SearchExtension.h"
#import "UserDefaultsExtension.h"

@implementation ItemSearchField

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];

	// set action
	[self setTarget:self];
	[self setAction:NSSelectorFromString(@"searchWithField:")];

	// NO:incremental
//	[[self cell] setSendsWholeSearchString:NO];
	[self setSearchIncremental];

	// create search text menu
	[self createSearchTextMenu];
	[self setSearchFormat:@""];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:@"optSearchIncremental" options:0 context:nil];

}
//=======================================================================
// Action
//=======================================================================
//------------------------------------
// searchWithField
//------------------------------------
- (IBAction)searchWithField:(id)sender
{

	NSString *searchString = [self stringValue];
	NSString *searchFormat = [self searchFormat];
	NSString *searchFormatString = @"";

	// selected format
	if(![searchFormat isEqualToString:@""]){
		searchFormatString = searchFormat;
	}
	// ALL
	else{
		searchFormatString = [self searchFormatAll];
	}

	// create pred
	NSPredicate *pred = [self createSearchStringPred:searchString 
									searchFormatString:searchFormatString
						];

	// search
	[viewMain searchWithString:searchString pred:pred];


}
//------------------------------------
// changeSearchFormat
//------------------------------------
- (IBAction)changeSearchFormat:(id)sender
{

	NSString *record = [sender representedObject];
	NSArray *cols = [record componentsSeparatedByString:@","];

	NSString *searchFormat = [cols objectAtIndex:1];
	int itemIndex = [[cols objectAtIndex:0] intValue];
	int i;

	for(i = 0; i < [[searchTextMenu_ itemArray] count]; i++){
		if(i == itemIndex){
			[[searchTextMenu_ itemAtIndex:i] setState:1];
		}else{
			[[searchTextMenu_ itemAtIndex:i] setState:0];
		}
	}

	// set menu template
	[[self cell] setSearchMenuTemplate:searchTextMenu_];

	[self setSearchFormat:searchFormat];

	// search
	if(![[self stringValue] isEqualToString:@""]){
		[self searchWithField:nil];
	}

}
//------------------------------------
// setSearchIncremental
//------------------------------------
- (void)setSearchIncremental
{
	BOOL inc = [self defaultBoolValue:@"optSearchIncremental"];

	// NO:incremental
	[[self cell] setSendsWholeSearchString:!inc];

}
//------------------------------------
// createSearchTextMenu
//------------------------------------
- (void)createSearchTextMenu
{
	NSArray *cols;
	NSString *itemTitle;
	NSString *itemFormat;
	NSString *itemRecord;
	int i;

	// create menu array
	NSArray *items = [self searchColumnItems];

    searchTextMenu_ = [[[NSMenu alloc] initWithTitle:@"searchTextMenu_"] autorelease];

	for(i = 0; i < [items count]; i++){

		cols = [[items objectAtIndex:i] componentsSeparatedByString:@","];
		itemTitle = [cols objectAtIndex:0];
		itemFormat = [cols objectAtIndex:2];
		itemRecord = [NSString stringWithFormat:@"%d,%@",i, itemFormat];

 		NSMenuItem *menuItem = [[NSMenuItem alloc] initWithTitle:itemTitle action:@selector(changeSearchFormat:) keyEquivalent:@""];

		[menuItem setTarget:self];
		[menuItem setRepresentedObject:itemRecord];

        [searchTextMenu_ addItem:menuItem];
		[menuItem release];
    }

	[[searchTextMenu_ itemAtIndex:0] setState:1];

	// set menu template
	[[self cell] setSearchMenuTemplate:searchTextMenu_];

	// set setSearchFormatAll
	[self setSearchFormatAll:[self createSearchFormatAll:items]];

	[items release];
}

//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:@"optSearchIncremental"]){
		[self setSearchIncremental];
	}

}
//------------------------------------
// clear Search Text
//------------------------------------
- (void)clearSearchText
{
	[self setStringValue:@""];
}

//------------------------------------
// searchFormat
//------------------------------------
- (void)setSearchFormat:(NSString *)searchFormat
{
	[searchFormat retain];
    [searchFormat_ release];
    searchFormat_ = searchFormat;

}
- (NSString *)searchFormat
{
    return searchFormat_;
}
//------------------------------------
// searchFormatAll
//------------------------------------
- (void)setSearchFormatAll:(NSString *)searchFormatAll
{
	[searchFormatAll retain];
    [searchFormatAll_ release];
    searchFormatAll_ = searchFormatAll;

}
- (NSString *)searchFormatAll
{
    return searchFormatAll_;
}

/*
//------------------------------------
// textDidChange
//------------------------------------
- (BOOL)textDidChange:(NSNotification *)aNotification
{
//	[self searchWithField:nil];
	return YES;
}
//------------------------------------
// textDidEndEditing
//------------------------------------
- (BOOL)textDidEndEditing:(NSNotification *)aNotification
{
//	[self searchWithField:nil];
	return YES;
}
*/
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:@"optSearchIncremental"];

	[searchFormat_ release];
	[searchFormatAll_ release];
	
    [super dealloc];
}

@end
