#import "MenuItemLangType.h"
#import "UserDefaultsExtension.h"

@implementation MenuItemLangType

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// createSubMenu
	subMenu_ = [[[NSMenu alloc] initWithTitle:@"langTypeMenu"] autorelease];
	[self createSubMenu:subMenu_];

	//set delegate menu
	[subMenu_ setDelegate:self];

}

//------------------------------------
// changeLangType
//------------------------------------
- (IBAction)changeLangType:(id)sender
{
	int langType = [[sender representedObject] intValue];

	if(langType != [self defaultLangType]){
		[self setDefaultIntValue:langType key:@"optLangType"];

		// post notification
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
		[nc postNotificationName:DEFAULT_NOTIF_LANG_TYPE_DID_CHANGED object:self];
	}
}
//------------------------------------
// createSubMenu
//------------------------------------
- (void)createSubMenu:(NSMenu*)aMenu
{

	NSArray *cols;
	NSString *itemName;
	NSString *keyEquiv;
	int itemValue;
	int i;
	id record;

	// remove menuItem
	while (record = [[[aMenu itemArray] objectEnumerator] nextObject]) {
		[aMenu removeItem:record];
	}

	NSArray *menuArray = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuArray count]; i++){

		cols = [[menuArray objectAtIndex:i] componentsSeparatedByString:@","];
		
		itemName = [cols objectAtIndex:0];
		itemValue = [[cols objectAtIndex:1] intValue];
		keyEquiv = [cols objectAtIndex:2];

		// set action
 		SEL sel = NSSelectorFromString(@"changeLangType:");

		menuItem = [[[NSMenuItem alloc] initWithTitle:itemName action:sel keyEquivalent:keyEquiv] autorelease];
		[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:[NSNumber numberWithInt:itemValue]];

		[aMenu addItem:menuItem];
	}

	[self setSubmenu:aMenu];

//	[menuArray release];

}
//------------------------------------
// updateSubMenu
//------------------------------------
- (void)updateSubMenu:(NSMenu*)aMenu
{

	int itemValue;
	id menuItem;
	int langType = [self defaultLangType];

	NSEnumerator *enumArray = [[aMenu itemArray] objectEnumerator];
	while (menuItem = [enumArray nextObject]) {
		itemValue = [[menuItem representedObject] intValue];
		// set state
		if(itemValue == langType){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}
	}

}
//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// title, value, keyEquiv
	return [[NSArray alloc] initWithObjects: 
			@"English,0,e",
			@"Japanese,1,j",
			nil
	];
}
//------------------------------------
// menuNeedsUpdate
//------------------------------------
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateSubMenu:menu];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[subMenu_ release];	
    [super dealloc];
}


@end
