#import "MenuSelectCategory.h"
#import "ViewCategory.h"
#import "ImageBindExtension.h"
#import "UserDefaultsExtension.h"

@implementation MenuSelectCategory

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[super setDelegate:self];

	[self createMenuItem];

}
//------------------------------------
// changeCategoryNo
//------------------------------------
- (IBAction)changeCategoryNo:(id)sender
{
	// change category
	[viewCategory changeCategory:sender];

}
//------------------------------------
// changeCategoryOption
//------------------------------------
- (IBAction)changeCategoryOption:(id)sender
{

	NSString *key = [self getDefaultKey:[viewCategory categoryNo]];

	if(![key isEqualToString:@""]){
		if([self defaultBoolValue:key] == YES){
			[self setDefaultBoolValue:NO key:key];
		}else{
			[self setDefaultBoolValue:YES key:key];
		}
	}

	[viewCategory reloadCategory:nil];
}
//------------------------------------
// nullAction
//------------------------------------
- (IBAction)nullAction:(id)sender
{
	// nullAction
}

//------------------------------------
// createMenuItem
//------------------------------------
- (void)createMenuItem
{

	NSArray *cols;
	NSString *title;
	int categoryNo;
	int i;
	id record;

	// remove menuItem
	while (record = [[[self itemArray] objectEnumerator] nextObject]) {
		[self removeItem:record];
	}

	NSArray *menuItems = [self menuItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		cols = [[menuItems objectAtIndex:i] componentsSeparatedByString:@","];

		// add item separator
        if([[cols objectAtIndex:0] isEqualToString:@"-"]){
			[self addItem:[NSMenuItem separatorItem]];
			continue;
		}

		categoryNo = [[cols objectAtIndex:1] intValue];

		// set title
		if(categoryNo == -1){
			title = [self getOptionTitle:[viewCategory categoryNo]];
		}else{
			title = [cols objectAtIndex:0];
		}

		// set action
		SEL sel;
        if([[cols objectAtIndex:2] isEqualToString:@""]){
			sel = NSSelectorFromString(@"changeCategoryNo:");
		}else{
			sel = NSSelectorFromString([cols objectAtIndex:2]);
		}

		menuItem = [[[NSMenuItem alloc] initWithTitle:title action:sel keyEquivalent:@""] autorelease];
		[menuItem setTarget:self];
		[menuItem setRepresentedObject:[NSNumber numberWithInt:categoryNo]];

		// set image
		if(categoryNo != -1){
			[menuItem setImage:[self getCategoryIconImage:categoryNo]];
		}else{
			[menuItem setImage:[NSImage imageNamed:@"icon_gear"]];
		}

		[self addItem:menuItem];
	}

//	[menuItems release];

}
//------------------------------------
// updateMenuItem
//------------------------------------
- (void)updateMenuItem
{

	id record;
	int categoryNo;
	NSString *key;
	int i;

	NSArray *menuArray = [self itemArray];

	for(i = 0; i < [menuArray count]; i++){

		record = [menuArray objectAtIndex:i];

		categoryNo = [[record representedObject] intValue];

		// set title
		if(categoryNo == -1){
			[record setTitle:[self getOptionTitle:[viewCategory categoryNo]]];
		}

		// set action
		if(categoryNo == -1){
			SEL sel;
			if(![[self getDefaultKey:[viewCategory categoryNo]] isEqualToString:@""]){
				sel = NSSelectorFromString(@"changeCategoryOption:");
			}else{
				sel = nil;
			}
			[record setAction:sel];
		}

		// set state
		// category no
		if(categoryNo != -1){
			if(categoryNo == [viewCategory categoryNo]){
				[record setState:1];
			}else{
				[record setState:0];
			}
		}
		// category option
		else{
			key = [self getDefaultKey:[viewCategory categoryNo]];
			if(![key isEqualToString:@""]){
				if([self defaultBoolValue:key] == YES){
					[record setState:1];
				}else{
					[record setState:0];
				}
			}else{
				[record setState:0];
			}
		}
	}

}
//------------------------------------
// getDefaultKey
//------------------------------------
- (NSString*)getDefaultKey:(int)no
{
	NSString *key = @"";

	switch(no){
		case CATEGORY_TYPE_MANUFACTURE:
			key = @"optCategoryAdjust";
			break;
		case CATEGORY_TYPE_ADDVERSION:
			key = @"optCategoryClone";
			break;
		case CATEGORY_TYPE_FILEDATE:
			key = @"optCategoryClone";
			break;
		case CATEGORY_TYPE_COMMENT:
			key = @"optCategorySplit";
			break;
	}

	return key;

}
//------------------------------------
// getOptionTitle
//------------------------------------
- (NSString*)getOptionTitle:(int)no
{
	NSString *title = @"No Option";

	switch(no){
		case CATEGORY_TYPE_MANUFACTURE:
			title = @"Adjust Title";
			break;
		case CATEGORY_TYPE_ADDVERSION:
			title = @"Include Clones";
			break;
		case CATEGORY_TYPE_FILEDATE:
			title = @"Include Clones";
			break;
		case CATEGORY_TYPE_COMMENT:
			title = @"Split Tag";
			break;
	}

	return title;

}
//------------------------------------
// menuItems
//------------------------------------
- (NSArray*)menuItems
{
	// titile, value, action, key
	return [[NSArray alloc] initWithObjects: 
				@"Playlist,0,,",
				@"-,,,",
				@"Letter,1,,",
				@"Manufacture,2,,",
				@"Year,3,,",
				@"Date,4,,",
				@"ImportVersion,5,,",
				@"Tag,6,,",
				@"-,,,",
				@"No Option,-1,changeCategoryOption:,",
			nil
			];
}
//=======================================================================
#pragma mark - Delegate -
//=======================================================================
-( void )menuNeedsUpdate:(NSMenu *)menu
{
	[self updateMenuItem];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
    [super dealloc];
}


@end
