#import "PreviewImageView.h"
//#import "ViewMain.h"
#import "ThumbsExtension.h"
#import "HelperExtension.h"
#import "FileAttributeExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation PreviewImageView

//------------------------------------
// init
//------------------------------------
- (id)init
{
    if (self = [super init])
	{
		// init code
		[self setIsExist:NO];
		[self setHasImage:NO];		
		[self setRomId:@""];
		[self setFileIndex:0];
		[self setMaxIndex:0];
		
     }
    return self;
}
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// delete focus ring
	[btnImageNext setFocusRingType:NSFocusRingTypeNone];
	[btnImageBack setFocusRingType:NSFocusRingTypeNone];
	[self setButtonImageIndexEnabled];
	[self setBoxImageIndexHidden];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	
	int tag = [self tag];
	if(tag == ITEM_TYPE_MAIN){
		[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED_MAIN object:nil];
		[nc addObserver:self selector:@selector(handleThumbsTypeDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_MAIN object:nil];
	}
	else if(tag == ITEM_TYPE_CLONE){
		[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED_CLONE object:nil];
		[nc addObserver:self selector:@selector(handleThumbsTypeDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_CLONE object:nil];
	}

	// set observer
	[targetArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];
}
//=======================================================================
// actions
//=======================================================================
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
}
//------------------------------------
// changeImageIndex
//------------------------------------
- (IBAction)changeImageIndex:(id)sender
{
	[self selectImageIndex:[sender tag]];
}
//------------------------------------
// showImageInFinder
//------------------------------------
- (IBAction)showImageInFinder:(id)sender
{

	NSString *romId = [self romId];

	if(!romId || [romId isEqualToString:@""]){
		return;
	}

	int tag = [self tag];
	int imageType = 0;

	if(tag == ITEM_TYPE_MAIN){
		imageType = [self defaultThumbsImageTypeMain];
	}
	else if(tag == ITEM_TYPE_CLONE){
		imageType = [self defaultThumbsImageTypeClone];
	}

	[self showFileInFinder:[self getThumbsImagePath:romId
											imageType:imageType
											fileIndex:[self fileIndex]
											maxIndex:nil
							]
	];

}
//------------------------------------
// changeBackGroundImage
//------------------------------------
- (IBAction)changeBackGroundImage:(id)sender
{

	NSString *romId = [self romId];

	if(!romId || [romId isEqualToString:@""]){
		return;
	}

	int tag = [self tag];
	int imageType = 0;

	if(tag == ITEM_TYPE_MAIN){
		imageType = [self defaultThumbsImageTypeMain];
	}
	else if(tag == ITEM_TYPE_CLONE){
		imageType = [self defaultThumbsImageTypeClone];
	}

	NSString *filePath = [self getThumbsImagePath:romId
											imageType:imageType
											fileIndex:[self fileIndex]
											maxIndex:nil
							];

	// set default
	if(![filePath isEqualToString:@""]){
		[self setDefaultStringValue:filePath key:@"pathBackGroundImage"];
//		if([self defaultBoolValue:@"optDrawBackGroundImage"] == NO){
//			[self setDefaultBoolValue:YES key:@"optDrawBackGroundImage"];
//		}
	}
}
//------------------------------------
// nullAction
//------------------------------------
- (IBAction)nullAction:(id)sender
{
	// null action
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// selectImageIndex
//------------------------------------
- (void)selectImageIndex:(int)tag
{
	int fileIndex = [self fileIndex];

	if(tag == 0){
		fileIndex--;
		if(fileIndex < 0){
			fileIndex = [self maxIndex];
		}
	}else{
		fileIndex++;
		if(fileIndex > [self maxIndex]){
			fileIndex = 0;
		}
	}

	// set index
	[self setFileIndex:fileIndex];

	// create image view
	[self createImageView:[self romId] fileIndex:fileIndex];

}
//------------------------------------
// changeImageView
//------------------------------------
- (void)changeImageViewWithArray
{
	int tag = [self tag];
	NSString *romId = @"";
	NSString *key = @"romId";

	if(tag == ITEM_TYPE_MAIN){
		key = @"defaultRomId";
	}
	else if(tag == ITEM_TYPE_CLONE){
		key = @"cloneId";
	}

	if([[targetArrayController selectedObjects] count] > 0){
		id record = [[targetArrayController selectedObjects] objectAtIndex:0];
		romId = [record valueForKey:key];
	}
	// create image view
	[self createImageView:romId fileIndex:0];
}
//------------------------------------
// createImageView
//------------------------------------
- (void)createImageView:(NSString*)romId fileIndex:(int)fileIndex
{

	int tag = [self tag];

	NSImage *image = nil;
	int maxIndex = 0;
	int imageType = 0;
	BOOL isExist = NO;
	BOOL hasImage = NO;

	if(![romId isEqualToString:@""]){
		// isExist
		if([self checkRomExist:romId] != FILE_EXIST_NOT_FOUND){
			isExist = YES;
		}

		if(tag == ITEM_TYPE_MAIN){
			imageType = [self defaultThumbsImageTypeMain];
		}
		else if(tag == ITEM_TYPE_CLONE){
			imageType = [self defaultThumbsImageTypeClone];
		}

		// get thumbs image
		image = [self getThumbsImage:[self getThumbsImagePath:romId
													imageType:imageType
													fileIndex:fileIndex
													maxIndex:&maxIndex
									]
							hasImage:&hasImage
				];
	}

	[self setImage:image];
	[self setIsExist:isExist];
	[self setHasImage:hasImage];		

	[self setRomId:romId];
	[self setFileIndex:fileIndex];
	[self setMaxIndex:maxIndex];

	[self setBoxImageIndexHidden];

}
//------------------------------------
// setButtonImageIndexEnabled
//------------------------------------
- (void)setButtonImageIndexEnabled
{
	BOOL enabled = [self isDisplay];

	[btnImageNext setEnabled:enabled];
	[btnImageBack setEnabled:enabled];
}
//------------------------------------
// setBoxImageIndexHidden
//------------------------------------
- (void)setBoxImageIndexHidden
{
	if([self maxIndex] > 0){
		[boxImageIndex setHidden:NO];
	}else{
		[boxImageIndex setHidden:YES];
	}
}
//------------------------------------
// drawRect
//------------------------------------
- (void)drawRect:(NSRect)rect
{

	// draw background
	[[NSColor blackColor] set];
//	[[NSColor colorWithCalibratedWhite:0.2 alpha:1.0] set];
	NSRectFill([self bounds]);

    NSPoint	imagePoint;
    imagePoint.x = [self bounds].origin.x;
    imagePoint.y = [self bounds].origin.y;

    // draw image
    if([self image]){

		NSImage *image = [[self image] copy];
 
		// set image size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:[self bounds].size
									paddingSize:NSMakeSize(2,2)
						]
		];
		[image setScalesWhenResized:YES];

		// set image rect
		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:[self bounds]
							];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

		// set image round rect

		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:[self bounds]
								];

		// draw shadow rect
/*
		NSRect shadowRect = imageRoundRect;
		shadowRect.origin.x += 5;
		shadowRect.origin.y += 5;
		NSBezierPath *shadowPath = [self setRoundPathForRect:shadowRect radius:5];
		[[NSColor blackColor] set];
		[shadowPath fill];
*/
		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		[NSGraphicsContext restoreGraphicsState];

		[image release];

		//
		// draw label
		//
		if([self isExist] == NO){
			// now origin.y is buggy
			NSRect imageBounds;
			imageBounds.origin.x = rect.origin.x + imageRect.origin.x;
			imageBounds.origin.y = rect.origin.y + imageRect.origin.y;
//			if([controlView isFlipped]) {
//				imageBounds.origin.y += (imageRect.size.height - imageRect.origin.y);
//			}
			imageBounds.size = imageRect.size;

/*
			// draw label
			NSString *labelTitle = LABEL_NAME_NOT_FOUND;
			if(![labelTitle isEqualToString:@""]){
				NSSize stringSize = [labelTitle sizeWithAttributes:nil];
				if( imageBounds.size.width > stringSize.width + 6 &&
					imageBounds.size.height > stringSize.height + 4
				){
					// draw label
					[self drawLabelAndString:imageBounds
									withString:labelTitle
									fontSize:LABEL_STATUS_FSIZE
									fontColor:[NSColor whiteColor]
									labelColor:[self getLabelColor:LABEL_COLOR_NOT_FOUND]
									align:CELL_ALIGN_RIGHT
									valign:CELL_VALIGN_BOTTOM
									hPadding:LABEL_STATUS_HPADDING
									vPadding:LABEL_STATUS_VPADDING
									radius:LABEL_STATUS_RADIUS
					];
				}
			}
*/
		}

    }

}
//------------------------------------
// handleItemInfoDidChanged
//------------------------------------
-(void)handleItemInfoDidChanged:(NSNotification *)notification
{
	// create image
	[self changeImageViewWithArray];
}
//------------------------------------
// handleThumbsTypeDidChanged
//------------------------------------
-(void)handleThumbsTypeDidChanged:(NSNotification *)notification
{
	// create image type
	[self createImageView:[self romId] fileIndex:0];
}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	// change image
	if([keyPath isEqualToString:@"selection"]){
		[self changeImageViewWithArray];
	}
}
//------------------------------------
// viewDidEndLiveResize
//------------------------------------
- (void)viewDidEndLiveResize
{
	[self setButtonImageIndexEnabled];
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)event
{
	// double clicked
	if([event type] == NSLeftMouseDown){
		if([event clickCount] == 2){
			[self playItem:nil];
		}
	}
}
//------------------------------------
// scrollWheel
//------------------------------------
- (void)scrollWheel:(NSEvent *)theEvent
{
	int wheelDelta;

	wheelDelta = [theEvent deltaY];

	if(wheelDelta <= 0){
		[targetArrayController selectNext:nil];
	}else{
		[targetArrayController selectPrevious:nil];
	}
}
//------------------------------------
// isExist
//------------------------------------
- (void)setIsExist:(BOOL)isExist
{
	isExist_ = isExist;
}
- (BOOL)isExist
{
	return isExist_;
}
//------------------------------------
// hasImage
//------------------------------------
- (void)setHasImage:(BOOL)hasImage
{
	hasImage_ = hasImage;
}
- (BOOL)hasImage
{
	return hasImage_;
}
//------------------------------------
// romId
//------------------------------------
- (void)setRomId:(NSString*)romId
{
	[romId retain];
    [romId_ release];
    romId_ = romId;

}
- (NSString*)romId
{
	return romId_;
}
- (BOOL)hasRomId
{
	if(![[self romId] isEqualToString:@""]){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// fileIndex
//------------------------------------
- (void)setFileIndex:(int)fileIndex
{
	fileIndex_ = fileIndex;
}
- (int)fileIndex
{
	return fileIndex_;
}
//------------------------------------
// maxIndex
//------------------------------------
- (void)setMaxIndex:(int)maxIndex
{
	maxIndex_ = maxIndex;
}
- (int)maxIndex
{
	return maxIndex_;
}
//--------------------------------------
// isDisplay
//--------------------------------------
- (BOOL)isDisplay
{
//	NSSize viewSize = [[self enclosingScrollView] contentSize];
	float width = [self bounds].size.width;
	float height = [self bounds].size.height;
//	NSLog(@"width=%f", viewSize.width);
//	NSLog(@"height=%f", viewSize.height);

	if(width > 0 && height > 0){
		return YES;
	}else{
		return NO;
	}
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[targetArrayController removeObserver:self forKeyPath:@"selection"];
	[romId_ release];
    [super dealloc];
}

@end
