#import "ReportRomStatus.h"
#import "ViewLog.h"
#import "ViewPrefs.h"
#import "TBArrayController.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ReportRomStatus

//=======================================================================
// awakeFromNib
//=======================================================================
- (void)awakeFromNib
{
	[self setReportType:0];
	[self setLogString:@""];

	//
	// set state from user default
	//
	// window rect
	[self setWindowRect:folderSelectWindow key:@"rectWindowSelectFolder"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:folderSelectWindow];
}
//=======================================================================
// IBAction
//=======================================================================
//------------------------------------
// openFolderSelectWindow
//------------------------------------
- (IBAction)openFolderSelectWindow:(id)sender
{
	[folderSelectWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// selectReportFolder
//------------------------------------
- (IBAction)selectReportFolder:(id)sender
{

	[folderSelectWindow close];

	NSString *folderPath = [[pbtnFolders selectedItem] representedObject];
	[self createReportRomsFolder:folderPath reportType:[self reportType]];

}
//------------------------------------
// reportRomsFolder
//------------------------------------
- (IBAction)reportRomsFolder:(id)sender
{

	int reportType = [sender tag];
	int result;

	NSArray *folderItems = [self defaultArrayValue:@"pathRomsFolderItems"];

	// no items
	if([folderItems count] <= 0){
		result = [self displayMessageAlertFolder:@"Roms"];
		// show preferences
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	// 1 item
	if([folderItems count] == 1){
		id record = [folderItems objectAtIndex:0];
		[self createReportRomsFolder:[record valueForKey:@"filePath"] reportType:reportType];
	}else{
		// create select window
		if([self createFolderSelectWindow:folderItems reportType:reportType] == YES){
			[self openFolderSelectWindow:nil];
		}
	}

}
//------------------------------------
// reportImagesFolder
//------------------------------------
- (IBAction)reportImagesFolder:(id)sender
{

	int reportType = [sender tag];
	int result;

	int folderType = [self getDefaultImageFolderType:reportType];
	NSString *folderPath = [self defaultSettingFolderPath:folderType];
	NSString *folderTitle = [self getDefaultImageFolderTitle:reportType];
	NSString *reportTitle = [NSString stringWithFormat:@"Unknown %@ images", folderTitle];

	// check folder path
	if([self checkFileExist:folderPath] != FILE_EXIST_DIR){
		result = [self displayMessageAlertFolder:folderTitle];
		// show preferences
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	[self setLogString:@""];

	// header
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[NSString stringWithFormat:@"%@\n", reportTitle]];
	[self appendLogString:[NSString stringWithFormat:@"Folder : %@\n", folderPath]];


	//
	// unknown
	//
	NSMutableDictionary *dictRoms = [self createDictRoms];
	NSMutableDictionary *dictClones = [self createDictClones];
	NSArray *unknownItems = [self createReportItemsUnknown:folderPath
													dictRoms:dictRoms
													dictClones:dictClones
							];

	// summary
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[NSString stringWithFormat:@"Unknown %@ : %d files.\n"
												, folderTitle
												, [unknownItems count]
						]
	];
	[self appendLogString:@"--------------------------------------------------------------------\n"];

	// detail
	[self appendLogString:[[unknownItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];

	// file name
	NSString *fileName = [NSString stringWithFormat:@"Report_Unknown_%@.txt", folderTitle];

	// open log window
	[viewLog setLogString:[self logString]];
	[viewLog setTitle:reportTitle];
	[viewLog setFileName:fileName];
	[viewLog setCanSave:YES];
	[viewLog openLogWindow:nil];

	[self setLogString:@""];

}
//------------------------------------
// reportRomsAuditStatus
//------------------------------------
- (IBAction)reportRomsAuditStatus:(id)sender
{
	int result;
	int reportType = [sender tag];

	result = [self displayMessage:@"alert"
				messageText:@"You need update all roms audit status before this report."
				infoText:@"Continue?"
				btnList:@"Report,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}

	[self createRomsAuditStatus:reportType];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createFolderSelectWindow
//------------------------------------
- (BOOL)createFolderSelectWindow:(NSArray*)folderItems reportType:(int)reportType
{

	// no items
	if([folderItems count] <= 0){
		return NO;
	}

	// create menu
	NSMenu *menu = [[[NSMenu alloc] initWithTitle:@"FolderSelectMenu"] autorelease];
	NSString *filePath;
	NSMenuItem *menuItem;
	id record;
	int i;
	for(i = 0; i < [folderItems count]; i++){

		record = [folderItems objectAtIndex:i];
		filePath = [record valueForKey:@"filePath"];

		menuItem = [[[NSMenuItem alloc] initWithTitle:filePath action:nil keyEquivalent:@""] autorelease];
		[menuItem setRepresentedObject:filePath];

		// set image
		NSImage *image = nil;
		if([self checkFileExist:filePath] != FILE_EXIST_NOT_FOUND){
			image = [[NSWorkspace sharedWorkspace] iconForFile:filePath];
		}else{
			image =[NSImage imageNamed:@"icon_file_not_found"];
		}
		[image setSize:NSMakeSize(16,16)];
		[image setScalesWhenResized:YES];

		[menuItem setImage:image];

		[menu addItem:menuItem];

	}

	[pbtnFolders setMenu:menu];

	NSString *reportTypeName = [self getRomsReportTypeName:reportType];
	NSString *reportTitle = [NSString stringWithFormat:@"%@ Roms", reportTypeName];
	[folderSelectWindow setTitle:reportTitle];

	[txtTitle setStringValue:@"Select Roms Folder"];

	[self setReportType:reportType];

	return YES;

}
//------------------------------------
// createReportRomsFolder
//------------------------------------
- (void)createReportRomsFolder:(NSString*)folderPath reportType:(int)reportType
{

	int result;

	// not folder
	if([self checkFileExist:folderPath] != FILE_EXIST_DIR){
		result = [self displayMessageAlertFolder:@"Roms"];
		// show preferences
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	[self setLogString:@""];

	NSString *reportTypeName = [self getRomsReportTypeName:reportType];
	NSString *reportTitle = [NSString stringWithFormat:@"%@ Roms", reportTypeName];
	NSString *reportFileName = [NSString stringWithFormat:@"Report_%@_Roms.txt", reportTypeName];
	
	// header
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[NSString stringWithFormat:@"%@\n", reportTitle]];
	[self appendLogString:[NSString stringWithFormat:@"Folder : %@\n", folderPath]];

	//
	// available / missing
	//
	if(reportType == MAME_REPORT_TYPE_ROM_AVAILABLE || reportType == MAME_REPORT_TYPE_ROM_MISSING){

		//
		// parent roms
		//
		NSArray *romItems = [tbArrayController getAllObjects:@"romlist"];
		NSArray *reportRomItems = [self createReportItemsRoms:folderPath
													items:romItems
													keyRomId:@"romId"
													isParent:YES
													reportType:reportType
									];

		//
		// clone roms
		//
		NSArray *cloneItems = [tbArrayController getAllObjects:@"clonelist"];
		NSArray *reportCloneItems = [self createReportItemsRoms:folderPath
														items:cloneItems
														keyRomId:@"cloneId"
														isParent:NO
														reportType:reportType
									];

		// summary
		[self appendLogString:[NSString stringWithFormat:@"Parent Roms : %d / %d files.\n"
													, [reportRomItems count]
													, [romItems count]
								]
		];
		[self appendLogString:[NSString stringWithFormat:@"Clone  Roms : %d / %d files.\n"
													, [reportCloneItems count]
													, [cloneItems count]
								]
		];

		// parent
		[self appendLogString:@"--------------------------------------------------------------------\n"];
		[self appendLogString:[NSString stringWithFormat:@"Parent Roms : %d / %d files.\n"
													, [reportRomItems count]
													, [romItems count]
								]
		];
		[self appendLogString:@"--------------------------------------------------------------------\n"];

		// detail
		[self appendLogString:[[reportRomItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];
		[self appendLogString:@"\n\n"];

		// clone
		[self appendLogString:@"--------------------------------------------------------------------\n"];
		[self appendLogString:[NSString stringWithFormat:@"Clone Roms : %d / %d files.\n"
													, [reportCloneItems count]
													, [cloneItems count]
								]
		];
		[self appendLogString:@"--------------------------------------------------------------------\n"];

		// detail
		[self appendLogString:[[reportCloneItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];

	}
	//
	// unknown
	//
	else if(reportType == MAME_REPORT_TYPE_ROM_UNKNOWN){

		NSMutableDictionary *dictRoms = [self createDictRoms];
		NSMutableDictionary *dictClones = [self createDictClones];
		NSArray *unknownItems = [self createReportItemsUnknown:folderPath
													dictRoms:dictRoms
													dictClones:dictClones
								];

		// summary
		[self appendLogString:@"--------------------------------------------------------------------\n"];
		[self appendLogString:[NSString stringWithFormat:@"Unknown Roms : %d files.\n"
													, [unknownItems count]
							]
		];
		[self appendLogString:@"--------------------------------------------------------------------\n"];

		// detail
		[self appendLogString:[[unknownItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];
	}

	// open log window
	[viewLog setLogString:[self logString]];
	[viewLog setTitle:reportTitle];
	[viewLog setFileName:reportFileName];
	[viewLog setCanSave:YES];
	[viewLog openLogWindow:nil];

	[self setLogString:@""];

}
//------------------------------------
// createRomsAuditStatus
//------------------------------------
- (void)createRomsAuditStatus:(int)reportType
{

	NSString *status = @"";
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];

	if(reportType == 0){
		status = @"good";
	}
	else if(reportType == 1){
		status = @"bad";
	}
	else if(reportType == 2){
		status = @"best available";
	}
	else{
		return;
	}
	pred = [NSPredicate predicateWithFormat:@"fileSize > 0 AND audit == %@", status];


	[self setLogString:@""];
	//
	// parent roms
	//
	NSArray *romItems = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];
	NSArray *reportRomItems = [self createReportItemsAuditStatus:romItems
												keyRomId:@"romId"
												isParent:YES
												reportType:reportType
								];
	//
	// clone roms
	//
	NSArray *cloneItems = [tbArrayController getObjectsWithPred:@"clonelist" pred:pred];
	NSArray *reportCloneItems = [self createReportItemsAuditStatus:cloneItems
												keyRomId:@"cloneId"
												isParent:NO
												reportType:reportType
								];

	// summary
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:@"Audit Report Summary\n"];
	[self appendLogString:@"Parent Roms:\n"];
	[self appendLogString:[NSString stringWithFormat:@"- %@ = %d\n"
												, status
												, [reportRomItems count]
							]
	];
	[self appendLogString:@"Clone  Roms:\n"];
	[self appendLogString:[NSString stringWithFormat:@"- %@ = %d\n"
												, status
												, [reportCloneItems count]
							]
	];

	// parent
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[NSString stringWithFormat:@"Parent Roms : %d\n"
												, [reportRomItems count]
							]
	];
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[[reportRomItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];
	[self appendLogString:@"\n\n"];

	// clone
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[NSString stringWithFormat:@"Clone  Roms : %d\n"
												, [reportCloneItems count]
							]
	];
	[self appendLogString:@"--------------------------------------------------------------------\n"];
	[self appendLogString:[[reportCloneItems sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)] componentsJoinedByString:@"\n"]];
	[self appendLogString:@"\n\n"];

	NSString *reportTitle = [NSString stringWithFormat:@"Audit Roms %@", status];
	NSString *reportFileName = [NSString stringWithFormat:@"Report_Audit_Roms_%@.txt", status];

	// open log window
	[viewLog setLogString:[self logString]];
	[viewLog setTitle:reportTitle];
	[viewLog setFileName:reportFileName];
	[viewLog setCanSave:YES];
	[viewLog openLogWindow:nil];

	[self setLogString:@""];

}
//------------------------------------
// createReportItemsRoms
//------------------------------------
- (NSMutableArray*)createReportItemsRoms:(NSString*)folderPath
								items:(NSArray*)items
								keyRomId:(NSString*)keyRomId
								isParent:(BOOL)isParent
								reportType:(int)reportType
{

	NSMutableArray *reportItems = [[[NSMutableArray alloc] init] autorelease];

	id record;
	NSString *romId;
	NSString *parentId;
	NSString *filePath;
	NSString *title;
	NSString *string;

	//
	// check records
	// 
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:keyRomId];
//		title = [record valueForKey:@"title"];
		title = [self getTitleString:[self defaultLangType]
								title:[record valueForKey:@"title"]
								titlejp:[record valueForKey:@"titlejp"]
				];

		if(isParent == YES){
			string = [NSString stringWithFormat: @"[%@]\t%@", romId, title];
		}else{
			parentId = [record valueForKey:@"parentId"];
			string = [NSString stringWithFormat: @"[%@\t%@]\t%@", parentId, romId, title];
		}

		// exists
//		filePath = [self createCurrentRomPath:romId];
		filePath = [self createRomPath:romId folderPath:folderPath];
		
		if(![filePath isEqualToString:@""]){
			if(reportType == MAME_REPORT_TYPE_ROM_AVAILABLE){
//				string = [NSString stringWithFormat: @"------------\n%@\nPath: %@", string, filePath];
				[reportItems addObject:string];
			}
		}
		// not exist
		else{
			if(reportType == MAME_REPORT_TYPE_ROM_MISSING){
				[reportItems addObject:string];
			}
		}

	}

	return reportItems;

}
//------------------------------------
// createReportItemsUnknown
//------------------------------------
- (NSMutableArray*)createReportItemsUnknown:(NSString *)filePath
									dictRoms:(NSMutableDictionary*)dictRoms
									dictClones:(NSMutableDictionary*)dictClones
{

	NSMutableArray *reportItems = [[[NSMutableArray alloc] init] autorelease];

	NSString *romId;
	NSString *fileName;

	NSArray *files = [[NSFileManager defaultManager] directoryContentsAtPath:filePath];
	NSEnumerator *enumFiles = [files objectEnumerator];

	while (fileName = [enumFiles nextObject]) {

		if([fileName isEqualToString:@""]){
			continue;
		}
		// hidden file
		if([[fileName substringToIndex:1] isEqualToString:@"."]){
			continue;
		}
		// icon file
		if([fileName isEqualToString:@"Icon\x0d"]){
			continue;
		}

		// delete extension
		romId = [[fileName stringByDeletingPathExtension] lowercaseString];

		if(![dictRoms valueForKey:romId] && ![dictClones valueForKey:romId]){
			[reportItems addObject:fileName];
		}
	}

	return reportItems;

}
//------------------------------------
// createReportItemsAuditStatus
//------------------------------------
- (NSMutableArray*)createReportItemsAuditStatus:(NSArray*)items
								keyRomId:(NSString*)keyRomId
								isParent:(BOOL)isParent
								reportType:(int)reportType
{

	NSMutableArray *reportItems = [[[NSMutableArray alloc] init] autorelease];

	id record;
	NSString *romId;
	NSString *parentId;
	NSString *title;
	NSString *string;

	//
	// check records
	// 
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:keyRomId];
		title = [self getTitleString:[self defaultLangType]
								title:[record valueForKey:@"title"]
								titlejp:[record valueForKey:@"titlejp"]
				];

		if(isParent == YES){
			string = [NSString stringWithFormat: @"[%@]\t%@", romId, title];
		}else{
			parentId = [record valueForKey:@"parentId"];
			string = [NSString stringWithFormat: @"[%@\t%@]\t%@", parentId, romId, title];
		}
		[reportItems addObject:string];

	}

	return reportItems;

}

//------------------------------------
// createDictRoms
//------------------------------------
- (NSMutableDictionary*)createDictRoms
{

	NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];
	NSString *romId;
	id record;

	NSArray *items = [tbArrayController getAllObjects:@"romlist"];
	NSEnumerator *enumItems = [items objectEnumerator];

	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:@"romId"];
		if(![romId isEqualToString:@""]){
			[dict setObject:romId forKey:romId];
		}

	}

	return dict;
}
//------------------------------------
// createDictClones
//------------------------------------
- (NSMutableDictionary*)createDictClones
{

	NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];
	NSString *cloneId;
	id record;

	NSArray *items = [tbArrayController getAllObjects:@"clonelist"];
	NSEnumerator *enumItems = [items objectEnumerator];

	while (record = [enumItems nextObject]) {

		cloneId = [record valueForKey:@"cloneId"];
		if(![cloneId isEqualToString:@""]){
			[dict setObject:cloneId forKey:cloneId];
		}

	}

	return dict;
}
//------------------------------------
// getRomsReportTypeName
//------------------------------------
- (NSString*)getRomsReportTypeName:(int)reportType
{
	NSString *name = @"";

	if(reportType == MAME_REPORT_TYPE_ROM_AVAILABLE){
		name = @"Available";
	}
	else if(reportType == MAME_REPORT_TYPE_ROM_MISSING){
		name = @"Missing";
	}
	else if(reportType == MAME_REPORT_TYPE_ROM_UNKNOWN){
		name = @"Unknown";
	}

	return name;
}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	if([notification object] == folderSelectWindow){
		[self saveWindowRect:folderSelectWindow key:@"rectWindowSelectFolder"];
	}
}
//------------------------------------
// reportType
//------------------------------------
- (void)setReportType:(int)reportType
{
	reportType_ = reportType;
}
- (int)reportType
{
	return reportType_;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
//------------------------------------
// logString
//------------------------------------
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[logString_ release];
    [super dealloc];
}
@end
