#import "RomlistTableView.h"
#import "ViewMain.h"
#import "RoundImageCell.h"
#import "ItemDescTextCell.h"
#import "TableColumnManager.h"
#import "TableColumnExtension.h"
#import "ThumbsExtension.h"
#import "CellAttributeExtension.h"
#import "FileAttributeExtension.h"
#import "ImageAttributeExtension.h"
#import "ImageBindExtension.h"
#import "ConvertExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation RomlistTableView

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set delegate
	[super setDelegate:self];
	[super setDataSource:self];

	// set row height
	float rowHeight = [self defaultFloatValue:@"optRowHeightMain"];
	[sliderRowHeight setFloatValue:rowHeight];
	[self setRowHeight:rowHeight];

	// image column
	NSTableColumn *imageColumn = [self tableColumnWithIdentifier:@"image"];
	RoundImageCell *imageCell = [[[RoundImageCell alloc] init] autorelease];
	[imageColumn setDataCell:imageCell];

	// description column
	NSTableColumn *descColumn = [self tableColumnWithIdentifier:@"description"];
	ItemDescTextCell *descTextCell = [[[ItemDescTextCell alloc] init] autorelease];
	[descColumn setDataCell:descTextCell];

	// set click action
	[self setTarget:viewMain];
	[self setDoubleAction:@selector(playItem:)];

	// change description column & sort descriptor
	[self setColumnDescriptionHeaderTitle:self langType:[self defaultLangType]];
	[self setColumnDescriptionSortDescriptor:self
									aController:romlistArrayController
									langType:[self defaultLangType]
	];

	// set table column state
	[self setTableColumnState:self key:@"stateTableColumnRomlist4_0"];

	// init column controller
	if (tcm_ == nil) {
		tcm_ = [[TableColumnManager alloc] initWithTable:self];
	}

	// showTableColumns
	// set default hide columns
	NSArray *defaultHideCols = [NSArray arrayWithObjects:
									@"auditIcon",
									@"auditStatus",
									@"addVersion",
									@"comment",
									nil
								];
	[self showTableColumns:tcm_ aTableView:self defaultHideCols:defaultHideCols key:@"stateTableColumnRomlist4_0"];

	// set sort descriptor
	[self setArrayControllerSortDescriptor:romlistArrayController key:@"sortDescColumnRomlist4_0"];

	// set header menu
	[[self headerView] setMenu:cmRomlistHeader];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:[self window]];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED_MAIN object:nil];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_STATUS_DID_CHANGED_MAIN object:nil];
//	[nc addObserver:self selector:@selector(handleImageItemLoaded:) name:IMAGE_ITEM_NOTIF_NAME_TABLE_MAIN object:nil];
	[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_MAIN object:nil];
	[nc addObserver:self selector:@selector(handleLangTypeDidChanged:) name:DEFAULT_NOTIF_LANG_TYPE_DID_CHANGED object:nil];

	// set registerForDraggedTypes
	[self registerForDraggedTypes:[NSArray arrayWithObjects:RomlistPboardType, nil]];

	// set drag operation
	[self setDraggingSourceOperationMask:NSDragOperationLink forLocal:NO];

}
//------------------------------------
// changeRowHeight
//------------------------------------
- (IBAction)changeRowHeight:(id)sender
{
	[self setRowHeight:[sender floatValue]];

	if([self numberOfSelectedRows] > 0){
		[self scrollRowToVisible:[self selectedRow]];
	}

}
//------------------------------------
// change table column state
//------------------------------------
- (IBAction)changeColumnState:(id)sender;
{
	[self setColumnState:tcm_ sender:sender];
}

//------------------------------------
// isShowColumn
//------------------------------------
- (BOOL)isShowColumn:(NSString*)identifier
{
	return [self isShowColumnState:tcm_ identifier:identifier];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{

	// table column state
	[self saveTableColumnState:self key:@"stateTableColumnRomlist4_0"];
	// save sort descriptor
	[self saveArrayControllerSortDescriptor:romlistArrayController key:@"sortDescColumnRomlist4_0"];

}
//------------------------------------
// handleItemInfoDidChanged
//------------------------------------
-(void)handleItemInfoDidChanged:(NSNotification *)notification
{
	// reload
	[self reloadData];
}
//------------------------------------
// handleImageItemLoaded
//------------------------------------
-(void)handleImageItemLoaded:(NSNotification *)notification
{
	// reload
	[self reloadData];
}

//------------------------------------
// handleLangTypeDidChanged
//------------------------------------
-(void)handleLangTypeDidChanged:(NSNotification *)notification
{
	// change description column & sort descriptor
	[self setColumnDescriptionHeaderTitle:self langType:[self defaultLangType]];
	[self setColumnDescriptionSortDescriptor:self
									aController:romlistArrayController
									langType:[self defaultLangType]
	];

//	[self reloadData];
	[romlistArrayController rearrangeObjects];
}
//=======================================================================
// bind to tableview column
//=======================================================================
//------------------------------------
// objectValueForTableColumn
//------------------------------------
- (id)tableView:(NSTableView *)aTableView
		objectValueForTableColumn:(NSTableColumn *)aTableColumn
		row:(int)rowIndex
{

	// get record
	id record = [[romlistArrayController arrangedObjects] objectAtIndex:rowIndex];
	NSString *curRomId;

	// get defaultSet
	BOOL isClone;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// clone
	if(isClone == YES){
		curRomId = [defaultSet valueForKey:@"cloneId"];
	}else{
		curRomId = [defaultSet valueForKey:@"romId"];
	}

	// isExist
	BOOL isExist = [[defaultSet valueForKey: @"isExist"] boolValue];
/*
	BOOL isExist = NO;
	if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
		isExist = YES;
	}
*/
/*
	// status
	else if([[aTableColumn identifier] isEqualToString:@"statusIcon"]){
		return [self getStatusIconImage:isClone isExist:isExist];
	}
*/
	// image
	if([[aTableColumn identifier] isEqualToString:@"image"]){

		// get thumbs image
		NSImage *image = [self getThumbsImage:[self getThumbsImagePath:curRomId
																imageType:[self defaultThumbsImageTypeMain]
																fileIndex:0
																maxIndex:nil
												]
									hasImage:nil
						];

		RoundImageCell *imageCell = [aTableColumn dataCell];
		[imageCell setImage:image];
		[imageCell setRomId:curRomId];
		[imageCell setImageType:[self defaultThumbsImageTypeMain]];
		[imageCell setIsExist:isExist];

	}
	// description
	else if([[aTableColumn identifier] isEqualToString:@"description"]){

		//
		// get value
		//
		NSString *title = [self getTitleString:[self defaultLangType]
								title:[defaultSet valueForKey:@"title"]
								titlejp:[defaultSet valueForKey:@"titlejp"]
				];
		NSString *manufacture = [defaultSet valueForKey:@"manufacture"];
		NSString *year = [defaultSet valueForKey:@"year"];
//		NSString *fileSizeStr = @"-";
//		int fileSize = [[defaultSet valueForKey: @"fileSize"] intValue];
//		if(fileSize > 0){
//			fileSizeStr = [self convertFileSizeToString:fileSize];
//		}
		// current exist
		BOOL isCurExist = NO;
		if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
			isCurExist = YES;
		}
		int labelColorNo = [[record valueForKey: @"labelColorNo"] intValue];

		// statusIcon
		NSImage *imageStatus = [self getStatusIconImage:isClone isExist:isExist isCurExist:isCurExist];

		NSString *desc1 = [NSString stringWithFormat:@"%@ / %@ ", manufacture, year];
//		NSString *desc2 = [NSString stringWithFormat:@"Size: %@", fileSizeStr];
		NSString *desc2 = @"";
		ItemDescTextCell *descTextCell = [aTableColumn dataCell];
		[descTextCell setStringValue:[NSString stringWithFormat:@"%@\t%@\t%@", title, desc1, desc2]];
		[descTextCell setIsExist:isExist];
		[descTextCell setLabelColorNo:labelColorNo];
		[descTextCell setImage:imageStatus];
		return descTextCell;

	}
	// manufacture
	else if([[aTableColumn identifier] isEqualToString:@"manufacture"]){
		return [defaultSet valueForKey:@"manufacture"];
	}
	// year
	else if([[aTableColumn identifier] isEqualToString:@"year"]){
		return [defaultSet valueForKey:@"year"];
	}
	// id
	else if([[aTableColumn identifier] isEqualToString:@"id"]){
		return curRomId;
	}
	// fileSize
	else if([[aTableColumn identifier] isEqualToString:@"fileSize"]){
		NSString *fileSizeStr = @"-";
		int fileSize = [[defaultSet valueForKey: @"fileSize"] intValue];
		// convert file size
		if(fileSize > 0){
			fileSizeStr = [self convertFileSizeToString:fileSize];
		}
		return fileSizeStr;
	}
	// fileDate
	else if([[aTableColumn identifier] isEqualToString:@"fileDate"]){
		return [defaultSet valueForKey:@"fileDate"];
	}
	// auditIcon
	else if([[aTableColumn identifier] isEqualToString:@"auditIcon"]){
		return [self getAuditIconImage:[defaultSet valueForKey: @"audit"]];
	}
	// auditStatus
	else if([[aTableColumn identifier] isEqualToString:@"auditStatus"]){
		return [defaultSet valueForKey:@"audit"];
	}
	// addVersion
	else if([[aTableColumn identifier] isEqualToString:@"addVersion"]){
		return [defaultSet valueForKey:@"addVersion"];
	}

	return nil;
}
//------------------------------------
// toolTipForCell
//------------------------------------
- (NSString *)tableView:(NSTableView *)aTableView toolTipForCell:(NSCell *)aCell rect:(NSRectPointer)rect tableColumn:(NSTableColumn *)aTableColumn row:(int)row mouseLocation:(NSPoint)mouseLocation
{
	if( [[aTableColumn identifier] isEqualToString:@"description"]){

		// get record
		id record = [[romlistArrayController arrangedObjects] objectAtIndex:row];

		// get defaultSet
		BOOL isClone;
		id defaultSet = [self getDefaultRomSet:record isClone:&isClone];
		return [self createToolTipString:defaultSet isClone:isClone];

	}
	return nil;
}
//------------------------------------
// numberOfRowsInTableView
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[romlistArrayController arrangedObjects] count];
}
//------------------------------------
// tableViewSelectionDidChange
//------------------------------------
-(void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	if([self numberOfSelectedRows] == 1){
		[self scrollRowToVisible:[[self selectedRowIndexes] firstIndex]];
	}
}
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

	BOOL success = NO;

	NSArray *objects = [[romlistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];
 	NSMutableArray *items = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		NSManagedObjectID *objectID = [item objectID];
		NSURL *representedURL = [objectID URIRepresentation];
		[items addObject:representedURL];

	}

	[pboard declareTypes:[NSArray arrayWithObjects:RomlistPboardType, nil] owner:nil];
	[pboard addTypes:[NSArray arrayWithObjects:RomlistPboardType, nil] owner:nil];
	[pboard setString:[items componentsJoinedByString:@","] forType:RomlistPboardType];

	if([items count] > 0){
		success = YES;
	}

	return success;

}
//------------------------------------
// dragImageForRowsWithIndexes
//------------------------------------
- (NSImage *)dragImageForRowsWithIndexes:(NSIndexSet*)dragRows
							tableColumns:(NSArray*)tableColumns
							event:(NSEvent*)dragEvent
							offset:(NSPointPointer)dragImageOffset
{
	// one item
/*
	if ([dragRows count] == 1){
		return [super dragImageForRowsWithIndexes:dragRows tableColumns:tableColumns event:dragEvent offset:dragImageOffset];
	}
	// many items
	else{
*/
		// create drag image
		NSImage	*dragImage = [[NSImage alloc] init];
		[dragImage setSize:NSMakeSize(65.0, 40.0)];

		// set count label bounds
		NSString *countStr = [NSString stringWithFormat:@"%i", [dragRows count]];
		NSRect imageBounds;
		imageBounds.origin = NSMakePoint(0, 0);
		imageBounds.size = [dragImage size];

		// create source image
		NSImage *sourceImage = [[[NSImage alloc] initByReferencingFile: [[NSBundle mainBundle] pathForResource:@"icon_video" ofType:@"png"]] autorelease];

		[sourceImage setScalesWhenResized:YES];
		[sourceImage setSize:NSMakeSize(24, 24)];

		NSPoint sourcePoint;
		sourcePoint.x = imageBounds.origin.x + ((imageBounds.size.width - [sourceImage size].width) / 2);
		sourcePoint.y = imageBounds.origin.y + ((imageBounds.size.height - [sourceImage size].height) / 2);

		[dragImage lockFocus];

		// draw label
		[self drawLabelAndString:imageBounds
						withString:countStr
						fontSize:LABEL_DRAG_FSIZE
						fontColor:[NSColor whiteColor]
						labelColor:[NSColor redColor]
						align:CELL_ALIGN_RIGHT
						valign:CELL_VALIGN_TOP
						hPadding:LABEL_DRAG_HPADDING
						vPadding:LABEL_DRAG_VPADDING
						radius:LABEL_DRAG_RADIUS
		];

		// draw source image
		[sourceImage compositeToPoint:sourcePoint operation:NSCompositeDestinationOver fraction:0.8];

		[dragImage unlockFocus];

		return [dragImage autorelease];
//	}
}
//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
		validateDrop:(id <NSDraggingInfo>)info 
		proposedRow:(int)row 
		proposedDropOperation:(NSTableViewDropOperation)operation
{

	// Get pboard
	NSPasteboard*   pboard;
	pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
		return NSDragOperationNone;
	}

	// ignore self
	if (draggingSource == self) {
		return NSDragOperationNone;
	}

	return NSDragOperationNone;

}
//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
						acceptDrop:(id <NSDraggingInfo>)info 
						row:(int)row 
						dropOperation:(NSTableViewDropOperation)operation
{
	
	NSPasteboard *pboard = [info draggingPasteboard];
	id draggingSource = [info draggingSource];

	if (!pboard) {
		return NO;
	}

	// ignore self
	if (draggingSource == self) {
		return NO;
	}

	return NO;
}

//------------------------------------
// keyDown
//------------------------------------
- (void)keyDown:(NSEvent *)theEvent
{	 
	BOOL isAction = NO;

	// no alternateKeyMask
	if(!([theEvent modifierFlags] & NSAlternateKeyMask)){
		// return key
		if([theEvent keyCode] == 36){
			[viewMain playItem:nil];
			isAction = YES;
		}
	}

	if(isAction == NO){
		[super keyDown:theEvent];
	}
}
//------------------------------------
// mouseDown
//------------------------------------
- (void)mouseDown:(NSEvent *)theEvent
{	
	BOOL isAction = NO;

	// control click
	if([theEvent modifierFlags] & NSControlKeyMask){
		[self rightMouseDown:theEvent];
		isAction = YES;
	}

	if(isAction == NO){
		[super mouseDown:theEvent];
	}
}
//------------------------------------
// rightMouseDown
//------------------------------------
- (void)rightMouseDown:(NSEvent *)theEvent
{	

	NSPoint p = [self convertPoint:[theEvent locationInWindow] fromView:nil];
	BOOL isAction = NO;
	
	int i = [self rowAtPoint:p];
	
	if (i < [self numberOfRows]){
		// single selected
		if ([self numberOfSelectedRows] <= 1){
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
		}
		// multiple selected
		else{
			[self selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:YES];
		}
		[NSMenu popUpContextMenu:cmRomlist withEvent:theEvent forView:self];
		isAction = YES;
	}

	if(isAction == NO){
		[super rightMouseDown:theEvent];
	}

}
//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}
@end
