#import "SearchExtension.h"
#import "ConvertExtension.h"

@implementation NSObject(searchExtension_)

//------------------------------------
// getSearchColumnTitle
//------------------------------------
- (NSString*)getSearchColumnTitle:(NSString*)column items:(NSArray*)items
{

	NSString *title = @"All";
	NSString *predTitle = @"";
	NSString *predColumn = @"";
	NSArray *cols;
	int i;

	for(i = 0; i < [items count]; i++){
		cols = [[items objectAtIndex:i] componentsSeparatedByString:@","];
		predTitle = [cols objectAtIndex:0];
		predColumn = [cols objectAtIndex:1];
		
		if([predColumn isEqualToString:column]){
			title = predTitle;
			break;
		}
	}

	return title;

}
//------------------------------------
// createSearchFormat
//------------------------------------
- (NSString*)createSearchFormat:(NSString*)column
						items:(NSArray*)items
						title:(NSString**)title
{

	NSString *formatString = @"";
	NSString *predTitle = @"";
	NSString *predColumn = @"";
	NSString *predString = @"";
	NSArray *cols;
	int i;

	for(i = 0; i < [items count]; i++){
		cols = [[items objectAtIndex:i] componentsSeparatedByString:@","];
		predTitle = [cols objectAtIndex:0];
		predColumn = [cols objectAtIndex:1];
		predString = [cols objectAtIndex:2];
		
		if([predColumn isEqualToString:column]){
			if(![predString isEqualToString:@""]){
				formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"(%@)", predString]];
				if(*title != nil){
					*title = predTitle;
				}
				break;
			}
		}
	}

	return formatString;

}
//------------------------------------
// createSearchFormatAll
//------------------------------------
- (NSString*)createSearchFormatAll:(NSArray*)items
{

	NSString *formatString = @"";
	NSString *predString = @"";
	NSArray *cols;
	int i;

	for(i = 0; i < [items count]; i++){
		cols = [[items objectAtIndex:i] componentsSeparatedByString:@","];
		predString = [cols objectAtIndex:2];
		if(![predString isEqualToString:@""]){
			if([formatString isEqualToString:@""]){
				formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"(%@)", predString]];
			}else{
				formatString = [formatString stringByAppendingString:[NSString stringWithFormat:@"OR (%@)", predString]];
			}
		}
	}

	return formatString;

}
//------------------------------------
// createSearchStringPred
//------------------------------------
- (NSPredicate*)createSearchStringPred:(NSString*)searchString
					searchFormatString:(NSString*)searchFormatString
{

	NSString *searchWord = @"";
	NSString *predString = @"";
	NSString *matchString = @"";
	int i;

	// escape charactoer
	searchString = [self replaceCharacter:searchString str1:@"\n" str2:@""];
	searchString = [self replaceCharacter:searchString str1:@"\\" str2:@"\\\\"];
	searchString = [self replaceCharacter:searchString str1:@"'" str2:@"\\'"];

	if(![searchString isEqualToString:@""]){

		// separate by " "
		NSArray *searchWords = [searchString componentsSeparatedByString:@" "];

		for (i = 0; i < [searchWords count]; i++){

			searchWord = [searchWords objectAtIndex:i];

			if([searchWord isEqualToString:@""]){
				continue;
			}

			// replace $value
			matchString = [self replaceCharacter:searchFormatString str1:@"$value" str2:searchWord];

			// create predicate string
			if([predString isEqualToString:@""]){
				predString = [predString stringByAppendingString:[NSString stringWithFormat:@"(%@)", matchString]];
			}else{
				predString = [predString stringByAppendingString:[NSString stringWithFormat:@"AND (%@)", matchString]];
			}
		}
	}

//	NSLog(@"predString=%@", predString);
 	NSPredicate *pred = nil;
	if(![predString isEqualToString:@""]){
		pred = [NSPredicate predicateWithFormat:predString];
	}

	return pred;

}
//------------------------------------
// searchColumnItems
//------------------------------------
- (NSArray*)searchColumnItems
{

	// title, column, predString
	return [[NSArray alloc] initWithObjects: 
		@"All,,",
//		@"Title,title,title contains[c] '$value' OR rlt_clonelist.title contains[c] '$value'",
//		@"Title(JP),titlejp,titlejp contains[c] '$value' OR rlt_clonelist.titlejp contains[c] '$value'",
		@"Title / Title(JP),title,(title contains[c] '$value' OR rlt_clonelist.title contains[c] '$value') OR (titlejp contains[c] '$value' OR rlt_clonelist.titlejp contains[c] '$value')",
		@"Manufacture,manufacture,manufacture contains[c] '$value' OR rlt_clonelist.manufacture contains[c] '$value'",
		@"ID,romId,romId contains[c] '$value' OR rlt_clonelist.cloneId contains[c] '$value'",
		@"Year,year,year contains[c] '$value' OR rlt_clonelist.year contains[c] '$value'",
		@"FileDate,fileDate,fileDate contains[c] '$value' OR rlt_clonelist.fileDate contains[c] '$value'",
		@"ImportVersion,addVersion,addVersion contains[c] '$value' OR rlt_clonelist.addVersion contains[c] '$value'",
		@"AuditStatus,audit,audit contains[c] '$value' OR rlt_clonelist.audit contains[c] '$value'",
		@"Tag,comment,comment contains[c] '$value'",
		nil];
}
@end
