#import "SearchlistTableView.h"
#import "ViewSearchlist.h"
#import "TBArrayController.h"

@implementation SearchlistTableView
//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[super setDelegate:self];
	[super setDataSource:self];

	// set sort descriptor
	NSSortDescriptor *sortDescriptor;
	sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES];
	[searchlistArrayController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	[sortDescriptor release];

	// set dragged type
	[self registerForDraggedTypes:[NSArray arrayWithObject:SearchlistPboardType]];

}

//=======================================================================
// bind to tableview column
//=======================================================================
/*
- (id)tableView:(NSTableView *)aTableView
		objectValueForTableColumn:(NSTableColumn *)aTableColumn
		row:(int)rowIndex
{

	// icon
	if([[aTableColumn identifier] isEqualToString:@"icon"]){
		return [NSImage imageNamed: @"icon_websearch"];
	}

	return nil;
}
//------------------------------------
// bind to tableview line count
//------------------------------------
- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [[searchlistArrayController arrangedObjects] count];
}
*/
//=======================================================================
// drag & drop method
//=======================================================================
//------------------------------------
// writeRowsWithIndexes
//------------------------------------
- (BOOL)tableView:(NSTableView *)tv writeRowsWithIndexes:(NSIndexSet *)rowIndexes toPasteboard:(NSPasteboard*)pboard {

	BOOL success = NO;

	NSArray *objects = [[searchlistArrayController arrangedObjects] objectsAtIndexes:rowIndexes];
	NSMutableArray *objectIDs = [NSMutableArray array];
	int i;
	for (i = 0; i < [objects count]; i++) {
		NSManagedObject *item = [objects objectAtIndex:i];
		NSManagedObjectID *objectID = [item objectID];
		NSURL *representedURL = [objectID URIRepresentation];
		[objectIDs addObject:representedURL];
	}

	[pboard declareTypes:[NSArray arrayWithObject:SearchlistPboardType] owner:nil];
	[pboard addTypes:[NSArray arrayWithObject:SearchlistPboardType] owner:nil];
	[pboard setString:[objectIDs componentsJoinedByString:@","] forType:SearchlistPboardType];		  
 
	if([objects count] > 0){
		success = YES;
	}

	return success;
}
//------------------------------------
// validateDrop
//------------------------------------
- (NSDragOperation)tableView:(NSTableView*)tableView 
		validateDrop:(id <NSDraggingInfo>)info 
		proposedRow:(int)row 
		proposedDropOperation:(NSTableViewDropOperation)operation
{

	// Get pboard
	NSPasteboard*	pboard;
	pboard = [info draggingPasteboard];
	if (!pboard) {
		return NSDragOperationNone;
	}

	// registerd type
	if ([[pboard types] containsObject:SearchlistPboardType]){
		// allow drop above
		if (operation == NSTableViewDropAbove) {
			return NSDragOperationCopy;
		}
	}

	return NSDragOperationNone;

}
//------------------------------------
// acceptDrop
//------------------------------------
- (BOOL)tableView:(NSTableView*)tableView 
		acceptDrop:(id <NSDraggingInfo>)info 
		row:(int)row 
		dropOperation:(NSTableViewDropOperation)operation
{
	
	NSPasteboard *pboard = [info draggingPasteboard];
	if (!pboard) {
		return NO;
	}

	// searchlist
	if ([[pboard types] containsObject:SearchlistPboardType]){

		NSMutableArray *objectArray = [[[NSMutableArray alloc] init] autorelease];
		NSManagedObjectContext *moc = [tbArrayController getManagedObjectContext:@"searchlist"];

		if(moc == nil){
			return NO;
		}

		// get array from paste board string
		NSArray *objectIDs = [[pboard stringForType:SearchlistPboardType] componentsSeparatedByString:@","];

		// create object array
		int i;
		for (i = 0; i < [objectIDs count]; i++) {
			NSURL *objectURL = [NSURL URLWithString:[objectIDs objectAtIndex:i]];
			NSManagedObjectID *objectID = [[moc persistentStoreCoordinator] managedObjectIDForURIRepresentation:objectURL];
			if (objectID != nil) {
				id object = [moc objectRegisteredForID:objectID];
				[objectArray addObject:object];
			}
		}

		// update lower index
		if([objectArray count] > 0){
			[viewSearchlist updateSearchlistLowerIndex:objectArray row:row];
			return YES;
		}
	}

	return YES;
}
//------------------------------------
// textDidEndEditing
//------------------------------------
- (void)textDidEndEditing:(NSNotification *)notification
{ 
	if([[[notification userInfo] objectForKey:@"NSTextMovement"] intValue] == NSReturnTextMovement){
		NSMutableDictionary * newUserInfo;
		NSNotification * newNotification;
		newUserInfo = [NSMutableDictionary dictionaryWithDictionary:[notification userInfo]]; 
		[newUserInfo setObject:[NSNumber numberWithInt:NSIllegalTextMovement] forKey:@"NSTextMovement"];
		newNotification = [NSNotification notificationWithName:[notification name] object:[notification object] userInfo:newUserInfo];
		[super textDidEndEditing:newNotification];
		[[self window] makeFirstResponder:self]; 
	} else {
		[super textDidEndEditing:notification]; 
	}
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[super dealloc];
}

@end
