#import "SetupAuditStatus.h"
#import "TBArrayController.h"
#import "ViewLog.h"
#import "ViewPrefs.h"
#import "ViewProcStatus.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "HelperExtension.h"
#import "DialogExtension.h"

@implementation SetupAuditStatus

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setLogString:@""];

}
//=======================================================================
// IBActions
//=======================================================================
//------------------------------------
// importAuditStatus
//------------------------------------
- (IBAction)importAuditStatus:(id)sender
{

	int result;
	BOOL ret;

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_MAME_AUDIT];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:defaultPath
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"Import Audit Status?"
					infoText:@""
					btnList:@"Import,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}
	
	NSString *filePath = [filePaths objectAtIndex:0];

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

	// get result file
	NSString *resultString = [[NSString alloc] initWithContentsOfFile:filePath];
	if(resultString == nil){
		[self displayMessage:@"error"
							messageText:@"Import file is invalid."
							infoText:@"Please check import file."
							btnList:@"Cancel"
		];
		return;
	}

	//
	// update all data
	//
	ret = [self updateAllAuditStatusWithString:resultString];

	// error
	if(ret == NO){
		result = [self displayMessage:@"alert"
					messageText:@"Import data is invalid."
					infoText:@"Please check log."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:resultString];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return;
	}

	// success
	result = [self displayMessage:@"info"
						messageText:@"Import process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Import Log"];
		[viewLog setLogString:resultString];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}

}
//------------------------------------
// exportAuditStatus
//------------------------------------
- (IBAction)exportAuditStatus:(id)sender
{
	int result;

	// appPath is not exist
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] != FILE_EXIST_FILE){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	// check roms folder
	NSString *folderPathString = @"";
	if( [self defaultBoolValue:@"optSdlMameEnable"] == YES &&
		[self defaultBoolValue:@"optSdlMameRomPath"] == YES) {

		NSString *defaultFolderPathString = [self getDefaultRomsFolderPathString];
		// not exist
		if([defaultFolderPathString isEqualToString:@""]){
			result = [self displayMessageAlertFolder:@"Roms"];
			// open prefs tab
			if(result == NSAlertFirstButtonReturn){
				[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
			}
			return;
		}
		folderPathString = defaultFolderPathString;
	}

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_MAME_AUDIT];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	// confirm save
	NSString *filePath = [self selectFilePathWithSavePanel:@"MAME_AuditStatus.txt"
												canCreateDir:YES
												defaultPath:defaultPath
						];

	if([filePath isEqualToString:@""]){
		return;
	}

	// check invalid character
	NSCharacterSet *cSet = [NSCharacterSet characterSetWithCharactersInString:@"'\""];
	if([self checkIsExistCharacterSet:appPath cSet:cSet] == YES){
		[self displayMessage:@"alert"
					messageText:[NSString stringWithFormat:@"Includes invalid character ' or \" in %@", appPath]
					infoText:@"Please change path"
					btnList:@"Cancel"
		];
		return;
	}
	if([self checkIsExistCharacterSet:filePath cSet:cSet] == YES){
		[self displayMessage:@"alert"
					messageText:[NSString stringWithFormat:@"Includes invalid character ' or \" in %@", filePath]
					infoText:@"Please change path"
					btnList:@"Cancel"
		];
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"It needs much time for exporting."
					infoText:@"Continue export?"
					btnList:@"Export,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// save defaults
	[self setDefaultStringValue:filePath key:defaultKey];

	// create export result wit apple script
	NSDictionary *errorInfo = [self createAuditResultWithAppleScript:appPath
									folderPathString:folderPathString
									filePath:filePath
								];
	// error
	if(errorInfo != nil){
		result = [self displayMessage:@"alert"
					messageText:@"Can not export audit result."
					infoText:@"Please check log."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Export Log"];
			[viewLog setLogString:[errorInfo description]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return;
	}


/*
	//
	// create audit result
	//
	NSString *resultString = [self createAuditResultString:@""
												folderPathString:folderPathString
												showProc:YES
							];
//	NSLog(@"resultString=%@", resultString);

	//
	// save file
	//
	NSData *resultData = [resultString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES]; 

	// error
	if(resultData == nil){
		result = [self displayMessage:@"alert"
					messageText:@"Audit result is invalid."
					infoText:@"Please check result."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Audit Log"];
			[viewLog setLogString:resultString];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return;
	}

	// writeToFile
	if([resultData writeToFile:[filePath stringByExpandingTildeInPath] atomically:YES] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not save file."
						infoText:@"Failed write to file."
						btnList:@"Cancel"
		];
		return;
	}

	// success
	[self displayMessage:@"info"
						messageText:@"Export MAME Audit Status is completed."
						infoText:@""
						btnList:@"OK"
	];
*/

}

//------------------------------------
// updateAllAuditStatus
//------------------------------------
- (IBAction)updateAllAuditStatus:(id)sender
{

	int result;
	BOOL ret;

	// appPath is not exist
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] != FILE_EXIST_FILE){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	// check roms folder
	NSString *folderPathString = @"";
	if( [self defaultBoolValue:@"optSdlMameEnable"] == YES &&
		[self defaultBoolValue:@"optSdlMameRomPath"] == YES) {

		NSString *defaultFolderPathString = [self getDefaultRomsFolderPathString];
		// not exist
		if([defaultFolderPathString isEqualToString:@""]){
			result = [self displayMessageAlertFolder:@"Roms"];
			// open prefs tab
			if(result == NSAlertFirstButtonReturn){
				[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
			}
			return;
		}
		folderPathString = defaultFolderPathString;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"Update all audit status?"
					infoText:@"It needs much time for updating."
					btnList:@"Update,Cancel"
			];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	//
	// create audit result
	//
	NSString *resultString = [self createAuditResultString:@""
												folderPathString:folderPathString
												showProc:YES
							];

	//
	// update all data
	//
	ret = [self updateAllAuditStatusWithString:resultString];

	// error
	if(ret == NO){
		result = [self displayMessage:@"alert"
					messageText:@"Audit result is invalid."
					infoText:@"Please check result."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Audit Log"];
			[viewLog setLogString:resultString];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return;
	}

	// success
	result = [self displayMessage:@"info"
						messageText:@"Update process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Audit Log"];
		[viewLog setLogString:resultString];
		[viewLog setFileName:@"AuditResult.txt"];
		[viewLog setCanSave:YES];
		[viewLog openLogWindow:nil];
	}

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// updateAuditStatusWithRomId
//------------------------------------
- (BOOL)updateAuditStatusWithRomId:(NSString*)romId isParent:(BOOL)isParent
{

	int result;

	// appPath is not exist
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] != FILE_EXIST_FILE){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	// check roms folder
	NSString *folderPathString = @"";
	if( [self defaultBoolValue:@"optSdlMameEnable"] == YES &&
		[self defaultBoolValue:@"optSdlMameRomPath"] == YES) {

		NSString *defaultFolderPathString = [self getDefaultRomsFolderPathString];
		// not exist
		if([defaultFolderPathString isEqualToString:@""]){
			result = [self displayMessageAlertFolder:@"Roms"];
			// open prefs tab
			if(result == NSAlertFirstButtonReturn){
				[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
			}
			return NO;
		}
		folderPathString = defaultFolderPathString;
	}

	//
	// create audit result
	//
	NSString *resultString = [self createAuditResultString:romId
												folderPathString:folderPathString
												showProc:NO
								];

	//
	// create audit dict
	//
	NSMutableDictionary *dict = [self createDictAuditStatus:resultString];

	// no data
	if([dict count] <= 0){
		result = [self displayMessage:@"alert"
					messageText:@"Audit result is invalid."
					infoText:@"Please check result."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Audit Log"];
			[viewLog setLogString:resultString];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return NO;
	}			

	//
	// get data
	//
	NSString *entName = @"";
	NSString *keyRomId = @"";
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];

	if(isParent == YES){
		entName = @"romlist";
		keyRomId = @"romId";
		pred = [NSPredicate predicateWithFormat:@"romId == %@", romId];
	}else{
		entName = @"clonelist";
		keyRomId = @"cloneId";
		pred = [NSPredicate predicateWithFormat:@"cloneId == %@", romId];
	}

	NSArray *items = [tbArrayController getObjectsWithPred:entName pred:pred];

	// no data
	if([items count] <= 0){
		return NO;
	}

	//
	// update data
	//
	if([self updateAuditStatusWithItems:items
									dict:dict
									keyRomId:keyRomId
									isParent:isParent
									showProc:NO
									isClose:YES
		] == NO
	){
		return NO;
	}

/*
	// success
	result = [self displayMessage:@"info"
						messageText:@"Audit process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
*/
	// show log
//	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Audit Log"];
		[viewLog setLogString:resultString];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
//	}

	// post notification
	if(isParent == YES){
		[self postItemStatusDidChangedMainNotification];
	}else{
		[self postItemStatusDidChangedCloneNotification];
	}

	return YES;

}
//------------------------------------
// createAuditResultWithAppleScript
//------------------------------------
- (NSDictionary*)createAuditResultWithAppleScript:(NSString*)appPath
							folderPathString:(NSString*)folderPathString
							filePath:(NSString*)filePath
{

	// escape " "
//	appPath = [self replaceCharacter:appPath str1:@" " str2:@"\\ "];
//	folderPathString = [self replaceCharacter:folderPathString str1:@" " str2:@"\\ "];
//	filePath = [self replaceCharacter:filePath str1:@" " str2:@"\\ "];

	// script string
	NSString *script = @"tell application \"Terminal\"\n";
	script = [script stringByAppendingFormat:@"do script \"'%@' -verifyroms", appPath];
	if(![folderPathString isEqualToString:@""]){
		script = [script stringByAppendingFormat:@" -rompath '%@'", folderPathString];
	}
	script = [script stringByAppendingFormat:@" > '%@'\"\n", filePath];
	script = [script stringByAppendingString:@"activate\n"];
	script = [script stringByAppendingString:@"end tell\n"];
//	NSLog(@"script=%@", script);

	return [self executeAppleScript:script];

}
//------------------------------------
// updateAllAuditStatusWithString
//------------------------------------
- (BOOL)updateAllAuditStatusWithString:(NSString*)resultString
{

	//
	// create audit dict
	//
	NSMutableDictionary *dict = [self createDictAuditStatus:resultString];

	// no data
	if([dict count] <= 0){
		return NO;
	}			

	//
	// update romlist
	//
	NSArray *romItems = [tbArrayController getAllObjects:@"romlist"];
	if([self updateAuditStatusWithItems:romItems
									dict:dict
									keyRomId:@"romId"
									isParent:YES
									showProc:YES
									isClose:NO
		] == NO
	){
		return NO;
	}

	//
	// update clonelist
	//
	NSArray *cloneItems = [tbArrayController getAllObjects:@"clonelist"];
	if([self updateAuditStatusWithItems:cloneItems
									dict:dict
									keyRomId:@"cloneId"
									isParent:NO
									showProc:YES
									isClose:NO
		] == NO
	){
		return NO;
	}

	return YES;

}
//------------------------------------
// createAuditResultString
//------------------------------------
- (NSString*)createAuditResultString:(NSString*)romId
							folderPathString:(NSString*)folderPathString
							showProc:(BOOL)showProc
{

	// path
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];
	NSString *curPath = [appPath stringByDeletingLastPathComponent];

	// set arguments
	NSMutableArray *args = [NSMutableArray arrayWithCapacity:1];

	// command
	[args addObject:@"-verifyroms"];

	// romId
	if(![romId isEqualToString:@""]){
		[args addObject:romId];
	}
	// rompath
	if(![folderPathString isEqualToString:@""]){
		[args addObject:@"-rompath"];
		[args addObject:folderPathString];
	}

	// proc start
	if(showProc == YES){
		[viewProcStatus initProcStatus:@"Updating Audit Status"
							targetName:@"SDLMAME \"-verifyroms\" wait long.."
							procCount:0
							totalCount:0
							showCount:NO
							isOpen:YES
		];
	}

//	NSLog(@"appPath=%@",appPath);
//	NSLog(@"curPath=%@",curPath);
//	NSLog(@"args=%@",args);

	NSTask *task = [[[NSTask alloc] init] autorelease]; 
	NSPipe *pipeErr = [NSPipe pipe];
	NSPipe *pipeOutput = [NSPipe pipe];

	// tack error
	[task setStandardError:pipeErr]; 
	// tack output
	[task setStandardOutput:pipeOutput];

	// launch task 
	[task setLaunchPath:appPath];
	[task setCurrentDirectoryPath:curPath];
	[task setArguments:args];
	[task launch];
	[task waitUntilExit];

	// get outputdata
	NSData *dataOutput = [[pipeOutput fileHandleForReading] availableData];

	// proc stop
	if(showProc == YES){
		[viewProcStatus closeProcStatus:NO];
	}

	return [[[NSString alloc] initWithData:dataOutput encoding:NSUTF8StringEncoding] autorelease];

}

//------------------------------------
// createDictAuditStatus
//------------------------------------
- (NSMutableDictionary*)createDictAuditStatus:(NSString*)string
{

	NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];

	NSArray *cols;
	NSArray *names;

	NSMutableString *muline;
	NSString *romId;
	NSString *auditStatus;

	NSRange rng;
	NSString *line;

	NSArray *lines = [string componentsSeparatedByString:@"\n"];
	NSEnumerator *enumLines = [lines objectEnumerator];

	while(line = [enumLines nextObject]) {

		muline = [NSMutableString stringWithString:line];

		if([muline isEqualToString:@""]){
			continue;
		}

		// check "romset" line
		rng = [muline rangeOfString:@"romset"];
		if(rng.length > 0){
			[muline replaceOccurrencesOfString:@"romset " withString:@"" options:0 range:NSMakeRange(0,[muline length])];
			cols = [muline componentsSeparatedByString:@" is "];
			if([cols count] < 2){
				continue;
			}
			names = [[cols objectAtIndex:0] componentsSeparatedByString:@" "];
			romId = [names objectAtIndex:0];
			auditStatus = [cols objectAtIndex:1];

			// trim space
			romId = [romId stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			auditStatus = [auditStatus stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

			// set dictionary
			if(![dict valueForKey:romId]){
				[dict setObject:auditStatus forKey:romId];
			}
		}

	}

	return dict;
}

//------------------------------------
// updateAuditStatusWithItems
//------------------------------------
- (BOOL)updateAuditStatusWithItems:(NSArray*)items
						dict:(NSMutableDictionary*)dict
						keyRomId:(NSString*)keyRomId
						isParent:(BOOL)isParent
						showProc:(BOOL)showProc
						isClose:(BOOL)isClose
{

	NSString *romId = @"";
	NSString *auditStatus = @"";
	id record;

	int procCount = 0;
	int totalCount = [items count];

	// show proc
	if(showProc == YES){
		NSString *procTitle = @"";
		if(isParent == YES){
			procTitle = @"Updating Parent Audit Status";
		}else{
			procTitle = @"Updating Clone Audit Status";
		}

		// process start
		[viewProcStatus initProcStatus:procTitle
							targetName:@""
							procCount:procCount
							totalCount:totalCount
							showCount:NO
							isOpen:YES
		];
	}

	//
	// update records
	// 
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:keyRomId];
		auditStatus = [dict valueForKey:romId];

		if(auditStatus){
			[record setValue:auditStatus forKey:@"audit"];
		}else{
			[record setValue:@"" forKey:@"audit"];
		}

		procCount++;
		// show proc
		if(showProc == YES){
			if(procCount % 100 == 0 && romId){
				[viewProcStatus updateProcStatus:procCount targetName:romId];
			}
		}
		
	}


	// show proc
	if(showProc == YES){
		if(romId){
			[viewProcStatus updateProcStatus:procCount targetName:romId];
		}
		[viewProcStatus closeProcStatus:isClose];
	}
	return YES;

}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
//------------------------------------
// logString
//------------------------------------
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
@end
