#import "SetupMAMEList.h"
#import "ViewPrefs.h"
#import "ViewProcStatus.h"
#import "ViewLog.h"
#import "TBArrayController.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation SetupMAMEList
//------------------------------------
// awake
//------------------------------------
- (void)awakeFromNib
{
	insertItemsParent_ = [[NSMutableArray alloc] init];
	deleteItemsParent_ = [[NSMutableArray alloc] init];
	unlinkItemsParent_ = [[NSMutableArray alloc] init];
	resetItemsParent_ = [[NSMutableArray alloc] init];
	insertCountParent_ = 0;
	updateCountParent_ = 0;
	deleteCountParent_ = 0;
	unlinkCountParent_ = 0;
	resetCountParent_ = 0;

	insertItemsClone_ = [[NSMutableArray alloc] init];
	deleteItemsClone_ = [[NSMutableArray alloc] init];
	insertCountClone_ = 0;
	updateCountClone_ = 0;
	deleteCountClone_ = 0;

	[self setLogString:@""];

}

//=======================================================================
// IBAction
//=======================================================================
//------------------------------------
// importMAMEList
//------------------------------------
- (IBAction)importMAMEList:(id)sender
{

	int result;
	int parseType = 0;
	BOOL ret;

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_MAME_LIST];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	// get app folder
	if([defaultPath isEqualToString:@""]){
		defaultPath = [self converToAppHomePath];
	}else{
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:defaultPath
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"It needs much time for importing."
					infoText:@"Continue import?"
					btnList:@"Import,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// confirm
	result = [self displayMessage:@"info"
					messageText:@"Select parse type."
					infoText:@"If importing is failed, please try other type."
					btnList:@"as XML,as Text,Cancel"
			];
	if(result == NSAlertFirstButtonReturn){
		parseType = 0;
	}
	else if(result == NSAlertSecondButtonReturn){
		parseType = 1;
	}
	else{
		return;
	}

	NSString *filePath = [filePaths objectAtIndex:0];

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

	// proc start
//	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	[viewProcStatus initProcStatus:@"Importing MAME Rom List"
						targetName:@"Reading XML.. wait"
						procCount:0
						totalCount:0
						showCount:NO
						isOpen:YES
	];

	id pool = [[NSAutoreleasePool alloc] init];

	//
	// import mamelist
	//
	ret = [self createRomlistFromXMLFile:filePath parseType:parseType];

	[pool release];

	if(ret == NO){
		[viewProcStatus closeProcStatus:NO];
		result = [self displayMessage:@"alert"
						messageText:@"Can not import MAME Rom List."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[self logString]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		[self setLogString:@""];
		return;
	}

	// proc stop
//	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];
	// close process window
	[viewProcStatus closeProcStatus:NO];

	// success
	result = [self displayMessage:@"info"
					messageText:@"Imported MAME Rom List."
					infoText:@"Please check import log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Import Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setFileName:@"ImportResult.txt"];
		[viewLog setCanSave:YES];
		[viewLog openLogWindow:nil];
	}

	[self setLogString:@""];

}

//------------------------------------
// exportMAMEList
//------------------------------------
- (IBAction)exportMAMEList:(id)sender
{
	int result;

	// appPath is not exist
	NSString *appPath = [self defaultSettingFilePath:MAME_FILE_TYPE_SDLMAME];

	// app is not found
	if([self checkFileExist:appPath] == FILE_EXIST_NOT_FOUND){
		result = [self displayMessageAlertExecutionFile];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_MAME_LIST];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	// get app folder
	if([defaultPath isEqualToString:@""]){
		defaultPath = [self converToAppHomePath];
	}else{
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}

	// confirm save
	NSString *filePath = [self selectFilePathWithSavePanel:@"MAME_RomList.xml"
												canCreateDir:YES
												defaultPath:defaultPath
						];

	if([filePath isEqualToString:@""]){
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"It needs much time for exporting."
					infoText:@"Continue export?"
					btnList:@"Export,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];
/*
	// create export result wit apple script
	NSDictionary *errorInfo = [self exportMAMEListWithAppleScript:appPath
									filePath:filePath
								];
	// error
	if(errorInfo != nil){
		result = [self displayMessage:@"alert"
					messageText:@"Can not export audit result."
					infoText:@"Please check log."
					btnList:@"Cancel,Log"
				];
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Export Log"];
			[viewLog setLogString:[errorInfo description]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}
		return;
	}
*/
	[viewProcStatus initProcStatus:@"Exporting MAME Rom List"
						targetName:@"SDLMAME \"-listxml\" wait long.."
						procCount:0
						totalCount:0
						showCount:NO
						isOpen:YES
	];

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

	// proc start
//	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	// current path
	NSString *curPath = [appPath stringByDeletingLastPathComponent];

	// set arguments
	NSMutableArray *args = [NSMutableArray arrayWithCapacity:1];

	// command
	[args addObject:@"-listxml"];

//	NSLog(@"appPath=%@",appPath);
//	NSLog(@"curPath=%@",curPath);
//	NSLog(@"args=%@",args);

	NSTask *task = [[[NSTask alloc] init] autorelease]; 
	NSPipe *pipeErr = [NSPipe pipe];

	// remove & create export file
	NSFileManager *myFile = [NSFileManager defaultManager];
	
	[myFile removeFileAtPath:filePath handler:nil];
	[myFile createFileAtPath:filePath contents:nil attributes:nil];

	NSFileHandle *outfile = [NSFileHandle fileHandleForWritingAtPath:filePath];
  
	// tack output
	[task setStandardOutput:outfile];

	// tack error
	[task setStandardError:pipeErr]; 

	// launch task 
	[task setLaunchPath:appPath];
	[task setCurrentDirectoryPath:curPath];
	[task setArguments:args];
	[task launch];
	[task waitUntilExit];

	// proc start
//	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];
	// close process window
	[viewProcStatus closeProcStatus:NO];

	[self displayMessage:@"info"
						messageText:@"Export MAME Rom List is completed."
						infoText:@""
						btnList:@"OK"
	];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// exportMAMEListWithAppleScript
//------------------------------------
- (NSDictionary*)exportMAMEListWithAppleScript:(NSString*)appPath
									filePath:(NSString*)filePath
{

	// escape " "
//	appPath = [self replaceCharacter:appPath str1:@" " str2:@"\\ "];
//	filePath = [self replaceCharacter:filePath str1:@" " str2:@"\\ "];
//	appPath = [self replaceCharacter:appPath str1:@"\"" str2:@"\\\""];
//	filePath = [self replaceCharacter:filePath str1:@"\"" str2:@"\\\""];

	// script string
	NSString *script = @"tell application \"Terminal\"\n";
	script = [script stringByAppendingFormat:@"do script \"'%@' -listxml", appPath];
	script = [script stringByAppendingFormat:@" > '%@'\"\n", filePath];
	script = [script stringByAppendingString:@"activate\n"];
	script = [script stringByAppendingString:@"end tell\n"];
//	NSLog(@"script=%@", script);

	return [self executeAppleScript:script];

}
//------------------------------------
// createRomlistFromXMLFile
//------------------------------------
- (BOOL)createRomlistFromXMLFile:(NSString*)filePath parseType:(int)parseType
{

	// clear log
	[self setLogString:@""];

	if([[NSFileManager defaultManager] fileExistsAtPath:filePath ] == NO){
		[self appendLogString:@"error=[File is not found]\n"];
		[self appendLogString:[NSString stringWithFormat: @"file=[%@]\n", filePath]];
		return NO;
	}

	//
	// create dict
	//
	NSString *newVersion = @"";
	NSString *oldVersion = [self defaultStringValue:@"optSdlMameVersion"];

	NSMutableDictionary *dictParents = [[[NSMutableDictionary alloc] init] autorelease];
	NSMutableDictionary *dictClones = [[[NSMutableDictionary alloc] init] autorelease];
	NSMutableDictionary *dictCloneCounts = [[[NSMutableDictionary alloc] init] autorelease];

	//
	// parse as xml
	//
	[viewProcStatus updateTargetName:@"Parsing XML.. wait"];

	if(parseType == 0){
		// create xml string
		BOOL isError = NO;
		NSString *xmlString = [self createRomlistFromXMLString:filePath isError:&isError];
		if(isError == YES || [xmlString isEqualToString:@""]){
			return NO;
		}
		if([self createDictRomsWithXMLString:xmlString
										dictParents:dictParents
										dictClones:dictClones
										dictCloneCounts:dictCloneCounts
										newVersion:&newVersion
			] == NO
		){
			return NO;
		}
	}
	//
	// parse as text file
	//
	else{
		if([self createDictRomsWithFile:filePath
										dictParents:dictParents
										dictClones:dictClones
										dictCloneCounts:dictCloneCounts
										newVersion:&newVersion
			] == NO
		){
			return NO;
		}
	}

	[viewProcStatus updateTargetName:@"Updating data.. wait"];

	//
	// clear results
	//
	insertCountParent_ = 0;
	updateCountParent_ = 0;
	deleteCountParent_ = 0;
	unlinkCountParent_ = 0;
	resetCountParent_ = 0;
	[insertItemsParent_ removeAllObjects];
	[deleteItemsParent_ removeAllObjects];
	[unlinkItemsParent_ removeAllObjects];
	[resetItemsParent_ removeAllObjects];

	insertCountClone_ = 0;
	updateCountClone_ = 0;
	deleteCountClone_ = 0;
	[insertItemsClone_ removeAllObjects];
	[deleteItemsClone_ removeAllObjects];

	//
	// update romlist
	//
	[self updateRomlist:dictParents
				dictClones:dictClones
				dictCloneCounts:dictCloneCounts
	];

	//
	// insert romlist
	//
	[self insertRomlist:dictParents
				dictCloneCounts:dictCloneCounts
				version:newVersion
	];
	
	//
	// update clonelist
	//
	[self updateClonelist:dictClones];

	//
	// insert clonelist
	//
	[self insertClonelist:dictClones version:newVersion];

	//
	// update rom & clonelist
	//
	[self updateRomAndCloneRelation];

	//
	// log string
	//
	// head
	NSString *headString = [self createImportHeadString:oldVersion newVersion:newVersion];

	// parent
	NSString *parentLogString = [self createImportResultString:insertItemsParent_
																	deletedItems:deleteItemsParent_
																	insertedCount:insertCountParent_
																	updatedCount:updateCountParent_
																	deletedCount:deleteCountParent_
																	isParent:YES
								];
	// clone
	NSString *cloneLogString = [self createImportResultString:insertItemsClone_
																	deletedItems:deleteItemsClone_
																	insertedCount:insertCountClone_
																	updatedCount:updateCountClone_
																	deletedCount:deleteCountClone_
																	isParent:NO
								];

	// unlink
	NSString *unlinkLogString = [self createUnlinkResultString:unlinkItemsParent_
																	unlinkCount:unlinkCountParent_
								];

	// reset
	NSString *resetLogString = [self createResetResultString:resetItemsParent_
																	resetCount:resetCountParent_
								];

	NSString *logString = [NSString stringWithFormat:@"%@%@%@%@%@"
													, headString
													, parentLogString
													, cloneLogString
													, unlinkLogString
													, resetLogString
													];

	[self setLogString:logString];

	//
	// clear results
	//
	[insertItemsParent_ removeAllObjects];
	[deleteItemsParent_ removeAllObjects];
	[unlinkItemsParent_ removeAllObjects];
	[resetItemsParent_ removeAllObjects];

	[insertItemsClone_ removeAllObjects];
	[deleteItemsClone_ removeAllObjects];

	// update version
	[self setDefaultStringValue:newVersion key:@"optSdlMameVersion"];

	return YES;
}

//------------------------------------
// createDictRomsWithFile
//------------------------------------
- (BOOL)createDictRomsWithFile:(NSString*)filePath
				dictParents:(NSMutableDictionary*)dictParents
				dictClones:(NSMutableDictionary*)dictClones
				dictCloneCounts:(NSMutableDictionary*)dictCloneCounts
				newVersion:(NSString**)newVersion
{

	NSArray *lines;
	NSArray *cols;
	NSEnumerator *enumLines;
	NSRange rng;
	id line;

	NSString *record = @"";
	NSString *driverRecord = @"";
	NSString *romId = @"";
	NSString *parentId = @"";
	NSString *description = @"";
	NSString *description_last = @"";
	NSString *manufacturer = @"";
	NSString *year = @"????";
	NSString *romof = @"";
	NSString *driverStatus = @"";
	NSString *driverEmulation = @"";
	NSString *disk_name = @"";

	BOOL isClone = NO;
	BOOL isSuccess = NO;
	int cloneCount = 0;
	int procCount = 0;

	// process status
	[viewProcStatus initProcStatus:@"Importing MAME Rom List"
						targetName:@""
						procCount:0
						totalCount:0
						showCount:YES
						isOpen:NO
	];


	// streaming reading
	#define STR_MAX  512

	FILE *fp;
	char cStr[STR_MAX];
	const  char  *cFilePath;
	int last;

	cFilePath = [[ NSFileManager defaultManager] fileSystemRepresentationWithPath:[filePath stringByExpandingTildeInPath]];

	// open file
	if ((fp = fopen(cFilePath, "r")) == NULL) {
		[self appendLogString:@"error=[File open error]\n"];
		[self appendLogString:[NSString stringWithFormat: @"file=[%@]\n", filePath]];
		return NO;
	}
	
	// read file
	while (fgets(cStr, STR_MAX, fp) != NULL) {

		// trancate last "\n"
		last = strlen(cStr) - 1;
		if(cStr[last] == '\n'){
			cStr[last] = '\0';
		}

		// cStr -> record
		record = [NSString stringWithCString:cStr];
		if(!record){
			continue;
		}
		// erase Tab Code
		record = [self replaceCharacter:record str1:@"\t" str2:@""];

		if([record isEqualToString:@""]){
			continue;
		}

		// <mame build 
		rng = [record rangeOfString:@"<mame build"];
		if(rng.length > 0){
			lines = [record componentsSeparatedByString:@"\""];
			if([lines count] > 1){
				cols = [[lines objectAtIndex:1] componentsSeparatedByString:@" "];
//				[[NSUserDefaults standardUserDefaults] setObject:[cols objectAtIndex:0] forKey:@"optSdlMameVersion"];
				*newVersion = [cols objectAtIndex:0];
			}

		}

		// <game 
		rng = [record rangeOfString:@"<game"];
		if(rng.length > 0){
			lines = [record componentsSeparatedByString:@" "];
			enumLines = [lines objectEnumerator];
			while (line = [enumLines nextObject]) {
				if([line isEqualToString:@""]){
					continue;
				}
				rng = [line rangeOfString:@"="];
				if(rng.length > 0){
					cols = [line componentsSeparatedByString:@"="];
					if([[cols objectAtIndex:0] isEqualTo:@"name"]){
						romId = [self replaceCharacter:[cols objectAtIndex:1] str1:@"\"" str2:@""];
					}
					if([[cols objectAtIndex:0] isEqualTo:@"cloneof"]){
						parentId = [self replaceCharacter:[cols objectAtIndex:1] str1:@"\"" str2:@""];
						isClone = YES;
					}
					if([[cols objectAtIndex:0] isEqualTo:@"romof"]){
						romof = [self replaceCharacter:[cols objectAtIndex:1] str1:@"\"" str2:@""];
					}
				}
			}

		}

		// <description>
		rng = [record rangeOfString:@"<description>"];
		if(rng.length > 0){
			description = record;
			description = [self replaceCharacter:description str1:@"<description>" str2:@""];
			description = [self replaceCharacter:description str1:@"</description>" str2:@""];
			description = [self replaceCharacter:description str1:@"&amp;" str2:@"&"];
			description = [self replaceCharacter:description str1:@"&lt;" str2:@"<"];
			description = [self replaceCharacter:description str1:@"&gt;" str2:@">"];
		}
		// <manufacturer>
		rng = [record rangeOfString:@"<manufacturer>"];
		if(rng.length > 0){
			manufacturer = record;
			manufacturer = [self replaceCharacter:manufacturer str1:@"<manufacturer>" str2:@""];
			manufacturer = [self replaceCharacter:manufacturer str1:@"</manufacturer>" str2:@""];
			manufacturer = [self replaceCharacter:manufacturer str1:@"&amp;" str2:@"&"];
			manufacturer = [self replaceCharacter:manufacturer str1:@"&lt;" str2:@"<"];
			manufacturer = [self replaceCharacter:manufacturer str1:@"&gt;" str2:@">"];
		}
		// <year>
		rng = [record rangeOfString:@"<year>"];
		if(rng.length > 0){
			year = record;
			year = [self replaceCharacter:year str1:@"<year>" str2:@""];
			year = [self replaceCharacter:year str1:@"</year>" str2:@""];
		}
		// <driver>
		rng = [record rangeOfString:@"<driver"];
		if(rng.length > 0){
			driverRecord = record;
			driverRecord = [self replaceCharacter:driverRecord str1:@"<driver " str2:@""];
			driverRecord = [self replaceCharacter:driverRecord str1:@"/>" str2:@""];
			driverRecord = [self replaceCharacter:driverRecord str1:@"\"" str2:@""];

			// search "status"
			lines = [driverRecord componentsSeparatedByString:@" "];
			enumLines = [lines objectEnumerator];
			while (line = [enumLines nextObject]) {
				if([line isEqualToString:@""]){
					continue;
				}
				rng = [line rangeOfString:@"="];
				if(rng.length > 0){
					cols = [line componentsSeparatedByString:@"="];
					if([[cols objectAtIndex:0] isEqualToString:@"status"]){
						driverStatus = [NSMutableString stringWithString:line];
					}
				}
			}
		}

		// </game>
		rng = [record rangeOfString:@"</game>"];
		if(rng.length > 0){
			if(romId && ![romId isEqualToString:@""]){

				NSDictionary *dictRom = [NSDictionary dictionaryWithObjectsAndKeys:
											romId, @"romId",
											parentId, @"parentId",
											romof, @"romof",
											description, @"title",
											year, @"year",
											manufacturer, @"manufacture",
											driverStatus, @"driverStatus",
											driverEmulation, @"driverEmulation",
											disk_name, @"disk_name",
											[NSNumber numberWithBool:isClone], @"isClone",
											nil
										];

				// parent
				if(isClone == NO){
					[dictParents setValue:dictRom forKey:romId];
				}
				// clone
				else{
					[dictClones setValue:dictRom forKey:romId];

					// clone count
					if([dictCloneCounts valueForKey:parentId]){
						cloneCount = [[dictCloneCounts valueForKey:parentId] intValue];
						cloneCount++;
					}else{
						cloneCount = 1;
					}
					[dictCloneCounts setObject:[NSNumber numberWithInt:cloneCount] forKey:parentId];
				}

				procCount++;
				if(procCount % 100 == 0 && description){
					[viewProcStatus updateProcStatus:procCount targetName:description];
				}

				description_last = description;
				// clear
				romId = @"";
				description = @"";
				manufacturer = @"";
				year = @"????";
				parentId = @"";
				romof = @"";
				driverStatus = @"";
				driverEmulation = @"";
				disk_name = @"";
				isClone = NO;

				isSuccess = YES;

			}
		}

	}

	// close file
	fclose(fp);

	if(description_last){
		[viewProcStatus updateProcStatus:procCount targetName:description_last];
	}

	if(isSuccess == NO){
		[self appendLogString:@"error=[xml list data is not found\n"];
		[self appendLogString:[NSString stringWithFormat: @"file=[%@]\n", filePath]];
	}

	return isSuccess;

}
//------------------------------------
// createRomlistFromXMLString
//------------------------------------
- (NSString*)createRomlistFromXMLString:(NSString*)filePath isError:(BOOL*)isError
{

	// create data
	NSError *error = nil;
	NSData *xmlData = [[[NSData alloc] initWithContentsOfFile:filePath options:nil error:&error] autorelease];

	// error
	if(xmlData == nil || error != nil){
		[self setLogString:[error description]];
		*isError = YES;
		return @"";
	}

	// create string
	NSString *xmlString = [[[NSString alloc] initWithData:xmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"xmlString=%@", xmlString);
	if(xmlString == nil){
		[self appendLogString:@"XML Data is invalid."];
		*isError = YES;
		return @"";
	}

	return xmlString;
}
//------------------------------------
// createDictRomsWithXMLString
//------------------------------------
- (BOOL)createDictRomsWithXMLString:(NSString*)xmlString
				dictParents:(NSMutableDictionary*)dictParents
				dictClones:(NSMutableDictionary*)dictClones
				dictCloneCounts:(NSMutableDictionary*)dictCloneCounts
				newVersion:(NSString**)newVersion
{

	// create xml document
	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xmlString options:nil error:&error] autorelease];

	if(error != nil){
		[self appendLogString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		[self appendLogString:[NSString stringWithFormat: @"xmlString=[%@]\n", xmlString]];
		return NO;
	}

	// parse xml document
	//
	// <mame>
	//
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);

	// null or not match
	if(rootNode == nil || ![[rootNode name] isEqualToString:@"mame"]){
		[self appendLogString:@"error=[<mame> is null]\n"];
		return NO;
	}

	NSString *build = [[(NSXMLElement*)rootNode attributeForName:@"build"] stringValue];
	*newVersion = [self getMameVersionFromString:build];

	NSString *romId = @"";
	NSString *parentId = @"";
	NSString *romof = @"";
	NSString *description = @"";
	NSString *year = @"";
	NSString *manufacture = @"";

	NSString *driverStatus;
	NSString *driverEmulation;
	NSString *disk_name;

	BOOL isClone;
	int cloneCount;

	NSXMLNode *childNode;
	NSXMLNode *driverNode;
	NSXMLNode *diskNode;

	BOOL isSuccess = NO;

	// process status
	int procCount = 0;
	[viewProcStatus initProcStatus:@"Importing MAME Rom List"
						targetName:@""
						procCount:0
						totalCount:[[rootNode children] count]
						showCount:NO
						isOpen:NO
	];


	NSEnumerator *enumNodes = [[rootNode children] objectEnumerator];
	while (childNode = [enumNodes nextObject]){

		//
		// <game>
		//
		if ([[childNode name] isEqualToString:@"game"]){

			// name, parentId, romof
			romId = [[(NSXMLElement*)childNode attributeForName:@"name"] stringValue];
			parentId = [[(NSXMLElement*)childNode attributeForName:@"cloneof"] stringValue];
			romof = [[(NSXMLElement*)childNode attributeForName:@"romof"] stringValue];

			// <description>
			description = [[self getChildNode:childNode name:@"description"] stringValue];
			// <year>
			year = [[self getChildNode:childNode name:@"year"] stringValue];
			// <manufacturer>
			manufacture = [[self getChildNode:childNode name:@"manufacturer"] stringValue];

			// <driver>
			driverNode = [self getChildNode:childNode name:@"driver"];
			if(driverNode){
				driverStatus = [[(NSXMLElement*)driverNode attributeForName:@"status"] stringValue];
				if(![driverStatus isEqualToString:@""]){
					driverStatus = [NSString stringWithFormat:@"status=%@", driverStatus];
				}
				driverEmulation = [[(NSXMLElement*)driverNode attributeForName:@"emulation"] stringValue];
				if(![driverEmulation isEqualToString:@""]){
					driverEmulation = [NSString stringWithFormat:@"emulation=%@", driverEmulation];
				}
			}else{
				driverStatus = @"";
				driverEmulation = @"";
			}

			// <disk>
			diskNode = [self getChildNode:childNode name:@"disk"];
			if(diskNode){
				disk_name = [[(NSXMLElement*)diskNode attributeForName:@"name"] stringValue];
			}else{
				disk_name = @"";
			}

			// null check
			if(!romId){
				continue;
			}
			if(!parentId){parentId = @"";}
			if(!romof){romof = @"";}
			if(!description){description = @"";}
			if(!year){year = @"????";}
			if(!manufacture){manufacture = @"";}

			if([parentId isEqualToString:@""]){
				isClone = NO;
			}else{
				isClone = YES;
			}
	
			NSDictionary *dictRom = [NSDictionary dictionaryWithObjectsAndKeys:
										romId, @"romId",
										parentId, @"parentId",
										romof, @"romof",
										description, @"title",
										year, @"year",
										manufacture, @"manufacture",
										driverStatus, @"driverStatus",
										driverEmulation, @"driverEmulation",
										disk_name, @"disk_name",
										[NSNumber numberWithBool:isClone], @"isClone",
										nil
									];

			// parent
			if(isClone == NO){
				[dictParents setValue:dictRom forKey:romId];
			}
			// clone
			else{
				[dictClones setValue:dictRom forKey:romId];

				// clone count
				if([dictCloneCounts valueForKey:parentId]){
					cloneCount = [[dictCloneCounts valueForKey:parentId] intValue];
					cloneCount++;
				}else{
					cloneCount = 1;
				}
				[dictCloneCounts setObject:[NSNumber numberWithInt:cloneCount] forKey:parentId];
			}

			isSuccess = YES;

			procCount++;
			if(procCount % 100 == 0 && description){
				[viewProcStatus updateProcStatus:procCount targetName:description];
			}
		}
	}

	if(description){
		[viewProcStatus updateProcStatus:procCount targetName:description];
	}

	if(isSuccess == NO){
		[self appendLogString:@"error=[xml list data is not found\n"];
		[self appendLogString:[NSString stringWithFormat: @"xmlString=[%@]\n", xmlString]];
	}

	return isSuccess;

}
//------------------------------------
// updateRomlist
//------------------------------------
- (BOOL)updateRomlist:(NSMutableDictionary*)dictParents
			dictClones:(NSMutableDictionary*)dictClones
			dictCloneCounts:(NSMutableDictionary*)dictCloneCounts
{
	
	NSString *romId;
	NSString *defaultRomId;
	NSString *title;

	int cloneCount;

	NSDictionary *dictRom;

	id record;
	id plistItem;

	//---------------------------
	//	update record
	//---------------------------
	NSArray *items = [tbArrayController getAllObjects:@"romlist"];
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:@"romId"];
		defaultRomId = [record valueForKey:@"defaultRomId"];
		title = [record valueForKey:@"title"];

		dictRom = [dictParents valueForKey:romId];

		// clone count
		cloneCount = 0;
		if([dictCloneCounts valueForKey:romId]){
			cloneCount = [[dictCloneCounts valueForKey:romId] intValue];
		}

		// exists in parent
		if(dictRom){

/*
			// check defaultRomId
			if(![defaultRomId isEqualToString:romId]){
				// not exists in clones -> reset
				if(![dictClones valueForKey:defaultRomId]){
					defaultRomId = romId;
					[record setValue:nil forKey:@"rlt_clonelist"];		
				}
			}
*/
			// update record
			[record setValue:defaultRomId forKey:@"defaultRomId"];		
			[record setValue:[dictRom valueForKey:@"title"] forKey:@"title"];		
			[record setValue:[dictRom valueForKey:@"manufacture"] forKey:@"manufacture"];		
			[record setValue:[dictRom valueForKey:@"year"] forKey:@"year"];		
			[record setValue:[dictRom valueForKey:@"driverStatus"] forKey:@"driverStatus"];		
			[record setValue:[NSNumber numberWithInt:cloneCount] forKey:@"cloneCount"];

			// delete from dictParents
			[dictParents removeObjectForKey:romId];

			updateCountParent_++;
		}
		// delete records
		else{

			// unlink log
			plistItem = [record valueForKey:@"rlt_playlist"];
			if([plistItem count] > 0){
				[self createUnlinkItems:romId title:title plistItem:plistItem];
			}

			[[MAME_Tunes_AppDelegate managedObjectContext] deleteObject:record];

			// item log
			[deleteItemsParent_ addObject:[NSString stringWithFormat:@"[%@]\t%@"
										,romId
										,title
									]
			];
			deleteCountParent_++;
		}
	}

	return YES;

}
//------------------------------------
// insertRomlist
//------------------------------------
- (BOOL)insertRomlist:(NSMutableDictionary*)dictParents
			dictCloneCounts:(NSMutableDictionary*)dictCloneCounts
			version:(NSString*)version
{

	NSString *romId;
	NSString *title;
	int cloneCount;
	NSDictionary *dictRom;

	NSManagedObject *mo;

	//---------------------------
	//	insert record
	//---------------------------
	NSEntityDescription *entName = [NSEntityDescription 
									entityForName:@"romlist"
									inManagedObjectContext:[MAME_Tunes_AppDelegate managedObjectContext]];

	// insert new records
	NSEnumerator *enumParents = [dictParents objectEnumerator];
	while (dictRom = [enumParents nextObject]) {


		romId = [dictRom valueForKey:@"romId"];
		title = [dictRom valueForKey:@"title"];

		// clone count
		cloneCount = 0;
		if([dictCloneCounts valueForKey:romId]){
			cloneCount = [[dictCloneCounts valueForKey:romId] intValue];
		}

		mo = [[[NSManagedObject alloc]
				initWithEntity:entName
				insertIntoManagedObjectContext:[MAME_Tunes_AppDelegate managedObjectContext]]
				autorelease];

		[mo setValue:romId forKey:@"romId"];
		[mo setValue:romId forKey:@"defaultRomId"];
		[mo setValue:[dictRom valueForKey:@"title"] forKey:@"title"];
		[mo setValue:[dictRom valueForKey:@"manufacture"] forKey:@"manufacture"];
		[mo setValue:[dictRom valueForKey:@"year"] forKey:@"year"];
		[mo setValue:[dictRom valueForKey:@"driverStatus"] forKey:@"driverStatus"];
		[mo setValue:@"" forKey:@"titlejp"];
		[mo setValue:@"" forKey:@"audit"];
		[mo setValue:[NSNumber numberWithBool:NO] forKey:@"isExist"];
		[mo setValue:[NSNumber numberWithInt:0] forKey:@"playCount"];
		[mo setValue:[NSNumber numberWithInt:cloneCount] forKey:@"cloneCount"];
		[mo setValue:version forKey:@"addVersion"];
		[mo setValue:[NSNumber numberWithInt:0] forKey:@"labelColorNo"];
		[mo setValue:@"" forKey:@"comment"];

		// delete from dictParents
		[dictParents removeObjectForKey:romId];

		// item log
		[insertItemsParent_ addObject:[NSString stringWithFormat:@"[%@]\t%@"
									,romId
									,title
								]
		];

		insertCountParent_++;

	}

	return YES;
}
//------------------------------------
// updateClonelist
//------------------------------------
- (BOOL)updateClonelist:(NSMutableDictionary*)dictClones
{
	
	NSString *cloneId;
	NSString *parentId;
	NSString *title;

	NSDictionary *dictRom;

	id record;

	//---------------------------
	//	update record
	//---------------------------
	NSArray *items = [tbArrayController getAllObjects:@"clonelist"];
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		cloneId = [record valueForKey:@"cloneId"];
		parentId = [record valueForKey:@"parentId"];
		title = [record valueForKey:@"title"];

		dictRom = [dictClones valueForKey:cloneId];

		// exists in clone
		if(dictRom){

			// update record
			[record setValue:[dictRom valueForKey:@"parentId"] forKey:@"parentId"];		
			[record setValue:[dictRom valueForKey:@"title"] forKey:@"title"];		
			[record setValue:[dictRom valueForKey:@"manufacture"] forKey:@"manufacture"];		
			[record setValue:[dictRom valueForKey:@"year"] forKey:@"year"];		
			[record setValue:[dictRom valueForKey:@"driverStatus"] forKey:@"driverStatus"];		

			// delete from dictClones
			[dictClones removeObjectForKey:cloneId];

			updateCountClone_++;

		}
		// delete records
		else{

			[[MAME_Tunes_AppDelegate managedObjectContext] deleteObject:record];

			// item log
			[deleteItemsClone_ addObject:[NSString stringWithFormat:@"[%@ %@]\t%@"
										,parentId
										,cloneId
										,title
									]
			];
			deleteCountClone_++;
		}
	}

	return YES;

}
//------------------------------------
// insertClonelist
//------------------------------------
- (BOOL)insertClonelist:(NSMutableDictionary*)dictClones
			version:(NSString*)version
{

	NSString *cloneId;
	NSString *parentId;
	NSString *title;
	NSDictionary *dictRom;

	NSManagedObject *mo;

	//---------------------------
	//	insert record
	//---------------------------
	NSEntityDescription *entName = [NSEntityDescription 
									entityForName:@"clonelist"
									inManagedObjectContext:[MAME_Tunes_AppDelegate managedObjectContext]];

	// insert new records
	NSEnumerator *enumClones = [dictClones objectEnumerator];
	while (dictRom = [enumClones nextObject]) {


		cloneId = [dictRom valueForKey:@"romId"];
		parentId = [dictRom valueForKey:@"parentId"];
		title = [dictRom valueForKey:@"title"];

		mo = [[[NSManagedObject alloc]
				initWithEntity:entName
				insertIntoManagedObjectContext:[MAME_Tunes_AppDelegate managedObjectContext]]
				autorelease];

		[mo setValue:cloneId forKey:@"cloneId"];
		[mo setValue:parentId forKey:@"parentId"];
		[mo setValue:[dictRom valueForKey:@"title"] forKey:@"title"];
		[mo setValue:[dictRom valueForKey:@"manufacture"] forKey:@"manufacture"];
		[mo setValue:[dictRom valueForKey:@"year"] forKey:@"year"];
		[mo setValue:[dictRom valueForKey:@"driverStatus"] forKey:@"driverStatus"];
		[mo setValue:@"" forKey:@"titlejp"];
		[mo setValue:@"" forKey:@"audit"];
		[mo setValue:[NSNumber numberWithBool:NO] forKey:@"isExist"];
		[mo setValue:version forKey:@"addVersion"];

		// delete from dictClones
		[dictClones removeObjectForKey:cloneId];

		// item log
		[insertItemsClone_ addObject:[NSString stringWithFormat:@"[%@ %@]\t%@"
									,parentId
									,cloneId
									,title
								]
		];
		insertCountClone_++;

	}

	return YES;
}
//------------------------------------
// updateRomAndCloneRelation
//------------------------------------
- (void)updateRomAndCloneRelation
{
	NSMutableDictionary *dictClones = [[[NSMutableDictionary alloc] init] autorelease];
	id record;
	id cloneObject;

	NSString *title;
	NSString *romId;
	NSString *defaultRomId;
	NSString *cloneId;

	//
	//	crean clonelist & create clone dict
	//
	NSArray *cloneItems = [tbArrayController getAllObjects:@"clonelist"];
	NSEnumerator *enumCloneItems = [cloneItems objectEnumerator];
	while (record = [enumCloneItems nextObject]){

		cloneId = [record valueForKey:@"cloneId"];
		[record setValue:nil forKey:@"rlt_romlist"];

		[dictClones setValue:record forKey:cloneId];
	}

	//
	//	update rom & clonelist
	//
	NSArray *romItems = [tbArrayController getAllObjects:@"romlist"];
	NSEnumerator *enumRomItems = [romItems objectEnumerator];
	while (record = [enumRomItems nextObject]){
		title = [record valueForKey:@"title"];
		romId = [record valueForKey:@"romId"];
		defaultRomId = [record valueForKey:@"defaultRomId"];

		// check defaultRomId
		if(!defaultRomId || [defaultRomId isEqualToString:@""]){
			[record setValue:romId forKey:@"defaultRomId"];
			[record setValue:nil forKey:@"rlt_clonelist"];
		}else{
			if(![defaultRomId isEqualToString:romId]){
				// set clone
				cloneObject = [dictClones valueForKey:defaultRomId];
				if(cloneObject){
					[record setValue:cloneObject forKey:@"rlt_clonelist"];
	//				[cloneObject setValue:record forKey:@"rlt_romlist"];
				}
				// reset
				else{
					[record setValue:romId forKey:@"defaultRomId"];
					[record setValue:nil forKey:@"rlt_clonelist"];
					// item log
					[resetItemsParent_ addObject:[NSString stringWithFormat:@"[%@ %@]\t%@"
														,romId
														,defaultRomId
														,title
												]
					];
					resetCountParent_++;
				}
			}
		}
	}
}
//------------------------------------
// createUnlinkItems
//------------------------------------
- (void)createUnlinkItems:(NSString*)romId
					title:(NSString*)title
					plistItem:(id)plistItem
{

	id record;
	NSString *plistTitle;

	if([plistItem count] > 0){

		NSEnumerator *enumItems = [plistItem objectEnumerator];
		while (record = [enumItems nextObject]) {

			plistTitle = [record valueForKey:@"title"];

			// item log
			[unlinkItemsParent_ addObject:[NSString stringWithFormat:@"[%@]\t%@\t%@"
										, plistTitle
										, romId
										, title
									]
			];
			unlinkCountParent_++;
		}

	}

}
//------------------------------------
// createImportHeadString
//------------------------------------
- (NSString *)createImportHeadString:(NSString*)oldVersion newVersion:(NSString*)newVersion
{

	NSString *string = @"";

	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:@"Importing Summary\n"];
	if([oldVersion isEqualToString:@""]){
		string = [string stringByAppendingString:[NSString stringWithFormat: @"SDLMAME Ver.%@\n", newVersion]];
	}else{
		string = [string stringByAppendingString:[NSString stringWithFormat: @"SDLMAME Ver.%@ -> %@\n", oldVersion, newVersion]];
	}
	string = [string stringByAppendingString:@"Parent Roms:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Insert = %d\n", insertCountParent_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Update = %d\n", updateCountParent_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Delete = %d\n", deleteCountParent_]];
	string = [string stringByAppendingString:@"Clone Roms:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Insert = %d\n", insertCountClone_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Update = %d\n", updateCountClone_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Delete = %d\n", deleteCountClone_]];
	string = [string stringByAppendingString:@"Misc:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Delete Parent Roms From Playlist = %d\n", unlinkCountParent_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Reset Default Roms = %d\n", resetCountParent_]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	return string;
}
//------------------------------------
// createImportResultString
//------------------------------------
- (NSString*)createImportResultString:(NSArray*)insertedItems
				deletedItems:(NSArray*)deletedItems
				insertedCount:(int)insertedCount
				updatedCount:(int)updatedCount
				deletedCount:(int)deletedCount
				isParent:(BOOL)isParent
{

	NSString *title = @"";
	NSString *string = @"";

	if(isParent == YES){
		title = @"Parent";
	}else{
		title = @"Clone";
	}

	// insert
	string = [string stringByAppendingString:@"\n"];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" Insert %@ Roms = %d\n", title, insertedCount]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	string = [string stringByAppendingString:[[insertedItems sortedArrayUsingSelector:@selector(compare:)] componentsJoinedByString:@"\n"]];
	string = [string stringByAppendingString:@"\n"];

	// update
	string = [string stringByAppendingString:@"\n"];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" Update %@ Roms = %d\n", title, updatedCount]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	if(updatedCount > 0){
		string = [string stringByAppendingString:@"Can not report update roms because too many updates.\n"];
	}

	// delete
	string = [string stringByAppendingString:@"\n"];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" Delete %@ Roms = %d\n", title, deletedCount]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	string = [string stringByAppendingString:[[deletedItems sortedArrayUsingSelector:@selector(compare:)] componentsJoinedByString:@"\n"]];
	string = [string stringByAppendingString:@"\n"];

	return string;

}
//------------------------------------
// createUnlinkResultString
//------------------------------------
- (NSString*)createUnlinkResultString:(NSArray*)unlinkItems
							unlinkCount:(int)unlinkCount
{

	NSString *string = @"";

	// unlink
	string = [string stringByAppendingString:@"\n"];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" Delete Parent Roms From Playlist = %d\n", unlinkCount]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	string = [string stringByAppendingString:[[unlinkItems sortedArrayUsingSelector:@selector(compare:)] componentsJoinedByString:@"\n"]];
	string = [string stringByAppendingString:@"\n"];

	return string;

}
//------------------------------------
// createResetResultString
//------------------------------------
- (NSString*)createResetResultString:(NSArray*)resetItems
							resetCount:(int)resetCount
{

	NSString *string = @"";

	// reset
	string = [string stringByAppendingString:@"\n"];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" Reset Default Roms = %d\n", resetCount]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	string = [string stringByAppendingString:[[resetItems sortedArrayUsingSelector:@selector(compare:)] componentsJoinedByString:@"\n"]];
	string = [string stringByAppendingString:@"\n"];

	return string;

}

//------------------------------------
// getMameVersionFromString
//------------------------------------
- (NSString*)getMameVersionFromString:(NSString*)string
{
	NSString *version = string;

	if(![string isEqualToString:@""]){
		NSArray *cols = [string componentsSeparatedByString:@" "];
		version = [cols objectAtIndex:0];
	}
	return version;
}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[insertItemsParent_ release];
	[deleteItemsParent_ release];
	[unlinkItemsParent_ release];
	[resetItemsParent_ release];

	[insertItemsClone_ release];
	[deleteItemsClone_ release];
	[logString_ release];
	[super dealloc];
}

@end
