#import "SetupMAMEListJP.h"
#import "ViewLog.h"
#import "ViewProcStatus.h"
#import "TBArrayController.h"
#import "UserDefaultsExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"

@implementation SetupMAMEListJP

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setLogString:@""];

}
//=======================================================================
// IBAction
//=======================================================================
//------------------------------------
// importMAMEListJP
//------------------------------------
- (IBAction)importMAMEListJP:(id)sender
{

	int result;

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_MAME_LIST_JP];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	// get app folder
	if([defaultPath isEqualToString:@""]){
		defaultPath = [self converToAppHomePath];
	}else{
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:defaultPath
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"Import MAME List JP?"
					infoText:@"Import Japanese titles."
					btnList:@"Import,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}
	
	NSString *filePath = [filePaths objectAtIndex:0];

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

/*
	// process start
	[viewProcStatus initProcStatus:@"Importing MAME List JP"
						targetName:@"Reading text.. wait"
						procCount:0
						totalCount:0
						showCount:NO
						isOpen:YES
	];
*/
	id pool = [[NSAutoreleasePool alloc] init];

	// create dict
	NSMutableDictionary *dict = [self createDictJP:filePath];

//	[viewProcStatus closeProcStatus:NO];

	// no data
	if([dict count] <= 0){

		[pool release];

		result = [self displayMessage:@"error"
							messageText:@"Import file is invalid."
							infoText:@"Please check import file."
							btnList:@"Cancel"
				];
		return;
	}			

	//
	// update romlist
	//
	NSArray *romItems = [tbArrayController getAllObjects:@"romlist"];
	[self updateTitleJPWithItems:romItems
							dict:dict
							keyRomId:@"romId"
							isParent:YES
							showProc:YES
							isClose:NO
	];

	//
	// update clonelist
	//
	NSArray *cloneItems = [tbArrayController getAllObjects:@"clonelist"];
	[self updateTitleJPWithItems:cloneItems
							dict:dict
							keyRomId:@"cloneId"
							isParent:NO
							showProc:YES
							isClose:NO
	];

	[pool release];

	// success
	result = [self displayMessage:@"info"
						messageText:@"Import process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Update Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}

	[self setLogString:@""];

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// updateTitleJPWithItems
//------------------------------------
- (void)updateTitleJPWithItems:(NSArray*)items
						dict:(NSMutableDictionary*)dict
						keyRomId:(NSString*)keyRomId
						isParent:(BOOL)isParent
						showProc:(BOOL)showProc
						isClose:(BOOL)isClose
{

	NSString *romId = @"";
	NSString *title = @"";

	id record;
	id dictRec;

	int procCount = 0;
	int updateCount = 0;
	int totalCount = [items count];

	// show proc
	if(showProc == YES){
		NSString *procTitle = @"";
		if(isParent == YES){
			procTitle = @"Updating Parent Title JP";
		}else{
			procTitle = @"Updating Clone Title JP";
		}
		[viewProcStatus initProcStatus:procTitle
							targetName:@""
							procCount:procCount
							totalCount:totalCount
							showCount:NO
							isOpen:YES
		];
	}

	// update all records
	NSEnumerator *enumItems = [items objectEnumerator];

	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:keyRomId];
		dictRec = [dict valueForKey:romId];
	
		if(dictRec){
			title = [dictRec valueForKey:@"titlejp"];
			[record setValue:title forKey:@"titlejp"];
			updateCount++;
		}else{
			title = [record valueForKey:@"title"];
		}

		procCount++;		
		// show proc
		if(showProc == YES){
			if(procCount % 100 == 0 && title){
				[viewProcStatus updateProcStatus:procCount targetName:title];
			}
		}
	}

	// show proc
	if(showProc == YES){
		if(title){
			[viewProcStatus updateProcStatus:procCount targetName:title];
		}
		[viewProcStatus closeProcStatus:isClose];
	}

	// append summary
	if(isParent == YES){
		[self appendLogString:@"Parent Roms : "];
	}else{
		[self appendLogString:@"Clone Roms : "];
	}
	[self appendLogString:[NSString stringWithFormat:@"%d / %d files update.\n"
												, updateCount
												, totalCount
							]
	];

}
//------------------------------------
// createDictJP
//------------------------------------
- (NSMutableDictionary*)createDictJP:(NSString*)filePath
{

	NSArray *cols;
	NSString *strSep = @"\t";

	NSMutableDictionary *dict = [[[NSMutableDictionary alloc] init] autorelease];

	NSString *romId;
	NSString *titlejp;
	NSString *lineString;
	int procCount = 0;

	// streaming reading
	#define STR_MAX  1024
	FILE *fp;
	char cStr[STR_MAX];
	const  char  *cFileName;
	int last;

	cFileName = [[NSFileManager defaultManager] fileSystemRepresentationWithPath:[filePath stringByExpandingTildeInPath]];

	// file open
	if ((fp = fopen(cFileName, "r")) == NULL) {
		return dict;
	}
	
	// read file
	while (fgets(cStr, STR_MAX, fp) != NULL) {
	
		// trancate last "\n"
		last = strlen(cStr) - 1;
		if(cStr[last] == '\n'){
			cStr[last] = '\0';
		}

		// cStr -> lineString
		lineString = [NSString stringWithCString:cStr];

		if([lineString isEqualToString:@""]){
			continue;
		}

		cols = [lineString componentsSeparatedByString:strSep];
		if([cols count] < 2){
			continue;
		}

		romId = [cols objectAtIndex:0];
		titlejp = [cols objectAtIndex:1];

		// replace character
//		titlejp = [self replaceCharacter:titlejp str1:@"&" str2:@"&"];
//		titlejp = [self replaceCharacter:titlejp str1:@"<" str2:@"<"];
//		titlejp = [self replaceCharacter:titlejp str1:@">" str2:@">"];

		// skip
		if([romId isEqualToString:@""] || [titlejp isEqualToString:@""]){
			continue;
		}

		// set dict
		[dict setObject:[NSDictionary dictionaryWithObjectsAndKeys:
									romId, @"romId",
									titlejp, @"titlejp",
									nil
						]
				forKey:romId
		];

		procCount++;
//		if(procCount % 100 == 0){
//			[viewProcStatus updateTargetName:titlejp];
//		}

	}

	// close file
	fclose(fp);

	return dict;
}


//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
//------------------------------------
// logString
//------------------------------------
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
@end
