#import "SetupPlaylist.h"
#import "TBArrayController.h"
#import "ViewLog.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation SetupPlaylist

//------------------------------------
// awake
//------------------------------------
- (void)awakeFromNib
{
	countImportSearches_ = 0;
	countImportPlaylists_ = 0;
	countImportRoms_ = 0;
	countErrorRoms_ = 0;

	[self setLogString:@""];

}
//------------------------------------
// importPlaylist
//------------------------------------
- (IBAction)importPlaylist:(id)sender;
{

	int result;

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:@""
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	NSString *filePath = [filePaths objectAtIndex:0];

	// create data
	NSError *error = nil;
	NSData *xmlData = [[[NSData alloc] initWithContentsOfFile:filePath options:nil error:&error] autorelease];

	// error
	if(xmlData == nil || error != nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not read data from file."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[error description]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}


	// create string
	NSString *xmlString = [[[NSString alloc] initWithData:xmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"xmlString=%@", xmlString);
	if(xmlString == nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		[self displayMessage:@"alert"
						messageText:@"Can not create XML from file."
						infoText:@"Data is invalid."
						btnList:@"Cancel"
		];
		return;
	}

	//
	// import playlist
	//
	if([self importPlaylistWithString:xmlString] == NO){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not import playlist."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[self logString]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// log
	NSString *headString = [self createImportHeadString];
	NSString *resultString = [self logString];
	[self setLogString:headString];
	[self appendLogString:resultString];

	// success
	result = [self displayMessage:@"info"
					messageText:@"Imported playlist."
					infoText:@"Please check import log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Import Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setFileName:@"ImportResult_Playlist.txt"];
		[viewLog setCanSave:YES];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

}

//------------------------------------
// exportPlaylist
//------------------------------------
- (IBAction)exportPlaylist:(id)sender
{
	int result = [self displayMessage:@"alert"
						messageText:@"Export all playlists?"
						infoText:@""
						btnList:@"Export,Cancel"
				];
	// cancel
	if(result == NSAlertSecondButtonReturn){
		return;
	}

//	NSArray *items = [tbArrayController getAllObjects:@"playlist"];

	[self exportPlaylistWithItem:nil title:@"All_Playlist" rootFolder:YES];

}
//------------------------------------
// importPlaylistWithString
//------------------------------------
- (BOOL)importPlaylistWithString:(NSString*)xmlString
{

	// clear counts
	countImportSearches_ = 0;
	countImportPlaylists_ = 0;
	countImportRoms_ = 0;
	countErrorRoms_ = 0;

	// clear log
	[self setLogString:@""];

	// create xml document
	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xmlString options:nil error:&error] autorelease];
	if(error != nil){
		[self appendLogString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		[self appendLogString:[NSString stringWithFormat: @"xmlString=[%@]\n", xmlString]];
		return NO;
	}

	// parse xml document
	//
	// <root>
	//
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);

	// null or not match
	if(rootNode == nil || ![[rootNode name] isEqualToString:@"root"]){
		[self appendLogString:@"error=[<root> is null]\n"];
		return NO;
	}

	//
	// <datainfo>
	//
	NSXMLNode *dataInfoNode = [self getChildNode:rootNode name:@"datainfo"];
	// null
	if(dataInfoNode == nil){
		[self appendLogString:@"error=[<datainfo> is null]\n"];
		return NO;
	}

	NSString *appname = [[self getChildNode:dataInfoNode name:@"appname"] stringValue];
	NSString *datatype = [[self getChildNode:dataInfoNode name:@"datatype"] stringValue];
	NSString *title = [[self getChildNode:dataInfoNode name:@"title"] stringValue];
	BOOL rootFolder = [[[self getChildNode:dataInfoNode name:@"rootfolder"] stringValue] intValue];
//	NSString *appversion = [[self getChildNode:dataInfoNode name:@"appversion"] stringValue];
//	NSString *date = [[self getChildNode:dataInfoNode name:@"date"] stringValue];

	// null or not match
	if(appname == nil || ![appname isEqualToString:[self defaultAppName]]){
		[self appendLogString:@"error=[<appname> is invalid value]\n"];
		return NO;
	}
	// null or not match
	if(datatype == nil || ![datatype isEqualToString:@"playlist"]){
		[self appendLogString:@"error=[<datatype> is invalid value]\n"];
		return NO;
	}

	//
	// <entries>
	//
	NSXMLNode *entriesNode = [self getChildNode:rootNode name:@"entries"];
	// null
	if(entriesNode == nil){
		[self appendLogString:@"error=[<entries> is null]\n"];
		return NO;
	}


	id parentItem = nil;
	id childItem = nil;

	//
	// create folder
	//
	if(rootFolder == YES){

		parentItem = [tbArrayController insertPlaylist:0
												itemType:ITEM_FOLDER
												itemSubType:0
												title:title
												value:@""
												labelColorNo:0
												isFolder:YES
												parentItem:nil
						];
		// add item to parent item
		[tbArrayController addItemToParent:nil item:parentItem maxNum:YES];

	}

	//
	// create playlist & romlist
	//
	if([self createPlaylistWithNode:entriesNode
								parentItem:parentItem
								childItem:&childItem
								indLevel:0
								index:0
		] == NO
	){
		return NO;
	}


	[playlistTreeController rearrangeObjects];	

	// select folder
	if(rootFolder == YES){
		[olvPlaylist selectItem:parentItem];
//		[olvPlaylist editSelectedItem:nil];
	}
	// select item
	else{
		if(childItem != nil){
			[olvPlaylist selectItem:childItem];
		}
	}

	return YES;

}
//------------------------------------
// createPlaylistWithNode
//------------------------------------
- (BOOL)createPlaylistWithNode:(NSXMLNode*)node
					parentItem:(id)parentItem
					childItem:(id*)childItem
					indLevel:(int)indLevel
					index:(int)index
{

	BOOL isSuccess = NO;

	NSString *title;
	NSString *value;
	NSString *entryType;
	NSString *entrySubType;
	NSString *uniqueId;
	id plistItem = nil;
	int itemType;
	int itemSubType;

	BOOL isFolder;
	int labelColorNo;

	NSString *description;

	// create indent
	NSString *indentString = [self createIndentString:indLevel];
	NSString *treeString;

	NSXMLNode *entryNode;
	NSEnumerator *enumNodes = [[node children] objectEnumerator];
	while (entryNode = [enumNodes nextObject]){

		//
		// <entry>
		//
		if ([[entryNode name] isEqualToString:@"entry"]){

			entryType = [[(NSXMLElement*)entryNode attributeForName:@"type"] stringValue];
			entrySubType = [[(NSXMLElement*)entryNode attributeForName:@"subtype"] stringValue];

			title = [[self getChildNode:entryNode name:@"title"] stringValue];
			value = [[self getChildNode:entryNode name:@"value"] stringValue];
			labelColorNo = [[[self getChildNode:entryNode name:@"labelcolorno"] stringValue] intValue];

			description = @"";
			treeString = @"-";

			isFolder = NO;
			itemType = 0;
			itemSubType = 0;

			// folder
			if([entryType isEqualToString:@"folder"]){
				itemType = ITEM_FOLDER;
				isFolder = YES;
				treeString = @"+";
			}
			// playlist
			else if([entryType isEqualToString:@"playlist"]){
				itemType = ITEM_PLAYLIST;
				isFolder = NO;
				treeString = @"+";
				countImportPlaylists_++;
			}
			// search
			else if([entryType isEqualToString:@"search"]){
				itemType = ITEM_SEARCH;
				isFolder = NO;

				// keyword
				if([entrySubType isEqualToString:@"keyword"]){
					itemSubType = ITEM_SEARCH_KEYWORD;
				}

				treeString = @"+";
				countImportSearches_++;
			}
			// unknown
			else{
				continue;
			}

			// description
			if([entryType isEqualToString:@"search"]){
				if([entrySubType isEqualToString:@"keyword"]){
					if(![value isEqualToString:@""]){
						description = [NSString stringWithFormat:@"%@ : %@", entryType, value];
					}else{
						description = [NSString stringWithFormat:@"%@", entryType];
					}
				}else{
					description = [NSString stringWithFormat:@"%@", entryType];
				}
			}else{
				description = [NSString stringWithFormat:@"%@", entryType];
			}

			// insert playlist
//			index++;
			plistItem = [tbArrayController insertPlaylist:0
													itemType:itemType
													itemSubType:itemSubType
													title:title
													value:value
													labelColorNo:labelColorNo
													isFolder:isFolder
													parentItem:nil
							];

			// add item to parent item
			[tbArrayController addItemToParent:parentItem item:plistItem maxNum:YES];			

			uniqueId = [plistItem valueForKey:@"uniqueId"];

			// append log
			[self appendLogString:[self createPlistLogString:indentString
												treeString:treeString
												title:title
												description:description
									]
			];

			isSuccess = YES;

			//
			// create child
			//
			if([entryType isEqualToString:@"folder"]){

				//
				// <entries>
				//
				NSXMLNode *entriesNode = [self getChildNode:entryNode name:@"entries"];

				if(entriesNode != nil){
					isSuccess = [self createPlaylistWithNode:entriesNode
													parentItem:plistItem
													childItem:childItem
													indLevel:(indLevel+1)
													index:0
								];
				}

			}

			//
			// create romlist
			//
			if([entryType isEqualToString:@"playlist"]){

				//
				// <items>
				//
				NSXMLNode *itemsNode = [self getChildNode:entryNode name:@"items"];

				if(itemsNode != nil){
					[self createItemlistWithNode:itemsNode plistItem:plistItem indLevel:(indLevel + 1)];
				}

			}

		}

	}

	// last child item of root
	if(indLevel == 0){
		*childItem = plistItem;
	}

	return isSuccess;

}
//------------------------------------
// createItemlistWithNode
//------------------------------------
- (void)createItemlistWithNode:(NSXMLNode*)node plistItem:(id)plistItem indLevel:(int)indLevel
{

	NSString *romId;
	NSXMLNode *itemNode;
	int countRomIds;
	int countItems;
	int countErrors;

	NSString *indentString = [self createIndentString:indLevel];
//	NSString *treeString = @"-";

	// add items
	NSMutableArray *romIds = [[[NSMutableArray alloc] init] autorelease];

	NSEnumerator *enumNodes = [[node children] objectEnumerator];
	while (itemNode = [enumNodes nextObject]){

		//
		// <item>
		//
		if ([[itemNode name] isEqualToString:@"item"]){
			romId = [[self getChildNode:itemNode name:@"romid"] stringValue];
			[romIds addObject:romId];
		}
	}
	countRomIds = [romIds count];
	if(countRomIds <= 0){
		return;
	}

	// fetch data
	NSPredicate *pred = nil;
	pred = [NSPredicate predicateWithFormat:@"romId IN %@", romIds];
	NSArray *items = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];

	countItems = [items count];
	countImportRoms_ += countItems;
	if(countItems > 0){
		// add relation
		[tbArrayController addItemsToPlaylist:plistItem items:items];
	}

	// log
/*
	NSEnumerator *enumItems = [items objectEnumerator];
	id record;
	while (record = [enumItems nextObject]){

		// append log
		[self appendLogString:[self createItemLogString:indentString
											treeString:treeString
											romId:[record valueForKey:@"romId"]
											title:[record valueForKey:@"title"]
								]
		];
	}
*/

	// append summary log
	[self appendLogString:
			[NSString stringWithFormat:@"%@%d / %d Roms\n"
												, indentString
												, countItems
												, countRomIds
			]
	];

	// append error log
	countErrors = countRomIds - countItems;
	countErrorRoms_ += countErrors;
	if(countErrors > 0){
		[self appendLogString:
				[NSString stringWithFormat:@"%@Error : Not Found in Parent Roms = %d\n"
													, indentString
													, countErrors
				]
		];

		// from romlist
		NSArray *itemRomIds = [items valueForKey:@"romId"];
		// from xml
		NSEnumerator *enumRomIds = [romIds objectEnumerator];

		while (romId = [enumRomIds nextObject]){

			// not contains
			if([itemRomIds containsObject:romId] == NO){
				[self appendLogString:
						[NSString stringWithFormat:@"%@[%@]\n"
														, indentString
														, romId
						]
				];
			}
		}
	}
}
//------------------------------------
// exportPlaylistWithItem
//------------------------------------
- (BOOL)exportPlaylistWithItem:(NSManagedObject*)item title:(NSString*)title rootFolder:(BOOL)rootFolder
{

	NSMutableArray *childItems = [[[NSMutableArray alloc] init] autorelease];

	// add self item
	if(item != nil){
		[childItems addObject:item];
	}
	// add child items
	else{
		[childItems addObjectsFromArray:[tbArrayController getChildObjects:item]];
	}

	// export playlist
	return [self exportPlaylistWithItems:childItems title:title rootFolder:rootFolder];

}

//------------------------------------
// exportPlaylistWithItems
//------------------------------------
- (BOOL)exportPlaylistWithItems:(NSArray*)items title:(NSString*)title rootFolder:(BOOL)rootFolder
{

	int result;

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	//
	// create xml
	//
	BOOL isError = NO;
	NSXMLDocument *document = [self createXMLDocumentWithItems:items
														title:title
														rootFolder:rootFolder
														isError:&isError
							];

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// error
	if(isError == YES){

		[self displayMessage:@"alert"
						messageText:@"Can not export playlist."
						infoText:@"No playlist for exporting."
						btnList:@"Cancel"
		];
		return NO;
	}

/*
	// create fileName
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y%m%d_%H%M%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];
*/
	// confirm save
	NSString *fileName = [NSString stringWithFormat:@"%@_%@.xml", [self defaultAppName], title];

	NSString *filePath = [self selectFilePathWithSavePanel:fileName
												canCreateDir:YES
												defaultPath:@""
						];

	if([filePath isEqualToString:@""]){
		return NO;
	}

	NSString *xmlString= [document XMLStringWithOptions:NSXMLNodePrettyPrint];
//	NSString* xmlString= [document XMLStringWithOptions:NSXMLNodeCompactEmptyElement];
//	NSLog(@"xmlString=%@", xmlString);

	// write data
//	NSData *xmlData = [[[NSData alloc] init] autorelease]; 
//	xmlData = [NSData dataWithBytes:[xmlString cString] length:[xmlString cStringLength]];
//	const NSStringEncoding *encoding = [NSString availableStringEncodings];
	NSData *xmlData = [xmlString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES]; 

	// writeToFile
	if([xmlData writeToFile:[filePath stringByExpandingTildeInPath] atomically:YES] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not export playlist."
						infoText:@"Failed write to file."
						btnList:@"Cancel"
		];
		return NO;
	}

	// success
	result = [self displayMessage:@"info"
					messageText:@"Exported playlist."
					infoText:@"Please check export log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Export Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

	return YES;

}
//------------------------------------
// createXMLDocumentWithItems
//------------------------------------
- (NSXMLDocument*)createXMLDocumentWithItems:(NSArray*)items
										title:(NSString*)title
										rootFolder:(BOOL)rootFolder
										isError:(BOOL*)isError
{

	NSXMLDocument *document = [NSXMLNode document];
	[document setVersion:@"1.0"];

//	NSXMLDTD *dtd= [[[NSXMLDTD alloc] init] autorelease];
//	[dtd setPublicID:@"__URL__"];
//	[document setDTD:dtd];
	[document setCharacterEncoding:@"UTF-8"];
//	[document addChild:[NSXMLNode commentWithStringValue:@"__Comment_String__"]];

	// <root>
	NSXMLElement *root = [NSXMLNode elementWithName:@"root"];
	[document setRootElement:root];

	// <datainfo>
	NSXMLElement *elemDataInfo = [self createDataInfoElement:title
													rootFolder:rootFolder
													dataType:@"playlist"
								];
	[root addChild:elemDataInfo];

	// clear log
	[self setLogString:@""];

	// <entry>
	NSXMLElement *elemEntry = [self createEntryElementWithItems:items indLevel:0 isError:isError];
	[root addChild:elemEntry];

	return document;

}
//------------------------------------
// createDataInfoElement
//------------------------------------
- (NSXMLElement*)createDataInfoElement:(NSString*)title
								rootFolder:(BOOL)rootFolder
								dataType:(NSString*)dataType
{

	// <datainfo>
	NSXMLElement *element = [NSXMLNode elementWithName:@"datainfo"];

	// <appname>
	NSXMLElement *elemAppname = [NSXMLNode elementWithName:@"appname"];
	[elemAppname setObjectValue:[self defaultAppName]];
	[element addChild:elemAppname];

	// <appver>
	NSXMLElement *elemAppver = [NSXMLNode elementWithName:@"appversion"];
	[elemAppver setObjectValue:[self defaultAppVersion]];
	[element addChild:elemAppver];

	// <title>
	NSXMLElement *elemTitle = [NSXMLNode elementWithName:@"title"];
	[elemTitle setObjectValue:title];
	[element addChild:elemTitle];

	// <rootfolder>
	NSXMLElement *elemRootFolder = [NSXMLNode elementWithName:@"rootfolder"];
	[elemRootFolder setObjectValue:[NSNumber numberWithBool:rootFolder]];
	[element addChild:elemRootFolder];

	// <datatype>
	NSXMLElement *elemDataType = [NSXMLNode elementWithName:@"datatype"];
	[elemDataType setObjectValue:dataType];
	[element addChild:elemDataType];

	// <date>
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y/%m/%d %H:%M:%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];

	NSXMLElement *elemDate = [NSXMLNode elementWithName:@"date"];
	[elemDate setObjectValue:dateStr];
	[element addChild:elemDate];

	return element;

}
//------------------------------------
// createEntryElementWithItems
//------------------------------------
- (NSXMLElement*)createEntryElementWithItems:(NSArray*)items indLevel:(int)indLevel isError:(BOOL*)isError
{

	// <entries>
	NSXMLElement *element = [NSXMLNode elementWithName:@"entries"];

	BOOL isSuccess = NO;
	BOOL isFolder;
	NSString *uniqueId;
	NSString *title;
	NSString *value;
	NSString *entryType;
	NSString *entrySubType;
	NSString *description;
	int itemType;
	int itemSubType;
	int labelColorNo;

	// create indent
	NSString *indentString = [self createIndentString:indLevel];
	NSString *treeString;

	// sort by index
	NSSortDescriptor *sortDesc1 = [[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES];
	NSArray *sortedItems = [items sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDesc1]];
	[sortDesc1 release];

	id record;

	NSEnumerator *enumItems = [sortedItems objectEnumerator];
	while (record = [enumItems nextObject]) {

		uniqueId = [record valueForKey:@"uniqueId"];
		title = [record valueForKey:@"title"];
		value = [record valueForKey:@"value"];
		isFolder = [[record valueForKey:@"isFolder"] boolValue];
		itemType = [[record valueForKey:@"itemType"] intValue];
		itemSubType = [[record valueForKey:@"itemSubType"] intValue];
		labelColorNo = [[record valueForKey:@"labelColorNo"] intValue];

		if(!value){value = @"";}
		if(!itemType){itemType = 0;}
		if(!itemSubType){itemSubType = 0;}

		entryType = @"";
		entrySubType = @"";
		description = @"";
		treeString = @"-";

		// folder
		if(isFolder == YES){
			entryType = @"folder";
			description = [NSString stringWithFormat:@"%@", entryType];
			treeString = @"+";
		}
		else{
			// playlist
			if(itemType == ITEM_PLAYLIST){
				entryType = @"playlist";
				entrySubType = @"";
			}
			// search
			else if(itemType == ITEM_SEARCH){
				entryType = @"search";

				// keyword
				if(itemSubType == ITEM_SEARCH_KEYWORD){
					entrySubType = @"keyword";
				}
			}
			// unknown
			else{
				entryType = @"playlist";
			}

			// description
			if([entryType isEqualToString:@"search"]){
				if([entrySubType isEqualToString:@"keyword"]){
					if(![value isEqualToString:@""]){
						description = [NSString stringWithFormat:@"%@ : %@", entryType, value];
					}else{
						description = [NSString stringWithFormat:@"%@", entryType];
					}
				}else{
					description = [NSString stringWithFormat:@"%@", entryType];
				}
			}else{
				description = [NSString stringWithFormat:@"%@", entryType];
			}

			treeString = @"+";
		}

		// <entry>
		NSXMLElement *elemEntry= [NSXMLNode elementWithName:@"entry"];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"type" stringValue:entryType]];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"subtype" stringValue:entrySubType]];
		[element addChild:elemEntry];

		// <title>
		NSXMLElement *elemTitle= [NSXMLNode elementWithName:@"title"];
		[elemTitle setObjectValue:title];
		[elemEntry addChild:elemTitle];

		// <value>
		NSXMLElement *elemValue= [NSXMLNode elementWithName:@"value"];
		[elemValue setObjectValue:value];
		[elemEntry addChild:elemValue];

		// <labelcolorno>
		NSXMLElement *elemColorNo= [NSXMLNode elementWithName:@"labelcolorno"];
		[elemColorNo setObjectValue:[NSString stringWithFormat:@"%d", labelColorNo]];
		[elemEntry addChild:elemColorNo];

		// append log
		[self appendLogString:[self createPlistLogString:indentString
											treeString:treeString
											title:title
											description:description
								]
		];

		// add child entries
		if(isFolder == YES && [record valueForKey:@"children"] != nil){
			NSArray *childItems = [[record valueForKey:@"children"] allObjects];
			NSXMLElement *elemChildEntries = [self createEntryElementWithItems:childItems indLevel:(indLevel + 1) isError:isError];
			[elemEntry addChild:elemChildEntries];
		}

		// add romlist
		if(isFolder == NO && itemType == ITEM_PLAYLIST){
			NSXMLElement *elemItems = [self createItemElementWithPlistId:uniqueId indLevel:(indLevel + 1)];
			[elemEntry addChild:elemItems];
		}

		// data is found
		isSuccess = YES;
	}

	*isError = !isSuccess;

	return element;

}

//------------------------------------
// createItemElementWithPlistId
//------------------------------------
- (NSXMLElement*)createItemElementWithPlistId:(NSString*)uniqueId indLevel:(int)indLevel
{

	// get romlist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"ANY rlt_playlist.uniqueId == %@", uniqueId];
	NSArray *items = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];

	// sort by romId
	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"romId" ascending:YES];
	NSArray *sortedItems = [items sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	[sortDescriptor release];

	// add item element
	return [self createItemElementWithItems:sortedItems indLevel:indLevel];

}
//------------------------------------
// createItemElementWithItems
//------------------------------------
- (NSXMLElement*)createItemElementWithItems:(NSArray*)items indLevel:(int)indLevel
{

	// <items>
	NSXMLElement *element = [NSXMLNode elementWithName:@"items"];

	NSString *romId;
//	NSString *title;
	id record;

	// create indent
	NSString *indentString = [self createIndentString:indLevel];
//	NSString *treeString = @"-";

	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:@"romId"];
//		title = [record valueForKey:@"title"];

		// <item>
		NSXMLElement *elemItem = [NSXMLNode elementWithName:@"item"];
		[element addChild:elemItem];

		// <romid>
		NSXMLElement *elemRomId = [NSXMLNode elementWithName:@"romid"];
		[elemRomId setObjectValue:romId];
		[elemItem addChild:elemRomId];

//		NSLog(@"romId=%@", romId);

/*
		// <title>
		NSXMLElement *elemTitle = [NSXMLNode elementWithName:@"title"];
		[elemTitle setObjectValue:title];
		[elemItem addChild:elemTitle];
*/
/*
		// append log
		[self appendLogString:[self createItemLogString:indentString
											treeString:treeString
											romId:romId
											title:title
								]
		];
*/
	}

	// append summary log
	[self appendLogString:
			[NSString stringWithFormat:@"%@%d Roms\n"
												, indentString
												, [items count]
			]
	];

	return element;
}
//------------------------------------
// createIndentString
//------------------------------------
- (NSString*)createIndentString:(int)indLevel
{
	NSString *indentString = @"";
	int i;
	for(i = 0; i < indLevel; i++){
		indentString = [indentString stringByAppendingString:@"\t"];
	}
	return indentString;
}
//------------------------------------
// createPlistLogString
//------------------------------------
- (NSString*)createPlistLogString:(NSString*)indentString
							treeString:(NSString*)treeString
							title:(NSString*)title
							description:(NSString*)description
{
	return [NSString stringWithFormat:@"%@%@ %@ [ %@ ]\n"
													, indentString
													, treeString
													, title
													, description
			];
}
//------------------------------------
// createItemLogString
//------------------------------------
- (NSString*)createItemLogString:(NSString*)indentString
							treeString:(NSString*)treeString
							romId:(NSString*)romId
							title:(NSString*)title
{
	return [NSString stringWithFormat:@"%@[%@]\t%@"
												, indentString
												, romId
												, title
			];
}
//------------------------------------
// createImportHeadString
//------------------------------------
- (NSString *)createImportHeadString
{

	NSString *string = @"";

	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:@"Importing Summary\n"];
	string = [string stringByAppendingString:@"Search:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Import = %d\n", countImportSearches_]];
	string = [string stringByAppendingString:@"Playlist:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Import = %d\n", countImportPlaylists_]];
	string = [string stringByAppendingString:@"Roms:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Import = %d\n", countImportRoms_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Error  = %d\n", countErrorRoms_]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	return string;
}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[logString_ release];

    [super dealloc];
}

@end
