#import "SetupRomSetting.h"
#import "TBArrayController.h"
#import "ViewLog.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation SetupRomSetting

//------------------------------------
// awake
//------------------------------------
- (void)awakeFromNib
{
	countImportRoms_ = 0;
	countErrorRoms_ = 0;
	countWarningRoms_ = 0;

	[self setLogString:@""];

}
//------------------------------------
// importRomSetting
//------------------------------------
- (IBAction)importRomSetting:(id)sender;
{

	int result;

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:@""
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	NSString *filePath = [filePaths objectAtIndex:0];

	// create data
	NSError *error = nil;
	NSData *xmlData = [[[NSData alloc] initWithContentsOfFile:filePath options:nil error:&error] autorelease];

	// error
	if(xmlData == nil || error != nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not read data from file."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[error description]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}


	// create string
	NSString *xmlString = [[[NSString alloc] initWithData:xmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"xmlString=%@", xmlString);
	if(xmlString == nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		[self displayMessage:@"alert"
						messageText:@"Can not create XML from file."
						infoText:@"Data is invalid."
						btnList:@"Cancel"
		];
		return;
	}

	//
	// import romSetting
	//
	if([self importRomSettingWithString:xmlString] == NO){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not import rom settings."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[self logString]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// log
	NSString *headString = [self createImportHeadString];
	NSString *resultString = [self logString];
	[self setLogString:headString];
	[self appendLogString:resultString];

	// success
	result = [self displayMessage:@"info"
					messageText:@"Imported rom settings."
					infoText:@"Please check import log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Import Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setFileName:@"ImportResult_RomSetting.txt"];
		[viewLog setCanSave:YES];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

}

//------------------------------------
// exportRomSetting
//------------------------------------
- (IBAction)exportRomSetting:(id)sender
{
	int result = [self displayMessage:@"alert"
						messageText:@"Export all roms settings?"
						infoText:@"Export default set, label color, tag."
						btnList:@"Export,Cancel"
				];
	// cancel
	if(result == NSAlertSecondButtonReturn){
		return;
	}

	// get romlist
	NSArray *items = [self getExportTargetItems:[tbArrayController getAllObjects:@"romlist"]];
	
	[self exportRomSettingWithItems:items title:@"All_RomSetting" rootFolder:NO];

}
//------------------------------------
// getExportTargetItems
//------------------------------------
- (NSArray*)getExportTargetItems:(NSArray*)items
{

	NSString *romId;
	NSString *defaultRomId;
	NSString *comment;
	int labelColorNo;

	id record;

	NSMutableArray *outItems = [[[NSMutableArray alloc] init] autorelease];

 	NSEnumerator *enumItems = [items objectEnumerator];
    while (record = [enumItems nextObject]) {

		romId = [record valueForKey:@"romId"];
		defaultRomId = [record valueForKey:@"defaultRomId"];
		comment = [record valueForKey:@"comment"];
		labelColorNo = [[record valueForKey:@"labelColorNo"] intValue];
		if(!comment){
			comment = @"";
		}
		if(!labelColorNo){
			labelColorNo = 0;
		}
		// no change
		if(([defaultRomId isEqualToString:@""] || [romId isEqualToString:defaultRomId]) &&
			[comment isEqualToString:@""] &&
			labelColorNo == 0
		){
			continue;
		}

		[outItems addObject:record];

	}

	return outItems;

}

//------------------------------------
// importRomSettingWithString
//------------------------------------
- (BOOL)importRomSettingWithString:(NSString*)xmlString
{

	// clear counts
	countImportRoms_ = 0;
	countErrorRoms_ = 0;
	countWarningRoms_ = 0;

	// clear log
	[self setLogString:@""];

	// create xml document
	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xmlString options:nil error:&error] autorelease];
	if(error != nil){
		[self appendLogString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		[self appendLogString:[NSString stringWithFormat: @"xmlString=[%@]\n", xmlString]];
		return NO;
	}

	// parse xml document
	//
	// <root>
	//
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);

	// null or not match
	if(rootNode == nil || ![[rootNode name] isEqualToString:@"root"]){
		[self appendLogString:@"error=[<root> is null]\n"];
		return NO;
	}

	//
	// <datainfo>
	//
	NSXMLNode *dataInfoNode = [self getChildNode:rootNode name:@"datainfo"];
	// null
	if(dataInfoNode == nil){
		[self appendLogString:@"error=[<datainfo> is null]\n"];
		return NO;
	}

	NSString *appname = [[self getChildNode:dataInfoNode name:@"appname"] stringValue];
	NSString *datatype = [[self getChildNode:dataInfoNode name:@"datatype"] stringValue];
//	NSString *title = [[self getChildNode:dataInfoNode name:@"title"] stringValue];
//	BOOL rootFolder = [[[self getChildNode:dataInfoNode name:@"rootfolder"] stringValue] intValue];
//	NSString *appversion = [[self getChildNode:dataInfoNode name:@"appversion"] stringValue];
//	NSString *date = [[self getChildNode:dataInfoNode name:@"date"] stringValue];

	// null or not match
	if(appname == nil || ![appname isEqualToString:[self defaultAppName]]){
		[self appendLogString:@"error=[<appname> is invalid value]\n"];
		return NO;
	}
	// null or not match
	if(datatype == nil || ![datatype isEqualToString:@"romSetting"]){
		[self appendLogString:@"error=[<datatype> is invalid value]\n"];
		return NO;
	}

	//
	// <entries>
	//
	NSXMLNode *entriesNode = [self getChildNode:rootNode name:@"entries"];
	// null
	if(entriesNode == nil){
		[self appendLogString:@"error=[<entries> is null]\n"];
		return NO;
	}

	//
	// create romSetting
	//
	if([self createRomSettingWithNode:entriesNode] == NO){
		[self appendLogString:@"No roms data for updating.\n"];
		[self appendLogString:@"Please check file.\n"];
		return NO;
	}

	// reload
	[romlistArrayController rearrangeObjects];	

	return YES;

}
//------------------------------------
// createRomSettingWithNode
//------------------------------------
- (BOOL)createRomSettingWithNode:(NSXMLNode*)node
{

	BOOL isSuccess = NO;
	BOOL isUpdated = NO;

	NSString *romId;
	NSString *defaultRomId;
	NSString *title;
	NSString *comment;
	NSString *errorString;
	int labelColorNo;

	NSString *entryType;
	NSString *entrySubType;

	NSMutableDictionary *dictRoms = [self createDictRoms];
	NSMutableDictionary *dictClones = [self createDictClones];
	
	NSXMLNode *entryNode;
	NSEnumerator *enumNodes = [[node children] objectEnumerator];
	while (entryNode = [enumNodes nextObject]){

		//
		// <entry>
		//
		if ([[entryNode name] isEqualToString:@"entry"]){

			entryType = [[(NSXMLElement*)entryNode attributeForName:@"type"] stringValue];
			entrySubType = [[(NSXMLElement*)entryNode attributeForName:@"subtype"] stringValue];

			title = [[self getChildNode:entryNode name:@"title"] stringValue];
			romId = [[self getChildNode:entryNode name:@"romid"] stringValue];
			defaultRomId = [[self getChildNode:entryNode name:@"defaultromid"] stringValue];
			labelColorNo = [[[self getChildNode:entryNode name:@"labelcolorno"] stringValue] intValue];
			comment = [[self getChildNode:entryNode name:@"comment"] stringValue];
			errorString = @"";

			// rom
			if([entryType isEqualToString:@"rom"]){
				// ok
			}
			// unknown
			else{
				continue;
			}

			// no romId
			if([romId isEqualToString:@""]){
				continue;
			}

			// update rom setting
			isUpdated = [self updateRomSetting:dictRoms
								dictClones:dictClones
								romId:romId
								defaultRomId:defaultRomId
								labelColorNo:labelColorNo
								comment:comment
								errorString:&errorString
						];

			// append log
			[self appendLogString:[self createLogString:title
													romId:romId
													defaultRomId:defaultRomId
													labelColorNo:labelColorNo
													comment:comment
													errorString:errorString
									]
			];

			if(isUpdated == NO){
				continue;
			}

			isSuccess = YES;

		}

	}

	return isSuccess;

}

//------------------------------------
// exportRomSettingWithItems
//------------------------------------
- (BOOL)exportRomSettingWithItems:(NSArray*)items title:(NSString*)title rootFolder:(BOOL)rootFolder
{

	int result;

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	//
	// create xml
	//
	BOOL isError = NO;
	NSXMLDocument *document = [self createXMLDocumentWithItems:items
														title:title
														rootFolder:rootFolder
														isError:&isError
							];

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// error
	if(isError == YES){

		[self displayMessage:@"alert"
						messageText:@"Can not export rom settings."
						infoText:@"No rom settings for exporting."
						btnList:@"Cancel"
		];
		return NO;
	}

/*
	// create fileName
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y%m%d_%H%M%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];
*/
	// confirm save
	NSString *fileName = [NSString stringWithFormat:@"%@_%@.xml", [self defaultAppName], title];

	NSString *filePath = [self selectFilePathWithSavePanel:fileName
												canCreateDir:YES
												defaultPath:@""
						];

	if([filePath isEqualToString:@""]){
		return NO;
	}

	NSString *xmlString= [document XMLStringWithOptions:NSXMLNodePrettyPrint];
//	NSString* xmlString= [document XMLStringWithOptions:NSXMLNodeCompactEmptyElement];
//	NSLog(@"xmlString=%@", xmlString);

	// write data
//	NSData *xmlData = [[[NSData alloc] init] autorelease]; 
//	xmlData = [NSData dataWithBytes:[xmlString cString] length:[xmlString cStringLength]];
//	const NSStringEncoding *encoding = [NSString availableStringEncodings];
	NSData *xmlData = [xmlString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES]; 

	// writeToFile
	if([xmlData writeToFile:[filePath stringByExpandingTildeInPath] atomically:YES] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not export rom settings."
						infoText:@"Failed write to file."
						btnList:@"Cancel"
		];
		return NO;
	}

	// success
	result = [self displayMessage:@"info"
					messageText:@"Exported rom settings."
					infoText:@"Please check export log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Export Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

	return YES;

}
//------------------------------------
// createXMLDocumentWithItems
//------------------------------------
- (NSXMLDocument*)createXMLDocumentWithItems:(NSArray*)items
										title:(NSString*)title
										rootFolder:(BOOL)rootFolder
										isError:(BOOL*)isError
{

	NSXMLDocument *document = [NSXMLNode document];
	[document setVersion:@"1.0"];

//	NSXMLDTD *dtd= [[[NSXMLDTD alloc] init] autorelease];
//	[dtd setPublicID:@"__URL__"];
//	[document setDTD:dtd];
	[document setCharacterEncoding:@"UTF-8"];
//	[document addChild:[NSXMLNode commentWithStringValue:@"__Comment_String__"]];

	// <root>
	NSXMLElement *root = [NSXMLNode elementWithName:@"root"];
	[document setRootElement:root];

	// <datainfo>
	NSXMLElement *elemDataInfo = [self createDataInfoElement:title
													rootFolder:rootFolder
													dataType:@"romSetting"
								];
	[root addChild:elemDataInfo];

	// clear log
	[self setLogString:@""];

	// <entry>
	NSXMLElement *elemEntry = [self createEntryElementWithItems:items isError:isError];
	[root addChild:elemEntry];

	return document;

}
//------------------------------------
// createDataInfoElement
//------------------------------------
- (NSXMLElement*)createDataInfoElement:(NSString*)title
								rootFolder:(BOOL)rootFolder
								dataType:(NSString*)dataType
{

	// <datainfo>
	NSXMLElement *element = [NSXMLNode elementWithName:@"datainfo"];

	// <appname>
	NSXMLElement *elemAppname = [NSXMLNode elementWithName:@"appname"];
	[elemAppname setObjectValue:[self defaultAppName]];
	[element addChild:elemAppname];

	// <appver>
	NSXMLElement *elemAppver = [NSXMLNode elementWithName:@"appversion"];
	[elemAppver setObjectValue:[self defaultAppVersion]];
	[element addChild:elemAppver];

	// <title>
	NSXMLElement *elemTitle = [NSXMLNode elementWithName:@"title"];
	[elemTitle setObjectValue:title];
	[element addChild:elemTitle];

	// <rootfolder>
	NSXMLElement *elemRootFolder = [NSXMLNode elementWithName:@"rootfolder"];
	[elemRootFolder setObjectValue:[NSNumber numberWithBool:rootFolder]];
	[element addChild:elemRootFolder];

	// <datatype>
	NSXMLElement *elemDataType = [NSXMLNode elementWithName:@"datatype"];
	[elemDataType setObjectValue:dataType];
	[element addChild:elemDataType];

	// <date>
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y/%m/%d %H:%M:%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];

	NSXMLElement *elemDate = [NSXMLNode elementWithName:@"date"];
	[elemDate setObjectValue:dateStr];
	[element addChild:elemDate];

	return element;

}
//------------------------------------
// createEntryElementWithItems
//------------------------------------
- (NSXMLElement*)createEntryElementWithItems:(NSArray*)items isError:(BOOL*)isError
{

	// <entries>
	NSXMLElement *element = [NSXMLNode elementWithName:@"entries"];

	BOOL isSuccess = NO;
	NSString *romId;
	NSString *defaultRomId;
	NSString *title;
	NSString *comment;
	int labelColorNo;

	NSString *entryType;
	NSString *entrySubType;

	// sort by romId
	NSSortDescriptor *sortDesc1 = [[NSSortDescriptor alloc] initWithKey:@"romId" ascending:YES];
	NSArray *sortedItems = [items sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDesc1]];
	[sortDesc1 release];

	id record;

	NSEnumerator *enumItems = [sortedItems objectEnumerator];
	while (record = [enumItems nextObject]) {

		romId = [record valueForKey:@"romId"];
		defaultRomId = [record valueForKey:@"defaultRomId"];
		title = [record valueForKey:@"title"];
		comment = [record valueForKey:@"comment"];
		labelColorNo = [[record valueForKey:@"labelColorNo"] intValue];
		if(!comment){
			comment = @"";
		}
		if(!labelColorNo){
			labelColorNo = 0;
		}

		entryType = @"rom";
		entrySubType = @"";

		// <entry>
		NSXMLElement *elemEntry= [NSXMLNode elementWithName:@"entry"];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"type" stringValue:entryType]];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"subtype" stringValue:entrySubType]];
		[element addChild:elemEntry];

		// <romId>
		NSXMLElement *elemRomId= [NSXMLNode elementWithName:@"romid"];
		[elemRomId setObjectValue:romId];
		[elemEntry addChild:elemRomId];

		// <defaultromid>
		NSXMLElement *elemDefaultRomId= [NSXMLNode elementWithName:@"defaultromid"];
		[elemDefaultRomId setObjectValue:defaultRomId];
		[elemEntry addChild:elemDefaultRomId];

		// <title>
		NSXMLElement *elemTitle= [NSXMLNode elementWithName:@"title"];
		[elemTitle setObjectValue:title];
		[elemEntry addChild:elemTitle];

		// <labelcolorno>
		NSXMLElement *elemColorNo= [NSXMLNode elementWithName:@"labelcolorno"];
		[elemColorNo setObjectValue:[NSString stringWithFormat:@"%d", labelColorNo]];
		[elemEntry addChild:elemColorNo];

		// <comment>
		NSXMLElement *elemComment= [NSXMLNode elementWithName:@"comment"];
		[elemComment setObjectValue:comment];
		[elemEntry addChild:elemComment];

		// append log
		[self appendLogString:[self createLogString:title
												romId:romId
												defaultRomId:defaultRomId
												labelColorNo:labelColorNo
												comment:comment
												errorString:@""
								]
		];

		// data is found
		isSuccess = YES;
	}

	*isError = !isSuccess;

	return element;

}
//------------------------------------
// updateRomSetting
//------------------------------------
- (BOOL)updateRomSetting:(NSMutableDictionary*)dictRoms
				dictClones:(NSMutableDictionary*)dictClones
				romId:(NSString*)romId
				defaultRomId:(NSString*)defaultRomId
				labelColorNo:(int)labelColorNo
				comment:(NSString*)comment
				errorString:(NSString**)errorString
{

	BOOL ret = NO;

	id romItem = [dictRoms valueForKey:romId];
	if(!romItem){
		// set error string
		*errorString = @"[Error] Parent set is not found. **";
		countErrorRoms_++;
		return YES;
	}

	// update default set
	if(![defaultRomId isEqualToString:@""]){
		if(![romId isEqualToString:defaultRomId]){
			id cloneItem = [dictClones valueForKey:defaultRomId];
			// clone is exists
			if(cloneItem){			
				// update column
				[romItem setValue:defaultRomId forKey:@"defaultRomId"];
				// update relation
				[romItem setValue:cloneItem forKey:@"rlt_clonelist"];
			}else{
				// clear column
				[romItem setValue:romId forKey:@"defaultRomId"];
				// clear relation
				[romItem setValue:nil forKey:@"rlt_clonelist"];

				// set error string
				*errorString = @"[Warning] Clone set is not found. Reset default rom. **";
				countWarningRoms_++;
			}
			ret = YES;
		}
	}

	// update comment
	if(![comment isEqualToString:@""]){
		[romItem setValue:comment forKey:@"comment"];
		ret = YES;
	}

	// update labelColorNo
	if(labelColorNo > 0){
		[romItem setValue:[NSNumber numberWithInt:labelColorNo] forKey:@"labelColorNo"];
		ret = YES;
	}

	if(ret == YES){
		countImportRoms_++;
	}

	return ret;
}
//------------------------------------
// createDictRoms
//------------------------------------
- (NSMutableDictionary*)createDictRoms
{

	NSMutableDictionary *dictRoms = [[[NSMutableDictionary alloc] init] autorelease];
	NSString *romId;
	id record;

	//
	//	create dict
	//
	NSArray *items = [tbArrayController getAllObjects:@"romlist"];
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]){

		romId = [record valueForKey:@"romId"];

		[dictRoms setValue:record forKey:romId];
	}

	return dictRoms;

}
//------------------------------------
// createDictClones
//------------------------------------
- (NSMutableDictionary*)createDictClones
{

	NSMutableDictionary *dictClones = [[[NSMutableDictionary alloc] init] autorelease];
	NSString *cloneId;
	id record;

	//
	//	create dict
	//
	NSArray *items = [tbArrayController getAllObjects:@"clonelist"];
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]){

		cloneId = [record valueForKey:@"cloneId"];

		[dictClones setValue:record forKey:cloneId];
	}

	return dictClones;
}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// createImportHeadString
//------------------------------------
- (NSString *)createImportHeadString
{

	NSString *string = @"";

	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];
	string = [string stringByAppendingString:@"Importing Summary\n"];
	string = [string stringByAppendingString:@"Roms:\n"];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Import  = %d\n", countImportRoms_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Error   = %d\n", countErrorRoms_]];
	string = [string stringByAppendingString:[NSString stringWithFormat: @" - Warning = %d\n", countWarningRoms_]];
	string = [string stringByAppendingString:@"--------------------------------------------------------------------\n"];

	return string;
}

//------------------------------------
// createLogString
//------------------------------------
- (NSString*)createLogString:(NSString*)title
						romId:(NSString*)romId
						defaultRomId:(NSString*)defaultRomId
						labelColorNo:(int)labelColorNo
						comment:(NSString*)comment
						errorString:(NSString*)errorString
{

	NSString *description = @"";
	NSString *logString = @"";

	// romId
	description = [description stringByAppendingFormat:@"romId : %@\n", romId];

	// defaultRomId
	if(![romId isEqualToString:defaultRomId]){
		description = [description stringByAppendingFormat:@"defaultRomId : %@\n", defaultRomId];
	}

	// labelColorNo
	if(labelColorNo > 0){
		description = [description stringByAppendingFormat:@"labelColorNo : %d\n", labelColorNo];
	}

	// comment
	if(![comment isEqualToString:@""]){
		description = [description stringByAppendingFormat:@"tag : %@\n", comment];
	}

	if(![errorString isEqualToString:@""]){
		logString = [NSString stringWithFormat:@"------------\n%@\n%@\n%@"
									, errorString
									, title
									, description
					];
	}else{
		logString = [NSString stringWithFormat:@"------------\n%@\n%@"
									, title
									, description
					];
	}

	return logString;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[logString_ release];

    [super dealloc];
}

@end
