#import "SetupRomStatus.h"
#import "TBArrayController.h"
#import "viewLog.h"
#import "ViewProcStatus.h"
#import "ViewPrefs.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation SetupRomStatus

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setLogString:@""];

}
//=======================================================================
// IBActions
//=======================================================================
//------------------------------------
// updateAllFileStatus
//------------------------------------
- (IBAction)updateAllFileStatus:(id)sender
{
	int result;

	// check folder
	NSString *filePath = [self getDefaultRomsFolderPath];
	if([self checkFileExist:filePath] != FILE_EXIST_DIR){
		result = [self displayMessageAlertFolder:@"Roms"];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return;
	}

	// confirm
	result = [self displayMessage:@"alert"
					messageText:@"Update all file status?"
					infoText:@""
					btnList:@"Update,Cancel"
			];
	if(result != NSAlertFirstButtonReturn){
		return;
	}

//	[self handleProcStatusChanged:PROC_IND_STATUS_START];
	[self setLogString:@""];

	//
	// update romlist
	//
	NSArray *romItems = [tbArrayController getAllObjects:@"romlist"];
	if([self updateFileStatusWithItems:romItems
								keyRomId:@"romId"
								isParent:YES
								isUpdateDefault:NO
								showSum:YES
								showDetail:NO
								showProc:YES
								isClose:NO
		] == NO
	){
		return;
	}

	//
	// update clonelist
	//
	NSArray *cloneItems = [tbArrayController getAllObjects:@"clonelist"];
	if([self updateFileStatusWithItems:cloneItems
								keyRomId:@"cloneId"
								isParent:NO
								isUpdateDefault:NO
								showSum:YES
								showDetail:NO
								showProc:YES
								isClose:NO
		] == NO
	){
		return;
	}

//	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// success
	result = [self displayMessage:@"info"
						messageText:@"Update process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Update Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}

	[self setLogString:@""];

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// updateFileStatus
//------------------------------------
- (BOOL)updateFileStatus:(NSArray*)items
						keyRomId:(NSString*)keyRomId
						isParent:(BOOL)isParent
						isUpdateDefault:(BOOL)isUpdateDefault
{
	int result;

	// check folder
//	NSString *filePath = [self defaultSettingFolderPath:MAME_FOLDER_TYPE_ROMS];
	NSString *filePath = [self getDefaultRomsFolderPath];
	if([self checkFileExist:filePath] != FILE_EXIST_DIR){
		result = [self displayMessageAlertFolder:@"Roms"];
		// open prefs tab
		if(result == NSAlertFirstButtonReturn){
			[viewPrefs openPrefsTabWithIdentifier:PREFS_TAB_ID_MAME];
		}
		return NO;
	}

	// over count
	if([items count] > 100){
		[self displayMessage:@"alert"
				messageText:@"Too many items have been selected."
				infoText:@"Please select less than 100 items."
				btnList:@"Cancel"
		];
		return NO;
	}

	[self setLogString:@""];

	//
	// update items
	//
	if([self updateFileStatusWithItems:items
								keyRomId:keyRomId
								isParent:isParent
								isUpdateDefault:isUpdateDefault
								showSum:NO
								showDetail:YES
								showProc:NO
								isClose:NO
		] == NO
	){
		return NO;
	}

/*
	// success
	result = [self displayMessage:@"info"
						messageText:@"Update process is completed."
						infoText:@"Please check log."
						btnList:@"OK,Log"
			];
*/
	// show log
//	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Update Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
//	}
	[self setLogString:@""];

	// post notification
	if(isParent == YES){
		[self postItemStatusDidChangedMainNotification];
	}else{
		[self postItemStatusDidChangedCloneNotification];
	}

	return YES;

}
//------------------------------------
// updateFileStatusWithItems
//------------------------------------
- (BOOL)updateFileStatusWithItems:(NSArray*)items
						keyRomId:(NSString*)keyRomId
						isParent:(BOOL)isParent
						isUpdateDefault:(BOOL)isUpdateDefault
						showSum:(BOOL)showSum
						showDetail:(BOOL)showDetail
						showProc:(BOOL)showProc
						isClose:(BOOL)isClose
{


	NSMutableDictionary *fileAttb = nil;
	NSString *filePath = @"";
	NSString *romId = @"";
	NSString *title = @"";
	int fileSize = 0;
	NSString *fileSizeStr = @"";
	NSString *fileDateStr = @"";
	BOOL isExist = NO;
	id record;

	int procCount = 0;
	int updateCount = 0;
	int totalCount = [items count];

	// show proc
	if(showProc == YES){
		NSString *procTitle = @"";
		if(isParent == YES){
			procTitle = @"Updating Parent File Status";
		}else{
			procTitle = @"Updating Clone File Status";
		}
		[viewProcStatus initProcStatus:procTitle
							targetName:@""
							procCount:procCount
							totalCount:totalCount
							showCount:NO
							isOpen:YES
		];
	}

	//
	// update records
	//
	id defaultSet;
	BOOL isClone;

	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		// change defaultSet
		if(isUpdateDefault == YES){
			defaultSet = [self getDefaultRomSet:record isClone:&isClone];
			// clone
			if(isClone == YES){
				keyRomId = @"cloneId";
			}else{
				keyRomId = @"romId";
			}
		}else{
			defaultSet = record;
		}

		romId = [defaultSet valueForKey:keyRomId];
//		title = [defaultSet valueForKey:@"title"];
		title = [self getTitleString:[self defaultLangType]
								title:[defaultSet valueForKey:@"title"]
								titlejp:[defaultSet valueForKey:@"titlejp"]
				];
		fileSize = 0;
		fileSizeStr = @"";
		fileDateStr = @"";
		filePath = @"";
		isExist = NO;

		// get file attribute
		fileAttb = [self getFileAttribute:[self createCurrentRomPath:romId]];
		if([[fileAttb valueForKey:@"fileExist"] boolValue] == YES){
			filePath = [fileAttb valueForKey:@"filePath"];
			fileSize = [[fileAttb valueForKey:@"fileSize"] intValue];
			fileSizeStr = [fileAttb valueForKey:@"fileSizeStr"];
			fileDateStr = [fileAttb valueForKey:@"fileDateStr"];
			isExist = YES;
			updateCount++;
		}

		// update column
		[defaultSet setValue:[NSNumber numberWithBool:isExist] forKey:@"isExist"];		
		[defaultSet setValue:[NSNumber numberWithInt:fileSize] forKey:@"fileSize"];		
		[defaultSet setValue:fileDateStr forKey:@"fileDate"];		

		// append detail
		if(showDetail == YES){
			[self appendLogString:@"----------------\n"];
			[self appendLogString:[NSString stringWithFormat:@"%@\nID: %@\nSize: %@ / Date: %@\nPath: %@\n"
														, title
														, romId
														, fileSizeStr
														, fileDateStr
														, filePath
									]
			];
		}

		procCount++;
		// show proc
		if(showProc == YES){
			if(procCount % 100 == 0 && title){
				[viewProcStatus updateProcStatus:procCount targetName:title];
			}
		}
	}

	// show proc
	if(showProc == YES){
		if(title){
			[viewProcStatus updateProcStatus:procCount targetName:title];
		}
		[viewProcStatus closeProcStatus:isClose];
	}

	// append summary
	if(showSum == YES){
		if(isParent == YES){
			[self appendLogString:@"Parent Roms : "];
		}else{
			[self appendLogString:@"Clone Roms : "];
		}
		[self appendLogString:[NSString stringWithFormat:@"%d / %d files exist.\n"
													, updateCount
													, totalCount
								]
		];
	}

	return YES;

}

//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
//------------------------------------
// logString
//------------------------------------
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString*)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
@end
