#import "SetupSearchlist.h"
#import "TBArrayController.h"
#import "ViewLog.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation SetupSearchlist

//------------------------------------
// awake
//------------------------------------
- (void)awakeFromNib
{
	[self setLogString:@""];

}
//------------------------------------
// importSearchlist
//------------------------------------
- (IBAction)importSearchlist:(id)sender;
{

	int result;

	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_WEB_SEARCH_LIST];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	// get app folder
	if([defaultPath isEqualToString:@""]){
		defaultPath = [self converToAppHomePath];
	}else{
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}

	// select file
	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
											isDir:NO
											isMultiSel:NO
											isPackage:NO
											isAlias:NO
											canCreateDir:NO
											defaultPath:defaultPath
						];

	// no select
	if([filePaths count] <= 0){
		return;
	}

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	NSString *filePath = [filePaths objectAtIndex:0];

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

	// create data
	NSError *error = nil;
	NSData *xmlData = [[[NSData alloc] initWithContentsOfFile:filePath options:nil error:&error] autorelease];

	// error
	if(xmlData == nil || error != nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not read data from file."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[error description]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}


	// create string
	NSString *xmlString = [[[NSString alloc] initWithData:xmlData encoding:NSUTF8StringEncoding] autorelease];
//	NSLog(@"xmlString=%@", xmlString);
	if(xmlString == nil){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		[self displayMessage:@"alert"
						messageText:@"Can not create XML from file."
						infoText:@"Data is invalid."
						btnList:@"Cancel"
		];
		return;
	}

	//
	// import searchlist
	//
	if([self importSearchlistWithString:xmlString] == NO){
		// proc stop
		[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

		result = [self displayMessage:@"alert"
						messageText:@"Can not import searchlist."
						infoText:@"Please check error log."
						btnList:@"Cancel,Log"
				];

		// show log
		if(result == NSAlertSecondButtonReturn){
			[viewLog setTitle:@"Import Log"];
			[viewLog setLogString:[self logString]];
			[viewLog setCanSave:NO];
			[viewLog openLogWindow:nil];
		}else{
			[self setLogString:@""];
		}
		return;
	}

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// success
	result = [self displayMessage:@"info"
					messageText:@"Imported searchlist."
					infoText:@"Please check import log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Import Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

}

//------------------------------------
// exportSearchlist
//------------------------------------
- (IBAction)exportSearchlist:(id)sender
{
/*
	int result = [self displayMessage:@"alert"
						messageText:@"Export all searchlists?"
						infoText:@""
						btnList:@"Export,Cancel"
				];
	// cancel
	if(result == NSAlertSecondButtonReturn){
		return;
	}
*/
	NSArray *items = [tbArrayController getAllObjects:@"searchlist"];

	[self exportSearchlistWithItems:items title:@"WebSearch" rootFolder:NO];

}
//------------------------------------
// importSearchlistWithString
//------------------------------------
- (BOOL)importSearchlistWithString:(NSString*)xmlString
{

	// clear log
	[self setLogString:@""];

	// create xml document
	NSError *error = nil;
	NSXMLDocument *xmlDoc = [[[NSXMLDocument alloc] initWithXMLString:xmlString options:nil error:&error] autorelease];
	if(error != nil){
		[self appendLogString:[NSString stringWithFormat:@"error=[%@]\n", error]];
		[self appendLogString:[NSString stringWithFormat: @"xmlString=[%@]\n", xmlString]];
		return NO;
	}

	// parse xml document
	//
	// <root>
	//
	NSXMLNode *rootNode = [xmlDoc rootElement];
//	NSLog(@"rootNode=%@", [rootNode name]);

	// null or not match
	if(rootNode == nil || ![[rootNode name] isEqualToString:@"root"]){
		[self appendLogString:@"error=[<root> is null]\n"];
		return NO;
	}

	//
	// <datainfo>
	//
	NSXMLNode *dataInfoNode = [self getChildNode:rootNode name:@"datainfo"];
	// null
	if(dataInfoNode == nil){
		[self appendLogString:@"error=[<datainfo> is null]\n"];
		return NO;
	}

	NSString *appname = [[self getChildNode:dataInfoNode name:@"appname"] stringValue];
	NSString *datatype = [[self getChildNode:dataInfoNode name:@"datatype"] stringValue];
//	NSString *title = [[self getChildNode:dataInfoNode name:@"title"] stringValue];
//	BOOL rootFolder = [[[self getChildNode:dataInfoNode name:@"rootfolder"] stringValue] intValue];
//	NSString *appversion = [[self getChildNode:dataInfoNode name:@"appversion"] stringValue];
//	NSString *date = [[self getChildNode:dataInfoNode name:@"date"] stringValue];

	// null or not match
	if(appname == nil || ![appname isEqualToString:[self defaultAppName]]){
		[self appendLogString:@"error=[<appname> is invalid value]\n"];
		return NO;
	}
	// null or not match
	if(datatype == nil || ![datatype isEqualToString:@"searchlist"]){
		[self appendLogString:@"error=[<datatype> is invalid value]\n"];
		return NO;
	}

	//
	// <entries>
	//
	NSXMLNode *entriesNode = [self getChildNode:rootNode name:@"entries"];
	// null
	if(entriesNode == nil){
		[self appendLogString:@"error=[<entries> is null]\n"];
		return NO;
	}

	//
	// create searchlist
	//

	// get max index
	int index = [tbArrayController getMaxIntValueFromEntityWithColumn:@"searchlist" column:@"index"];

	if([self createSearchlistWithNode:entriesNode
								index:index
		] == NO
	){
		return NO;
	}

	// reload
	[searchlistArrayController rearrangeObjects];	

	return YES;

}
//------------------------------------
// createSearchlistWithNode
//------------------------------------
- (BOOL)createSearchlistWithNode:(NSXMLNode*)node
							index:(int)index
{

	BOOL isSuccess = NO;

	NSString *entryType;
	NSString *entrySubType;

	NSString *title;
	NSString *url;
	int keywordType;
	BOOL isTrim;

	NSString *description;

	NSXMLNode *entryNode;
	NSEnumerator *enumNodes = [[node children] objectEnumerator];
	while (entryNode = [enumNodes nextObject]){

		//
		// <entry>
		//
		if ([[entryNode name] isEqualToString:@"entry"]){

			entryType = [[(NSXMLElement*)entryNode attributeForName:@"type"] stringValue];
			entrySubType = [[(NSXMLElement*)entryNode attributeForName:@"subtype"] stringValue];

			title = [[self getChildNode:entryNode name:@"title"] stringValue];
			url = [[self getChildNode:entryNode name:@"url"] stringValue];
			isTrim = [[[self getChildNode:entryNode name:@"istrim"] stringValue] intValue];
			keywordType = [[[self getChildNode:entryNode name:@"keywordtype"] stringValue] intValue];

			description = [NSString stringWithFormat:@"url : %@\nkeywordType : %d\ntrim : %d", url, keywordType, isTrim];

			// searchlist
			if([entryType isEqualToString:@"searchlist"]){
				// ok
			}
			// unknown
			else{
				continue;
			}

			// insert searchlist
			index++;
			[tbArrayController insertSearchlist:index
											title:title
											url:url
											keywordType:keywordType
											isTrim:isTrim
			];

			// append log
			[self appendLogString:[self createLogString:title
												description:description
									]
			];

			isSuccess = YES;

		}

	}

	return isSuccess;

}

//------------------------------------
// exportSearchlistWithItems
//------------------------------------
- (BOOL)exportSearchlistWithItems:(NSArray*)items title:(NSString*)title rootFolder:(BOOL)rootFolder
{

	int result;

	// proc start
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	//
	// create xml
	//
	BOOL isError = NO;
	NSXMLDocument *document = [self createXMLDocumentWithItems:items
														title:title
														rootFolder:rootFolder
														isError:&isError
							];

	// proc stop
	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// error
	if(isError == YES){

		[self displayMessage:@"alert"
						messageText:@"Can not export searchlist."
						infoText:@"No searchlist for exporting."
						btnList:@"Cancel"
		];
		return NO;
	}

/*
	// create fileName
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y%m%d_%H%M%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];
*/
	// get defaultPath
	NSString *defaultKey = [self getDefaultFilePathKey:MAME_FILE_TYPE_WEB_SEARCH_LIST];
	NSString *defaultPath = [self defaultStringValue:defaultKey];
	// get app folder
	if([defaultPath isEqualToString:@""]){
		defaultPath = [self converToAppHomePath];
	}else{
		defaultPath = [defaultPath stringByDeletingLastPathComponent];
	}

	NSString *fileName = [NSString stringWithFormat:@"%@_%@.xml", [self defaultAppName], title];

	// confirm save
	NSString *filePath = [self selectFilePathWithSavePanel:fileName
												canCreateDir:YES
												defaultPath:defaultPath
						];

	if([filePath isEqualToString:@""]){
		return NO;
	}

	// save defaultPath
	[self setDefaultStringValue:filePath key:defaultKey];

	NSString *xmlString= [document XMLStringWithOptions:NSXMLNodePrettyPrint];
//	NSString* xmlString= [document XMLStringWithOptions:NSXMLNodeCompactEmptyElement];
//	NSLog(@"xmlString=%@", xmlString);

	// write data
//	NSData *xmlData = [[[NSData alloc] init] autorelease]; 
//	xmlData = [NSData dataWithBytes:[xmlString cString] length:[xmlString cStringLength]];
//	const NSStringEncoding *encoding = [NSString availableStringEncodings];
	NSData *xmlData = [xmlString dataUsingEncoding:NSUTF8StringEncoding allowLossyConversion:YES]; 

	// writeToFile
	if([xmlData writeToFile:[filePath stringByExpandingTildeInPath] atomically:YES] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not export searchlist."
						infoText:@"Failed write to file."
						btnList:@"Cancel"
		];
		return NO;
	}

	// success
	result = [self displayMessage:@"info"
					messageText:@"Exported searchlist."
					infoText:@"Please check export log."
					btnList:@"OK,Log"
			];

	// show log
	if(result == NSAlertSecondButtonReturn){
		[viewLog setTitle:@"Export Log"];
		[viewLog setLogString:[self logString]];
		[viewLog setCanSave:NO];
		[viewLog openLogWindow:nil];
	}else{
		[self setLogString:@""];
	}

	return YES;

}
//------------------------------------
// createXMLDocumentWithItems
//------------------------------------
- (NSXMLDocument*)createXMLDocumentWithItems:(NSArray*)items
										title:(NSString*)title
										rootFolder:(BOOL)rootFolder
										isError:(BOOL*)isError
{

	NSXMLDocument *document = [NSXMLNode document];
	[document setVersion:@"1.0"];

//	NSXMLDTD *dtd= [[[NSXMLDTD alloc] init] autorelease];
//	[dtd setPublicID:@"__URL__"];
//	[document setDTD:dtd];
	[document setCharacterEncoding:@"UTF-8"];
//	[document addChild:[NSXMLNode commentWithStringValue:@"__Comment_String__"]];

	// <root>
	NSXMLElement *root = [NSXMLNode elementWithName:@"root"];
	[document setRootElement:root];

	// <datainfo>
	NSXMLElement *elemDataInfo = [self createDataInfoElement:title
													rootFolder:rootFolder
													dataType:@"searchlist"
								];
	[root addChild:elemDataInfo];

	// clear log
	[self setLogString:@""];

	// <entry>
	NSXMLElement *elemEntry = [self createEntryElementWithItems:items isError:isError];
	[root addChild:elemEntry];

	return document;

}
//------------------------------------
// createDataInfoElement
//------------------------------------
- (NSXMLElement*)createDataInfoElement:(NSString*)title
								rootFolder:(BOOL)rootFolder
								dataType:(NSString*)dataType
{

	// <datainfo>
	NSXMLElement *element = [NSXMLNode elementWithName:@"datainfo"];

	// <appname>
	NSXMLElement *elemAppname = [NSXMLNode elementWithName:@"appname"];
	[elemAppname setObjectValue:[self defaultAppName]];
	[element addChild:elemAppname];

	// <appver>
	NSXMLElement *elemAppver = [NSXMLNode elementWithName:@"appversion"];
	[elemAppver setObjectValue:[self defaultAppVersion]];
	[element addChild:elemAppver];

	// <title>
	NSXMLElement *elemTitle = [NSXMLNode elementWithName:@"title"];
	[elemTitle setObjectValue:title];
	[element addChild:elemTitle];

	// <rootfolder>
	NSXMLElement *elemRootFolder = [NSXMLNode elementWithName:@"rootfolder"];
	[elemRootFolder setObjectValue:[NSNumber numberWithBool:rootFolder]];
	[element addChild:elemRootFolder];

	// <datatype>
	NSXMLElement *elemDataType = [NSXMLNode elementWithName:@"datatype"];
	[elemDataType setObjectValue:dataType];
	[element addChild:elemDataType];

	// <date>
	NSDateFormatter *formatter = [[[NSDateFormatter alloc] 
									initWithDateFormat:@"%Y/%m/%d %H:%M:%S" 
									allowNaturalLanguage:FALSE
								] autorelease];
	NSString *dateStr = [formatter stringFromDate:[NSDate date]];

	NSXMLElement *elemDate = [NSXMLNode elementWithName:@"date"];
	[elemDate setObjectValue:dateStr];
	[element addChild:elemDate];

	return element;

}
//------------------------------------
// createEntryElementWithItems
//------------------------------------
- (NSXMLElement*)createEntryElementWithItems:(NSArray*)items isError:(BOOL*)isError
{

	// <entries>
	NSXMLElement *element = [NSXMLNode elementWithName:@"entries"];

	BOOL isSuccess = NO;
	NSString *title;
	NSString *url;
	int keywordType;
	BOOL isTrim;

	NSString *entryType;
	NSString *entrySubType;
	NSString *description;

	// sort by index
	NSSortDescriptor *sortDesc1 = [[NSSortDescriptor alloc] initWithKey:@"index" ascending:YES];
	NSArray *sortedItems = [items sortedArrayUsingDescriptors:[NSArray arrayWithObject:sortDesc1]];
	[sortDesc1 release];

	id record;

	NSEnumerator *enumItems = [sortedItems objectEnumerator];
	while (record = [enumItems nextObject]) {

		title = [record valueForKey:@"title"];
		url = [record valueForKey:@"url"];
		keywordType = [[record valueForKey:@"keywordType"] intValue];
		isTrim = [[record valueForKey:@"isTrim"] boolValue];

		entryType = @"searchlist";
		entrySubType = @"";
		description = [NSString stringWithFormat:@"url : %@\nkeywordType : %d\ntrim : %d", url, keywordType, isTrim];

		// <entry>
		NSXMLElement *elemEntry= [NSXMLNode elementWithName:@"entry"];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"type" stringValue:entryType]];
		[elemEntry addAttribute:[NSXMLNode attributeWithName:@"subtype" stringValue:entrySubType]];
		[element addChild:elemEntry];

		// <title>
		NSXMLElement *elemTitle= [NSXMLNode elementWithName:@"title"];
		[elemTitle setObjectValue:title];
		[elemEntry addChild:elemTitle];

		// <url>
		NSXMLElement *elemUrl= [NSXMLNode elementWithName:@"url"];
		[elemUrl setObjectValue:url];
		[elemEntry addChild:elemUrl];

		// <keywordtype>
		NSXMLElement *elemKeywordType = [NSXMLNode elementWithName:@"keywordtype"];
		[elemKeywordType setObjectValue:[NSString stringWithFormat:@"%d", keywordType]];
		[elemEntry addChild:elemKeywordType];

		// <istrim>
		NSXMLElement *elemIsTrim = [NSXMLNode elementWithName:@"istrim"];
		[elemIsTrim setObjectValue:[NSString stringWithFormat:@"%d", isTrim]];
		[elemEntry addChild:elemIsTrim];

		// append log
		[self appendLogString:[self createLogString:title
											description:description
								]
		];

		// data is found
		isSuccess = YES;
	}

	*isError = !isSuccess;

	return element;

}

//------------------------------------
// createLogString
//------------------------------------
- (NSString*)createLogString:(NSString*)title
							description:(NSString*)description
{
	return [NSString stringWithFormat:@"------------\n%@\n%@\n"
									, title
									, description
			];
}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString*)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;
}
- (NSString*)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[logString_ release];

    [super dealloc];
}

@end
