#import "TableColumnExtension.h"

@implementation NSObject(tableColumnExtension_)

//------------------------------------
// setColumnState
//------------------------------------
- (void)setColumnState:(TableColumnManager*)tcm sender:(id)sender
{
	NSString *identifier;
	int state;

	NSString *record = [sender representedObject];
	NSArray *cols = [record componentsSeparatedByString:@","];

	identifier = [cols objectAtIndex:0];
	state = [[cols objectAtIndex:1] intValue];

	// change table column
	if(state == 0){
		[tcm showColumn:identifier];
	}else{
		[tcm hideColumn:identifier];
	}
}
//------------------------------------
// showTableColumns
//------------------------------------
- (void)showTableColumns:(TableColumnManager*)tcm
				aTableView:(NSTableView*)aTableView
				defaultHideCols:(NSArray*)defaultHideCols
				key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *identifier;
	int i;
	BOOL isKey = NO;
	id record;

	// default is available
	NSArray *defaultColumns = [defaults objectForKey:key];
/*
	if(defaultColumns != nil){
		if([defaultColumns isKindOfClass:[NSArray class]] == NO){
//			NSLog(@"defaults is not array");
//			return;
		}
	}
*/
	// set dict 
	NSMutableDictionary *dictColumns = [[NSMutableDictionary alloc] init];
	if([defaultColumns isKindOfClass:[NSArray class]] == YES){
		for (i = 0; i < [defaultColumns count]; i++){
			record = [defaultColumns objectAtIndex:i];
			if(![record valueForKey:@"identifier"]){
				continue;
			}
			identifier = [record valueForKey:@"identifier"];

			// column is exists
			if([aTableView tableColumnWithIdentifier:identifier]){
				[dictColumns setObject:identifier forKey:identifier];
			}
		}
	}

	if([dictColumns count] > 0){
		isKey = YES;
	}

//	NSLog([dictColumns description]);

	// using copy because columns will decrease
	NSArray *tableColumns = [[aTableView tableColumns] copy];

	for (i = 0; i < [tableColumns count]; i++){

		identifier =[[tableColumns objectAtIndex:i] identifier];

		// switch show / hide
		if(isKey == YES){
			if([dictColumns objectForKey:identifier]){
				[tcm showColumn:identifier];
			}else{
				[tcm hideColumn:identifier];
			}
		}
		// show all
		else{
			// default hide
			if([defaultHideCols indexOfObject:identifier] != NSNotFound){
				[tcm hideColumn:identifier];
			}else{
				[tcm showColumn:identifier];
			}
		}
	}

	[tableColumns release];
	[dictColumns release];

}
//------------------------------------
// isShowColumnState
//------------------------------------
- (BOOL)isShowColumnState:(TableColumnManager*)tcm identifier:(NSString*)identifier
{
	return [tcm isShowColumn:identifier];
}
//------------------------------------
// setColumnDescriptionHeaderTitle
//------------------------------------
- (void)setColumnDescriptionHeaderTitle:(NSTableView*)aTableView langType:(int)langType
{

	NSTableColumn *column = [aTableView tableColumnWithIdentifier:@"description"];
	if(!column){
		return;
	}

	NSString *title = @"";

	if(langType == DEFAULT_LANG_TYPE_JP){
		title = @"Description (JP)";
	}else{
		title = @"Description";
	}

	[[column headerCell] setStringValue:title];

	// force redraw column
	[column setWidth:[column width] + 1];
	[column setWidth:[column width] - 1];
	
}
//------------------------------------
// setColumnDescriptionSortDescriptor
//------------------------------------
- (void)setColumnDescriptionSortDescriptor:(NSTableView*)aTableView
							aController:(NSArrayController*)aController
							langType:(int)langType
{

//	NSLog(@"oldarray = %@", [[aController sortDescriptors] description]);

	// change sort key
	NSTableColumn *column = [aTableView tableColumnWithIdentifier:@"description"];
	if(!column){
		return;
	}

	NSString *newKey = @"";
	NSString *oldKey = @"";

	if(langType == DEFAULT_LANG_TYPE_JP){
		newKey = @"titlejp";
		oldKey = @"title";
	}else{
		newKey = @"title";
		oldKey = @"titlejp";
	}

	// create new descriptors
	NSMutableArray *newDescs = [[[NSMutableArray alloc] init] autorelease];
	NSArray *oldDescs = [aController sortDescriptors];
	int i;
	NSSortDescriptor *newDesc = nil;
	NSSortDescriptor *oldDesc = nil;
	for(i = 0; i < [oldDescs count]; i++){
		oldDesc = [oldDescs objectAtIndex:i];
		// replace new descriptor
		if([[oldDesc key] isEqualToString:oldKey]){
			// add new descriptor
			newDesc = [[[NSSortDescriptor alloc] initWithKey:newKey ascending:[oldDesc ascending] selector:@selector(caseInsensitiveCompare:)] autorelease];
			[newDescs addObject:newDesc];
			// add old(en) descriptor
//			if(langType == DEFAULT_LANG_TYPE_JP){
//				[newDescs addObject:oldDesc];
//			}
		}
/*
		else if(![[oldDesc key] isEqualToString:newKey]){
			[newDescs addObject:oldDesc];
		}
*/
		else{
			[newDescs addObject:oldDesc];
		}
	}

	// not replaced , create new descriptor
	if(newDesc == nil){
		newDesc = [[[NSSortDescriptor alloc] initWithKey:newKey ascending:YES selector:@selector(caseInsensitiveCompare:)] autorelease];
	}
	// set new descriptor to column
	[column setSortDescriptorPrototype:newDesc];
	[aController setSortDescriptors:newDescs];

//	NSLog(@"newarray = %@", [[aController sortDescriptors] description]);

}
@end
