#import "ThumbsExtension.h"
#import "FileAttributeExtension.h"
#import "UserDefaultsExtension.h"

@implementation NSObject(thumbsExtension_)

//------------------------------------
// getThumbsImagePath
//------------------------------------
- (NSString*)getThumbsImagePath:(NSString*)romId
						imageType:(int)imageType
						fileIndex:(int)fileIndex
						maxIndex:(int*)maxIndex
{
	if([romId isEqualToString:@""]){
		return @"";
	}

	int folderType = [self getDefaultImageFolderType:imageType];
	NSString *folderPath = [self defaultSettingFolderPath:folderType];

	// no path
	if([folderPath isEqualToString:@""]){
		return @"";
	}

	NSString *filePath = [NSString stringWithFormat: @"%@/%@.png", folderPath, romId];

	// get from fplder
	if([self checkFileExist:filePath] != FILE_EXIST_FILE){
		filePath = [self getThumbsImagePathWithFolder:romId
										folderPath:folderPath
										fileIndex:fileIndex
										maxIndex:maxIndex
					];
	}

	return filePath;

}
//------------------------------------
// getThumbsImagePathWithFolder
//------------------------------------
- (NSString*)getThumbsImagePathWithFolder:(NSString*)romId
								folderPath:(NSString*)folderPath
								fileIndex:(int)fileIndex
								maxIndex:(int*)maxIndex
{

	NSString *filePath = @"";

	if(maxIndex != nil){
		*maxIndex = 0;
	}

	if(![folderPath isEqualToString:@""]){

		NSFileManager *myFile = [NSFileManager defaultManager];

		// folder check
		NSString *thumbsPath = [NSString stringWithFormat: @"%@/%@", folderPath, romId];
		if([self checkFileExist:thumbsPath] == FILE_EXIST_DIR){
			NSMutableArray *fileNames = [[self sortFolderFiles:[myFile directoryContentsAtPath:thumbsPath]] retain];
			if(fileIndex < [fileNames count]){
				filePath = [NSString stringWithFormat: @"%@/%@", thumbsPath, [fileNames objectAtIndex:fileIndex]];
			}
			if([fileNames count] > 0){
				if(maxIndex != nil){
					*maxIndex = [fileNames count] - 1;
				}
			}
			[fileNames release];
		}
	}

	return filePath;
}
//------------------------------------
// getThumbsImage
//------------------------------------
- (NSImage*)getThumbsImage:(NSString*)filePath hasImage:(BOOL*)hasImage
{

	NSString *imagePath = @"";
	BOOL isCreate = NO;

	// thumbnail is exist
	if([self checkFileExist:filePath] == FILE_EXIST_FILE){
		imagePath = filePath;
		isCreate = YES;
	}else{
		imagePath = [[NSBundle mainBundle] pathForResource:@"image_noimage" ofType:@"png"];
//		imagePath = [[NSBundle mainBundle] pathForResource:@"image_null" ofType:@"png"];
		isCreate = NO;
	}

	NSImage *image = [[[NSImage alloc] initByReferencingFile:imagePath] autorelease];	

	if(hasImage != nil){
		*hasImage = isCreate;
	}
	return image;
	
}
//------------------------------------
// sortFolderFiles
//------------------------------------
- (NSMutableArray*)sortFolderFiles:(NSArray*)files
{
	NSMutableArray *sortedFiles = [[[NSMutableArray alloc] init] autorelease];

	id record;
	NSEnumerator *enumFiles = [files objectEnumerator];

    while (record = [enumFiles nextObject]) {
		// hidden file
		if([[record substringToIndex:1] isEqualToString:@"."]){
			continue;
		}
		[sortedFiles addObject:record];
	}

	return sortedFiles;

}
//------------------------------------
// thumbsImageTypeMenuItems
//------------------------------------
- (NSArray*)thumbsImageTypeMenuItems
{
	// title, value
	return [[NSArray alloc] initWithObjects: 
			@"Snap,0",
			@"Title,1",
			@"Flyer,2",
			@"Inst Cards,3",
			@"Cabinet,4",
//			@"Web,2",
			nil
	];
}
@end