#import "ThumbsMatrixCell.h"
#import "ConvertExtension.h"
#import "ThumbsExtension.h"
#import "ImageAttributeExtension.h"
#import "FileAttributeExtension.h"
#import "CellAttributeExtension.h"
#import "UserDefaultsExtension.h"

static float CELL_PADDING_WIDTH = 6.0;
static float IMAGE_PADDING_HEIGHT = 15;
static float IMAGE_SHADOW_OFFSET = 5;
//static float IMAGE_POINT_Y_OFFSET = -13;
static float IMAGE_POINT_Y_OFFSET = -8;
//static float TEXT_BLANK_HEIGHT = 0.0;
//static float TEXT_POINT_Y_OFFSET = -5;

@implementation ThumbsMatrixCell

//------------------------------------
// init
//------------------------------------
- (id)initWithItemObject:(id)itemObject
				itemIndex:(int)itemIndex
				isEnabled:(BOOL)isEnabled
{
	if (self = [super init])
	{
		[self setItemObject:itemObject];
		[self setItemIndex:itemIndex];
		[self setIsEnabled:isEnabled];
		[self setIsSelected:NO];
		[self setRomId:@""];

		// create image item
		// create at first
		if([self defaultIntValue:@"optMatrixDrawType"] == DEFAULT_MATRIX_DRAW_TYPE_FIRST){
			[self createImageItem:itemObject image:nil isCheck:NO];
		}
		// create at showing
		else{
			[self setImageItem:nil];
		}

		// set notification
		NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(handleItemInfoDidChanged:) name:ITEM_NOTIF_INFO_DID_CHANGED_MAIN object:nil];
		[nc addObserver:self selector:@selector(handleItemImageDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_MAIN object:nil];
		[nc addObserver:self selector:@selector(handleItemImageDidChanged:) name:SLIDER_NOTIF_NAME_DID_STOPPED object:nil];

	}
	return self;
}
//------------------------------------
// createImageItem
//------------------------------------
- (void)createImageItem:(id)itemObject image:(NSImage*)image isCheck:(BOOL)isCheck
{

	if(itemObject){

		// get defaultSet
		BOOL isClone;
		id defaultSet = [self getDefaultRomSet:itemObject isClone:&isClone];
		NSString *curRomId;

		// clone
		if(isClone == YES){
			curRomId = [defaultSet valueForKey:@"cloneId"];
		}else{
			curRomId = [defaultSet valueForKey:@"romId"];
		}

		// check same romId
		if(isCheck == YES){
			if([curRomId isEqualToString:[self romId]]){
//				NSLog(@"same=%@", curRomId);
				return;
			}
//			NSLog(@"changed=%@", curRomId);
		}

		NSString *filePath = [self getThumbsImagePath:curRomId
											imageType:[self defaultThumbsImageTypeMain]
											fileIndex:0
											maxIndex:nil
								];

		[self setImageItem:[[[ImageItem alloc] initWithFilePath:filePath
													image:image
													notifType:IMAGE_ITEM_NOTIF_MATRIX_MAIN
							] autorelease]
		];
		[self setRomId:curRomId];
	}else{
		[self setImageItem:nil];
		[self setRomId:@""];
	}

}
//------------------------------------
// reloadImageItem
//------------------------------------
- (void)reloadImageItem:(BOOL)isCheck
{

	// re-create image
	if([self itemObject] && [self imageItem]){
		// image is not equal
//		if([[self imageItem] image] != [self image]){
			[self createImageItem:[self itemObject] image:[[self imageItem] image] isCheck:isCheck];
//		}
	}
}
//------------------------------------
// handleItemInfoDidChanged
//------------------------------------
-(void)handleItemInfoDidChanged:(NSNotification *)notification
{
	// reload
	[self reloadImageItem:YES];
}
//------------------------------------
// handleItemImageDidChanged
//------------------------------------
-(void)handleItemImageDidChanged:(NSNotification *)notification
{
	// reload
	[self reloadImageItem:NO];
}
//------------------------------------
// drawInteriorWithFrame
//------------------------------------
- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView
{

	if([self isEnabled] == NO || ![self itemObject]){
		return;
	}

	// innerRect
	float innerSize = [self defaultMainMatrixCellSize];
	NSRect innerRect;
	innerRect.origin.x = cellFrame.origin.x + ((cellFrame.size.width - innerSize) / 2);
	innerRect.origin.y = cellFrame.origin.y + ((cellFrame.size.height - innerSize) / 2);
	innerRect.size.width = innerSize;
	innerRect.size.height = innerSize;

	// get record
	id record = [self itemObject];

	// get defaultSet
	BOOL isClone;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// value
	NSString *title = [self getTitleString:[self defaultLangType]
									title:[defaultSet valueForKey:@"title"]
									titlejp:[defaultSet valueForKey:@"titlejp"]
						];
//	NSString *desc = [defaultSet valueForKey:@"year"];;

	int labelColorNo = [[record valueForKey: @"labelColorNo"] intValue];
	BOOL isExist = [[defaultSet valueForKey: @"isExist"] boolValue];
/*
	BOOL isExist = NO;
	if([self checkRomExist:curRomId] != FILE_EXIST_NOT_FOUND){
		isExist = YES;
	}
*/

	//
	// get image
	//
/*
	NSImage *image = [[self imageItem] image];
*/

	NSImage *image = nil;
	// replace image
	if([self imageItem]){
		// image is not equal
//		if(image != [[self imageItem] image]){
			image = [[self imageItem] image];
//			NSLog(@"chaged=%@", title);
//			[self setImage:image];
//		}
	}
	else{
		// create image item
		[self createImageItem:record image:nil isCheck:NO];
	}

	NSPoint	imagePoint;
	imagePoint.x = innerRect.origin.x;
	imagePoint.y = innerRect.origin.y;

	//
	// draw image
	//
	if(image){

		// set image size from original size
		[image setSize:[self setCellImageSize:[image size]
									frameSize:innerRect.size
									paddingSize:NSMakeSize(CELL_PADDING_WIDTH,IMAGE_PADDING_HEIGHT)
						]
		];

		// set image rect
		NSRect imageRect = [self setCellImageRect:[image size]
										frameRect:innerRect
							];

		// shift image rect
		imageRect.origin.y += IMAGE_POINT_Y_OFFSET;

		[image setScalesWhenResized:YES];

		imagePoint.x += imageRect.origin.x;
		imagePoint.y += imageRect.origin.y;

		if([controlView isFlipped]) {
			imagePoint.y += [image size].height;
		}

		// set image round rect
		NSRect imageRoundRect = [self setCellImageRoundRect:[image size]
													frameRect:innerRect
								];

		// shift image round rect
		imageRoundRect.origin.y += IMAGE_POINT_Y_OFFSET;

		if([self isSelected] == YES){
			//
			// draw highlight
			//
			NSRect highlightRect = imageRoundRect;
			highlightRect.origin.x -= IMAGE_SHADOW_OFFSET;
			highlightRect.origin.y -= IMAGE_SHADOW_OFFSET;
			highlightRect.size.width += (IMAGE_SHADOW_OFFSET * 2);
			highlightRect.size.height += (IMAGE_SHADOW_OFFSET * 2);

//			[[NSColor colorWithCalibratedRed:0.1 green:0.1 blue:1.0 alpha:1.0] set];
			[[NSColor blueColor] set];
			NSBezierPath *selectedBoxPath = [self setRoundPathForRect:highlightRect radius:10];
			[selectedBoxPath fill];
		}else{
			//
			// draw shadow rect
			//
//			if(hasImage == YES){
				NSRect shadowRect = imageRoundRect;
				shadowRect.origin.x += IMAGE_SHADOW_OFFSET;
				shadowRect.origin.y += IMAGE_SHADOW_OFFSET;
				NSBezierPath *shadowPath = [self setRoundPathForRect:shadowRect radius:5];
				[[NSColor blackColor] set];
				[shadowPath fill];
//			}
		}

		[[NSGraphicsContext currentContext] setImageInterpolation:NSImageInterpolationNone];

		// draw image rect
		NSBezierPath *imagePath = [self setRoundPathForRect:imageRoundRect radius:5];
		[NSGraphicsContext saveGraphicsState];

		[imagePath addClip];

		// draw image
		[image compositeToPoint:imagePoint operation:NSCompositeSourceOver];

		[NSGraphicsContext restoreGraphicsState];

/*
		//
		// draw label
		//
		if(isExist == NO){
			// now origin.y is buggy
			NSRect imageBounds;
			imageBounds.origin.x = innerRect.origin.x + imageRect.origin.x;
			imageBounds.origin.y = innerRect.origin.y + imageRect.origin.y;
//			if([controlView isFlipped]) {
//				imageBounds.origin.y += (imageRect.size.height - imageRect.origin.y);
//			}
			imageBounds.size = imageRect.size;

			// draw label
			[self drawLabelAndString:imageBounds
							withString:LABEL_NAME_NOT_FOUND
							fontSize:LABEL_STATUS_FSIZE
							fontColor:[NSColor whiteColor]
							labelColor:[self getLabelColor:LABEL_COLOR_NOT_FOUND]
							align:CELL_ALIGN_RIGHT
							valign:CELL_VALIGN_TOP
							hPadding:LABEL_STATUS_HPADDING
							vPadding:LABEL_STATUS_VPADDING
							radius:LABEL_STATUS_RADIUS
			];
		}
*/

	}

//	[image release];

	//
	// draw text
	//
//	if(title){

		NSMutableParagraphStyle *lineStyle = [[[NSMutableParagraphStyle alloc] init] autorelease];
		[lineStyle setLineBreakMode:NSLineBreakByTruncatingMiddle];
		[lineStyle setAlignment:NSCenterTextAlignment];

		// color
		NSColor *titleColor = [NSColor whiteColor];
//		NSColor *descColor = [NSColor colorWithCalibratedWhite:0.7 alpha:1.0];
		// not exist
		if(isExist == NO){
			titleColor = [NSColor grayColor];
//			descColor = [NSColor grayColor];
		}

		// attributes
		NSDictionary *titleAttr = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSFont systemFontOfSize:11.5], NSFontAttributeName,
										titleColor, NSForegroundColorAttributeName,
										lineStyle, NSParagraphStyleAttributeName,
										nil
									];

/*
		NSDictionary *descAttr = [NSDictionary dictionaryWithObjectsAndKeys:
										[NSFont systemFontOfSize:10.5], NSFontAttributeName,
										descColor, NSForegroundColorAttributeName,
										lineStyle, NSParagraphStyleAttributeName,
										nil
									];
*/
		// size
		NSSize titleSize = [title sizeWithAttributes:titleAttr];
//		NSSize descSize = [desc sizeWithAttributes:descAttr];

		// text rect
		float textHeight;
		NSPoint textPoint;
		textPoint.x = innerRect.origin.x + CELL_PADDING_WIDTH;
		if(image){
//			textHeight = innerRect.origin.y + innerRect.size.height - imagePoint.y + TEXT_POINT_Y_OFFSET;
//			textPoint.y = imagePoint.y - TEXT_POINT_Y_OFFSET;
			textHeight = titleSize.height;
			textPoint.y = innerRect.origin.y + innerRect.size.height - textHeight;
		}else{
//			textHeight = titleSize.height + TEXT_BLANK_HEIGHT + descSize.height;
			textHeight = titleSize.height;
			textPoint.y = innerRect.origin.y + innerRect.size.height - textHeight;
		}

		// rect
		NSRect textRect = NSMakeRect(textPoint.x,
										textPoint.y,
//										innerRect.origin.y + (innerRect.size.height - textHeight) - TEXT_POINT_Y_OFFSET,
//										innerRect.origin.y + (innerRect.size.height - textHeight),
										innerRect.size.width - (CELL_PADDING_WIDTH * 2),
										textHeight
									);

		NSRect titleRect = NSMakeRect(textRect.origin.x, 
										textRect.origin.y,
										textRect.size.width,
										titleSize.height
									);

/*
		NSRect descRect = NSMakeRect(textRect.origin.x,
										textRect.origin.y + titleRect.size.height + TEXT_BLANK_HEIGHT,
										textRect.size.width,
										descSize.height
									);
*/
		float total1Height = titleRect.size.height;
//		float total2Height = titleRect.size.height + (descRect.size.height / 2) + TEXT_BLANK_HEIGHT;

		if(total1Height <= textRect.size.height){

			//
			// label color
			//
			if(labelColorNo > 0){
				NSColor *labelColor = [self getLabelColor:labelColorNo];
				if(labelColor != nil){
					NSRect labelRect = titleRect;
					NSSize stringSize = titleSize;

					if(!NSEqualSizes(stringSize, NSZeroSize)){
						labelRect.size.width = stringSize.width + 5;
					}
					if(labelRect.size.width > titleRect.size.width){
						labelRect.size.width = titleRect.size.width;
					}
					labelRect.origin.x = titleRect.origin.x + floor((titleRect.size.width - labelRect.size.width) / 2);

					[labelColor set];
					NSBezierPath *labelBoxPath = [self setRoundPathForRect:labelRect radius:6];
					[labelBoxPath fill];
				}
			}

			// draw title
			[title drawInRect:titleRect withAttributes:titleAttr];

		}
//		if(total2Height <= textRect.size.height){
//			[desc drawInRect:descRect withAttributes:descAttr];
//		}

//	}


}
//------------------------------------
// itemObject
//------------------------------------
- (void)setItemObject:(id)itemObject
{
	itemObject_ = itemObject;
}
- (id)itemObject
{
	return itemObject_;
}
//------------------------------------
// imageItem
//------------------------------------
- (void)setImageItem:(ImageItem*)imageItem
{
	[imageItem retain];
	[imageItem_ release];
	imageItem_ = imageItem;
}
- (ImageItem*)imageItem
{
	return imageItem_;
}
//------------------------------------
// romId
//------------------------------------
- (void)setRomId:(NSString*)romId
{
	[romId retain];
	[romId_ release];
	romId_ = romId;
}
- (NSString*)romId
{
	return romId_;
}

//------------------------------------
// itemIndex
//------------------------------------
- (void)setItemIndex:(int)itemIndex
{
	itemIndex_ = itemIndex;
}
- (int)itemIndex
{
	return itemIndex_;
}
//------------------------------------
// isEnabled
//------------------------------------
- (void)setIsEnabled:(BOOL)isEnabled
{
	isEnabled_ = isEnabled;
}
- (BOOL)isEnabled
{
	return isEnabled_;
}
//------------------------------------
// isSelected
//------------------------------------
- (void)setIsSelected:(BOOL)isSelected
{
	isSelected_ = isSelected;
}
- (BOOL)isSelected
{
	return isSelected_;
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[romId_ release];
	[imageItem_ release];
	[super dealloc];
}

@end
