#import "UserDefaultsExtension.h"
#import "FileAttributeExtension.h"

@implementation NSObject(userDefaultsExtension_)

//------------------------------------
// setWindowRect 
//------------------------------------
- (void)setWindowRect:(NSWindow*)aWindow key:(NSString*)key
{
	
	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// set rect string
	NSString *recStrWindow = [defaults stringForKey:key];
	if(recStrWindow){
		NSRect rectWindow = NSRectFromString(recStrWindow);
		[aWindow setFrame:rectWindow display:YES animate:NO];
	}else{
		[aWindow center];
	}

}
//------------------------------------
// setWindowPosition 
//------------------------------------
- (void)setWindowPosition:(NSWindow*)aWindow key:(NSString*)key
{

	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// set rect string
	NSString *recStrWindow = [defaults stringForKey:key];
	if(recStrWindow){
		NSRect rectWindow = NSRectFromString(recStrWindow);
		[aWindow setFrameOrigin:NSMakePoint(rectWindow.origin.x,rectWindow.origin.y)];
	}else{
		[aWindow center];
	}

}
//------------------------------------
// saveWindowRect 
//------------------------------------
- (void)saveWindowRect:(NSWindow*)aWindow key:(NSString*)key
{
	if(aWindow == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *rectStrWindow = NSStringFromRect([aWindow frame]);
	[defaults setObject:rectStrWindow forKey:key];
	[defaults synchronize];

}
//-------------------------
// setSplitViewRect
//-------------------------
- (void)setSplitViewRect:(NSSplitView *)aSplitView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *array = [defaults objectForKey:key];
	if (array == nil || [array isKindOfClass:[NSArray class]] == NO){
		return;
	}

	NSArray *subviews = [aSplitView subviews];

	int i;
	for (i = 0; i < [subviews count] && i < [array count]; i++){
		NSRect rect = NSRectFromString([array objectAtIndex:i]);
		[[subviews objectAtIndex:i] setFrame:rect];
	}
}

//-------------------------
// saveSplitViewRect
//-------------------------
- (void)saveSplitViewRect:(NSSplitView *)aSplitView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSMutableArray *array= [[[NSMutableArray alloc] init] autorelease];
	int i;

	// create array
	NSArray *subviews = [aSplitView subviews];
	for (i = 0; i < [subviews count]; i++){
		[array addObject:NSStringFromRect([[subviews objectAtIndex:i] frame])];
	}

	// save array
	if([array count] > 0){
		[defaults setObject:array forKey:key];
	}else{
		if([defaults objectForKey:key]){
			[defaults removeObjectForKey:key];
		}
	}
	[defaults synchronize];
}
//-------------------------
// splitViewRectString
//-------------------------
- (NSString*)splitViewRectString:(NSString*)key index:(int)index
{

	NSString *rectString = @"";

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([defaults objectForKey:key]){
		NSArray *array = [defaults objectForKey:key];
		if (array != nil && [array isKindOfClass:[NSArray class]] == YES){
			if (index >= 0 && index < [array count]){
				rectString = [array objectAtIndex:index];
			}
		}
	}

	return rectString;
}
//------------------------------------
// setTableColumnState
//------------------------------------
- (void)setTableColumnState:(NSTableView*)aTableView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *defaultColumns = [defaults objectForKey:key];
	if (defaultColumns == nil || [defaultColumns isKindOfClass:[NSArray class]] == NO){
//		NSLog(@"defaults is not array");
		return;
	}

	NSTableColumn *aColumn;
	NSString *identifier;
	id record;
	int i;
	int columnIndex = 0;
	int currentIndex;
	int lastColumnIndex = [[aTableView tableColumns] count] - 1;

	for (i = 0; i < [defaultColumns count]; i++){

		record = [defaultColumns objectAtIndex:i];

		if(![record valueForKey:@"identifier"]){
			continue;
		}

		identifier =[record valueForKey:@"identifier"];

		aColumn = [aTableView tableColumnWithIdentifier:identifier];

		// column is exist
		if(aColumn != nil){

			columnIndex++;
			currentIndex = [aTableView columnWithIdentifier:identifier];

//			NSLog(@"identifier=%@", identifier);
//			NSLog(@"lastColumnIndex=%d", lastColumnIndex);
//			NSLog(@"currentIndex=%d -> columnIndex=%d", currentIndex, columnIndex);

			// move column
			if(columnIndex <= lastColumnIndex){
				[aTableView moveColumn:currentIndex toColumn:columnIndex];
			}

			// set column width
			if([record valueForKey:@"width"]){
				[aColumn setWidth:[[record valueForKey:@"width"] intValue]];
			}
		}
	}

}
//-------------------------
// saveTableColumnState
//-------------------------
- (void)saveTableColumnState:(NSTableView*)aTableView key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSMutableArray *defaultColumns = [[[NSMutableArray alloc] init] autorelease];

	NSArray *tableColumns = [aTableView tableColumns];
	int i;

	// create array
	for (i = 0; i < [tableColumns count]; i++){
		[defaultColumns addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[[tableColumns objectAtIndex:i] identifier], @"identifier",
				[NSNumber numberWithInt:[[tableColumns objectAtIndex:i] width]], @"width",
				nil
			]
		];
	}

	// save array
	if([defaultColumns count] > 0){
		[defaults setObject:defaultColumns forKey:key];
	}else{
		if([defaults objectForKey:key]){
			[defaults removeObjectForKey:key];
		}
	}
	[defaults synchronize];
}
//------------------------------------
// setArrayControllerSortDescriptor
//------------------------------------
- (void)setArrayControllerSortDescriptor:(NSArrayController*)aController key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSString *column;
	BOOL isAsc = YES;

	NSDictionary *sortDict = [defaults objectForKey:key];
	if (sortDict == nil || [sortDict isKindOfClass:[NSDictionary class]] == NO){
		return;
	}

	// value not found
	if(![sortDict valueForKey:@"column"]){
		return;
	}

	// colmun not found
	column = [sortDict valueForKey:@"column"];
/*
	if([aTableView tableColumnWithIdentifier:column] == nil){
		return;
	}
*/
	if([sortDict valueForKey:@"isAsc"]){
		isAsc = [[sortDict valueForKey:@"isAsc"] boolValue];
	}

	NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:column ascending:isAsc selector:@selector(compare:)];
	[sortDescriptor autorelease];
	[aController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];

}
//------------------------------------
// saveArrayControllerSortDescriptor
//------------------------------------
- (void)saveArrayControllerSortDescriptor:(NSArrayController*)aController key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([[aController sortDescriptors] count] > 0){
		NSSortDescriptor *aDescriptor = [[aController sortDescriptors] objectAtIndex:0];

		[defaults setObject:
			[NSDictionary dictionaryWithObjectsAndKeys:
						[aDescriptor key], @"column",
						[NSNumber numberWithBool:[aDescriptor ascending]], @"isAsc",
						nil
			]
		forKey:key];
	}
	[defaults synchronize];
}
//------------------------------------
// setSearchFieldRecentSearches
//------------------------------------
- (void)setSearchFieldRecentSearches:(NSSearchField*)searchField key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	NSArray *array = [defaults objectForKey:key];
	if (array == nil || [array isKindOfClass:[NSArray class]] == NO){
		return;
	}

	[searchField setRecentSearches:array];

}
//------------------------------------
// saveSearchFieldRecentSearches
//------------------------------------
- (void)saveSearchFieldRecentSearches:(NSSearchField*)searchField key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[searchField recentSearches] forKey:key];

	[defaults synchronize];
}

//------------------------------------
// saveLastSelectedIndex 
//------------------------------------
- (void)saveLastSelectedIndex:(int)index key:(NSString*)key
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// last uniqueId
	[defaults setObject:[NSNumber numberWithInt:index] forKey:key];
}
//------------------------------------
// getLastSelectedIndex 
//------------------------------------
- (int)getLastSelectedIndex:(NSString*)key
{
	int index = -1;
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if([defaults objectForKey:key]){
		index = [[defaults objectForKey:key] intValue];
	}

	return index;

}

//------------------------------------
// defaultMaxResults 
//------------------------------------
- (int)defaultMaxResults
{

	int maxResultsType = [self defaultIntValue:@"optMaxResultsType"];
	int maxResults = 0;

	// no limit
	if(maxResultsType == 0){
		maxResults = 0;
	}
	else if(maxResultsType == 1){
		maxResults = 10;
	}
	else if(maxResultsType == 2){
		maxResults = 30;
	}
	else if(maxResultsType == 3){
		maxResults = 50;
	}
	else if(maxResultsType == 4){
		maxResults = 100;
	}


/*
	int maxResults = [self defaultIntValue:@"optMaxResults"];

//	if(maxResults <= 0){
//		maxResults = 10;
//	}
*/

	return maxResults;

}
//------------------------------------
// defaultMainMatrixCellSize
//------------------------------------
- (float)defaultMainMatrixCellSize
{
	float scale = [self defaultFloatValue:@"optMainMatrixCellScale"];
	return (100 + scale * scale);
}

//------------------------------------
// defaultExecuteMameNo
//------------------------------------
- (int)defaultExecuteMameNo
{
	return [self defaultIntValue:@"optExecuteMameNo"];
}
//------------------------------------
// defaultShowRomsType
//------------------------------------
- (int)defaultShowRomsType
{
	return [self defaultIntValue:@"optShowRomsType"];
}
//------------------------------------
// defaultThumbsImageTypeMain
//------------------------------------
- (int)defaultThumbsImageTypeMain
{
	return [self defaultIntValue:@"optThumbsImageTypeMain"];
}
//------------------------------------
// defaultThumbsImageTypeClone
//------------------------------------
- (int)defaultThumbsImageTypeClone
{
	return [self defaultIntValue:@"optThumbsImageTypeClone"];
}
//------------------------------------
// defaultLangType
//------------------------------------
- (int)defaultLangType
{
	return [self defaultIntValue:@"optLangType"];
}

//------------------------------------
// setDefaultStringValue 
//------------------------------------
- (void)setDefaultStringValue:(NSString*)string key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:string forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultStringValue 
//------------------------------------
- (NSString*)defaultStringValue:(NSString*)key
{
	NSString *string = @"";

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// string value 
	if([defaults objectForKey:key]){
		string = [defaults objectForKey:key];
	}

	return string;
}
//------------------------------------
// setDefaultIntValue 
//------------------------------------
- (void)setDefaultIntValue:(int)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithInt:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultIntValue
//------------------------------------
- (int)defaultIntValue:(NSString*)key
{
	int value = 0;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// int value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] intValue];
	}

	return value;
}
//------------------------------------
// setDefaultFloatValue 
//------------------------------------
- (void)setDefaultFloatValue:(float)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithFloat:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultFloatValue
//------------------------------------
- (float)defaultFloatValue:(NSString*)key
{
	float value = 0.0;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// float value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] floatValue];
	}

	return value;
}
//------------------------------------
// setDefaultBoolValue 
//------------------------------------
- (void)setDefaultBoolValue:(BOOL)value key:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:[NSNumber numberWithBool:value] forKey:key];

	[defaults synchronize];
}
//------------------------------------
// defaultBoolValue
//------------------------------------
- (BOOL)defaultBoolValue:(NSString*)key
{
	BOOL value = NO;

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	// BOOL value 
	if([defaults objectForKey:key]){
		value = [[defaults objectForKey:key] boolValue];
	}

	return value;
}

//-------------------------
// setDefaultArrayValue
//-------------------------
- (void)setDefaultArrayValue:(NSArray*)array key:(NSString*)key
{

	if(array == nil || [key isEqualToString:@""]){
		return;
	}

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setObject:array forKey:key];
	[defaults synchronize];

}
//-------------------------
// defaultArrayValue
//-------------------------
- (NSArray*)defaultArrayValue:(NSString*)key
{

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if([defaults objectForKey:key]){
		NSArray *array = [defaults objectForKey:key];
		if (array != nil && [array isKindOfClass:[NSArray class]] == YES){
			return array;
		}
	}

	return nil;
}
//------------------------------------
// defaultSettingFilePath
//------------------------------------
- (NSString*)defaultSettingFilePath:(int)tag
{
	NSString *defaultKey = [self getDefaultFilePathKey:tag];
	return [self defaultStringValue:defaultKey];
}
//------------------------------------
// defaultSettingFolderPath
//------------------------------------
- (NSString*)defaultSettingFolderPath:(int)tag
{
	NSString *defaultKey = [self getDefaultFolderPathKey:tag];
	return [self defaultStringValue:defaultKey];
}
//------------------------------------
// getDefaultFilePathKey
//------------------------------------
- (NSString*)getDefaultFilePathKey:(int)tag
{

	NSString *value = @"";

	switch(tag) {
		case MAME_FILE_TYPE_SDLMAME:
			value = @"pathSdlMame";
			break;
		case MAME_FILE_TYPE_MAMEOSX:
			value = @"pathMameOSX";
			break;
		case MAME_FILE_TYPE_MACMAME:
			value = @"pathMacMame";
			break;
		case MAME_FILE_TYPE_MAME_LIST:
			value = @"pathMAMEList";
			break;
		case MAME_FILE_TYPE_MAME_LIST_JP:
			value = @"pathMAMEListJP";
			break;
		case MAME_FILE_TYPE_MAME_AUDIT:
			value = @"pathMAMEAuditList";
			break;
//		case MAME_FILE_TYPE_MAME_ADD_VERSION:
//			value = @"pathMAMEAddVersionList";
//			break;
		case MAME_FILE_TYPE_SDLMAME_CHEAT:
			value = @"pathSdlMameCheat";
			break;
		case MAME_FILE_TYPE_WEB_SEARCH_LIST:
			value = @"pathWebSearchList";
			break;
		case MAME_FILE_TYPE_BACKGROUND_IMAGE:
			value = @"pathBackGroundImage";
			break;

	}

	return value;
}
//------------------------------------
// getDefaultFolderPathKey
//------------------------------------
- (NSString*)getDefaultFolderPathKey:(int)tag
{

	NSString *value = @"";

	switch(tag) {
//		case MAME_FOLDER_TYPE_ROMS:
//			value = @"pathRomsFolder";
//			break;
		case MAME_FOLDER_TYPE_SNAPS:
			value = @"pathSnapsFolder";
			break;
		case MAME_FOLDER_TYPE_TITLES:
			value = @"pathTitlesFolder";
			break;
		case MAME_FOLDER_TYPE_FLYERS:
			value = @"pathFlyersFolder";
			break;
		case MAME_FOLDER_TYPE_INSTS:
			value = @"pathInstsFolder";
			break;
		case MAME_FOLDER_TYPE_CABINETS:
			value = @"pathCabinetsFolder";
			break;
		case MAME_FOLDER_TYPE_SDLMAME_SAMPLE:
			value = @"pathSdlMameSample";
			break;
	}

	return value;
}
//------------------------------------
// getDefaultRomsFolderPath
//------------------------------------
- (NSString*)getDefaultRomsFolderPath
{

	NSArray *folderItems = [self defaultArrayValue:@"pathRomsFolderItems"];

	NSString *filePath;
	NSString *folderPath = @"";
	BOOL enabled;
	id record;

	int i;

	for(i = 0; i < [folderItems count]; i++){

		record = [folderItems objectAtIndex:i];
		enabled = [[record valueForKey:@"enabled"] boolValue];
		filePath = [record valueForKey:@"filePath"];
		if(enabled == NO){
			continue;
		}
		if(!filePath || [filePath isEqualToString:@""]){
			continue;
		}

		// folder
		if([self checkFileExist:filePath] == FILE_EXIST_DIR){
			folderPath = filePath; 
			break; 
		}

	}

	return folderPath;
}
//------------------------------------
// getDefaultRomsFolderPathString
//------------------------------------
- (NSString*)getDefaultRomsFolderPathString
{

	NSArray *folderItems = [self defaultArrayValue:@"pathRomsFolderItems"];

	NSMutableArray *pathItems = [[[NSMutableArray alloc] init] autorelease];
	NSString *filePath;
	BOOL enabled;
	id record;

	int i;

	for(i = 0; i < [folderItems count]; i++){

		record = [folderItems objectAtIndex:i];
		enabled = [[record valueForKey:@"enabled"] boolValue];
		filePath = [record valueForKey:@"filePath"];
		if(enabled == NO){
			continue;
		}
		if(!filePath || [filePath isEqualToString:@""]){
			continue;
		}

		// folder
		if([self checkFileExist:filePath] == FILE_EXIST_DIR){
			[pathItems addObject:filePath];
		}

	}
	return [pathItems componentsJoinedByString:@";"];
}
//------------------------------------
// getDefaultImageFolderType
//------------------------------------
- (int)getDefaultImageFolderType:(int)no
{

	int type = MAME_FOLDER_TYPE_SNAPS;

	switch(no) {
		case DEFAULT_IMAGE_TYPE_SNAP:
			type = MAME_FOLDER_TYPE_SNAPS;
			break;
		case DEFAULT_IMAGE_TYPE_TITLE:
			type = MAME_FOLDER_TYPE_TITLES;
			break;
		case DEFAULT_IMAGE_TYPE_FLYER:
			type = MAME_FOLDER_TYPE_FLYERS;
			break;
		case DEFAULT_IMAGE_TYPE_INST:
			type = MAME_FOLDER_TYPE_INSTS;
			break;
		case DEFAULT_IMAGE_TYPE_CABINET:
			type = MAME_FOLDER_TYPE_CABINETS;
			break;
	}

	return type;
}
//------------------------------------
// getDefaultImageFolderTitle
//------------------------------------
- (NSString*)getDefaultImageFolderTitle:(int)no
{

	NSString *title = @"";

	switch(no) {
		case DEFAULT_IMAGE_TYPE_SNAP:
			title = @"Snaps";
			break;
		case DEFAULT_IMAGE_TYPE_TITLE:
			title = @"Titles";
			break;
		case DEFAULT_IMAGE_TYPE_FLYER:
			title = @"Flyers";
			break;
		case DEFAULT_IMAGE_TYPE_INST:
			title = @"Inst Cards";
			break;
		case DEFAULT_IMAGE_TYPE_CABINET:
			title = @"Cabinets";
			break;
	}

	return title;
}
//------------------------------------
// getDefaultCategoryColumn
//------------------------------------
- (NSString*)getDefaultCategoryColumn:(int)type
{

	NSString *value = @"";

	switch(type) {
//		case CATEGORY_TYPE_PLAYLIST:
//			value = @"";
//			break;
//		case CATEGORY_TYPE_LETTER:
//			value = @"";
//			break;
		case CATEGORY_TYPE_MANUFACTURE:
			value = @"manufacture";
			break;
		case CATEGORY_TYPE_YEAR:
			value = @"year";
			break;
		case CATEGORY_TYPE_FILEDATE:
			value = @"fileDate";
			break;
		case CATEGORY_TYPE_ADDVERSION:
			value = @"addVersion";
			break;
		case CATEGORY_TYPE_COMMENT:
			value = @"comment";
			break;
	}

	return value;
}

//------------------------------------
// getAppSupportFolderPath
//------------------------------------
- (NSString*)getAppSupportFolderPath
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
    NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
    return [basePath stringByAppendingPathComponent:@"MAME_Tunes"];
}
//-------------------------
// defaultOSVersionNo
//-------------------------
- (NSString*)defaultOSVersionNo
{
	// versionString = Version 10.4.11 (Build 8S165)
	NSString *versionString = [[NSProcessInfo processInfo] operatingSystemVersionString];
	NSScanner *scan = [NSScanner scannerWithString:versionString];

	NSString *versionNo = nil;
	[scan scanUpToString:@" " intoString:nil];
	[scan scanUpToString:@" " intoString:&versionNo];

	if(versionNo == nil){
		versionNo = @"";
	}

	return versionNo;
}
//-------------------------
// defaultAppName
//-------------------------
- (NSString*)defaultAppName
{
	NSString *appName = [self defaultStringValue:@"optAppName"];
	if([appName isEqualToString:@""]){
		appName = @"MAME_Tunes";
	}
	return appName;
}
//------------------------------------
// defaultAppVersion
//------------------------------------
- (NSString*)defaultAppVersion
{
	return (NSString*)[self defaultInfoPlistValue:@"CFBundleShortVersionString"];
}

//------------------------------------
// defaultInfoPlistValue
//------------------------------------
- (id)defaultInfoPlistValue:(NSString*)key
{
	id value = nil;

	NSDictionary *dict = [[NSBundle mainBundle] infoDictionary];

	if([dict valueForKey:key]){
		value = [dict valueForKey:key];
	}

	return value;
}
@end