#import "ViewCategory.h"
#import "ViewMain.h"
#import "ViewKeywordSetting.h"
#import "TBArrayController.h"
#import "ConvertExtension.h"
#import "ImageBindExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewCategory

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	// alloc itemList and set content
	itemList_ = [[NSMutableArray alloc] init];
	[categorylistArrayController setContent:itemList_];

	[self setCategoryNo:CATEGORY_TYPE_PLAYLIST];

	[self setCategoryButtonEnable];

	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(viewFrameDidChange:) name:NSViewFrameDidChangeNotification object:tabViewPlaylist];
}
//=======================================================================
// IBActions
//=======================================================================
//------------------------------------
// changeCategory
//------------------------------------
- (IBAction)changeCategory:(id)sender
{
	int categoryNo = [[sender representedObject] intValue];
	[self selectCategory:categoryNo];
}
//------------------------------------
// reloadCategory
//------------------------------------
- (IBAction)reloadCategory:(id)sender
{
	[self selectCategory:[self categoryNo]];
}
//------------------------------------
// searchItem
//------------------------------------
- (IBAction)searchItem:(id)sender
{

	NSArray *items = [categorylistArrayController selectedObjects];
	id item = nil;

	// no select
	if([items count] > 0){
		item = [items objectAtIndex:0];
	}

	// search
	[self searchItemWithItem:item];

}
//------------------------------------
// searchWithDeselectItem
//------------------------------------
- (IBAction)searchWithDeselectItem:(id)sender
{
	// deselect
	[self deselectItem:nil];
	// search
	[self searchItem:nil];
}

//------------------------------------
// deselectItem
//------------------------------------
- (IBAction)deselectItem:(id)sender
{
	[tbvCategorylist deselectItem];
}
//------------------------------------
// addSearchlist
//------------------------------------
- (IBAction)addSearchlist:(id)sender
{
	
	NSArray *items = [categorylistArrayController selectedObjects];
	// no select
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];
	NSString *title = [record valueForKey:@"title"];
	NSString *column = [self getDefaultCategoryColumn:[self categoryNo]];

	if([viewKeywordSetting createKeywordSettingWindow:YES
								uniqueId:@""
								keyword:title
								value:column
		] == YES
	){
		[viewKeywordSetting openKeywordSettingWindow:nil];
	}
}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// selectCategory
//------------------------------------
- (void)selectCategory:(int)categoryNo
{

	[self setCategoryNo:categoryNo];

	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	NSString *column = [self getDefaultCategoryColumn:categoryNo];

	switch(categoryNo) {

		case CATEGORY_TYPE_PLAYLIST:
				break;

		case CATEGORY_TYPE_LETTER:
				[self createCategoryWithLetter];
				[self setColumnHeaderTitle:@"Letter"];
				break;

		case CATEGORY_TYPE_MANUFACTURE:
				[self createCategoryWithColumn:column
										adjust:YES
										split:NO
										trim:NO
										trimLen:0
										isSearchClone:NO
										searchPattern:@"contains"
				];
				[self setColumnHeaderTitle:@"Manufacture"];
				break;

		case CATEGORY_TYPE_YEAR:
				[self createCategoryWithColumn:column
										adjust:NO
										split:NO
										trim:NO
										trimLen:0
										isSearchClone:NO
										searchPattern:@"equal"
				];
				[self setColumnHeaderTitle:@"Year"];
				break;

		case CATEGORY_TYPE_FILEDATE:
				[self createCategoryWithColumn:column
										adjust:NO
										split:NO
										trim:YES
										trimLen:7
										isSearchClone:YES
										searchPattern:@"contains"
				];
				[self setColumnHeaderTitle:@"Date"];
				break;

		case CATEGORY_TYPE_ADDVERSION:
				[self createCategoryWithColumn:column
										adjust:NO
										split:NO
										trim:YES
										trimLen:0
										isSearchClone:YES
										searchPattern:@"equal"
				];
				[self setColumnHeaderTitle:@"Version"];
				break;

		case CATEGORY_TYPE_COMMENT:
				[self createCategoryWithColumn:column
										adjust:NO
										split:YES
										trim:NO
										trimLen:0
										isSearchClone:NO
										searchPattern:@"contains"
				];
				[self setColumnHeaderTitle:@"Comment"];
				break;
	}

	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

	// select tabViewItem
	NSString *identifier = @"";
	if(categoryNo == CATEGORY_TYPE_PLAYLIST){
		identifier = CATEGORY_TAB_ID_PLAYLIST;
	}else{
		identifier = CATEGORY_TAB_ID_BROWSER;
	}
	if([tabViewPlaylist indexOfTabViewItemWithIdentifier:identifier] != NSNotFound){
		[tabViewPlaylist selectTabViewItemWithIdentifier:identifier];
	}
	if([tabViewControl indexOfTabViewItemWithIdentifier:identifier] != NSNotFound){
		[tabViewControl selectTabViewItemWithIdentifier:identifier];
	}

	// set image
	[btnMenuCategory setImage:[self getCategoryButtonImage:categoryNo]];

	// post notification
	NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
	[nc postNotificationName:CATEGORY_NOTIF_TAB_ID_DID_CHANGED object:nil];

}
//------------------------------------
// createCategoryWithLetter
//------------------------------------
- (void)createCategoryWithLetter
{

	NSMutableArray *insertItems = [[[NSMutableArray alloc] init] autorelease];
	NSArray *cols;
	int i;
	NSArray *items = [self searchLetterItems];

	for(i = 0; i < [items count]; i++){

		cols = [[items objectAtIndex:i] componentsSeparatedByString:@","];
		[insertItems addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[cols objectAtIndex:0], @"title" ,
				[cols objectAtIndex:1], @"searchString" ,
				@"grep", @"searchPattern" ,
				nil
			]
		];
		
    }

	[self addItemList:insertItems column:@"title" isSearchClone:NO];

}
//------------------------------------
// createCategoryWithColumn
//------------------------------------
- (void)createCategoryWithColumn:(NSString*)column 
								adjust:(BOOL)adjust 
								split:(BOOL)split 
								trim:(BOOL)trim
								trimLen:(int)trimLen
								isSearchClone:(BOOL)isSearchClone 
								searchPattern:(NSString*)searchPattern
{

	BOOL isAdjust = [self defaultBoolValue:@"optCategoryAdjust"];
	BOOL isSplit = [self defaultBoolValue:@"optCategorySplit"];
	BOOL isClone = [self defaultBoolValue:@"optCategoryClone"];
//	BOOL isAdjust = YES;
//	BOOL isSplit = NO;
//	BOOL isClone = YES;

	NSMutableArray *fetchItems = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *insertItems = nil;

	// get distinct objects
	NSString *keyPath = [NSString stringWithFormat:@"@distinctUnionOfObjects.%@", column];
	[fetchItems addObjectsFromArray:[[tbArrayController getAllObjects:@"romlist"] valueForKeyPath:keyPath]];

	// get clone objects
	if(isSearchClone == YES){
		if(isClone == YES){
			[fetchItems addObjectsFromArray:[[tbArrayController getAllObjects:@"clonelist"] valueForKeyPath:keyPath]];
		}else{
			isSearchClone = NO;
		}
	}

	// split column
	if(split == YES && isSplit == YES){
		fetchItems = [self createSplitItems:fetchItems];
	}

	// sort array
	[fetchItems sortUsingSelector:@selector(caseInsensitiveCompare:)];

	// adjust
	if(adjust == YES && isAdjust == YES){
		insertItems = [self createInsertItems:[self createAdjustItems:fetchItems] searchPattern:searchPattern];
	}else{
		// trim 
		if(trim == YES){
			insertItems = [self createInsertItems:[self createTrimItems:fetchItems trimLen:trimLen] searchPattern:searchPattern];
		}
		// normal
		else{
			insertItems = [self createInsertItems:fetchItems searchPattern:searchPattern];
		}
	}

	[self addItemList:insertItems column:column isSearchClone:isSearchClone];

}
//------------------------------------
// createAdjustItems
//------------------------------------
- (NSMutableArray*)createAdjustItems:(NSArray*)items
{

	NSMutableArray *trimItems = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *outItems = [[[NSMutableArray alloc] init] autorelease];

	NSString *strSep1 = @"\t";
	NSString *strSep2 = @" ";
	NSString *string;
	NSArray *cols;
	NSArray *array1;
	NSArray *array2;

	NSString *value;
	NSString *lowerValue = @"";
	NSString *prevValue = @"";

	// trim column
    NSEnumerator *enumItems = [items objectEnumerator];

	while(string = [enumItems nextObject]) {

		// trim character
		if(![string isEqualToString:@""]){
			string = [self replaceCharacter:string str1:@"[" str2:@""];
			string = [self replaceCharacter:string str1:@"]" str2:@""];
			string = [self replaceCharacter:string str1:@"<" str2:@""];
			string = [self replaceCharacter:string str1:@">" str2:@""];
			string = [self replaceCharacter:string str1:@"/" str2:strSep1];
			string = [self replaceCharacter:string str1:@"(" str2:strSep1];
	
			// replace "?" -> TAB
			if(![[string substringToIndex:1] isEqualToString:@"?"]){
				string = [self replaceCharacter:string str1:@"?" str2:strSep1];
			}

			cols = [string componentsSeparatedByString:strSep1];
			value = [[cols objectAtIndex:0] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];

			[trimItems addObject:value];
		}
	}

	// sort column
	[trimItems sortUsingSelector:@selector(caseInsensitiveCompare:)];
	NSEnumerator *enumSortedItems = [trimItems objectEnumerator];

	// delete duplicate column
	while(string = [enumSortedItems nextObject]) {
		lowerValue = [string lowercaseString];
//		NSLog(@"%@",lowerValue);
		if(![lowerValue isEqualToString:@""]){
			if(![prevValue isEqualToString:@""] && [lowerValue length] >= [prevValue length]){
				array1 = [[self replaceSeparateCharacters:lowerValue strSep:strSep2] componentsSeparatedByString:strSep2];
				array2 = [[self replaceSeparateCharacters:prevValue strSep:strSep2] componentsSeparatedByString:strSep2];
				if([[array1 objectAtIndex:0] isEqualToString:[array2 objectAtIndex:0]]){
					NSRange rng = [lowerValue rangeOfString:prevValue];
					if(rng.length > 0){
						continue;
					}
				}
			}
		}
		[outItems addObject:string];
		prevValue = lowerValue;
	}

//	NSLog([outItems description]);

	return outItems;
}

//------------------------------------
// createTrimItems
//------------------------------------
- (NSMutableArray*)createTrimItems:(NSArray*)items trimLen:(int)trimLen
{
	NSMutableArray *outItems = [[[NSMutableArray alloc] init] autorelease];

	id record;
	NSString *value;
	NSString *lowerValue;
	NSString *prevValue = @"";

    NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {
		if(![record isEqualToString:@""]){
			if(trimLen > 0 && [record length] >= trimLen){
				value = [record substringToIndex:trimLen];
			}else{
				value = record;
			}
			value = [value lowercaseString];
			lowerValue = [value lowercaseString];
			if(![prevValue isEqualToString:@""] && [prevValue isEqualToString:lowerValue]){
				continue;
			}
			[outItems addObject:value];
			prevValue = lowerValue;
		}
	}

//	NSLog([outItems description]);

	return outItems;
}
//------------------------------------
// createSplitItems
//------------------------------------
- (NSMutableArray*)createSplitItems:(NSArray*)items
{
	NSMutableArray *outItems = [[[NSMutableArray alloc] init] autorelease];
	NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];

	id record;
	NSString *strSep = @" ";
	NSArray *cols;

	int i;

	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		// skip blank
		if([record isEqualToString:@""]){
			continue;
		}

		// split " "
		cols = [record componentsSeparatedByString:strSep];
		for(i = 0; i < [cols count]; i++){
			// skip blank
			if([[cols objectAtIndex:i] isEqualToString:@""]){
				continue;
			}
			// add object
			if(![dict valueForKey:[cols objectAtIndex:i]]){
				[outItems addObject:[cols objectAtIndex:i]];
				[dict setValue:[cols objectAtIndex:i] forKey:[cols objectAtIndex:i]];
			}
		}
	}

	[dict release];

	return outItems;
}
//------------------------------------
// createInsertItems
//------------------------------------
- (NSMutableArray*)createInsertItems:(NSArray*)items searchPattern:(NSString*)searchPattern
{

	NSMutableArray *outItems = [[[NSMutableArray alloc] init] autorelease];

	id record;

	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		// skip blank
		if([record isEqualToString:@""]){
			continue;
		}

		[outItems addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				record, @"title" ,
				record, @"searchString" ,
				searchPattern, @"searchPattern" ,
				nil
			]
		];
	}

//	NSLog([outItems description]);

	return outItems;
}
//------------------------------------
// searchItemWithItem
//------------------------------------
- (void)searchItemWithItem:(NSManagedObject*)item
{

	NSPredicate *pred = nil;
	NSString *title = @"";

	// get value
	if(item != nil){
		NSString *searchColumn = [item valueForKey:@"searchColumn"];
		NSString *searchString = [item valueForKey:@"searchString"];
		NSString *searchPattern = [item valueForKey:@"searchPattern"];
		BOOL isSearchClone = [[item valueForKey:@"isSearchClone"] boolValue];

		title = [item valueForKey:@"title"];

		// replace character
		searchString = [self replaceCharacter:searchString str1:@"\\" str2:@"\\\\"];
		searchString = [self replaceCharacter:searchString str1:@"'" str2:@"\\'"];

		// create pred
		NSMutableArray *predItems = [[[NSMutableArray alloc] init] autorelease];

		// search roms
		NSPredicate *searchPred = [self createSearchPredicate:searchColumn searchString:searchString searchPattern:searchPattern];
		[predItems addObject:searchPred];

		// search clones
		if(isSearchClone == YES){
			NSArray *cloneItems = [[tbArrayController getObjectsWithPred:@"clonelist" pred:searchPred] valueForKey:@"parentId"];
			if([cloneItems count] > 0){
				[predItems addObject:[NSPredicate predicateWithFormat:@"(romId IN %@)", cloneItems]];
			}
		}

		if([predItems count] > 0){
			pred = [NSCompoundPredicate orPredicateWithSubpredicates:predItems];
		}
	}

	[viewMain searchWithCategorylist:title pred:pred];

}
//------------------------------------
// createSearchPredicate
//------------------------------------
- (NSPredicate*)createSearchPredicate:(NSString*)searchColumn
						searchString:(NSString*)searchString
						searchPattern:(NSString*)searchPattern
{

 	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];

	// create pred
	if([searchPattern isEqualToString:@"contains"]){
		pred = [NSPredicate predicateWithFormat:@"(%K contains[c] %@)", searchColumn, searchString];
	}
	else if([searchPattern isEqualToString:@"equal"]){
		pred = [NSPredicate predicateWithFormat:@"(%K == %@)", searchColumn, searchString];
	}
	else if([searchPattern isEqualToString:@"grep"]){
		pred = [NSPredicate predicateWithFormat:@"(%K MATCHES %@)", searchColumn, searchString];
	}
	else{
		pred = [NSPredicate predicateWithFormat:@"(%K contains[c] %@)", searchColumn, searchString];
	}

//	NSLog(@"pred = %@", [pred description]);

	return pred;
}
//--------------------------------------
// addItemList
//--------------------------------------
- (void)addItemList:(NSArray*)items column:(NSString*)column isSearchClone:(BOOL)isSearchClone
{

	id record;
	int index = 0;

	// remove all objects
	[self removeItemList];

	// insert objects
	NSEnumerator *enumItems = [items objectEnumerator];
	while (record = [enumItems nextObject]) {

		[itemList_ addObject:
			[NSMutableDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithInt:index], @"index" ,
				[record valueForKey:@"title"], @"title" ,
				[record valueForKey:@"searchString"], @"searchString" ,
				[record valueForKey:@"searchPattern"], @"searchPattern" ,
				[NSNumber numberWithBool:isSearchClone], @"isSearchClone" ,
				column, @"searchColumn" ,
				nil
			]
		];
		index++;
	}

	// reload
	[categorylistArrayController rearrangeObjects];

//	[categorylistArrayController setSortDescriptors:[categorylistArrayController sortDescriptors]];
//	[categorylistArrayController setSelectionIndex:0];

}
//------------------------------------
// removeItemList 
//------------------------------------
- (void)removeItemList
{

	[itemList_ removeAllObjects];

	// reload
	[categorylistArrayController rearrangeObjects];

}
//------------------------------------
// setColumnHeaderTitle
//------------------------------------
- (void)setColumnHeaderTitle:(NSString*)title
{
	[[[tbvCategorylist tableColumnWithIdentifier:@"title"] headerCell] setStringValue:title];
}

//------------------------------------
// setCategoryButtonEnable
//------------------------------------
- (void)setCategoryButtonEnable
{
	if([self isDisplayCategorylist] == YES){
		[btnMenuCategory setEnabled:YES];
		[txtSearchCategory setEnabled:YES];
	}else{
		[btnMenuCategory setEnabled:NO];
		[txtSearchCategory setEnabled:NO];
	}
}

//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// replaceSeparateCharacters
//------------------------------------
- (NSString*)replaceSeparateCharacters:(NSString*)string strSep:(NSString*)strSep
{

	string = [self replaceCharacter:string str1:@"/" str2:strSep];
	string = [self replaceCharacter:string str1:@"(" str2:strSep];
	string = [self replaceCharacter:string str1:@")" str2:strSep];
	string = [self replaceCharacter:string str1:@"-" str2:strSep];
	string = [self replaceCharacter:string str1:@"," str2:strSep];
//	string = [self replaceCharacter:string str1:@"?" str2:strSep];

	return string;
}

//------------------------------------
// isDisplayTabView
//------------------------------------
- (BOOL)isDisplayCategorylist
{
	return [tbvCategorylist isDisplay];
}
//------------------------------------
// isSelectTabViewPlaylist
//------------------------------------
- (BOOL)isSelectTabViewPlaylist
{
	if([[[tabViewPlaylist selectedTabViewItem] identifier] isEqualToString:CATEGORY_TAB_ID_PLAYLIST]){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// canCreateSearchlist
//------------------------------------
- (BOOL)canCreateSearchlist
{

	int categoryNo = [self categoryNo];

	if( categoryNo != CATEGORY_TYPE_PLAYLIST &&
		categoryNo != CATEGORY_TYPE_LETTER
	){
		return YES;
	}else{
		return NO;
	}
}

//------------------------------------
// setCategoryNo
//------------------------------------
- (void)setCategoryNo:(int)categoryNo
{
	categoryNo_ = categoryNo;
}
- (int)categoryNo
{
	return categoryNo_;
}

//------------------------------------
// viewFrameDidChange
//------------------------------------
-(void)viewFrameDidChange:(NSNotification *)notification
{
	[self setCategoryButtonEnable];
}
//------------------------------------
// searchLetterItems
//------------------------------------
- (NSArray*)searchLetterItems
{
	// title, searchString
	return [[NSArray alloc] initWithObjects: 
		//		@"ALL,",@"0",
				@"#,[^a-zA-Z].*",
				@"A,[aA].*",
				@"B,[bB].*",
				@"C,[cC].*",
				@"D,[dD].*",
				@"E,[eE].*",
				@"F,[fF].*",
				@"G,[gG].*",
				@"H,[hH].*",
				@"I,[iI].*",
				@"J,[jJ].*",
				@"K,[kK].*",
				@"L,[lL].*",
				@"M,[mM].*",
				@"N,[nN].*",
				@"O,[oO].*",
				@"P,[pP].*",
				@"Q,[qQ].*",
				@"R,[rR].*",
				@"S,[sS].*",
				@"T,[tT].*",
				@"U,[uU].*",
				@"V,[vV].*",
				@"W,[wW].*",
				@"X,[xX].*",
				@"Y,[yY].*",
				@"Z,[zZ].*",
				nil
	];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{

	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[itemList_ release];
	[super dealloc];
}
@end
