#import "ViewClone.h"
#import "ViewPlaylist.h"
#import "ViewItemInfo.h"
#import "ExecuteMAME.h"
#import "TBArrayController.h"
#import "SetupAuditStatus.h"
#import "SetupRomStatus.h"
#import "FileAttributeExtension.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewClone

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:cloneWindow key:@"rectWindowClone"];
	// split view rect
	[self setSplitViewRect:spvClone key:@"rectSplitViewClone"];

	[self setParentId:@""];
	[self setCloneId:@""];
	[self setParentPred:nil];

	[self setResetButtonEnable];

	[txtStatus setStringValue:@""];

	// set first responder
	id listView = [[spvClone subviews] objectAtIndex:1];
	[cloneWindow makeFirstResponder:listView];
//	[cloneWindow makeFirstResponder:btnClose];

	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];
	[btnUpdate setKeyEquivalent:@"s"];
	[btnUpdate setKeyEquivalentModifierMask:NSCommandKeyMask];
	[btnReset setKeyEquivalent:@"s"];
	[btnReset setKeyEquivalentModifierMask:(NSCommandKeyMask|NSShiftKeyMask)];

	[btnShowInfo setKeyEquivalent:@" "];
	[btnShowInfo setKeyEquivalentModifierMask:NSAlternateKeyMask];
	[btnClose setKeyEquivalent:@"\033"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(cloneWindowWillClose:) name:NSWindowWillCloseNotification object:cloneWindow];

	// set observer
	[clonelistArrayController addObserver:self forKeyPath:@"selection" options:0 context:nil];

	// search with null
	[self searchWithParentId:@"" cloneId:@""];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// openCloneWindow 
//------------------------------------
- (IBAction)openCloneWindow:(id)sender
{
	[cloneWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// closeCloneWindow 
//------------------------------------
- (IBAction)closeCloneWindow:(id)sender
{
	[cloneWindow close];
}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	[self playSelectedItem];
}
//------------------------------------
// showItemInfo
//------------------------------------
- (IBAction)showItemInfo:(id)sender
{
	// no select
	if([[clonelistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[clonelistArrayController selectedObjects] objectAtIndex:0];
	if([viewItemInfo createItemInfoWithObject:record arrayNo:BIND_ARRAY_CLONE showDefault:NO] == YES){
		[viewItemInfo openInfoWindow:nil];
	}
}
//------------------------------------
// updateItemAuditStatus
//------------------------------------
- (IBAction)updateItemAuditStatus:(id)sender;
{

	NSArray *items = [[clonelistArrayController selectedObjects] valueForKey:@"cloneId"];

	// no select
	if([items count] <= 0){
		return;
	}

	NSString *romId = [items objectAtIndex:0];

	// update audit status
	[setupAuditStatus updateAuditStatusWithRomId:romId isParent:NO];

}
//------------------------------------
// updateItemFileStatus
//------------------------------------
- (IBAction)updateItemFileStatus:(id)sender
{

	NSArray *items = [clonelistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	// update file status
	[setupRomStatus updateFileStatus:items
							keyRomId:@"cloneId"
							isParent:NO
							isUpdateDefault:NO
	];

}

//------------------------------------
// showItemInFinder
//------------------------------------
- (IBAction)showItemInFinder:(id)sender
{

	NSArray *items = [[clonelistArrayController selectedObjects] valueForKey:@"cloneId"];

	// no select
	if([items count] <= 0){
		return;
	}
	NSString *filePath = [self createCurrentRomPath:[items objectAtIndex:0]];

	[self showFileInFinder:filePath];

}
//------------------------------------
// searchItemWithWebSearch
//------------------------------------
- (IBAction)searchItemWithWebSearch:(id)sender
{

	id object = [sender representedObject];

	// get value from sender
	NSString *url = [object valueForKey:@"url"];
	int keywordType = [[object valueForKey:@"keywordType"] intValue];
	BOOL isTrim = [[object valueForKey:@"isTrim"] boolValue];

	// no url
	if([url isEqualToString:@""]){
		return;
	}

	NSArray *items = [clonelistArrayController selectedObjects];
	// no item
	if([items count] <= 0){
		return;
	}
	id record = [items objectAtIndex:0];

	NSString *cloneId = [record valueForKey:@"cloneId"];
	NSString *title = [record valueForKey:@"title"];
	NSString *titlejp = [record valueForKey:@"titlejp"];

	// search item
	[self searchItemWithParams:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithInt:keywordType], @"keywordType",
										url, @"url",
										cloneId, @"romId",
										title, @"title",
										titlejp, @"titlejp",
										[NSNumber numberWithBool:isTrim], @"isTrim",
										nil
									]
	];

}
//------------------------------------
// updateParentItem
//------------------------------------
- (IBAction)updateParentItem:(id)sender
{

	NSArray *items = [clonelistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];
	NSString *cloneId = [record valueForKey:@"cloneId"];
	NSString *parentId = [record valueForKey:@"parentId"];

	// update romlist
	if([self updateParentRomlist:parentId defaultRomId:cloneId cloneObject:record] == NO){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Failed updating parent rom."
										infoText:@""
										btnList:@"Cancel"
		];
		return;
	}

	[self setCloneId:cloneId];
	[self setResetButtonEnable];

	[txtStatus setStringValue:@"Default set is changed."];

	// post notification
	[self postItemInfoDidChangedMainNotification];
	[self postItemInfoDidChangedCloneNotification];

}
//------------------------------------
// resetParentItem
//------------------------------------
- (IBAction)resetParentItem:(id)sender
{

	NSString *parentId = [self parentId];
 
	// update romlist
	if([self updateParentRomlist:parentId defaultRomId:parentId cloneObject:nil] == NO){
		[self displayMessageWithIcon:@"icon_alert"
										messageText:@"Failed updating parent rom."
										infoText:@""
										btnList:@"Cancel"
		];
		return;
	}

	[self setCloneId:parentId];
	[self setResetButtonEnable];

	[txtStatus setStringValue:@"Default set is resetted."];

	// post notification
	[self postItemInfoDidChangedMainNotification];
	[self postItemInfoDidChangedCloneNotification];

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playSelectedItem
//------------------------------------
- (void)playSelectedItem
{
	// no select
	if([[clonelistArrayController selectedObjects] count] != 1){
		return;
	}

	id record = [[clonelistArrayController selectedObjects] objectAtIndex:0];

	NSString *cloneId = [record valueForKey:@"cloneId"];
	NSString *parentId = [record valueForKey:@"parentId"];
	NSString *title = [self getTitleString:[self defaultLangType]
							title:[record valueForKey:@"title"]
							titlejp:[record valueForKey:@"titlejp"]
					];

	// create params
	int mameNo = [self defaultExecuteMameNo];
	NSDictionary *playParams = [NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:mameNo], @"mameNo",
									cloneId, @"romId",
									parentId, @"parentId",
									title, @"title",
									[NSNumber numberWithBool:YES], @"isClone",
									[NSNumber numberWithInt:ITEM_TYPE_CLONE], @"itemType",
									nil
								];

	// playItem
	[executeMAME playItem:playParams];

}
//------------------------------------
// searchWithParentId
//------------------------------------
- (void)searchWithParentId:(NSString*)parentId cloneId:(NSString*)cloneId
{

	NSPredicate *pred = [NSPredicate predicateWithFormat:@"parentId == %@", parentId];

	[self setParentId:parentId];
	[self setCloneId:cloneId];
	[self setParentPred:pred];

	// set pred
	[self searchWithPredicate:YES];

	[self setResetButtonEnable];

}

//------------------------------------
// searchWithPredicate
//------------------------------------
- (void)searchWithPredicate:(BOOL)withParent
{

	NSMutableArray *predItems = [[[NSMutableArray alloc] init] autorelease];

	// clear pred
	if(withParent == NO){
		[self setParentId:@""];
		[self setCloneId:@""];
		[self setParentPred:nil];
	}

	NSPredicate *pred = nil;
	NSPredicate *parentPred = [self parentPred];

	// add preds 
	if(parentPred != nil){
		[predItems addObject:parentPred];
	}

	// create pred
	if([predItems count] > 0){
		pred = [NSCompoundPredicate andPredicateWithSubpredicates:predItems];
	}

//	NSLog(@"pred=%@", [pred description]);

	// filter
	[clonelistArrayController setFilterPredicate:pred];

}
//------------------------------------
// updateParentRomlist
//------------------------------------
- (BOOL)updateParentRomlist:(NSString*)romId
				defaultRomId:(NSString*)defaultRomId
				cloneObject:(id)cloneObject
{

	BOOL ret = NO;

	// fetch parent record
 	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"romId == %@", romId];
	NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];

	// update column
	if([fetchArray count] > 0){
		id record = [fetchArray objectAtIndex:0];
		[record setValue:defaultRomId forKey:@"defaultRomId"];
		// update relation
		if(cloneObject){
			[record setValue:cloneObject forKey:@"rlt_clonelist"];
//			[cloneObject setValue:record forKey:@"rlt_romlist"];
		}else{
			[record setValue:nil forKey:@"rlt_clonelist"];
//			[cloneObject setValue:nil forKey:@"rlt_romlist"];
		}
		ret = YES;
	}

	return ret;
}
//------------------------------------
// setWindowStatus
//------------------------------------
- (void)setWindowStatus
{

	id record = nil;
	NSString *title = @"No Select";

	NSArray *items = [clonelistArrayController selectedObjects];

	// selected
	if([items count] > 0){

		record = [items objectAtIndex:0];

		title = [self getTitleString:[self defaultLangType]
							title:[record valueForKey:@"title"]
							titlejp:[record valueForKey:@"titlejp"]
				];

	}else{
/*
		// fetch parent record
	 	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
		pred = [NSPredicate predicateWithFormat:@"romId == %@", [self parentId]];
		NSArray *fetchArray = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];
		if([fetchArray count] > 0){
			record = [fetchArray objectAtIndex:0];
			title = [self getTitleString:[self defaultLangType]
								title:[record valueForKey:@"title"]
								titlejp:[record valueForKey:@"titlejp"]
					];
		}
*/
	}

	[cloneWindow setTitle:title];

	[txtStatus setStringValue:@""];

}
//------------------------------------
// setResetButtonEnable
//------------------------------------
- (void)setResetButtonEnable
{

	BOOL enable = NO;
	NSString *parentId = [self parentId];
	NSString *cloneId = [self cloneId];

	// changed
	if(![parentId isEqualToString:cloneId]){
		enable = YES;
	}

	[btnReset setEnabled:enable];

}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([[object entityName] isEqualToString:@"clonelist"]){
		if([keyPath isEqualToString:@"selection"]){
			[self setWindowStatus];
		}
	}
}
//------------------------------------
// cloneWindowWillClose
//------------------------------------
-(void)cloneWindowWillClose:(NSNotification *)notification
{

	// save state from user default
	// window rect
	[self saveWindowRect:cloneWindow key:@"rectWindowClone"];

	// split view rect
	[self saveSplitViewRect:spvClone key:@"rectSplitViewClone"];

}

//------------------------------------
// parentId
//------------------------------------
- (void)setParentId:(NSString*)parentId
{
	[parentId retain];
	[parentId_ release];
	parentId_ = parentId;

}
- (NSString*)parentId
{
	return parentId_;
}

//------------------------------------
// parentPred
//------------------------------------
- (void)setParentPred:(NSPredicate*)parentPred
{
	[parentPred retain];
	[parentPred_ release];
	parentPred_ = parentPred;

}
- (NSPredicate*)parentPred
{
	return parentPred_;
}
//------------------------------------
// cloneId
//------------------------------------
- (void)setCloneId:(NSString*)cloneId
{
	[cloneId retain];
	[cloneId_ release];
	cloneId_ = cloneId;

}
- (NSString*)cloneId
{
	return cloneId_;
}
//------------------------------------
// isKeyCloneWindow
//------------------------------------
- (BOOL)isKeyCloneWindow
{
	return [cloneWindow isKeyWindow];
}
//------------------------------------
// isVisibleCloneWindow
//------------------------------------
- (BOOL)isVisibleCloneWindow
{
	return [cloneWindow isVisible];
}

//------------------------------------
// hasPreviewRomId
//------------------------------------
- (BOOL)hasPreviewRomId
{
	if([imagePreview hasRomId] == YES){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// hasPreviewImage
//------------------------------------
- (BOOL)hasPreviewImage
{
	if([imagePreview hasImage] == YES){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[clonelistArrayController removeObserver:self forKeyPath:@"selection"];
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[parentId_ release];
	[cloneId_ release];
	[parentPred_ release];

	[super dealloc];
}
@end

