#import "ViewGallery.h"
#import "ViewClone.h"
#import "ViewItemInfo.h"
#import "ExecuteMAME.h"
#import "TBArrayController.h"
#import "DialogExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewGallery

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	[self setIsPlay:NO];
	[self setItemObject:nil];

	// set roundedView attribute
//	[roundedView setViewAttr:0.5 radius:20.0];

	// set key equivalent
	[btnClose setKeyEquivalent:@"\033"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(handleWindowGalleryDidClosed:) name:WINDOW_NOTIF_GALLERY_DID_CLOSED object:nil];
	[nc addObserver:self selector:@selector(handleWindowDialogDidOpened:) name:WINDOW_NOTIF_DIALOG_DID_OPENED object:nil];
	[nc addObserver:self selector:@selector(handleMameExecDidStarted:) name:MAME_NOTIF_EXEC_DID_STARTED object:nil];

}

//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openGalleryWindow
//------------------------------------
- (IBAction)openGalleryWindow:(id)sender
{

	[self setIsPlay:NO];

	// set roundedView attribute
	[roundedView setViewAttr:0.5 radius:20.0];


	[galleryMatrix initMatrix:nil];
	[galleryWindow open];
}

//------------------------------------
// closeGalleryWindow
//------------------------------------
- (IBAction)closeGalleryWindow:(id)sender
{
	[galleryWindow close];
}
//------------------------------------
// changeGalleryWindow
//------------------------------------
- (IBAction)changeGalleryWindow:(id)sender
{
	if([galleryWindow isVisible] == NO){
		[self openGalleryWindow:nil];
	}else{
		[self closeGalleryWindow:nil];
	}
}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{

	id itemObject = [galleryMatrix selectedObject];
	if(itemObject == nil){
/*
		[self displayMessage:@"alert"
					messageText:@"Can not play item."
					infoText:@"Item is not selected."
					btnList:@"Cancel"
		];
*/
		return;
	}

	[self setIsPlay:YES];
	[self setItemObject:itemObject];

	// select index
	int selectedIndex = [galleryMatrix selectedIndex];
	int totalResults = [[romlistArrayController arrangedObjects] count];
	if(selectedIndex >= 0 && selectedIndex < totalResults){
		[romlistArrayController setSelectionIndex:selectedIndex];
	}

	if([self defaultBoolValue:@"optGalleryAutoClose"] == YES){
		// post notification when window is closed
		[galleryWindow close];
	}else{
		[self playSelectedItem];
	}

}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playSelectedItem
//------------------------------------
- (void)playSelectedItem
{
	id itemObject = [self itemObject];

	// no item
	if(itemObject == nil){
		return;
	}

	// get defaultSet
	BOOL isClone;
	NSString *romId;
	NSString *parentId;
	id defaultSet = [self getDefaultRomSet:itemObject isClone:&isClone];

	// clone
	if(isClone == YES){
		romId = [defaultSet valueForKey:@"cloneId"];
		parentId = [defaultSet valueForKey:@"parentId"];
	}else{
		romId = [defaultSet valueForKey:@"romId"];
		parentId = [defaultSet valueForKey:@"romId"];
	}
	NSString *title = [self getTitleString:[self defaultLangType]
							title:[defaultSet valueForKey:@"title"]
							titlejp:[defaultSet valueForKey:@"titlejp"]
					];

/*
	// change status
	if([viewClone isVisibleCloneWindow] == YES){
		[viewClone searchWithParentId:parentId cloneId:romId];
	}
	if([viewItemInfo isVisibleInfoWindow] == YES){
		[viewItemInfo createItemInfoWithObject:itemObject arrayNo:BIND_ARRAY_ROM showDefault:YES];
	}
*/

	// create params
	int mameNo = [self defaultExecuteMameNo];
	NSDictionary *playParams = [NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:mameNo], @"mameNo",
									romId, @"romId",
									parentId, @"parentId",
									title, @"title",
									[NSNumber numberWithBool:isClone], @"isClone",
									[NSNumber numberWithInt:ITEM_TYPE_MAIN], @"itemType",
									nil
								];

	// playItem
	[executeMAME playItem:playParams];

}
//------------------------------------
// handleWindowGalleryDidClosed
//------------------------------------
- (void)handleWindowGalleryDidClosed:(NSNotification *)notification
{

	// claer matrix
	[galleryMatrix clearMatrix];

	if([self defaultBoolValue:@"optGalleryAutoClose"] == YES){
		// play item
		if([self isPlay] == YES){
			[self playSelectedItem];
		}
	}
	[self setIsPlay:NO];

}
//------------------------------------
// handleWindowDialogDidOpened
//------------------------------------
- (void)handleWindowDialogDidOpened:(NSNotification *)notification
{
	// close
	[self closeGalleryWindow:nil];
}
//------------------------------------
// handleMamePlayDidStarted
//------------------------------------
- (void)handleMameExecDidStarted:(NSNotification *)notification
{
	// close
	[self closeGalleryWindow:nil];
}

//------------------------------------
// itemObject
//------------------------------------
- (void)setItemObject:(id)itemObject
{
	itemObject_ = itemObject;
}
- (id)itemObject
{
	return itemObject_;
}
//------------------------------------
// isPlay
//------------------------------------
- (void)setIsPlay:(BOOL)isPlay
{
	isPlay_ = isPlay;
}
- (BOOL)isPlay
{
	return isPlay_;
}
//------------------------------------
// isVisibleWindow
//------------------------------------
- (BOOL)isVisibleWindow
{
	return [galleryWindow isVisible];
}
//------------------------------------
// isNotVisibleWindow
//------------------------------------
- (BOOL)isNotVisibleWindow
{
	return ![self isVisibleWindow];
}
//------------------------------------
// isKeyWindow
//------------------------------------
- (BOOL)isKeyWindow
{
	return [galleryWindow isKeyWindow];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

@end
