#import "ViewItemInfo.h"
#import "TBArrayController.h"
#import "ExecuteMAME.h"
#import "SetupAuditStatus.h"
#import "SetupRomStatus.h"
#import "ThumbsExtension.h"
#import "FileAttributeExtension.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "ImageBindExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"
#import "MAMEAttributeExtension.h"

@implementation ViewItemInfo

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:infoWindow key:@"rectWindowInfo"];

	// set binding array no
	[self setArrayNo:BIND_ARRAY_ROM];
	[self setTargetArrayController:[self arrayNo]];
	[self bindControlButtons];

	[self setParams:nil];
	[self setRomId:@""];
	[self setIsParent:NO];
	[self setShowDefault:NO];

	[self setComment:@""];
	[self setCanEditComment:NO];

	[self setLabelColorNo:0];
	[self setCanEditLabelColorNo:NO];

	[self setImageIndex:0];
	[self setImageMaxIndex:0];

	// set first responder
	[infoWindow makeFirstResponder:btnCancel];
	// set key equivalent
//	[btnCancel setKeyEquivalent:@"\r"];
//	[btnCancel setKeyEquivalentModifierMask:NSAlternateKeyMask];
	[btnCancel setKeyEquivalent:@"\033"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillCloseInfo:) name:NSWindowWillCloseNotification object:infoWindow];

	[nc addObserver:self selector:@selector(handleItemStatusDidChanged:) name:ITEM_NOTIF_STATUS_DID_CHANGED_MAIN object:nil];
	[nc addObserver:self selector:@selector(handleItemStatusDidChanged:) name:ITEM_NOTIF_STATUS_DID_CHANGED_CLONE object:nil];
	[nc addObserver:self selector:@selector(handleImageTypeDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_MAIN object:nil];
	[nc addObserver:self selector:@selector(handleImageTypeDidChanged:) name:DEFAULT_NOTIF_IMAGE_TYPE_DID_CHANGED_CLONE object:nil];
	[nc addObserver:self selector:@selector(handleLangTypeDidChanged:) name:DEFAULT_NOTIF_LANG_TYPE_DID_CHANGED object:nil];

}

//------------------------------------
// openInfoWindow
//------------------------------------
- (IBAction)openInfoWindow:(id)sender
{
	[infoWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// closeInfoWindow 
//------------------------------------
- (IBAction)closeInfoWindow:(id)sender
{
	[infoWindow close];
}
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	NSDictionary *params = [self params];

	// no select
	if(params == nil){
		return;
	}

	NSString *romId;
	NSString *parentId;

	// parent
	BOOL isParent = [self isParent];
	if(isParent == YES){
		romId = [params valueForKey:@"romId"];
		parentId = [params valueForKey:@"romId"];
	}else{
		romId = [params valueForKey:@"cloneId"];
		parentId = [params valueForKey:@"parentId"];
	}
	NSString *title = [self getTitleString:[self defaultLangType]
							title:[params valueForKey:@"title"]
							titlejp:[params valueForKey:@"titlejp"]
					];

	// create params
	int mameNo = [self defaultExecuteMameNo];
	int itemType = ITEM_TYPE_MAIN;
	if([self arrayNo] == BIND_ARRAY_CLONE){
		itemType= ITEM_TYPE_CLONE;
	}
	NSDictionary *playParams = [NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:mameNo], @"mameNo",
									romId, @"romId",
									parentId, @"parentId",
									title, @"title",
									[NSNumber numberWithBool:!isParent], @"isClone",
									[NSNumber numberWithInt:itemType], @"itemType",
									nil
								];

	// playItem
	[executeMAME playItem:playParams];

}
//------------------------------------
// selectItem
//------------------------------------
- (IBAction)selectItem:(id)sender
{

	// update
	if([self isParent] == YES){
		if([self updateItemInfo] == NO){
			return;
		}
	}

	int tag = [sender tag];
	BOOL canSelect = NO;

	// previous
	if(tag == 0){
		if([targetArrayController_ canSelectPrevious] == YES){
			[targetArrayController_ selectPrevious:nil];
			canSelect = YES;
		}
	}
	// next
	else if(tag == 1){
		if([targetArrayController_ canSelectNext] == YES){
			[targetArrayController_ selectNext:nil];
			canSelect = YES;
		}
	}

	// create info
	if(canSelect == YES){
		SEL sel = NSSelectorFromString(@"createItemInfoWithSelectedObject");
		[self performSelector:sel withObject:nil afterDelay:0.1];
	}
}
//------------------------------------
// changeItemInfo
//------------------------------------
- (IBAction)changeItemInfo:(id)sender
{

	// update
	if([self updateItemInfo] == YES){
		[infoWindow performClose:nil];
	}

}
//------------------------------------
// changeItemSet
//------------------------------------
- (IBAction)changeItemSet:(id)sender;
{

	int tag = [[sender selectedItem] tag];

	NSDictionary *params = [self params];
	BOOL isParent = [self isParent];

	NSString *romId;
	NSString *cloneId;
	NSString *parentId;

	// current is parent
	if(isParent == YES){
		parentId = [params valueForKey:@"romId"];
		cloneId = [params valueForKey:@"defaultRomId"];
		if([parentId isEqualToString:cloneId]){
			tag = 0;
		}
	}
	// current is clone
	else{
		parentId = [params valueForKey:@"parentId"];
		cloneId = [params valueForKey:@"cloneId"];
	}
		
	// change to parent
	if(tag == 0){
		romId = parentId;
		isParent = YES;
	}
	// change to clone
	else{
		romId = cloneId;
		isParent = NO;
	}

	[self createItemInfoWithRomId:romId isParent:isParent];

}

//------------------------------------
// changeImageIndex
//------------------------------------
- (IBAction)changeImageIndex:(id)sender
{
	[self selectImageIndex:[sender tag]];
}
//------------------------------------
// searchItemInfoWeb
//------------------------------------
- (IBAction)searchItemInfoWeb:(id)sender
{
	NSString *webInfoURL = [txtWebInfoURL stringValue];
	[self createItemInfoWeb:webInfoURL];
}

//------------------------------------
// updateItemAuditStatus
//------------------------------------
- (IBAction)updateItemAuditStatus:(id)sender;
{

	NSDictionary *params = [self params];

	// no select
	if(params == nil){
		return;
	}

	NSString *romId;
	BOOL isParent = [self isParent];

	if(isParent == YES){
		romId = [params valueForKey:@"romId"];
	}else{
		romId = [params valueForKey:@"cloneId"];
	}

	// update audit status
	[setupAuditStatus updateAuditStatusWithRomId:romId isParent:isParent];

}
//------------------------------------
// updateItemFileStatus
//------------------------------------
- (IBAction)updateItemFileStatus:(id)sender
{

	NSDictionary *params = [self params];

	// no select
	if(params == nil){
		return;
	}

	NSString *keyRomId;
	BOOL isParent = [self isParent];

	if(isParent == YES){
		keyRomId = @"romId";
	}else{
		keyRomId = @"cloneId";
	}

	// update file status
	[setupRomStatus updateFileStatus:[NSArray arrayWithObject:params]
							keyRomId:keyRomId
							isParent:isParent
							isUpdateDefault:NO
	];

}
//------------------------------------
// showItemInFinder
//------------------------------------
- (IBAction)showItemInFinder:(id)sender
{

	NSDictionary *params = [self params];

	// no select
	if(params == nil){
		return;
	}

	NSString *romId;
	if([self isParent] == YES){
		romId = [params valueForKey:@"romId"];
	}else{
		romId = [params valueForKey:@"cloneId"];
	}

	[self showFileInFinder:[self createCurrentRomPath:romId]];

}
//------------------------------------
// showImageInFinder
//------------------------------------
- (IBAction)showImageInFinder:(id)sender
{

	NSDictionary *params = [self params];

	// no select
	if(params == nil){
		return;
	}

	int imageType;

	if([self arrayNo] == BIND_ARRAY_ROM){
		imageType = [self defaultThumbsImageTypeMain];
	}else{
		imageType = [self defaultThumbsImageTypeClone];
	}

	[self showFileInFinder:[self getThumbsImagePath:[self romId]
											imageType:imageType
											fileIndex:[self imageIndex]
											maxIndex:nil
							]
	];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createItemInfoWithSelectedObject
//------------------------------------
- (void)createItemInfoWithSelectedObject
{

	NSArray *fetchedArray = [targetArrayController_ selectedObjects];
	int count = [fetchedArray count];

	// no select
	if(count != 1){
		[self displayMessage:@"alert"
				messageText:@"Item is not selected or multiple selected."
				infoText:@""
				btnList:@"Cancel"
		];
		return;
	}

	id record = [fetchedArray objectAtIndex:0];

	[self createItemInfoWithObject:record arrayNo:[self arrayNo] showDefault:YES];

}
//------------------------------------
// createItemInfoWithObject
//------------------------------------
- (BOOL)createItemInfoWithObject:(id)record arrayNo:(int)arrayNo showDefault:(BOOL)showDefault
{

	// set binding array no
	if(arrayNo != [self arrayNo] ){
		[self setArrayNo:arrayNo];
		[self setTargetArrayController:arrayNo];
		[self bindControlButtons];
	}

	//
	// set attribute
	//
	id attribute = nil;
	BOOL isParent = NO;
	BOOL hasDefaultClone = NO;

	// from romlist
	if(arrayNo == BIND_ARRAY_ROM){

		// get defaultSet
		id defaultSet = [self getDefaultRomSet:record isClone:&hasDefaultClone];

		// clone
		if(hasDefaultClone == YES && showDefault == YES){
			attribute = defaultSet;
			isParent = NO;
		}
		// parent
		else{
			attribute = record;
			isParent = YES;
		}

	}
	// from clonelist
	else if(arrayNo == BIND_ARRAY_CLONE){
		attribute = record;
		isParent = NO;
		hasDefaultClone = NO;
	}
	// unknown
	else{
		return NO;
	}

	// set enable
	[pbtnSelectSet setEnabled:hasDefaultClone];

	// create item info
	return [self createItemInfoRom:attribute isParent:isParent];


}
//------------------------------------
// createItemInfoWithRomId
//------------------------------------
- (void)createItemInfoWithRomId:(NSString*)romId isParent:(BOOL)isParent
{

	// fetch records
	NSPredicate* pred = [[[NSPredicate alloc] init] autorelease];
	NSString *entName = @"";

	if(isParent == YES){
		entName = @"romlist";
		pred = [NSPredicate predicateWithFormat:@"romId == %@", romId];
	}
	else{
		entName = @"clonelist";
		pred = [NSPredicate predicateWithFormat:@"cloneId == %@", romId];
	}

	NSArray *items = [tbArrayController getObjectsWithPred:entName pred:pred];

	// no item
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];

	// create item info
	[self createItemInfoRom:record isParent:isParent];


}
//------------------------------------
// createItemInfoRom
//------------------------------------
- (BOOL)createItemInfoRom:(id)record isParent:(BOOL)isParent
{

	// no select
	if(record == nil){
		[self displayMessage:@"alert"
									messageText:@"Item is not selected."
									infoText:@""
									btnList:@"Cancel"
		];
		return NO;
	}


	NSString *keyRomId;
//	NSString *keyParentId;
	BOOL canEditLabelColorNo = NO;
	BOOL canEditComment = NO;
	int tag;

	// parent
	if(isParent == YES){
		keyRomId = @"romId";
		canEditLabelColorNo = YES;
		canEditComment = YES;
		tag = 0;
	}
	// clone
	else{
		keyRomId = @"cloneId";
		canEditLabelColorNo = NO;
		canEditComment = NO;
		tag = 1;
	}

	// value
	NSString *romId = [record valueForKey:keyRomId];
	NSString *manufacture = [record valueForKey:@"manufacture"];
	NSString *year = [record valueForKey:@"year"];
	NSString *audit = [record valueForKey:@"audit"];

	NSString *driverStatus = @"";
	NSString *driverStatusString = [record valueForKey:@"driverStatus"];
	NSDictionary *driverParams = [self getDriverStatusParams:driverStatusString];
	if([driverParams valueForKey:@"status"]){
		driverStatus = [driverParams valueForKey:@"status"];
	}

	NSString *comment = @"";
	int labelColorNo = 0;

	if(isParent == YES){
		comment = [record valueForKey:@"comment"];
		labelColorNo = [[record valueForKey:@"labelColorNo"] intValue];
	}

	int fileSize = [[record valueForKey:@"fileSize"] intValue];
	NSString *fileSizeStr = @"-";
	if(fileSize > 0){
		fileSizeStr = [NSString stringWithFormat:@"%@ (%@ B)",
									[self convertFileSizeToString:fileSize],
									[self convertToComma:fileSize]
						];
	}
	NSString *fileDate = [record valueForKey:@"fileDate"];
	NSString *filePath = [self createCurrentRomPath:romId];
	NSString *fileName = @"-";

	if(!fileDate || [fileDate isEqualToString:@""]){
		fileDate = @"-";
	}
	if(!filePath || [filePath isEqualToString:@""]){
		filePath = @"-";
	}else{
		fileName = [filePath lastPathComponent];
	}

	// null check
	if(!romId){romId = @"";}
	if(!manufacture){manufacture = @"";}
	if(!year){year = @"";}
	if(!audit){audit = @"";}
	if(!driverStatus){driverStatus = @"";}
	if(!comment){comment = @"";}

//	BOOL isExist = [self checkFileExist:filePath];
	BOOL isExist = [[record valueForKey:@"isExist"] boolValue];
	// current exist
	BOOL isCurExist = NO;
	if([self checkRomExist:romId] != FILE_EXIST_NOT_FOUND){
		isCurExist = YES;
	}

	NSImage *iconRomStatus = [self getStatusIconImage:!isParent isExist:isExist isCurExist:isCurExist];
	NSImage *iconAuditStatus = [self getAuditIconImage:audit];

	// fileStatus
	NSString *fileStatus = @"";
//	if(isExist == YES){
		fileStatus = [NSString stringWithFormat:@"File: %@\nSize: %@\nDate: %@\nPath: %@\n",
												fileName,
												fileSizeStr,
												fileDate,
												filePath
					];
//	}else{
//		fileStatus = @"Not Found";
//	}
	// set status
	[txtRomId setStringValue:romId];

//	[infoWindow setTitle:title];
//	[txtTitle setStringValue:title];
	[self setTitle:record isParent:isParent langType:[self defaultLangType]];

	[txtManufacture setStringValue:manufacture];
	[txtYear setStringValue:year];
	[txtAuditStatus setStringValue:audit];
	[txtDriverStatus setStringValue:driverStatus];

	[pbtnSelectSet selectItemWithTag:tag];
//	[pbtnSelectSet setEnabled:canChangeSet];

	// create comment box
	[self createCommentComboBox:comment isCreate:canEditComment];

	[imageRomStatus setImage:iconRomStatus];
	[imageAuditStatus setImage:iconAuditStatus];

	[txtFileStatus setString:fileStatus];

	[self setParams:record];
	[self setRomId:romId];
	[self setIsParent:isParent];
//	[self setShowDefault:showDefault];

	[self setComment:comment];
	[self setCanEditComment:canEditComment];

	[self setLabelColorNo:labelColorNo];
	[self setCanEditLabelColorNo:canEditLabelColorNo];

	// create image view
	[self createPreviewImage:romId imageIndex:0];

	// enable/disable
	[pbtnLabelColor selectItemWithTag:labelColorNo];
	[pbtnLabelColor setEnabled:canEditLabelColorNo];
	[cbxComment setEnabled:canEditComment];

	[btnUpdate setEnabled:isParent];
	[btnSearchItem setEnabled:isExist];

	NSString *webInfoURL = [self defaultStringValue:@"optWebInfoURL"];
	// replace parameter
	if(![webInfoURL isEqualToString:@""]){
		webInfoURL = [self replaceCharacter:webInfoURL str1:@"{$romId}" str2:romId];
	}
	[txtWebInfoURL setStringValue:webInfoURL];

	// webviewinfo
	if([[[tabViewInfo selectedTabViewItem] identifier] isEqualToString:@"webInfo"]){
		[self createItemInfoWeb:webInfoURL];
	}
	
	return YES;

}
//------------------------------------
// createItemInfoWeb
//------------------------------------
- (BOOL)createItemInfoWeb:(NSString*)urlString
{

	if([urlString isEqualToString:@""]){
		[self displayMessage:@"alert"
									messageText:@"Web info URL is not entered."
									infoText:@""
									btnList:@"Cancel"
		];
		return NO;
	}

	// already loaded
	NSString *loadedUrlString = [[[[[webViewInfo mainFrame] dataSource] request] URL] absoluteString];
	if([urlString isEqualToString:loadedUrlString]){
		return YES;
	}

	NSURL *infoURL = [NSURL URLWithString:urlString];
	if(infoURL == nil){
		return NO;
	}

	// create request
	NSMutableURLRequest *req = [NSMutableURLRequest requestWithURL:infoURL
														cachePolicy:NSURLRequestUseProtocolCachePolicy
														timeoutInterval:10.0
									];
	if(req == nil){
		return NO;
	}

	// set referer
	NSString *refString = [infoURL host];
//	NSLog(@"host=%@", refString);
	[req setValue:refString forHTTPHeaderField:@"Referer"];

	// load
	[[webViewInfo mainFrame] loadRequest:req];

	return YES;

}
//------------------------------------
// selectImageIndex
//------------------------------------
- (void)selectImageIndex:(int)tag
{
	int imageIndex = [self imageIndex];

	if(tag == 0){
		imageIndex--;
		if(imageIndex < 0){
			imageIndex = [self imageMaxIndex];
		}
	}else{
		imageIndex++;
		if(imageIndex > [self imageMaxIndex]){
			imageIndex = 0;
		}
	}

	// create image view
	[self createPreviewImage:[self romId] imageIndex:imageIndex];

}
//------------------------------------
// createPreviewImage
//------------------------------------
- (void)createPreviewImage:(NSString*)romId imageIndex:(int)imageIndex
{

	int imageType = 0;
	int imageMaxIndex = 0;
	BOOL hasImage = NO;

	if([self arrayNo] == BIND_ARRAY_CLONE){
		imageType = [self defaultThumbsImageTypeClone];
	}else{
		imageType = [self defaultThumbsImageTypeMain];
	}

	// preview image
	NSImage *image = [self getThumbsImage:[self getThumbsImagePath:romId
													imageType:imageType
													fileIndex:imageIndex
													maxIndex:&imageMaxIndex
											]
								hasImage:&hasImage
					];

	[imagePreview setImage:image];
	[btnSearchImage setEnabled:hasImage];

	[self setImageIndex:imageIndex];
	[self setImageMaxIndex:imageMaxIndex];
	[self setBoxImageIndexHidden];

}
//------------------------------------
// createCommentComboBox
//------------------------------------
- (void)createCommentComboBox:(NSString*)comment isCreate:(BOOL)isCreate
{

	// remove items
	[cbxComment removeAllItems];
	[cbxComment setStringValue:comment];

	if(isCreate == YES){
		// get all comments
		id record;
		int index = 0;

		NSArray *items = [[tbArrayController getAllObjects:@"romlist"] valueForKeyPath:@"@distinctUnionOfObjects.comment"];

		NSEnumerator *enumItems = [items objectEnumerator];
		while (record = [enumItems nextObject]) {
			if(![record isEqualToString:@""]){
				[cbxComment addItemWithObjectValue:record];
				index++;
			}
		}

		[cbxComment setNumberOfVisibleItems:index];
	}

}

//------------------------------------
// setTitle
//------------------------------------
- (void)setTitle:(id)record isParent:(BOOL)isParent langType:(int)langType
{

	NSString *title = [self getTitleString:langType
							title:[record valueForKey:@"title"]
							titlejp:[record valueForKey:@"titlejp"]
					];

	NSString *windowTitle = @"";

	if(isParent == YES){
		windowTitle = title;
	}else{
		windowTitle = [NSString stringWithFormat:@"Clone : %@", title];
	}

	[infoWindow setTitle:windowTitle];
	[txtTitle setStringValue:title];
	
}
//------------------------------------
// setBoxImageIndexHidden
//------------------------------------
- (void)setBoxImageIndexHidden
{
	if([self imageMaxIndex] > 0){
		[boxImageIndex setHidden:NO];
	}else{
		[boxImageIndex setHidden:YES];
	}
}
//------------------------------------
// updateItemInfo
//------------------------------------
- (BOOL)updateItemInfo
{

	NSString *romId = [self romId];
	BOOL canEditLabelColorNo = [self canEditLabelColorNo];
	BOOL canEditComment = [self canEditComment];

	int result;

	// no select
	if([romId isEqualToString:@""]){
		[self displayMessage:@"alert"
									messageText:@"Item is not selected."
									infoText:@""
									btnList:@"Cancel"
		];
		return NO;
	}

	// not parent 
	if([self isParent] == NO){
		[self displayMessage:@"alert"
									messageText:@"Can not update clone roms."
									infoText:@""
									btnList:@"Cancel"
		];
		return NO;
	}

	// get values
	int labelColorNo = [[pbtnLabelColor selectedItem] tag];
	NSString *comment = [cbxComment stringValue];

	BOOL isChangedLabelColorNo = NO;
	BOOL isChangedComment = NO;

	// check changed value
	if(canEditLabelColorNo == YES && labelColorNo != [self labelColorNo]){
		isChangedLabelColorNo = YES;
	}
	if(canEditComment == YES && ![comment isEqualToString:[self comment]]){
		isChangedComment = YES;
	}

	// no change
	if(isChangedLabelColorNo == NO && isChangedComment == NO){
//		NSLog(@"no changed value");
		return YES;
	}

	// confirm
//	if([self defaultBoolValue:@"optShowAlertBeforeChange"] == YES){
		result = [self displayMessage:@"alert"
										messageText:@"Change item info?"
										infoText:@""
										btnList:@"Change,Cancel"
				];
		if(result != NSAlertFirstButtonReturn){
			return NO;
		}
//	}

/*
	NSLog(@"isChangedLabelColorNo=%d", isChangedLabelColorNo);
*/
	// updateRomlist
	BOOL ret = [self updateRomlist:romId
					labelColorNo:labelColorNo
					comment:comment
					isChangedLabelColorNo:isChangedLabelColorNo
					isChangedComment:isChangedComment
				];

	if(ret == YES){
		// post notification
		NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
		[nc postNotificationName:ITEM_NOTIF_INFO_DID_CHANGED_MAIN object:nil];
	}

	return ret;
}
//------------------------------------
// updateRomlist
//------------------------------------
- (BOOL)updateRomlist:(NSString*)romId
					labelColorNo:(int)labelColorNo
					comment:(NSString*)comment
					isChangedLabelColorNo:(BOOL)isChangedLabelColorNo
					isChangedComment:(BOOL)isChangedComment
{

	// fetch update records
	NSPredicate* pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"romId == %@", romId];

	NSArray *items = [tbArrayController getObjectsWithPred:@"romlist" pred:pred];

	// no item
	if([items count] <= 0){
		[self displayMessage:@"alert"
									messageText:@"Item is not found."
									infoText:@""
									btnList:@"Cancel"
		];
		return NO;
	}

	id record;

	NSEnumerator *enumItems = [items objectEnumerator];
    while (record = [enumItems nextObject]) {

		// labelColorNo
		if(isChangedLabelColorNo == YES){
			[record setValue:[NSNumber numberWithInt:labelColorNo] forKey:@"labelColorNo"];
		}
		// comment
		if(isChangedComment == YES){
			[record setValue:comment forKey:@"comment"];
		}
	}

	return YES;

}
//------------------------------------
// bindControlButtons
//------------------------------------
- (void)bindControlButtons
{
	if(targetArrayController_ != nil){
		[btnPrev bind:@"enabled" toObject:targetArrayController_ withKeyPath:@"canSelectPrevious" options:nil];
	//	[btnPrev bind:@"enabled2" toObject:targetArrayController_ withKeyPath:@"selectedObjects.@count" options:nil];
		[btnNext bind:@"enabled" toObject:targetArrayController_ withKeyPath:@"canSelectNext" options:nil];
	//	[btnNext bind:@"enabled2" toObject:targetArrayController_ withKeyPath:@"selectedObjects.@count" options:nil];
	}else{
		[btnPrev unbind:@"enabled"];
		[btnNext unbind:@"enabled"];
		[btnPrev setEnabled:NO];
		[btnNext setEnabled:NO];
	}
}

//------------------------------------
// setTargetArrayController
//------------------------------------
- (void)setTargetArrayController:(int)arrayNo
{
	if(arrayNo == BIND_ARRAY_ROM){
		targetArrayController_ = romlistArrayController;
	}
	else if(arrayNo == BIND_ARRAY_CLONE){
		targetArrayController_ = clonelistArrayController;
	}else{
		targetArrayController_ = nil;
	}
}
//------------------------------------
// handleItemStatusDidChanged
//------------------------------------
-(void)handleItemStatusDidChanged:(NSNotification *)notification
{
	if([self isVisibleInfoWindow] == YES && ![[self romId] isEqualToString:@""]){
		[self createItemInfoWithRomId:[self romId] isParent:[self isParent]];
	}
}
//------------------------------------
// handleImageTypeDidChanged
//------------------------------------
-(void)handleImageTypeDidChanged:(NSNotification *)notification
{
	if([self isVisibleInfoWindow] == YES && ![[self romId] isEqualToString:@""]){
		[self createPreviewImage:[self romId] imageIndex:0];
	}
}

//------------------------------------
// handleLangTypeDidChanged
//------------------------------------
-(void)handleLangTypeDidChanged:(NSNotification *)notification
{
	if([self isVisibleInfoWindow] == YES && [self params] != nil){
		[self setTitle:[self params] isParent:[self isParent] langType:[self defaultLangType]];
	}
}
//------------------------------------
// windowWillCloseInfo
//------------------------------------
-(void)windowWillCloseInfo:(NSNotification *)notification
{

	// save window rect
	[self saveWindowRect:infoWindow key:@"rectWindowInfo"];

}

//------------------------------------
// setArrayNo
//------------------------------------
- (void)setArrayNo:(int)arrayNo
{
	arrayNo_ = arrayNo;
}
- (int)arrayNo
{
	return arrayNo_;
}

//------------------------------------
// params
//------------------------------------
- (void)setParams:(NSDictionary*)params
{
	[params retain];
    [params_ release];
    params_ = params;
}
- (NSDictionary*)params
{
    return params_;
}
//------------------------------------
// romId
//------------------------------------
- (void)setRomId:(NSString*)romId
{
	[romId retain];
	[romId_ release];
	romId_ = romId;
}
- (NSString*)romId
{
	return romId_;
}
//------------------------------------
// comment
//------------------------------------
- (void)setComment:(NSString*)comment
{
	[comment retain];
	[comment_ release];
	comment_ = comment;
}
- (NSString*)comment
{
	return comment_;
}
//------------------------------------
// canEditComment
//------------------------------------
- (void)setCanEditComment:(BOOL)canEditComment
{
    canEditComment_ = canEditComment;
}
- (BOOL)canEditComment
{
    return canEditComment_;
}

//------------------------------------
// labelColorNo
//------------------------------------
- (void)setLabelColorNo:(int)labelColorNo
{
    labelColorNo_ = labelColorNo;
}
- (int)labelColorNo
{
    return labelColorNo_;
}
//------------------------------------
// canEditLabelColorNo
//------------------------------------
- (void)setCanEditLabelColorNo:(BOOL)canEditLabelColorNo
{
    canEditLabelColorNo_ = canEditLabelColorNo;
}
- (BOOL)canEditLabelColorNo
{
    return canEditLabelColorNo_;
}
//------------------------------------
// isParent
//------------------------------------
- (void)setIsParent:(BOOL)isParent
{
    isParent_ = isParent;
}
- (BOOL)isParent;
{
    return isParent_;
}
//------------------------------------
// showDefault
//------------------------------------
- (void)setShowDefault:(BOOL)showDefault
{
    showDefault_ = showDefault;
}
- (BOOL)showDefault;
{
    return showDefault_;
}

//------------------------------------
// imageIndex
//------------------------------------
- (void)setImageIndex:(int)imageIndex
{
    imageIndex_ = imageIndex;
}
- (int)imageIndex
{
	return imageIndex_;
}

//------------------------------------
// imageMaxIndex
//------------------------------------
- (void)setImageMaxIndex:(int)imageMaxIndex
{
    imageMaxIndex_ = imageMaxIndex;
}
- (int)imageMaxIndex
{
	return imageMaxIndex_;
}
//------------------------------------
// isKeyInfoWindow
//------------------------------------
- (BOOL)isKeyInfoWindow
{
	return [infoWindow isKeyWindow];
}
//------------------------------------
// isVisibleInfoWindow
//------------------------------------
- (BOOL)isVisibleInfoWindow
{
	return [infoWindow isVisible];
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[romId_ release];
	[comment_ release];
	[params_ release];

	[super dealloc];
}
@end
