#import "ViewKeywordSetting.h"
#import "ViewPlaylist.h"
#import "TBArrayController.h"
#import "SearchExtension.h"
#import "DialogExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewKeywordSetting

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	[self setUniqueId:@""];
	[self setIsNewList:NO];
	[txtKeyword setStringValue:@""];

	// window position
	[self setWindowRect:keywordSettingWindow key:@"rectWindowKeywordSetting"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:keywordSettingWindow];

}
//=======================================================================
// IBAction
//=======================================================================
//------------------------------------
// openKeywordSettingWindow
//------------------------------------
- (IBAction)openKeywordSettingWindow:(id)sender
{
	[keywordSettingWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// openKeywordNewSettingWindow
//------------------------------------
- (IBAction)openKeywordNewSettingWindow:(id)sender
{
	if([self createKeywordSettingWindow:YES
								uniqueId:@""
								keyword:@""
								value:@""
		] == YES
	){
		[self openKeywordSettingWindow:nil];
	}
}
//------------------------------------
// openKeywordEditSettingWindow
//------------------------------------
- (IBAction)openKeywordEditSettingWindow:(id)sender
{

	NSArray *items = [tbArrayController getSelectedObjects:@"playlist"];

	// not result
	if([items count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}
	// get value
	id record = [items objectAtIndex:0];
	NSString *uniqueId = [record valueForKey:@"uniqueId"];
	NSString *keyword = [record valueForKey:@"title"];
	NSString *value = [record valueForKey:@"value"];
	int itemType = [[record valueForKey:@"itemType"] intValue];
	int itemSubType = [[record valueForKey:@"itemSubType"] intValue];

	// not keyword
	if(itemType != ITEM_SEARCH || itemSubType != ITEM_SEARCH_KEYWORD){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	if([self createKeywordSettingWindow:NO
								uniqueId:uniqueId
								keyword:keyword
								value:value
		] == YES
	){
		[self openKeywordSettingWindow:nil];
	}
}
//------------------------------------
// closeWindow
//------------------------------------
- (IBAction)closeWindow:(id)sender
{
	[keywordSettingWindow close];
}

//------------------------------------
// changeKeywordSetting
//------------------------------------
- (IBAction)changeKeywordSetting:(id)sender
{

	NSString *keyword = [txtKeyword stringValue];
	NSString *value = [[btnSearchColumns selectedItem] representedObject];

	// new
	if([self isNewList] == YES){

		// add item
		[viewPlaylist addItem:ITEM_SEARCH
					itemSubType:ITEM_SEARCH_KEYWORD
					title:keyword
					value:value
					isFolder:NO
					isSelect:YES
					isEdit:NO
					isSearch:YES
		];

		[self closeWindow:nil];
	}
	// edit
	else{

		NSString *uniqueId = [self uniqueId];
		if([uniqueId isEqualToString:@""]){
			[self displayMessage:@"alert"
								messageText:@"Keyword list is not found"
								infoText:@""
								btnList:@"Cancel"
			];
			return;
		}

		// update playlist
		if([self updatePlaylist:uniqueId title:keyword value:value] == YES){
			[self closeWindow:nil];
			// search
			[viewPlaylist searchWithSelectedItem:nil];
		}
	}

}

//=======================================================================
// methods
//=======================================================================
//------------------------------------
// createKeywordSettingWindow
//------------------------------------
- (BOOL)createKeywordSettingWindow:(BOOL)isNewList
							uniqueId:(NSString*)uniqueId
							keyword:(NSString*)keyword
							value:(NSString*)value
{

	// create menu
	NSMenu *menu = [[[NSMenu alloc] initWithTitle:@"SearchColumnMenu"] autorelease];
	NSArray *cols;
	NSString *itemName;
	NSString *itemValue;
	int i;

	NSArray *menuItems = [self searchColumnItems];
	NSMenuItem *menuItem;

	for(i = 0; i < [menuItems count]; i++){

		cols = [[menuItems objectAtIndex:i] componentsSeparatedByString:@","];
		itemName = [cols objectAtIndex:0];
		itemValue = [cols objectAtIndex:1];

		menuItem = [[[NSMenuItem alloc] initWithTitle:itemName action:nil keyEquivalent:@""] autorelease];
		[menuItem setRepresentedObject:itemValue];

		// set state
		if([itemValue isEqualToString:value]){
			[menuItem setState:1];
		}else{
			[menuItem setState:0];
		}

		[menu addItem:menuItem];

	}

	[btnSearchColumns setMenu:menu];

	[txtKeyword setStringValue:keyword];

	[self setIsNewList:isNewList];
	[self setUniqueId:uniqueId];

	return YES;

}

//------------------------------------
// updatePlaylist
//------------------------------------
- (BOOL)updatePlaylist:(NSString*)uniqueId title:(NSString*)title value:(NSString*)value
{

	// get playllist
	NSPredicate *pred = [[[NSPredicate alloc] init] autorelease];
	pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", uniqueId];
	NSArray *items = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not result
	if([items count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// get value
	id record = [items objectAtIndex:0];

	int itemType = [[record valueForKey:@"itemType"] intValue];
	int itemSubType = [[record valueForKey:@"itemSubType"] intValue];

	// not keyword
	if(itemType != ITEM_SEARCH || itemSubType != ITEM_SEARCH_KEYWORD){
		[self displayMessage:@"alert"
							messageText:@"Keyword list is not selected"
							infoText:@""
							btnList:@"Cancel"
		];
		return NO;
	}

	// update value
	[record setValue:title forKey:@"title"];
	[record setValue:value forKey:@"value"];

	return YES;

}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	[self saveWindowRect:keywordSettingWindow key:@"rectWindowKeywordSetting"];

}

//------------------------------------
// uniqueId
//------------------------------------
- (void)setUniqueId:(NSString*)uniqueId
{
	[uniqueId retain];
	[uniqueId_ release];
	uniqueId_ = uniqueId;
}
- (NSString*)uniqueId
{
	return uniqueId_;
}
//------------------------------------
// isNewList
//------------------------------------
- (void)setIsNewList:(BOOL)isNewList
{
	isNewList_ = isNewList;
}
- (BOOL)isNewList
{
	return isNewList_;
}

//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[uniqueId_ release];
	[super dealloc];
}
@end
