#import "ViewLog.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewLog

//------------------------------------
// awake
//------------------------------------
- (void)awakeFromNib
{

	[self setTitle:@""];
	[self setLogString:@""];
	[self setFileName:@""];
	[self setCanSave:NO];

	// set window rect
	[self setWindowRect:logWindow key:@"rectWindowLog"];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:logWindow];
}

//------------------------------------
// openLogWindow
//------------------------------------
- (IBAction)openLogWindow:(id)sender
{

	[logWindow setTitle:[self title]];
	[txtLogString setString:[self logString]];
	[btnSave setHidden:![self canSave]];

	[logWindow makeKeyAndOrderFront:self];

}
//------------------------------------
// saveLogFile
//------------------------------------
- (IBAction)saveLogFile:(id)sender
{

	// confirm save
	NSString *filePath = [self selectFilePathWithSavePanel:[self fileName]
												canCreateDir:YES
												defaultPath:@""
						];

	if([filePath isEqualToString:@""]){
		return;
	}

	NSString *fileName = [[NSFileManager defaultManager] displayNameAtPath:filePath];

	NSString *logString = [self logString];

	NSData* data = [[[NSData alloc] init] autorelease]; 
	data = [NSData dataWithBytes:[logString cString] length:[logString cStringLength]];

	// writeToFile
	if([data writeToFile:[filePath stringByExpandingTildeInPath] atomically:YES] == NO){
		[self displayMessage:@"alert"
						messageText:@"Can not save log."
						infoText:@"Failed write to file."
						btnList:@"Cancel"
		];
		return;
	}

	// ok
	[self displayMessage:@"info"
					messageText:[NSString stringWithFormat:@"\"%@\" is saved.", fileName]
					infoText:@""
					btnList:@"OK"
	];

}
//------------------------------------
// title
//------------------------------------
- (void)setTitle:(NSString *)title
{
	[title retain];
	[title_ release];
	title_ = title;

}
- (NSString *)title
{
	return title_;
}
//------------------------------------
// logString
//------------------------------------
- (void)setLogString:(NSString *)logString
{
	[logString retain];
	[logString_ release];
	logString_ = logString;

}
- (NSString *)logString
{
	return logString_;
}
//------------------------------------
// appendLogString
//------------------------------------
- (void)appendLogString:(NSString *)logString
{
	[self setLogString:[[self logString] stringByAppendingString:logString]];
}
//------------------------------------
// fileName
//------------------------------------
- (void)setFileName:(NSString *)fileName
{
	[fileName retain];
	[fileName_ release];
	fileName_ = fileName;

}
- (NSString *)fileName
{
	return fileName_;
}
//------------------------------------
// canSave
//------------------------------------
- (void)setCanSave:(BOOL)canSave
{
	canSave_ = canSave;

}
- (BOOL)canSave
{
	return canSave_;
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// clear log
	[txtLogString setString:@""];
	[self setLogString:@""];

	// save window rect
	[self saveWindowRect:logWindow key:@"rectWindowLog"];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[title_ release];
	[logString_ release];
	[fileName_ release];
	
	[super dealloc];
}

@end
