#import "ViewMain.h"
#import "ViewClone.h"
#import "ViewItemInfo.h"
#import "ViewPlaylist.h"
#import "ViewCategory.h"
#import "executeMAME.h"
#import "TBArrayController.h"
#import "FileAttributeExtension.h"
#import "SetupAuditStatus.h"
#import "SetupRomStatus.h"
#import "ConvertExtension.h"
#import "DialogExtension.h"
#import "HelperExtension.h"
#import "ThumbsExtension.h"
#import "UserDefaultsExtension.h"

@implementation ViewMain

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// window rect
	[self setWindowRect:mainWindow key:@"rectWindowMain"];
	// split view rect
	[self setSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self setSplitViewRect:spvMainNavi key:@"rectSplitViewMainNavi"];
	[self setSplitViewRect:spvMainHead key:@"rectSplitViewMainHead"];

	[self setSearchString:@""];
	[self setSearchPred:nil];
	[self setPlistUniqueId:@""];
	[self setPlistItemType:0];
	
	[self setPlistIsFolder:NO];
	[self setPlistPred:nil];
	[self setClistPred:nil];
	[self setItemsPred:nil];
	[self setItemsTitle:@""];
	[self setWindowTitle:[self defaultAppName]];

	// set first responder
	id listView = [[spvMain subviews] objectAtIndex:1];
	[mainWindow makeFirstResponder:listView];
//	[mainWindow makeFirstResponder:txtSearchField];
	
	// set key equivalent
	[btnPlay setKeyEquivalent:@" "];
	[btnShowInfo setKeyEquivalent:@" "];
	[btnShowInfo setKeyEquivalentModifierMask:NSAlternateKeyMask];
	[btnSearchClone setKeyEquivalent:@"\r"];
	[btnSearchClone setKeyEquivalentModifierMask:NSAlternateKeyMask];

	[self changeTabViewMainResultIndex];
	[self changeTabViewMainSliderIndex];
	[self changeTabViewMainStatusIndex];
	[self changeTabViewMainButtonEnable];

	// add observer
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults addObserver:self forKeyPath:@"optShowRomsType" options:0 context:nil];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(mainWindowWillClose:) name:NSWindowWillCloseNotification object:mainWindow];


	// search
	[self searchWithRomsType:[self defaultShowRomsType]];

}
//=======================================================================
// Actions
//=======================================================================
//------------------------------------
// playItem
//------------------------------------
- (IBAction)playItem:(id)sender
{
	[self playSelectedItem];
}
//------------------------------------
// showItemInfo
//------------------------------------
- (IBAction)showItemInfo:(id)sender
{
	// no select
	if([[romlistArrayController selectedObjects] count] <= 0){
		return;
	}

	id record = [[romlistArrayController selectedObjects] objectAtIndex:0];
	if([viewItemInfo createItemInfoWithObject:record arrayNo:BIND_ARRAY_ROM showDefault:YES] == YES){
		[viewItemInfo openInfoWindow:nil];
	}
}
//------------------------------------
// showCloneItem
//------------------------------------
- (IBAction)showCloneItem:(id)sender
{

	NSArray *items = [romlistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	id record = [items objectAtIndex:0];

	// get defaultSet
	BOOL isClone;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	NSString *romId = [record valueForKey:@"romId"];
	NSString *defaultRomId;
	// clone
	if(isClone == YES){
		defaultRomId = [defaultSet valueForKey:@"cloneId"];
	}else{
		defaultRomId = [defaultSet valueForKey:@"romId"];
	}

	[viewClone searchWithParentId:romId cloneId:defaultRomId];
	[viewClone openCloneWindow:nil];

}
//------------------------------------
// changeInfoWindow
//------------------------------------
- (IBAction)changeInfoWindow:(id)sender
{
	// not visible
	if([viewItemInfo isVisibleInfoWindow] == NO){
		[self showItemInfo:nil];
	}else{
		// not key window
		if([viewItemInfo isKeyInfoWindow] == NO){
			[self showItemInfo:nil];
		}
		// key window
		else{
			[viewItemInfo closeInfoWindow:nil];
		}
	}
}
//------------------------------------
// changeCloneWindow
//------------------------------------
- (IBAction)changeCloneWindow:(id)sender
{
	// not visible
	if([viewClone isVisibleCloneWindow] == NO){
		[self showCloneItem:nil];
	}else{
		// not key window
		if([viewClone isKeyCloneWindow] == NO){
			[self showCloneItem:nil];
		}
		// key window
		else{
			[viewClone closeCloneWindow:nil];
		}
	}
}

//------------------------------------
// updateItemAuditStatus
//------------------------------------
- (IBAction)updateItemAuditStatus:(id)sender;
{

	NSArray *items = [romlistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	// get record
	id record = [items objectAtIndex:0];

	// get defaultSet
	BOOL isClone;
	NSString *curRomId;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// clone
	if(isClone == YES){
		curRomId = [defaultSet valueForKey:@"cloneId"];
	}else{
		curRomId = [defaultSet valueForKey:@"romId"];
	}

	// update audit status
	[setupAuditStatus updateAuditStatusWithRomId:curRomId isParent:!isClone];

}
//------------------------------------
// updateItemFileStatus
//------------------------------------
- (IBAction)updateItemFileStatus:(id)sender
{

	NSArray *items = [romlistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	// update file status
	[setupRomStatus updateFileStatus:items
							keyRomId:@"romId"
							isParent:YES
							isUpdateDefault:YES
	];

}
//------------------------------------
// showItemInFinder
//------------------------------------
- (IBAction)showItemInFinder:(id)sender
{

	NSArray *items = [romlistArrayController selectedObjects];

	// no select
	if([items count] <= 0){
		return;
	}

	// get record
	id record = [items objectAtIndex:0];

	// get defaultSet
	BOOL isClone;
	NSString *curRomId;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// clone
	if(isClone == YES){
		curRomId = [defaultSet valueForKey:@"cloneId"];
	}else{
		curRomId = [defaultSet valueForKey:@"romId"];
	}

	NSString *filePath = [self createCurrentRomPath:curRomId];

	[self showFileInFinder:filePath];

}
//------------------------------------
// searchItemWithWebSearch
//------------------------------------
- (IBAction)searchItemWithWebSearch:(id)sender
{

	id object = [sender representedObject];

	// get value from sender
	NSString *url = [object valueForKey:@"url"];
	int keywordType = [[object valueForKey:@"keywordType"] intValue];
	BOOL isTrim = [[object valueForKey:@"isTrim"] boolValue];

	// no url
	if([url isEqualToString:@""]){
		return;
	}

	NSArray *items = [romlistArrayController selectedObjects];
	// no item
	if([items count] <= 0){
		return;
	}

	// get record
	id record = [items objectAtIndex:0];

	// get defaultSet
	BOOL isClone;
	NSString *curRomId;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// clone
	if(isClone == YES){
		curRomId = [defaultSet valueForKey:@"cloneId"];
	}else{
		curRomId = [defaultSet valueForKey:@"romId"];
	}

	NSString *title = [defaultSet valueForKey:@"title"];
	NSString *titlejp = [defaultSet valueForKey:@"titlejp"];

	// search item
	[self searchItemWithParams:[NSDictionary dictionaryWithObjectsAndKeys:
										[NSNumber numberWithInt:keywordType], @"keywordType",
										url, @"url",
										curRomId, @"romId",
										title, @"title",
										titlejp, @"titlejp",
										[NSNumber numberWithBool:isTrim], @"isTrim",
										nil
									]
	];

}
//------------------------------------
// addItemToPlaylist
//------------------------------------
- (IBAction)addItemToPlaylist:(id)sender
{

	NSManagedObject *targetItem = [sender representedObject];

	NSArray *items = [romlistArrayController selectedObjects];
 
	[tbArrayController addItemsToPlaylist:targetItem items:items];

}
//------------------------------------
// removeItem
//------------------------------------
- (IBAction)removeItem:(id)sender
{

	// remove from playlist
	if([self isSearchWithPlaylist] == YES){
		[self removeItemFromPlaylist:nil];
	}

}

//------------------------------------
// removeItemFromPlaylist
//------------------------------------
- (IBAction)removeItemFromPlaylist:(id)sender
{

	NSArray *items = [romlistArrayController selectedObjects];

	int count = [items count];
	// no select
	if(count <= 0){
		return;
	}

	// not result from playlist
	if([self isSearchWithPlaylist] == NO){
		return;
	}

	// get playllist
	NSPredicate *pred = [NSPredicate predicateWithFormat:@"uniqueId == %@", [self plistUniqueId]];
	NSArray *fetchedArray = [tbArrayController getObjectsWithPred:@"playlist" pred:pred];

	// not found
	if([fetchedArray count] <= 0){
		[self displayMessage:@"alert"
							messageText:@"Playlist is not found"
							infoText:@""
							btnList:@"Cancel"
		];
		return;
	}

	id selectedItem = [fetchedArray objectAtIndex:0];
	NSString *title = [selectedItem valueForKey:@"title"];

	NSString *message = @"";
	if(count == 1){
//		message = [NSString stringWithFormat:@"Delete %d item from Playlist?", count];
		message = [NSString stringWithFormat:@"Delete %d item from \"%@\"?", count, title];
	}else{
		message = [NSString stringWithFormat:@"Delete %d items from \"%@\"?", count, title];
	}

	// confirm
	int result = [self displayMessageWithIcon:@"icon_alert_playlist"
							messageText:message
							infoText:@"Delete from only playlist."
							btnList:@"Delete,Cancel"
				];

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove relation from playlist
	[tbArrayController removeItemsFromPlaylist:selectedItem items:items];

	// reload
	[romlistArrayController rearrangeObjects];	
//	[self searchWithPredicate:YES withPlist:YES withClist:YES withItems:YES];

}

//------------------------------------
// changeTabViewMainIndex
//------------------------------------
- (IBAction)changeTabViewMainIndex:(id)sender
{
	int index = [sender tag];
	[self setDefaultIntValue:index key:@"optTabViewMainIndex"];
	[self changeTabViewMainResultIndex];
	[self changeTabViewMainSliderIndex];
	[self changeTabViewMainStatusIndex];
	[self changeTabViewMainButtonEnable];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// playSelectedItem
//------------------------------------
- (void)playSelectedItem
{
	// no select
	if([[romlistArrayController selectedObjects] count] != 1){
		return;
	}

	id record = [[romlistArrayController selectedObjects] objectAtIndex:0];

	// get defaultSet
	BOOL isClone;
	NSString *romId;
	NSString *parentId;
	id defaultSet = [self getDefaultRomSet:record isClone:&isClone];

	// clone
	if(isClone == YES){
		romId = [defaultSet valueForKey:@"cloneId"];
		parentId = [defaultSet valueForKey:@"parentId"];
	}else{
		romId = [defaultSet valueForKey:@"romId"];
		parentId = [defaultSet valueForKey:@"romId"];
	}
	NSString *title = [self getTitleString:[self defaultLangType]
							title:[defaultSet valueForKey:@"title"]
							titlejp:[defaultSet valueForKey:@"titlejp"]
					];

	// create params
	int mameNo = [self defaultExecuteMameNo];
	NSDictionary *playParams = [NSDictionary dictionaryWithObjectsAndKeys:
									[NSNumber numberWithInt:mameNo], @"mameNo",
									romId, @"romId",
									parentId, @"parentId",
									title, @"title",
									[NSNumber numberWithBool:isClone], @"isClone",
									[NSNumber numberWithInt:ITEM_TYPE_MAIN], @"itemType",
									nil
								];
	// playItem
	[executeMAME playItem:playParams];
}
//------------------------------------
// searchWithString
//------------------------------------
- (void)searchWithString:(NSString*)searchString pred:(NSPredicate*)pred
{

	[self setSearchString:searchString];
	[self setSearchPred:pred];

	// set pred
	[self searchWithPredicate:YES withPlist:YES withClist:YES withItems:YES];

}
//------------------------------------
// searchWithPlaylist
//------------------------------------
- (void)searchWithPlaylist:(NSString*)uniqueId
					title:(NSString*)title
					itemType:(int)itemType
					isFolder:(BOOL)isFolder
					pred:(NSPredicate*)pred
{

	[self setPlistUniqueId:uniqueId];
	[self setPlistItemType:itemType];
	[self setPlistIsFolder:isFolder];
	[self setPlistPred:pred];

	// set window title
	if([uniqueId isEqualToString:@""]){
		[self setWindowTitle:[self defaultAppName]];
	}else{
		[self setWindowTitle:title];
	}

	// set pred
	[self searchWithPredicate:YES withPlist:YES withClist:NO withItems:YES];

}
//------------------------------------
// searchWithCategorylist
//------------------------------------
- (void)searchWithCategorylist:(NSString*)title pred:(NSPredicate*)pred
{

	[self setClistPred:pred];

	// set window title
	if([title isEqualToString:@""]){
		[self setWindowTitle:[self defaultAppName]];
	}else{
		[self setWindowTitle:title];
	}

	// set pred
	[self searchWithPredicate:YES withPlist:NO withClist:YES withItems:YES];

}
//------------------------------------
// searchWithRomsType
//------------------------------------
- (void)searchWithRomsType:(int)romsType
{

	NSPredicate *pred = nil;
	NSString *title = @"";

	if(romsType == DEFAULT_ROMS_TYPE_AVAILABLE){
		pred = [NSPredicate predicateWithFormat:@"isExist == YES"];
		title = @"Available Roms";
	}
	else if(romsType == DEFAULT_ROMS_TYPE_MISSING){
		pred = [NSPredicate predicateWithFormat:@"isExist == NO"];
		title = @"Missing Roms";
	}

	// set pred
	[self setItemsPred:pred];
	[self setItemsTitle:title];
	[self searchWithPredicate:YES withPlist:YES withClist:YES withItems:YES];

}
//------------------------------------
// searchWithItems
//------------------------------------
- (void)searchWithItems:(NSArray*)items title:(NSString*)title
{

	NSPredicate *pred = nil;

	if([items count] > 0){
		pred = [NSPredicate predicateWithFormat:@"defaultRomId IN %@", items];
	}

	// set pred
	[self setItemsPred:pred];
	[self setItemsTitle:title];
	[self searchWithPredicate:YES withPlist:YES withClist:YES withItems:YES];

}
//------------------------------------
// searchWithPredicate
//------------------------------------
- (void)searchWithPredicate:(BOOL)withString
					withPlist:(BOOL)withPlist
					withClist:(BOOL)withClist
					withItems:(BOOL)withItems
{

	NSMutableArray *predItems = [[[NSMutableArray alloc] init] autorelease];
	NSString *windowTitle = [self windowTitle];

	// clear pred
	if(withString == NO){
		[txtSearchField setStringValue:@""];
		[self setSearchString:@""];
		[self setSearchPred:nil];
	}
	if(withPlist == NO){
		[viewPlaylist deselectItem:nil];
		[self setPlistUniqueId:@""];
		[self setPlistItemType:0];
		[self setPlistPred:nil];
	}
	if(withClist == NO){
		[viewCategory deselectItem:nil];
		[self setClistPred:nil];
	}
	if(withItems == NO){
		[self setItemsPred:nil];
	}

	NSPredicate *pred = nil;
	NSPredicate *searchPred = [self searchPred];
	NSPredicate *plistPred = [self plistPred];
	NSPredicate *clistPred = [self clistPred];
	NSPredicate *itemsPred = [self itemsPred];

	// add preds 
	if(searchPred != nil){
		[predItems addObject:searchPred];
	}
	if(plistPred != nil){
		[predItems addObject:plistPred];
	}
	if(clistPred != nil){
		[predItems addObject:clistPred];
	}
	// items pred
	if(itemsPred != nil){
		[predItems addObject:itemsPred];
		windowTitle = [windowTitle stringByAppendingFormat:@" [%@]", [self itemsTitle]];
	}

	// create pred
	if([predItems count] > 0){
		pred = [NSCompoundPredicate andPredicateWithSubpredicates:predItems];
	}

//	NSLog(@"pred=%@", [pred description]);

	// window title
	[mainWindow setTitle:windowTitle];

	// filter
	[self handleProcStatusChanged:PROC_IND_STATUS_START];

	[romlistArrayController setFilterPredicate:pred];

	[self handleProcStatusChanged:PROC_IND_STATUS_STOP];

}

//------------------------------------
// changeTabViewMainResultIndex
//------------------------------------
- (void)changeTabViewMainResultIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainResult selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainSliderIndex
//------------------------------------
- (void)changeTabViewMainSliderIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainSlider selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainStatusIndex
//------------------------------------
- (void)changeTabViewMainStatusIndex
{
	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	[tabViewMainStatus selectTabViewItemAtIndex:index];

}
//------------------------------------
// changeTabViewMainButtonEnable
//------------------------------------
- (void)changeTabViewMainButtonEnable
{

	int index = [self defaultIntValue:@"optTabViewMainIndex"];

	NSString *btnTableName = @"btn_view_table_off";
	NSString *btnGrigName = @"btn_view_grid_off";
	BOOL tableEnabled = NO;
	BOOL gridEnabled = NO;

	if(index == 0){
		gridEnabled = YES;
		btnTableName = @"btn_view_table_on";
	}
	else if(index == 1){
		tableEnabled = YES;
		btnGrigName = @"btn_view_grid_on";
	}

	[btnTabViewMainTable setImage:[NSImage imageNamed:btnTableName]];
	[btnTabViewMainGrid setImage:[NSImage imageNamed:btnGrigName]];

}
//------------------------------------
// handleProcStatusChanged
//------------------------------------
- (void)handleProcStatusChanged:(int)status
{
	if(status == PROC_IND_STATUS_START){
		[indProc startAnimation:nil];
	}
	else{
		[indProc stopAnimation:nil];
	}
}
//------------------------------------
// observeValueForKeyPath
//------------------------------------
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
	if([keyPath isEqualToString:@"optShowRomsType"]){
		[self searchWithRomsType:[self defaultShowRomsType]];
	}
	
}
//------------------------------------
// mainWindowWillClose
//------------------------------------
-(void)mainWindowWillClose:(NSNotification *)notification
{

	// save state from user default
	// window rect
	[self saveWindowRect:mainWindow key:@"rectWindowMain"];

	// split view rect
	[self saveSplitViewRect:spvMain key:@"rectSplitViewMain"];
	[self saveSplitViewRect:spvMainNavi key:@"rectSplitViewMainNavi"];
	[self saveSplitViewRect:spvMainHead key:@"rectSplitViewMainHead"];

}
//----------------------------------------
// windowTitle
//----------------------------------------
- (void)setWindowTitle:(NSString*)windowTitle
{
	[windowTitle retain];
	[windowTitle_ release];
	windowTitle_ = windowTitle;
}
- (NSString*)windowTitle
{
	return windowTitle_;
}
//------------------------------------
// searchString
//------------------------------------
- (void)setSearchString:(NSString*)searchString
{
	[searchString retain];
	[searchString_ release];
	searchString_ = searchString;

}
- (NSString*)searchString
{
	return searchString_;
}
//------------------------------------
// searchPred
//------------------------------------
- (void)setSearchPred:(NSPredicate*)searchPred
{
	[searchPred retain];
	[searchPred_ release];
	searchPred_ = searchPred;

}
- (NSPredicate*)searchPred
{
	return searchPred_;
}
//------------------------------------
// plistUniqueId
//------------------------------------
- (void)setPlistUniqueId:(NSString*)plistUniqueId
{
	[plistUniqueId retain];
	[plistUniqueId_ release];
	plistUniqueId_ = plistUniqueId;

}
- (NSString*)plistUniqueId
{
	return plistUniqueId_;
}
//------------------------------------
// plistItemType
//------------------------------------
- (void)setPlistItemType:(int)plistItemType
{
	plistItemType_ = plistItemType;

}
- (int)plistItemType
{
	return plistItemType_;
}
//------------------------------------
// plistIsFolder
//------------------------------------
- (void)setPlistIsFolder:(BOOL)plistIsFolder
{
	plistIsFolder_ = plistIsFolder;

}
- (BOOL)plistIsFolder
{
	return plistIsFolder_;
}
//------------------------------------
// plistPred
//------------------------------------
- (void)setPlistPred:(NSPredicate*)plistPred
{
	[plistPred retain];
	[plistPred_ release];
	plistPred_ = plistPred;

}
- (NSPredicate*)plistPred
{
	return plistPred_;
}
//------------------------------------
// clistPred
//------------------------------------
- (void)setClistPred:(NSPredicate*)clistPred
{
	[clistPred retain];
	[clistPred_ release];
	clistPred_ = clistPred;

}
- (NSPredicate*)clistPred
{
	return clistPred_;
}
//------------------------------------
// itemsPred
//------------------------------------
- (void)setItemsPred:(NSPredicate*)itemsPred
{
	[itemsPred retain];
	[itemsPred_ release];
	itemsPred_ = itemsPred;

}
- (NSPredicate*)itemsPred
{
	return itemsPred_;
}
//------------------------------------
// itemsTitle
//------------------------------------
- (void)setItemsTitle:(NSString*)itemsTitle
{
	[itemsTitle retain];
	[itemsTitle_ release];
	itemsTitle_ = itemsTitle;

}
- (NSString*)itemsTitle
{
	return itemsTitle_;
}
//------------------------------------
// canRemoveItemFromPlaylist
//------------------------------------
- (BOOL)canRemoveItemFromPlaylist
{
	if( [self isSearchWithPlaylist] == YES &&
		[[romlistArrayController selectedObjects] count] > 0){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
//------------------------------------
// isSearchWithPlaylist
//------------------------------------
- (BOOL)isSearchWithPlaylist
{
	if(![[self plistUniqueId] isEqualToString:@""] &&
		[self plistIsFolder] == NO &&
		[self plistItemType] == ITEM_PLAYLIST){
		return YES;
	}else{
		return NO;
	}

}

//------------------------------------
// hasPreviewRomId
//------------------------------------
- (BOOL)hasPreviewRomId
{
	if([imagePreview hasRomId] == YES){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// hasPreviewImage
//------------------------------------
- (BOOL)hasPreviewImage
{
	if([imagePreview hasImage] == YES){
		return YES;
	}else{
		return NO;
	}

}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	[defaults removeObserver:self forKeyPath:@"optShowRomsType"];

	[searchString_ release];
	[searchPred_ release];
	[plistUniqueId_ release];
	[plistPred_ release];
	[clistPred_ release];
	[itemsPred_ release];
	[itemsTitle_ release];
	[windowTitle_ release];

	[super dealloc];
}
@end

