#import "ViewPlaylist.h"
#import "ViewMain.h"
#import "ViewCategory.h"
#import "SetupPlaylist.h"
#import "TBArrayController.h"
#import "SearchExtension.h"
#import "HelperExtension.h"
#import "ConvertExtension.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewPlaylist

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	prevUniqueId_ = @"";

	// set display setting
	[self setControlButtonEnable];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(frameDidResize:) name:NSViewFrameDidChangeNotification object:[olvPlaylist enclosingScrollView]];
	[nc addObserver:self selector:@selector(handleCategoryTabIdDidChanged:) name:CATEGORY_NOTIF_TAB_ID_DID_CHANGED object:nil];


}
//=======================================================================
// action methods
//=======================================================================
//------------------------------------
// addPlaylist
//------------------------------------
- (IBAction)addPlaylist:(id)sender
{

	// add item
	[self addItem:ITEM_PLAYLIST
				itemSubType:0
				title:@"New Playlist"
				value:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
				isSearch:NO
	];

}
//------------------------------------
// addSearchlist
//------------------------------------
- (IBAction)addSearchlist:(id)sender
{

	// add item
	[self addItem:ITEM_SEARCH
				itemSubType:ITEM_SEARCH_KEYWORD
				title:@"New Keyword"
				value:@""
				isFolder:NO
				isSelect:YES
				isEdit:YES
				isSearch:NO
	];

}
//------------------------------------
// addFolder
//------------------------------------	  
- (IBAction)addFolder:(id)sender
{

	// add folder item
	[self addItem:ITEM_FOLDER
				itemSubType:0
				title:@"New Folder"
				value:@""
				isFolder:YES
				isSelect:YES
				isEdit:YES
				isSearch:NO
	];

}
//------------------------------------
// searchItem
//------------------------------------
- (IBAction)searchItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	NSString *uniqueId = @"";

	// selected
	if(selectedItem != nil){
		uniqueId = [selectedItem valueForKey:@"uniqueId"];
		// same playlist
		if([uniqueId isEqualToString:prevUniqueId_]){
			return;
		}
	}

	// search
	[self searchItemWithItem:selectedItem];

}
//------------------------------------
// searchWithSelectedItem
//------------------------------------
- (IBAction)searchWithSelectedItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];

	// search
	[self searchItemWithItem:selectedItem];

}
//------------------------------------
// searchWithDeselectItem
//------------------------------------
- (IBAction)searchWithDeselectItem:(id)sender
{
	// deselect
	[self deselectItem:nil];
	// search
	[self searchItem:nil];
}
//------------------------------------
// removeItem
//------------------------------------	  
- (IBAction)removeItem:(id)sender
{

	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];
	// no select
	if(!selectedItem){
		return;
	}

	BOOL isFolder = [[selectedItem valueForKey:@"isFolder"] boolValue];
	int itemType = 0;
	if([selectedItem valueForKey:@"itemType"]){
		itemType = [[selectedItem valueForKey:@"itemType"] intValue];
	}

	int result;

	// Alert before remove
	// folder
	if(isFolder == YES){
		result = [self displayMessageWithIcon:@"icon_alert_folder"
											messageText:@"Delete Folder?"
											infoText:@"Delete all playlists in folder."
											btnList:@"Delete,Cancel"
											];
	}
	else{
		// playlist
		if(itemType == ITEM_PLAYLIST){
			result = [self displayMessageWithIcon:@"icon_alert_playlist"
												messageText:@"Delete Playlist?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		// search
		else if(itemType == ITEM_SEARCH){
			result = [self displayMessageWithIcon:@"icon_alert_search"
												messageText:@"Delete Search Keyword?"
												infoText:@""
												btnList:@"Delete,Cancel"
												];
		}
		else{
			return;
		}
	}

	if(result != NSAlertFirstButtonReturn){
		return;
	}

	// remove tree of selectedItem
	[tbArrayController removeTreeItems:selectedItem];

	// reload
	[playlistTreeController rearrangeObjects];

	// searchWithDeselectItem
	[self searchWithDeselectItem:nil];

	// deselect
//	[self deselectItem:nil];
	// search
//	[self searchItem:nil];

}
//------------------------------------
// exportItem
//------------------------------------
- (IBAction)exportItem:(id)sender
{
	// get selected item
	NSManagedObject *selectedItem = [olvPlaylist selectedObservedObject];

	// no select
	if(!selectedItem){
		return;
	}

	NSString *title = [selectedItem valueForKey:@"title"];

	[setupPlaylist exportPlaylistWithItem:selectedItem title:title rootFolder:NO];

}
//------------------------------------
// showItem
//------------------------------------
- (IBAction)showItem:(id)sender
{

	// select tabViePlaylist
	if([self isSelectTabViewPlaylist] == NO){
		[viewCategory selectCategory:CATEGORY_TYPE_PLAYLIST];
	}

	// get object from representedObject
	NSManagedObject *targetItem = [sender representedObject];
	// deselect
	if(targetItem == nil){
		// searchWithDeselectItem
		[self searchWithDeselectItem:nil];
//		[self deselectItem:nil];
//		[self searchItem:nil];
		return;
	}

	id parentItem = [targetItem valueForKey:@"parent"];
	NSMutableArray *parentItems = [[NSMutableArray alloc] init];

	// add parentItems until parent is null
	while(parentItem){
		[parentItems addObject:parentItem];
		parentItem = [parentItem valueForKey:@"parent"];
	}

	int i;
	int numberOfRows = [olvPlaylist numberOfRows];

	for(i = 0; i < numberOfRows; i++){
		id item = [olvPlaylist itemAtRow:i];
		id columnItem = [item observedObject];
		// folder
		if([[columnItem valueForKey:@"isFolder"] boolValue] == YES){
			// expand
			if([parentItems containsObject:columnItem]){
				[olvPlaylist expandItem:item expandChildren:NO];
				numberOfRows = [olvPlaylist numberOfRows];
			}
		}
		// select row
		if(targetItem == columnItem){
			[olvPlaylist selectRowIndexes:[NSIndexSet indexSetWithIndex:i] byExtendingSelection:NO];
			// search
			[self searchItemWithItem:columnItem];
			break;
		}

	}

	[parentItems release];

}

//------------------------------------
// editItem
//------------------------------------
- (IBAction)editItem:(id)sender
{

	[olvPlaylist editSelectedItem];

}
//------------------------------------
// deselectItem
//------------------------------------
- (IBAction)deselectItem:(id)sender
{
	[olvPlaylist deselectItem];
	prevUniqueId_ = @"";
}

//------------------------------------
// nullAction
//------------------------------------
- (IBAction)nullAction:(id)sender
{
	// null action
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// addItem
//------------------------------------
- (void)addItem:(int)itemType
			itemSubType:(int)itemSubType
			title:(NSString*)title
			value:(NSString*)value
			isFolder:(BOOL)isFolder
			isSelect:(BOOL)isSelect
			isEdit:(BOOL)isEdit
			isSearch:(BOOL)isSearch
{


	// get selected item
	id selectedRowItem = [olvPlaylist selectedRowObject];
	NSManagedObject *selectedItem = nil;
	if(selectedRowItem){
		selectedItem = [olvPlaylist selectedObservedObject];
	}

	// create item
	id item = [tbArrayController insertPlaylist:0
									itemType:itemType
									itemSubType:itemSubType
									title:title
									value:value
									labelColorNo:0
									isFolder:isFolder
									parentItem:nil
				];

	// add item to patent of selected item
	[tbArrayController addItemToParent:selectedItem item:item maxNum:YES];			

	// reload
	[playlistTreeController rearrangeObjects];

	// expand selectedRowItem
	if(selectedItem){
		if([[selectedItem valueForKey:@"isFolder"] boolValue] ==YES){
			[olvPlaylist expandItem:selectedRowItem expandChildren:YES];
		}
	}

	// select add item
	if(isSelect == YES){
		[olvPlaylist selectItem:item];
	}

	// edit selected row
	if(isEdit == YES){
		[olvPlaylist editSelectedItem];
	}

	// search item
	if(isSearch == YES){
		[self searchItemWithItem:item];
	}
}
//------------------------------------
// searchItemWithItem
//------------------------------------
- (void)searchItemWithItem:(NSManagedObject*)item
{

	NSPredicate *pred = nil;
	NSString *uniqueId = @"";
	NSString *title = @"";
	NSString *value = @"";
	BOOL isFolder = NO;
	int itemType = 0;
	int itemSubType = 0;
	NSString *searchTitle = @"";

	if(item != nil){
		if([item valueForKey:@"itemType"]){
			itemType = [[item valueForKey:@"itemType"] intValue];
		}
		if([item valueForKey:@"itemSubType"]){
			itemSubType = [[item valueForKey:@"itemSubType"] intValue];
		}
		if([item valueForKey:@"value"]){
			value = [item valueForKey:@"value"];
		}

		uniqueId = [item valueForKey:@"uniqueId"];
		title = [item valueForKey:@"title"];
		isFolder = [[item valueForKey:@"isFolder"] boolValue];
		searchTitle = title;

		// folder / playlist
		if(itemType == ITEM_FOLDER || itemType == ITEM_PLAYLIST){
			NSSet *uniqueIdSet = [NSSet setWithArray:[[tbArrayController getTreeItems:item] valueForKey:@"uniqueId"]];
			pred = [NSPredicate predicateWithFormat:@"ANY rlt_playlist.uniqueId IN %@", uniqueIdSet];
		}
		// search
		else if(itemType == ITEM_SEARCH){
			// keyword
			if(itemSubType == ITEM_SEARCH_KEYWORD){
				NSString *searchFormatString = @"";
				NSString *searchFormatTitle = @"";
				// search value columns
				if(![value isEqualToString:@""]){
					searchFormatString = [self createSearchFormat:value
															items:[self searchColumnItems]
															title:&searchFormatTitle
										];
//					searchTitle = [NSString stringWithFormat:@"%@ : %@", searchFormatTitle, title];
				}
				// search all columns
				if([searchFormatString isEqualToString:@""]){
					searchFormatString = [self createSearchFormatAll:[self searchColumnItems]];
				}
				pred = [self createSearchStringPred:title searchFormatString:searchFormatString];
			}
		}
	}

	prevUniqueId_ = uniqueId;

	// search
	[viewMain searchWithPlaylist:uniqueId
							title:searchTitle
							itemType:itemType
							isFolder:isFolder
							pred:pred
	];

	// select tabViePlaylist
	if([self isSelectTabViewPlaylist] == NO){
		[viewCategory selectCategory:CATEGORY_TYPE_PLAYLIST];
	}

}
//------------------------------------
// addItemToPlaylistWithParams
//------------------------------------
- (void)addItemToPlaylistWithParams:(NSDictionary*)params
{

	id targetItem = [params valueForKey:@"targetItem"];
	NSArray *items = [params valueForKey:@"items"];

	[tbArrayController addItemsToPlaylist:targetItem items:items];

}

//------------------------------------
// setControlButton
//------------------------------------
- (void)setControlButtonEnable
{

	if([self isSelectTabViewPlaylist] == YES){

		BOOL canAdd = [playlistTreeController canAdd];
		int selectedRow = [olvPlaylist selectedRow];
		BOOL isDisplay = [olvPlaylist isDisplay];

		if(canAdd == YES && isDisplay == YES){
			[btnAdd setEnabled:YES];
		}else{
			[btnAdd setEnabled:NO];
		}

		if(selectedRow >= 0 && isDisplay == YES){
			[btnRemove setEnabled:YES];
		}else{
			[btnRemove setEnabled:NO];
		}

		[btnMenuCategory setEnabled:isDisplay];

	}else{
		[btnAdd setEnabled:NO];
		[btnRemove setEnabled:NO];
	}

}
//------------------------------------
// frameDidResize
//------------------------------------
-(void)frameDidResize:(NSNotification *)notification
{
	[self setControlButtonEnable];
}
//------------------------------------
// handleCategoryTabIdDidChanged
//------------------------------------
-(void)handleCategoryTabIdDidChanged:(NSNotification *)notification
{
	[self setControlButtonEnable];
}

//--------------------------------------
// isSelectItem
//--------------------------------------
- (BOOL)isSelectItem
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == NO){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectFolder
//--------------------------------------
- (BOOL)isSelectFolder
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if([[item valueForKey:@"isFolder"] boolValue] == YES){
			ret = YES;
		}
	}

	return ret;
}
//--------------------------------------
// isSelectSearchlist
//--------------------------------------
- (BOOL)isSelectSearchlist
{
	BOOL ret = NO;

	// get selected item
	NSManagedObject *item = [olvPlaylist selectedObservedObject];
	if(item){
		if( [[item valueForKey:@"isFolder"] boolValue] == NO &&
			[[item valueForKey:@"itemType"] intValue] == ITEM_SEARCH
		){
			ret = YES;
		}
	}

	return ret;
}

//------------------------------------
// isKeyMainWindow
//------------------------------------
- (BOOL)isKeyMainWindow
{
	return [mainWindow isKeyWindow];
}
//------------------------------------
// canAddPlaylist
//------------------------------------
- (BOOL)canAddPlaylist
{
	if( [self isKeyMainWindow] == YES &&
		[self isDisplayPlaylist] == YES &&
		[self isSelectTabViewPlaylist] == YES 
		){
		return YES;
	}else{
		return NO;
	}
}
//------------------------------------
// isDisplayPlaylist
//------------------------------------
- (BOOL)isDisplayPlaylist
{
	return [olvPlaylist isDisplay];
}
//------------------------------------
// isSelectTabViewPlaylist
//------------------------------------
- (BOOL)isSelectTabViewPlaylist
{
	return [viewCategory isSelectTabViewPlaylist];
}

//----------------------
// dealloc
//----------------------
- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[prevUniqueId_ release];
	[super dealloc];
}
@end
