#import "ViewPrefs.h"
#import "UserDefaultsExtension.h"
#import "DialogExtension.h"

@implementation ViewPrefs

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{

	//
	// set state from user default
	//
	// window position
	[self setWindowPosition:prefsWindow key:@"rectWindowPrefs"];
	[self setWindowPosition:helpWindow key:@"rectWindowHelp"];

	// alloc romsFolderList
	romsFolderList_ = [[NSMutableArray alloc] init];
	// add converted array from defaults
	[romsFolderList_ addObjectsFromArray:[self convertToRomsFolderItems:[self defaultArrayValue:@"pathRomsFolderItems"]]];

	[romsFolderArrayController setContent:romsFolderList_];
	[romsFolderArrayController rearrangeObjects];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:prefsWindow];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:helpWindow];

}
//=======================================================================
// Event Actions
//=======================================================================
//------------------------------------
// openPrefsWindow
//------------------------------------
- (IBAction)openPrefsWindow:(id)sender
{
	[prefsWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// openPrefsWindowWithSender
//------------------------------------
- (IBAction)openPrefsWindowWithSender:(id)sender
{
	if([sender representedObject]){
		int tabIndex = [[sender representedObject] intValue];
		[tabViewPrefs selectTabViewItemAtIndex:tabIndex];
	}
	[prefsWindow makeKeyAndOrderFront:self];
}

//------------------------------------
// openHelpWindow
//------------------------------------
- (IBAction)openHelpWindow:(id)sender
{
	[helpWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// changeWindowTheme
//------------------------------------
- (IBAction)changeWindowTheme:(id)sender
{
	[mainWindow changeWindowTheme:nil];
	[cloneWindow changeWindowTheme:nil];
	[logWindow changeWindowTheme:nil];
	
}
//------------------------------------
// selectFilePath
//------------------------------------
- (IBAction)selectFilePath:(id)sender
{
	int tag = [sender tag];

	// get default file path
	NSString *defaultPath = [self defaultSettingFilePath:tag];
	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	NSArray *filePaths = [self selectFilePathWithOpenPanel:YES
													isDir:NO
													isMultiSel:NO
													isPackage:NO
													isAlias:NO
													canCreateDir:NO
													defaultPath:defaultPath
						];

	if([filePaths count] <= 0){
		return;
	}

	NSString *filePath = [filePaths objectAtIndex:0];

	// set default file path
	[self setDefaultStringValue:filePath key:[self getDefaultFilePathKey:tag]];

}
//------------------------------------
// selectFolderPath
//------------------------------------
- (IBAction)selectFolderPath:(id)sender
{
	int tag = [sender tag];

	// get default file path
	NSString *defaultPath = [self defaultSettingFolderPath:tag];
	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	NSArray *filePaths = [self selectFilePathWithOpenPanel:NO
													isDir:YES
													isMultiSel:NO
													isPackage:NO
													isAlias:NO
													canCreateDir:NO
													defaultPath:defaultPath
						];

	if([filePaths count] <= 0){
		return;
	}

	NSString *filePath = [filePaths objectAtIndex:0];

	// set default file path
	[self setDefaultStringValue:filePath key:[self getDefaultFolderPathKey:tag]];

}
//------------------------------------
// addRomsFolderItem
//------------------------------------
- (IBAction)addRomsFolderItem:(id)sender
{

	// get default file path
	NSString *defaultPath = @"";
//	defaultPath = [defaultPath stringByDeletingLastPathComponent];

	NSArray *filePaths = [self selectFilePathWithOpenPanel:NO
													isDir:YES
													isMultiSel:NO
													isPackage:NO
													isAlias:NO
													canCreateDir:NO
													defaultPath:defaultPath
						];

	if([filePaths count] <= 0){
		return;
	}

	NSString *filePath = [filePaths objectAtIndex:0];

	// add object
	[romsFolderList_ addObject:
		[NSMutableDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithInt:0], @"index" ,
			filePath, @"filePath" ,
			[NSNumber numberWithBool:YES], @"enabled" ,
			nil
		]
	];

	// re-number
	[self renumRomsFolderList];

	// reload
	[romsFolderArrayController rearrangeObjects];

	// save
	[self saveRomsFolderItem:nil];

	// select and editable
//	int col = [tbvRomsFolder columnWithIdentifier:@"path"];
	int row = [romsFolderList_ count] - 1;
	[tbvRomsFolder selectRowIndexes:[NSIndexSet indexSetWithIndex:row] byExtendingSelection:NO];
//	[tbvRomsFolder editColumn:col row:row withEvent:nil select:YES];

}

/*
//------------------------------------
// editRomsFolderItem
//------------------------------------
- (IBAction)editRomsFolderItem:(id)sender
{

	[self saveRomsFolderItem:nil];

}
*/
//------------------------------------
// moveRomsFolderItem
//------------------------------------
- (IBAction)moveRomsFolderItem:(id)sender
{
	// no select
	if([tbvRomsFolder numberOfSelectedRows] <= 0){
		return;
	}

	int tag = [sender tag];

	int selectedIndex = [[tbvRomsFolder selectedRowIndexes] firstIndex];
	int targetIndex = selectedIndex;
	// up
	if(tag == 0){
		targetIndex = selectedIndex - 1;
	}
	// down
	else if(tag == 1){
		targetIndex = selectedIndex + 1;
	}
	// over
	if(targetIndex < 0 || targetIndex > [romsFolderList_ count] - 1){
		return;
	}

	id selectedRecord = [romsFolderList_ objectAtIndex:selectedIndex];
	id targetRecord = [romsFolderList_ objectAtIndex:targetIndex];

	// replace record
	[romsFolderList_ replaceObjectAtIndex:targetIndex withObject:selectedRecord];
	[romsFolderList_ replaceObjectAtIndex:selectedIndex withObject:targetRecord];

	// re-number
	[self renumRomsFolderList];

	// reload
	[romsFolderArrayController rearrangeObjects];

	// save
	[self saveRomsFolderItem:nil];

}
//------------------------------------
// removeRomsFolderItem
//------------------------------------
- (IBAction)removeRomsFolderItem:(id)sender
{
	// no select
	if([tbvRomsFolder numberOfSelectedRows] <= 0){
		return;
	}

	// remove object
	[romsFolderList_ removeObjectsAtIndexes:[tbvRomsFolder selectedRowIndexes]];

	// re-number
	[self renumRomsFolderList];

	// reload
	[romsFolderArrayController rearrangeObjects];

	// save
	[self saveRomsFolderItem:nil];

}

//------------------------------------
// saveRomsFolderItem
//------------------------------------
- (IBAction)saveRomsFolderItem:(id)sender
{
//	NSLog(@"saveRomsFolderItem");

	[self setDefaultArrayValue:[self romsFolderList] key:@"pathRomsFolderItems"];
}
//=======================================================================
// methods
//=======================================================================
//------------------------------------
// convertToRomsFolderItems
//------------------------------------
- (NSMutableArray*)convertToRomsFolderItems:(NSArray*)array
{

	NSMutableArray *muArray = [NSMutableArray array];
	if(array != nil){
		int i;
		id record;
		for(i = 0; i < [array count]; i++){
			record = [array objectAtIndex:i];
			// add object
			[muArray addObject:
				[NSMutableDictionary dictionaryWithObjectsAndKeys:
					[record valueForKey:@"index"], @"index" ,
					[record valueForKey:@"filePath"], @"filePath" ,
					[record valueForKey:@"enabled"], @"enabled" ,
					nil
				]
			];
		}
	}

	return muArray;
}
//------------------------------------
// renumRomsFolderList
//------------------------------------
- (void)renumRomsFolderList
{
	// re-number
	int i;
	id record;
	
	for(i = 0; i < [romsFolderList_ count]; i++){
		record = [romsFolderList_ objectAtIndex:i];
		[record setValue:[NSNumber numberWithInt:i] forKey:@"index"];
	}

}
//------------------------------------
// romsFolderList
//------------------------------------
- (NSMutableArray*)romsFolderList
{
	return romsFolderList_;
}
//------------------------------------
// openPrefsTabWithIdentifier
//------------------------------------
- (void)openPrefsTabWithIdentifier:(NSString*)identifier
{

	if([tabViewPrefs indexOfTabViewItemWithIdentifier:identifier] != NSNotFound){
		[tabViewPrefs selectTabViewItemWithIdentifier:identifier];
	}

	[prefsWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// save window rect
	if([notification object] == prefsWindow){
		[self saveWindowRect:prefsWindow key:@"rectWindowPrefs"];
	}
	else if([notification object] == helpWindow){
		[self saveWindowRect:helpWindow key:@"rectWindowHelp"];
	}
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{	
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[romsFolderList_ release];
	[super dealloc];
}

@end
