#import "ViewProcStatus.h"
#import "UserDefaultsExtension.h"

@implementation ViewProcStatus

//=======================================================================
// awake App
//=======================================================================
- (void)awakeFromNib
{
	// set window rect
	[self setWindowPosition:procStatusWindow key:@"rectWindowProc"];
	[procStatusWindow setLevel:NSStatusWindowLevel];
	[procStatusWindow center];

	// init value
	[self setShowCount:NO];
	[self setTotalCount:0];
	[self setCloseTimer:nil];

	[txtProcStatus setStringValue:@""];
	[txtTargetName setStringValue:@""];

	[progProcStatus setUsesThreadedAnimation:YES];

	// set notification
	NSNotificationCenter *nc=[NSNotificationCenter defaultCenter];
	[nc addObserver:self selector:@selector(windowWillClose:) name:NSWindowWillCloseNotification object:procStatusWindow];

}

//------------------------------------
// openProcStatusWindow
//------------------------------------
- (IBAction)openProcStatusWindow:(id)sender;
{
	[procStatusWindow makeKeyAndOrderFront:self];
}
//------------------------------------
// closeProcStatusWindow
//------------------------------------
- (IBAction)closeProcStatusWindow:(id)sender;
{
	[procStatusWindow close];

}

//------------------------------------
// initProcStatus
//------------------------------------
- (void)initProcStatus:(NSString*)title
					targetName:(NSString*)targetName
					procCount:(int)procCount
					totalCount:(int)totalCount
					showCount:(BOOL)showCount
					isOpen:(BOOL)isOpen
{
	// process status
	[self setTotalCount:totalCount];
	[self setShowCount:showCount];

	[self clearCloseTimer];

	if(totalCount > 0){
		[progProcStatus setIndeterminate:NO];
	}else{
		[progProcStatus setIndeterminate:YES];
		[progProcStatus animate:nil];
	}

	[self updateProcStatus:procCount targetName:targetName];

	[procStatusWindow setTitle:title];
	[progProcStatus startAnimation:self]; 

	if(isOpen == YES){
		[self openProcStatusWindow:nil];
	}

}
//------------------------------------
// updateProcStatus
//------------------------------------
- (void)updateProcStatus:(int)procCount
				targetName:(NSString*)targetName
{

	double	value = 0.0;
	NSString *status = @"";
	int totalCount = [self totalCount];
	BOOL showCount = [self showCount];

	if(totalCount > 0) {
		value = procCount  / (double)totalCount;
		status = [NSString stringWithFormat:@"%d of %d", procCount, totalCount];
	}else{
		value = 1.0;
		if(showCount == YES){
			status = [NSString stringWithFormat:@"%d", procCount];
		}
	}

	// set proc status
	[progProcStatus setDoubleValue:value];	
	[txtTargetName setStringValue:targetName];
	[txtProcStatus setStringValue:status];

	[progProcStatus displayIfNeeded];
	[txtTargetName displayIfNeeded];
	[txtProcStatus displayIfNeeded];
}
//------------------------------------
// updateTargetName
//------------------------------------
- (void)updateTargetName:(NSString*)targetName
{
	[txtTargetName setStringValue:targetName];
	[txtTargetName displayIfNeeded];
}
//------------------------------------
// closeProcStatus
//------------------------------------
- (void)closeProcStatus:(BOOL)withTimer
{
	if(withTimer == YES){
		[self closeProcStatusWindowWithTimer];
	}else{
		[self closeProcStatusWindow:nil];
	}
}
//------------------------------------
// closeProcStatusWindowWithTimer
//------------------------------------
- (void)closeProcStatusWindowWithTimer
{
//	if([self defaultBoolValue:@"optProcWindowCloseAuto"] == YES){
		// set timer for update closer
		[self setCloseTimer:[NSTimer scheduledTimerWithTimeInterval:1.5
								target: self 
								selector:@selector(closeProcStatusWindow:)
								userInfo:nil
								repeats: NO]
		];
		[[NSRunLoop currentRunLoop] addTimer:[self closeTimer] forMode:(NSString*)kCFRunLoopCommonModes];
//	}
}
//------------------------------------
// totalCount
//------------------------------------
- (void)setTotalCount:(int)totalCount
{
	totalCount_ = totalCount;
}
- (int)totalCount
{
	return totalCount_;
}
//------------------------------------
// showCount
//------------------------------------
- (void)setShowCount:(BOOL)showCount
{
	showCount_ = showCount;
}
- (BOOL)showCount
{
	return showCount_;
}

//------------------------------------
// closeTimer
//------------------------------------
- (void)setCloseTimer:(NSTimer*)closeTimer
{
	[closeTimer retain];
	[closeTimer_ release];
	closeTimer_ = closeTimer;
}
//------------------------------------
// clearCloseTimer
//------------------------------------
- (void)clearCloseTimer
{
	if([[self closeTimer] isValid] == YES){
		[[self closeTimer] invalidate];
//		NSLog(@"clearCloseTimer");
	}
	[self setCloseTimer:nil];

}
//------------------------------------
// closeTimer
//------------------------------------
- (NSTimer*)closeTimer
{
	return closeTimer_;
}
//------------------------------------
// windowWillClose
//------------------------------------
-(void)windowWillClose:(NSNotification *)notification
{
	// clearCloseTimer
	[self clearCloseTimer];

	// save window rect
	[self saveWindowRect:procStatusWindow key:@"rectWindowProc"];
}
//------------------------------------
// dealloc
//------------------------------------
- (void)dealloc
{
	[closeTimer_ release];

	[super dealloc];
}

@end
